.\"	NIST-PCTS:151-2 Installation and Testing Guide
.ds T1 NIST-PCTS:151-2
.ds T2 Installation and Testing Guide
.so ../macros
.ds HP 14 12 12 12 10
.ds HF 3 3 3 3 3 3 3
.nr Hc 1				\" Center chapter titles
.nr Hs 7
.nr ## 1 1
.ds Cv \f2Certificate of Validation\fP
.ds Rd \f1IEEE Std 2003.1-1992\fP
.ds Nt \f1NIST-PCTS:151-2\fP
.ds Np \f1NIST-PCTS:151-2\fP
.ds Nl \f1NIST/CSL\fP
.ds Fp \f1FIPS 151-2\fP
.ds Gt NIST POSIX Testing Policy\(em General Information
.ds Ft NIST POSIX Testing Policy\(em Certificate of Validation Requirements \
for FIPS 151-2
.ds Ig NIST-PCTS:151-2\ \(em\ Installation and Testing Guide
.ds Em \f2151-2@sst.ncsl.nist.gov\fP
.ds Px POSIX.1
.ds P3 POSIX.3.1
.ds Pd PCD
.ds Pp PCTP
.ds Ps PCTS
.ds t3 \f1Tests Methods for Measuring Conformance to POSIX\fP
.ds T3 \f1Tests Methods for Measuring Conformance to POSIX.1\fP
.ds Tt \f1Portable Operating Systems Interface(POSIX)\(em Part 1: System Application Program Interface (API) [C Language]\fP
.ds s3 IEEE Std 1003.3-1991
.ds S3 IEEE Std 2003.1-1992
.nr XS 11		\" Point size for document
.\"
.\"    Start of Document
.\"
.rs 0
.PH "'\s12Installation and Testing Guide''\*(Dt'"
.SP 5.0i
.bp
.nr % 1
.EF "''Page \\\\n%''"
.OF "''Page \\\\n%''"
.ce 1
.S 14
\f2ABSTRACT\fP
.SP
.S \\n(XS
The \*(Np,
is a POSIX conformance test
suite (\*(Ps) written by the National Institute of Standards and Technology.
It tests system implementations for conformance
to \*(Fp.
\*(Fp
is based on the \*(S1 (\*(Tt) and the additional requirements
restated in Appendix A of this document.
The assertions tested by this \*(Ps are only those of the \*(Rd (\*(T3)
that relate to \*(Fp.
.P
This Installation Guide specifies the implementation
resources needed and documents the procedures to be
followed to install and execute this \*(Ps.
.SP
.H 1 Introduction
The \*(Np is a POSIX.1 conformance test suite (\*(Ps)
that verifies the compliance of an implementation to \*(Fp.
It is based on the requirements of the 
\*(s3 (\*(t3) and \*(S3 (\*(T3).
The \*(Np only tests the \*(S3 assertions that are applicable to the
requirements of \*(Fp.
.P
To fully understand the limitations and
requirements imposed on the \*(Np, the documents \*(s3, \*(S3, \*(S1,
and the \*(Fp should be studied.
This document assumes that the tester is knowledgeable with the
terms and concepts used in these documents.
.P
If you have questions, suggestions, suggested code, please send
electronic mail to  \*(Em.
Other correspondence may be U.S. mailed to:
.DS
.R
.ta 1.0i
.in 0.5i
National Institute of Standards and Technology
Attn: \*(Np
\*(Nl (Code 872)
Bldg 225,  Room B266
Gaithersburg, MD 20899
.DE
.SK
.rs
.sp |10P
.H 1 General
.SP 2P
.H 2 Notations
The specification of pathnames as used by this document are relative
to the directory
name \f(CWNIST-PCTS\fP.
Therefore, a pathname of \f(CW./src/driver\fP
refers to the relative pathname
\f(CWNIST-PCTS/src/driver\fP.
The header files for this \*(Ps are denoted as \f(CW<pathname_hdr>\fP.
The pathname of these files is
\f(CWNIST-PCTS/include/pathname_hdr\fP.
.P
The \*(S3 will be referenced in this document as \*(P3.
.H 2 "General Terms and Abbreviations"
.BL 4
.LI assertion
.br
A statement of functionality or behavior for a \*(Px
.I element
that is derived from
the \*(P3 standard being tested and that is true for a conforming \*(Px
implementation.
.LI "assertion test"
.br
The software or procedural methods that ascertain
the conformance of a
\*(Px implementation to an
.I assertion. 
.LI "C Standard"
.br
FIPS 160.
The reference text for FIPS 160 is ISO/IEC 9899,
\f2Information technology\(emProgramming languages\(emC.\fP
.LI "element"
.br
A functional interface or a namespace allocation.
Examples of elements are C functions or utility programs.
Examples of namespace allocation include headers or error return value
constants.
.LI "extended assertion"
.br
An
.I assertion
that is not required to be tested.
.LI GTI
\(em general terminal interface
.LI IUT
\(em Implementation Under Test
.LI PCD.1
\(em The POSIX conformance document required by the \*(Px standard.
.LI "POSIX Conformance Test Procedure (PCTP)"
.br
The nonsoftware procedures used in conjunction with the PCTS
to measure conformance.
.LI "POSIX Conformance Test Suite (PCTS)"
.br
The collection of software possibly used in conjunction
with other test methods to measure conformance.
.LI ROFS
\(em read-only file system
.LI "test method"
.br
The software, procedures, or other means specified by a \*(Px standard
to measure conformance.
.P
Test methods
may include a \*(Ps, \*(Pp, or an audit of a \*(Pd.
.LI "test result code"
.br
A value that describes the result of an
.I "assertion test."
.LE
.H 2 Users
This \*(Ps can be used by a vendor or a user to test conformance of
an operating system to the \*(Fp.
This \*(Ps assumes that the environment under test is a \*(Px
compatible environment.
.H 3 "Minimum Implementation Requirements"
This \*(Ps requires the following:
.BL 4
.BL 1
.LI
\*(Fp
.br
The implementation must support FIPS 160 (ISO/IEC 9899: Information
technology\(emProgramming languages\(emC).
Implementations that do not support FIPS 160 may not attain successful
compilation of this PCTS.
.LI
{CHILD_MAX}
.br
This PCTS requires a value of {CHILD_MAX} of at least 10 to successfully
run a number of the assertion tests.
\*(Fp requires a minimum value of {CHILD_MAX} of 25.
.LI
headers
.br
Must be \*(Px compatible.
The tester, \f1for checking only, not for verification\fP, may
copy the system's include files to 
\f(CWNIST-PCTS/include\fP
and then alter as desired to obtain \*(Px compatibility.
.LI
functions/macros
.br
This \*(Ps will test that the required \*(Px functional interfaces
and optionally-provided macros conform to \*(Fp requirements.
.LI
appropriate privileges
.br
This \*(Ps requires the implementation to provide the appropriate
privileges synonymous with the super-user concept.
.LE
.LE
.SK
.rs
.sp |10P
.H 1 "Contents"
.SP 2P
The documentation for this release is this Installation Guide.
.P
This document specifies the procedures required
to configure this \*(Ps and execute its tests.
These procedures are numbered excerpts specified in italics.
.P
This release contains no source code control identifiers.
.H 2 "\*(P3 Assertion Changes"
.H 3 "Assertion Updates"
Assertions conditional on a conditional feature not being documented is
not applicable to \*(Fp.
\*(Fp requires all supported conditional features to be
documented (See \(scA item ``p'').
Therefore, a feature which is not documented is not supported by the
implementation.
Assertions of this nature are not tested and are assigned a test result
code of \f2NOT_APPLICABLE\fP.
.P
The following assertions are expected to be altered or should be altered in
the ensuing IEEE 1003.3.1 standard (see comments preceding the instantiation 
of the ensuing test result code for the element stated).
.BL 4
.BL 1
.LI
.At sym_const 06-08
\(em Replace with:
.BL 4
06(C) If {_POSIX_CHOWN_RESTRICTED} is defined when <unistd.h> is included:
.BL 4
The value of {_POSIX_CHOWN_RESTRICTED} is other than -1.
.LE
.P
07(C) If {_POSIX_NO_TRUNC} is defined when <unistd.h> is included:
.BL 4
The value of {_POSIX_NO_TRUNC} is other than -1.
.LE
.P
08(C) If {_POSIX_VDISABLE} is defined when <unistd.h> is included:
.BL 4
The value of {_POSIX_VDISABLE} is other than -1.
.LE
.LE
.LI
.At fork 27
\(em Replace ``27(PCTS_PROCESS_LIMIT?A:UNTESTED)'' with ``27(A)''.
.LI
.At signal_con 02
\(em Replace ``void(*)(\^)'' with ``void(*)(int)''.
.LI
.At sigaction 4
\(em Replace ``void(*)(\^)'' with ``void(*)(int)''.
.LI
.At sigaction 5-18
\(em Replace ``Testing Requirement'' with ``Note''.
.LI
.At link 65
\(em Replace ``(C)'' with ``(PCTS_FS?C:UNTESTED)''.
.LI
.At rename 68
\(em Replace ``(C)'' with ``(PCTS_FS?C:UNTESTED)''.
.LI
.At fpathconf 28-29
\(em Replace ``(C)'' with ``(PCTS_GTI_DEVICE?C:UNTESTED)''.
.LI
.At settable 53
\(em Replace ``53(PCTS_GTI_DEVICE?C:UNTESTED)'' with ``R08''.
.br
        Append to assertion the text ``See Assertion 69 in 7.1.1.11.)''
.LI
.At tcsetattr 11
\(em Replace ``A:UNTESTED)'' with
.br
        ``C:UNTESTED) If the implementation
supports input and output baud rates that differ:''.
.LI
.At fgetc 8
\(em Replace ``08(C)'' with ``08(PCTS_GTI_DEVICE?C:UNTESTED)''.
.LI
.At fgets 8
\(em Replace ``08(C)'' with ``08(PCTS_GTI_DEVICE?C:UNTESTED)''.
.LI
.At fread 8
\(em Replace ``08(C)'' with ``08(PCTS_GTI_DEVICE?C:UNTESTED)''.
.LI
.At freopen 34
\(em Replace ``34(B) If {OPEN_MAX)\(<={PCTS_OPEN_MAX}:'' with
.br
        ``34({OPEN_MAX)\(<={PCTS_OPEN_MAX}?B:A)''.
.LI
.At getc 8
\(em Replace ``08(C)'' with ``08(PCTS_GTI_DEVICE?C:UNTESTED)''.
.LI
.At getchar 8
\(em Replace ``08(C)'' with ``08(PCTS_GTI_DEVICE?C:UNTESTED)''.
.LI
.At gets 8
\(em Replace ``08(C)'' with ``08(PCTS_GTI_DEVICE?C:UNTESTED)''.
.\"	Need an IEEE Interpretation
.\".LI
.\".At remove 9-10
.\"\(em Replace ``(A)'' with ``(C) If {PCD_LINK_TO_DIRECTORY} is TRUE:".
.LI
.At scanf 8-9
\(em Replace ``(C)'' with ``(PCTS_GTI_DEVICE?C:UNTESTED)''.
.LI
.At fscanf 8-9
\(em Replace ``(C)'' with ``(PCTS_GTI_DEVICE?C:UNTESTED)''.
.LI
.At sigsetjmp 4
\(em Replace ``04(A)'' with ``04(B)''.
.LI
.At tar 20
\(em Replace ``20(C)'' with ``20(D)''.
.LI
.At tar 44
\(em Replace ``44(C)  If the ... ustar:'' with ``44(A)''.
.LI
.At cpio 23
\(em Replace ``23(A)'' with ``23(B)''.
.LI
.At cpio 24
\(em Replace ``24(C)'' with ``24(D)''.
.LE
.LE
.H 3 Interpretations
No interpretations have been rendered at the time of the printing of this
document.
.SP 2
.H 2 Documents
.H 3 \*(Nl
This document and the ``Certificate of Validation Requirements for FIPS 151-2''
document are distributed as printed matter and are provided in \f2./DOCS\fP.
These documents are formatted for use with \f(CWtroff\fP and the
memorandum macro package.
To generate copies of these documents from the sources provided, the user is
required to have the necessary expertise, software, and hardware
to modify and execute the scripts provided in \f2./DOCS\fP.
.H 3 Assertions
The assertions tested by the \*(Np are those in \*(S3.
.H 2 "Source Controls"
The complete set of files whose contents may be changed are:
.BL 4
.BL 1
.LI
Header files in \f(CW./include\fP (See \(sc6. and \(scB):
.br
\h'1.0i' <config>, <install.h>, <iso646_natv.h>, and <pcts_time.h>.
.LI
Implementation dependent file for compiling and installing the \f(CWinstall\fP
module.
(See \(sc6.1):
.br
\h'1.0i' \f(CW./src/install/s_install\fP.
.LI
Compilation scripts:
.br
\h'1.0i' \f(CW./*/*/src/*/svcomp\fP\ \(em
.br
\h'1.0i' Changes restricted to library references only.
.LI
Implementation dependent file for interfacing with
.Fn mtrwfs ,
.Fn mtrofs ,
.Fn unmtfs ,
.R and
.Fn open_ctty
(See \(sc6.2):
.br
\h'1.0i' \f(CW./src/tools/tssf/tssf.c\fP
.LI
Implementation dependent file for Data Interchange Format testing.
.R
(See \(sc7.4):
.br
\h'1.0i' \f(CW./bin/dif_script\fP
.LE
.LE
.P
A \f1tester\fP shall not change, replace, or alter any other files or
permission modes of any file in this \*(Ps.
The exception is those files in \f(CW./CERT_data\fP which are templates
for the actual APTL generated files (see \*(Ft).
.SK
.rs
.sp |10P
.H 1 "Resources"
.SP 2P
You will need the following resources to
successfully install this \*(Ps on a development or target system:
.BL 4
.BL 1
.LI
.B "User Account"
- A new user account should be made available for loading and testing
this \*(Ps.
See \(sc5.1.
.LI
.B "Time"
- The time required to install and execute the \*(Np
depends on the IUT's speed and peculiarities. 
On our in-house 25 SPECmark system, we installed the entire test suite
in less than three hours and executed in less than two hours.
.LI
.B "Disk Space"
- The disk storage required when reading this distribution
is approximately 8 MB.
The DOCS and ass_NIST-PCTS directories, 1MB, may be deleted.
Disk storage for execution modules will vary from one implementation
to another.
Implementations we have tested total
storage requirements varied from 75MB to 200MB.
.P
The Certification Report can be generated on a \*(Px Section basis when
the IUT has inadequate disk space to process all sections contiguously.
.LI
.B "Utilities"
- \*(Np requires the following utilities:
.BL 4 1
.BL 1
.LI
\f(CWar\fP
or a compatible library archiver;
.LI
\f(CWgrep\fP or a compatible pattern retriever, 
.LI
\f(CWcc -E\fP or a compatible macro preprocessor,
.LI
extended \f(CWcpio\fP creating and reading utility
to read the \*(Np distribution and create an extended \f(CWcpio\fP archive;
.LI
extended \f(CWtar\fP format creating and reading utility
to read and create an extended \f(CWtar\fP archive;
.LI
\f(CWmkdir\fP
or a compatible directory maker;
.LI
\f(CWmknod\fP
or a compatible special file builder;
.LI
\f(CWmv\fP
or a compatible file mover;
.LI
\f(CWranlib\fP,
or the System V
\f(CWar/ld\fP
combination;
.LI
\f(CWrm\fP
or a compatible file remover;
.LI
\f(CWrm -r -f\fP
or a compatible directory tree remover;
.LE
.LE
.LI
.B Printer
- for documents and possibly source listings;
.LI
.B "Terminal Ports"
- The \*(Np requires two terminal ports connected \f1closed-loop\fP (\(sc6.2.1)
for performing general terminal interface testing.
.LI
.B "POSIX Headers"
- The installation modules require those elements and structures
defined in \*(Px headers to be present for compilation to be successful.
.LI
.B "POSIX Functionality"
- The library module \f(CWsv_sys/sv_setup.c\fP
used in every test component module requires 
.Fn sysconf .
The modules in the \f(CWz_base\fP
library require a functional interface to most of the 
\*(Px functions.
.LI
.B "File Systems"
- This \*(Ps requires an implementation to provide an additional file system
to perform [EROFS] and [ENOSPC] testing (see \(sc6.2.2).
.LI
.B "Idle System"
- This PCTS should be run on an idle system.
.LE
.LE
.SK
.rs
.sp |10P
.H 1 "Getting Started"
.SP 2P
.H 2 "User Accounts"
This \*(Ps should be installed into a newly created \f1test\fP user account.
The sole purpose of this account should be to run the \*(Ps test suite.
.P
The user-ID and group-ID for the \f1test\fP user account should
contain different values for user-ID and group-ID.
The test directory where the \*(Ps is to be loaded must
be set to the \f1test\fP user's
user-ID and group-ID.
The user account \f1login\fP name must agree with the \f(CWconfig\fP
(\(scB) required parameter
.B LOGNAME .
.H 2 Environment
This \*(Ps requires that the \f1tester\fP execute with no restrictions
on created files.
.BL 10
.BL 1
.I
.LI "<\\n(##>
.nr ## +1
.LI
.ta 1.0i
Execute:	umask\h'0.3i'0
.R
.LE
.LE
.H 2 "Reading the Distribution"
The \*(Nt is distributed in extended \f(CWcpio\fP data interchange format.
.P
Login as the tester and transfer the PCTS to the implementation to be tested.
Modification time of files on the \*(Np distribution must be preserved
or failures will be reported during verification.
.BL 10
.BL 1
.I
.LI "<\\n(##>
.nr ## +1
.LI
.ta 1.0i
Execute:	cd\h'0.3i'tester_login_directory
.br
	cpio\h'0.3i'-icBdmu[v]\h'0.3i'<\h'0.3i'distribution_file
.P
Note1: The actual command to read the \*(Np distribution may vary 
on your implementation.
.P
Note2: We read the \*(Np distribution from a networked remote tape drive
with the command:
.br
\ \ \ rsh\h'0.1i'remote_host\h'0.1i'dd\h'0.1i'if=/dev/rmt0\h'0.1i'ibs=5120\h'0.2i'|\h'0.2i'cpio\h'0.1i'-icBdumv
.P
Note3: The \*(Np distribution contains 7360 blocks.
.R
.LE
.LE
.P
.H 2 "Apply Applicable Patch"
Apply the latest FIXES Patch and BETA Patch if applicable.
These patches are distrbuted by e-mail to those who have purchased
the \*(Np and have registered with \*(Nl.
To register with \*(Nl send an e-mail message to 151-2@sst.ncsl.nist.gov.
(See Appendix G for procedures for installing patches).
.BL 10
.BL 1
.ta 1.0i
.I
.LI "<\\n(##>"
.nr ## +1
.LI
Strip off e-mail header info from FIXES_PATCH_mm_dd_yy.
.LI
Execute:	sh\h'0.1i'FIXES_PATCH_mm_dd_yy\h'0.1i'2>FP

	Ensure no problems were encountered.

.BL 3
.BL 1
.LI "<\(*F>"
This step installs the active BETA Patch.
This step should only be performed for BETA testing.
.LI
Execute:	sh\h'0.3i'BETA_PATCH_mm_dd_yy\h'0.1i'2>BP

	Ensure no problems were encountered.
.LE
.LE
.LE
.LE
.H 2 "Owner and Group Adjustment"
The files read must be provided with the proper user ID and group ID of
the tester.
The procedure for accomplishing this can vary among implementations.
.BL 10
.BL 1
.I
.LI "<\\n(##>"
.nr ## +1
.LI
.ta 1.0i
Ensure:	The \*(Ps files are owned by the tester and have the tester's group ID.
.P
Note: On our implementation we perform the following commands.
.P
	chown\h'0.1i'-R\h'0.1i'tester\h'0.1i'NIST-PCTS
.br
	chgrp\h'0.1i'-R\h'0.1i'tester\h'0.1i'NIST-PCTS
.R
.LE
.LE
.H 2 "Tape Contents"
The \*(Np distribution contains the following directory trees:
.in +0.5i
.DS
.ta 1.0i 2.5i
.R
CERT_data	- certification modules
DOCS	- documents (written for \f(CWtroff\fP and mm macros)
STD	- POSIX.1 source test modules
STD/DEF	- \*(Px Chapter 2  (Definitions & General Requirements)
STD/PP	- \*(Px Chapter 3  (Process Primitives)
STD/PE	- \*(Px Chapter 4  (Process Environment)
STD/FD	- \*(Px Chapter 5  (Files & Directories)
STD/IO	- \*(Px Chapter 6  (Input & Output)
STD/DC	- \*(Px Chapter 7  (Device & Class-Specific Functions)
STD/LS	- \*(Px Chapter 8  (C Programming Language)
STD/PW	- \*(Px Chapter 9  (Passwords)
STD/DIF	- \*(Px Chapter 10 (Data Interchange Format)
bin	- executable file depository
data	- used by utility \f(CWverify\fP
include	- headers
lib	- archive library files depository
src	- commands, libraries, etc
src/Aids	- configuration files to test IUT for specific operating system
src/tools	- libraries
scripts	- used by utility \f(CWinstall\fP
tmp	- used by utility \f(CWverify\fP
.in -0.5i
.DE
.H 2 "\*(Np Modules"
The \*(Np provides the following utilities:
.BL 4
.BL 1
.LI
\f(CWinstall\fP
.br
The \f(CWinstall\fP command builds the executable modules of the
\*(Np.
.LI
\f(CWverify\fP
.br
The \f(CWverify\fP command is the \*(Np driver.
It can execute test components on an
individual or on a SECTION-by-SECTION basis.
The \f(CWverify\fP command has some options,
when executed on a SECTION-by-SECTION basis only,
which control
the amount of data to be retained from running a test component.
The \f1-d\fP option retains the temporary files.
The \f1-k\fP option retains the Raw Journal Report files for each test
component in the associated \f(CWadm\fP directory
under the name \f(CW\.component_test_name\fP.
The -K option retains the Raw Journal Report only if
an error in testing was detected.
The default is no options.
On completion of each test component,
\f(CWverify\fP will produce an output report.
.P
When \f(CWverify\fP is executed on a component basis the Raw Journal Report
is the only journal provided, and this is sent to the standard
output device.
.LI
\f(CWsetaccess\fP
.br
The \f(CWsetaccess\fP module traverses the entire \*(Np directory tree and
assigns the proper ownership and protection modes to selected component
test modules, data files and directories.
Module \f(CWsetaccess\fP touches each data file and will report
all missing data files.
.LI
\f(CWclraccess\fP
.br
The \f(CWclraccess\fP module traverses the entire \*(Np directory tree and
resets the ownership and protection modes set by \f(CWsetaccess\fP to their
original modes.
.LI
\f(CWvalidate\fP
.br
The \f(CWvalidate\fP module compares the tests result codes from the
output journals with the expected test result codes and reports all
discrepancies to \f(CWstdout\fP.
.LE
.LE
.SK
.rs
.sp |10P
.H 1 "\*(Np Configuration Procedures"
.SP 2P
When the \*(Np is to be rerun for an IUT, then skip to \(sc6.4
to reinstall the configuration files.
.P
.H 2 \f(CWinstall\fP
The \f(CWinstall\fP utility builds the executable modules of the
\*(Np.
.P
The \*(Np support utilities are Strictly Conforming Applications.
These utilities will compile and execute on all \*(Px Conforming
Implementations.
To allow an implementation's utilities to interface with the \*(Np
support
utilities, this \*(Ps allows the \f1tester\fP to define the command
syntax for the implementations utilities.
The \f(CW<install.h>\fP header file 
specifies the syntax for the implementation utilities.
.P
The symbol \f(CWPATH_CMD\fP will be prefixed to commands which do
not begin with a ``/''.
This allows commands to be specified without a common
absolute pathname.
Commands must be regular, executable files, not 
.I
shell
.R
scripts.
.DS
.BL 10
.BL 1
.I
.LI "<\\n(##>"
.nr ## +1
.LI
Edit \f(CW<install.h>\fP to define \f(CWPATH_CMD\fP and
the command syntax for all the commands specified in \f(CWstruct cmd_imp\fP.
.P
.Ns
Some of the assertion tests rely on the \f(CW'\fPC\f(CW'\fP compiler
to provide the required testing.
The APTL must exert proper judgement to ensure that if compilation flags
are used for compilation commands they do not suppress warnings that
are actually \*(Fp errors.
.Ne
.LI
If your implementation supports the System V \f(CWar/ld\fP combination, then the
\f(CWranlib\fP command is not required.
.R
.LE
.LE
.DE
.P
The module \f(CW./src/install/s_install\fP is a script file
that contains the commands
for compiling and loading the \f(CWinstall\fP program.
.DS
.BL 10
.BL 1
.I
.LI "<\\n(##>"
.nr ## +1
.LI
Modify \f(CWs_install\fP as needed by the IUT to compile and install
the \f(CWinstall\fP program..
.R
.LE
.LE
.DE
.H 2 "Target System Support Library"
The \*(Np uses a library to interface required
.I "target system support facilities."
File \f2tssf.c\fP in directory \f(CW./src/tools/tssf\fP
is provided as an example.
The target system support routine will be archived in
library \f(CW./lib/tssf.a\fP.
.H 3 "Terminal Device Configuration"
Many tests in this \*(Ps require two interconnected terminal device files.
This section specifies the configuration and what is required
from the installed configuration.
.P
Two interconnected terminal device ports on the same machine, with each port
simulating the presence of a terminal provides for error-free and
repeatable testing.
Currently, only those implementations which have a hardware
configuration with at least two asynchronous communications ports available
for use by this \*(Ps will
be able to show compliance with those assertions that require this
configuration.
.P
The terminal special files corresponding to \f1PORT_TTY\fP and
\f1PORT_LOOP\fP in ``./include/config'' should
be configured on the system with modem control enabled.
The special files' modes should be
set to give read and write permission to the \f1test\fP user account.
.P
Terminal devices are required to test those assertion numbers listed in
the tables of Section 12 of this document, under the column marked GTI_dev.
.P
When enabling asynchronous ports with a RS232 loop-back cable connection,
GTI_dev marked assertion tests expect to open each side
on the loop-back connector
without blocking, such that a blocking \f1open()\fP would succeed.
Data read/written on one port can be written/read on
the other port respectively.
Also a last \f1close()\fP on one side (with HUPCL set) would de-assert the line
such that the other side detects it.
This implies that the system asserts a RS232 line with RTS/CTS
or DTR/DSR on \f1open()\fP, but also that any form of hardware
handshaking will not de-assert the opened line.
.DS
.BL 10
.BL 1
.I
.LI "<\\n(##>"
.nr ## +1
.LI
Connect two terminal device ports back-to-back (\f1closed-loop\fP).
.R
.LE
.LE
.DE
.H 3 "Mounting and Unmounting File Systems"
Ability to mount and unmount file systems is an implementation feature
not provided by \*(Px.
\*(Px does require a conforming implementation to support
read-only file systems and read-write file systems.
.P
To test these file systems 
the \*(Np requires the tester to provide the interface routines
.Fn mtrwfs ,
.Fn mtrofs ,
.R
and
.Fn unmtfs .
This interface must successfully perform the functionality as described below.
The functionality can be performed programmatically or semi-manually.
.P
Modules that use these interfaces are provided appropriate privileges.
.H 4 "Mount with Read/Write Capabilities"
.H 5 "Synopsis"
.BL 6
.I
.LI " "
int mtrwfs(dirname, fsname)
.LI " "
char *dirname;
.LI " "
char *fsname;
.R
.LE
.H 5 "Description"
The
.Fn mtrwfs
function will mount the file system specified by \f(CWfsname\fP
on the mount point specified by \f(CWdirname\fP.
The file system mounted should be of size 10 kilobytes or larger.
A mountable filesystem much larger than 10 kilobytes will only
result in a larger setup time when preparing to test
assertions which depend on a full file system.
The mount point \f(CWdirname\fP will be specified by \*(Np.
The file system must be mounted in a manner that allows both read and 
write access to the file system.
.H 5 Returns
Upon successful completion,
.Fn mtrwfs
shall return a value of zero.
Otherwise, a value of -1 is returned.
.H 4 "Mount with Read Capabilities"
.H 5 "Synopsis"
.BL 6
.I
.LI " "
int mtrofs(dirname, fsname)
.LI " "
char *dirname;
.LI " "
char *fsname;
.R
.LE
.H 5 "Description"
The
.Fn mtrofs
function will mount the file system specified by \f(CWfsname\fP
on the mount point specified by \f(CWdirname\fP.
The mount point \f(CWdirname\fP will be specified by \*(Np.
The file system must be mounted in a manner that allows only read
access to the file system.
.H 5 Returns
Upon successful completion,
.Fn mtrofs
shall return a value of zero.
Otherwise, a value of -1 is returned.
.H 4 "Unmount a File System"
.H 5 "Synopsis"
.BL 6
.I
.LI " "
int unmtfs(dirname, fsname)
.LI " "
char *dirname;
.LI " "
char *fsname;
.R
.LE
.H 5 "Description"
The \f1unmtfs()\fP function will unmount the file system
specified by \f(CWfsname\fP
which is currently mounted on the mount point specified by \f(CWdirname\fP.
On completion the specified directory \f(CWdirname\fP may be used as the mount
point for another file system.
.P
Both the directory name and the file system are passed to this function
as it is unknown which of these will be used to effect the unmount.
.H 5 Returns
Upon successful completion,
.I unmtfs()
shall return a value of zero.
Otherwise, a value of -1 is returned.
.H 3 "Establish a Controlling Terminal"
Ability to establish a controlling terminal
is an implementation feature not provided by \*(Px.
\*(Px does allow a conforming implementation to support a general
terminal interface for controlling asynchronous communications ports.
.P
A conforming implementation that supports asynchronous communications ports
must interface to the routine 
.Fn open_ctty .
.R
This interface must successfully perform the functionality as described below.
.H 4 "Establish a Controlling Terminal"
.H 5 "Synopsis"
.BL 6
.I
.LI " "
int open_ctty(path, flags, mode)
.LI " "
char *path;
.LI " "
int flags;
.LI " "
mode_t mode;
.R
.LE
.H 5 "Description"
The
.Fn open_ctty
function will establish the terminal device
specified by \f(CWpath\fP as the controlling terminal
with permissions of \f(CWmode\fP.
.H 5 Returns
Upon successful completion,
.Fn open_ctty
shall open the controlling terminal device and shall return a non-negative
integer representing the file descriptor for the controlling terminal
device.
Otherwise, a value of -1 is returned.
.H 3 "Configuring Target System Support Facilities"
.BL 10
.BL 1
.I
.LI "<\\n(##>"
.nr ## +1
.LI
Edit file \f(CWtssf.c\fP in \f(CW./src/tools/tssf\fP
as required for your implementation.
.R
.LE
.LE
.H 2 "config"
The \*(Np uses the file \f(CW<config>\fP to specify all the installation
parameters required for testing \*(Px conformance.  
Appendix B provides the explanation of the symbols used in \f(CW<config>\fP
and the details on the specifications of what is required of the tester
to define the IUT.
.P
.Ns
APTL's should complete the template in Appendix A of \f2\*(Ft\fP before
tackling the configuration of \f(CW<config>\fP.
By following this systematic approach, much of the input required by
\f(CW<config>\fP will have been derived from the \*(Pd and the data
reported by the APTL, for this template, will agree
with that used by \f(CW<config>\fP.
.Ne
.DS
.BL 10
.BL 1
.I
.LI "<\\n(##>"
.nr ## +1
.LI
Edit \fR<config>\fP to define all required parameters.
.R
.LE
.LE
.DE
.P
The \f(CW./scripts\fP directory contains script files which are used by 
\f(CWinstall\fP to compile and install the \*(Np modules.
Scripts exist for compiling and installing this \*(Ps.
.H 2 "Reinstall of Configuration Files"
Once the IUT has been configured, steps <5> thru <9> completed,
all the configuration files required for the IUT should be placed into
a non-hierarchial test directory.
.P
When the configuration files to be modified for the IUT are available
in a single directory,
the script file, \f(CW./bin/config_iut\fP, is available for the tester to use.
When this script is executed the \*(Np configuration files present
in the directory specified will be installed.
The \*(Np distribution configuration files will be retained with a
trailor label name of ``_DISTR''.
.P
.DS
.BL 10
.BL 1
.I
.LI "<5-9>"
.LI
Example:\f2\h'0.3i'./bin/config_iut\h'1m'./src/Aids/oper_system\fP
.R
.LE
.LE
.DE
.P
This script file may be executed repeatedly during configuration procedures.
.P
A tester's procedure should be to copy the changed configuration files
to a test
directory once the configuration files are initially established.
If configuration are to be modified, then only perform the changes in the
test directory and execute \f2./bin/config_iut\fP to reinstall all
configuration files needed.
This procedure simplifies the task of updating and maintenance of
configuration files for an IUT.
.P
The directory \f(CW./src/Aids\fP contains anonymous contributions of
configuration files that have
been used on specific implementations.
No guarantees or claims are associated with these files that when
used on the hardware and operating system stated that the IUT will
sucessfully run the \*(Nt.
Further more, no guarantees or claims are associated with these files
that they were configured properly for the \*(Nt nor the IUT tested.
These configuration files are provided as examples of what a tester did
use to run a version of the \*(Nt during its development.
A tester may be able to use one of the sets of configuration files
provided to test an IUT.
.SK
.rs
.sp |10P
.H 1 "Installing \*(Np"
.SP 2P
.H 2 "Install Modules"
Install the \f(CWinstall\fP program.
.DS
.BL 10
.BL 1
.I
.LI "<\\n(##>"
.nr ## +1
.LI
Execute:\h'0.3i'cd ./src/install
.LI
Execute:\h'0.3i'./s_install
.LI
Execute:\h'0.3i'cd ../..
.R
.LE
.LE
.DE
.P
The \*(Np libraries, \f(CWverify\fP, \f(CWsetaccess\fP and \f(CWclraccess\fP modules
can now be installed.
The time required on a 20 SPECmark system was approximately 20 minutes
(120 SPECmark system, 8 minutes).
.P
The output from \f(CWinstall\fP, Installation Report, is generated as
the filename \f(CWjournal\fP.
To ensure a prior Installation Report is retained, rename the output filename.
If a \*(Cv is to be obtained,
the Installation Report obtained from the execution of \f(CW./scripts/utilities\fP
must be renamed to \f2util_inst_rep\fP.
.DS
.BL 10
.BL 1
.I
.LI "<\\n(##>"
.nr ## +1
.LI
Execute:\h'0.3i'./bin/install\h'1m'./scripts/utilities
.LI
Correct any Installation Report errors and warnings that violate
\*(Fp conformance requirements before continuing. 
.LI
Execute:\h'0.3i'mv\h'1m'journal\h'1m'util_inst_rep
.R
.LE
.LE
.DE
.H 2 "Install Test Components"
The test components can now be installed.
The time required on a 20 SPECmark system was approximately 2.25 hours
(120 SPECmark system, 35 minutes).
.DS
.BL 10
.BL 1
.I
.LI "<\\n(##>"
.nr ## +1
.LI
Execute:\h'0.3i'./bin/install\h'1m'./scripts/NIST-PCTS
.R
.LI
Correct any Installation Report errors and warnings that violate
\*(Fp conformance requirements before continuing. 
.LE
.LE
.DE
.Ns
Test load module names have a length less than {POSIX_NAME_MAX}.
Each test module's name is derived from the \*(Px subclause designation
and the corresponding \*(P3 starting assertion number being tested.
(Ex. The name for the module that tests assertion #4 of
.Fn fork
would be 3.1.1_04.)
When multiple functions have the same \*(Px subclause designation, then
an additional alphabetic letter follows the \*(Px subclause designation.
(Ex. The name for the module that starts with code to test assertion #4 of
.Fn wait
and
.Fn waitpid
would be 3.2.1a_04 and 3.2.1b_04 respectively.)
When a test requires multiple load modules then succeeding modules
are designated with a trailing alphabetic letter starting with ``x''.
.Ne
.SK
.Ns
When multiple \*(Px functions have similar requirements for testing.
Instead of the assertion number, a name designation is provided.
The complete set of tests that have this designation are:
.in +1.0i
.nf
.ta 0.50i
syn	synopsis section testing
pth	pathname testing
acc	access testing
.in -1.0i
.Ne
.H 2 "Special Files"
.DS
.BL 10
.BL 1
.I
.LI "<\\n(##>"
.nr ## +1
.LI
If IUT supports character special devices, then create a character
special file with pathname of
\&\f(CW./STD/DIF/data/dif.d/type.d/mode_cks.d/S_ISCHR\fP
with the user ID and group ID of the tester.
.P
Example:
.br
.BL 2
mknod\h'0.1i'./STD/DIF/data/dif.d/type.d/mode_cks.d/S_ISCHR c 77 77
.br
chown\h'0.1i' tester ./STD/DIF/data/dif.d/type.d/mode_cks.d/S_ISCHR
.br
chgrp\h'0.1i' tester ./STD/DIF/data/dif.d/type.d/mode_cks.d/S_ISCHR
.LE
.LI
If IUT supports block special devices, then create a block
special file with pathname of
\&\f(CW./STD/DIF/data/dif.d/type.d/mode_cks.d/S_ISBLK\fP
with the user ID and group ID of the tester.
.P
Example:
.br
.BL 2
mknod\h'0.1i'./STD/DIF/data/dif.d/type.d/mode_cks.d/S_ISBLK b 77 77
.br
chown\h'0.1i' tester ./STD/DIF/data/dif.d/type.d/mode_cks.d/S_ISBLK
.br
chgrp\h'0.1i' tester ./STD/DIF/data/dif.d/type.d/mode_cks.d/S_ISBLK
.LE
.R
.LE
.LE
.DE
.H 2 "Special Permissions"
The \*(Np requires many of the modules in the test suite to execute as
a specific user and many of the data files and directories to possess
special ownership and/or permissions.
The \f(CWsetaccess\fP module provides these attributes.
This module must be setup to execute as the super-user.
.DS
.BL 10
.BL 1
.I
.LI "<\\n(##>"
.nr ## +1
.LI
As the super-user, set the owner of \f(CW./bin/setaccess\fP to be super-user
and set the mode bits S_ISUID, S_IXUSR and S_IXGRP.
.R
.LE
.LE
.DE
.DS
.BL 10
.BL 1
.I
.LI "<\\n(##>"
.nr ## +1
.LI
Execute:\h'0.3i'./bin/setaccess
.R
.LE
.LE
.DE
.H 2 "Data Interchange Format"
A conforming implementation must
be able to read and write
both \f(CWcpio\fP and extended \f(CWtar\fP data interchange formats.
The \*(Np uses a set of data files to create \f(CWcpio\fP and extended
\f(CWtar\fP archives.
A Bourne shell script is used to establish test files.
This script creates the \f(CWcpio\fP and extended \f(CWtar\fP archives
with its header and source data using characters as represented in
ISO/IEC 646 IRV, and extracts data files from the \*(Ps distributed
\f(CWcpio\fP and extended \f(CWtar\fP archives.
.DS
.BL 10
.BL 1
.I
.LI "<\\n(##>"
.nr ## +1
.LI
Header file <iso646_natv.h> specifies a mapping between ISO/IEC 646 IRV and the
IUT's local codeset.
Edit <iso646_natv.h> if needed.
.R
.LE
.LE
.DE
.DS
.BL 10
.BL 1
.I
.LI "<\\n(##>"
.nr ## +1
.LI
Edit the script \f(CW./bin/dif_script\fP for proper functioning on the
implementation to be tested.
.P
.LI
Execute script file:\h'0.3i'./bin/dif_script
.R
.LE
.LE
.DE
.SK
.rs
.sp |10P
.H 1 "Testing \*(Np"
.SP 2P
.H 2 "Functional Interfaces"
The \*(Np provides for the \f2tester\fP to sequentially run all
the test components by executing a single command (\f(CWverify\fP).
The Raw Journal Report for each component tested is placed in the file 
\&\f(CW./STD/POSIX_CHAPTER/adm/.test_component\fP
and the Output Report is placed in the file
\&\f(CW./STD/POSIX_CHAPTER/adm/journal\fP.
.R
The POSIX_CHAPTER designation is the appropriate directory
DEF, PP, PE, FD, IO, DC, LS, PW, DIF corresponding to the POSIX chapter where
the test component is specified.
The time required to execute Step <18> on a 20 SPECmark system 
was approximately 90 minutes.
.DS
.BL 10
.BL 1
.I
.LI "<\\n(##>"
.nr ## +1
.LI
Execute:\h'0.3i'./bin/setaccess\h'7m'(Additional insurance)
.LI
Execute:\h'0.3i'./bin/verify\h'1m'IUT
.R
.LE
.LE
.DE
.Ns
A report submitted to \*(Nl for \*(Fp Validation, when adequate
storage exists, must have been
generated by the preceding command.
When the implementation has limited storage and can only compile and execute
the PCTS in sections, then the sections of the report must be installed
and verified in the following order: STD/DEF, STD/PP, STD/PE, STD/FD,
STD/IO, STD/DC, STD/LS, STD/PW, STD/DIF
(See \f2\*(Ft\fP).
.Ne
.P
Test components may be run a setion or multiple section at a time
by specifying the appropriate test directory to \f(CWverify\fP.
.DS
.BL 10
.BL 1
.I
.LI "<Ex.>"
.LI
Execute section ``Files and Directories'' tests:
.br
\h'1.2i'./bin/verify\h'1m'-k\h'1m'STD/FD
.LI
Execute multiple section tests:
.br
\h'1.2i'./bin/verify\h'1m'-k\h'1m'STD/DEF STD/PP STD/PE STD/FD
.R
.LE
.LE
.DE
.P
Test components may be run a component at a time by specifying the
test component to \f(CWverify\fP.
When \f(CWverify\fP is executed by test component the Raw Journal Report
is only outputted to the standard output device.
.DS
.BL 10
.BL 1
.I
.LI "<Ex.>"
.LI
Execute component:\h'0.3i'./bin/verify\h'1m'STD/FD/link
.R
.LE
.LE
.DE
.SK
.rs
.sp |10P
.H 1 Reports
.SP 2P
The \*(Np produces three reports.
.H 2 "Installation Report"
The Installation Report captures all the commands that are used to generate
the \*(Np utilities and test modules and all output results reported
by the implementations utilites used.
Installation reports are placed in the file \f(CWjournal\fP.
Move any journals you want to save to another file before
running the \f(CWinstall\fP command again.
.P
An example of the Installation Report
is provided in Appendix C.
.H 2 "Raw Journal Report"
The Raw Journal Report provides a step-by-step log of what the \*(Np requires
for each test, the preparations taken for each test, and the results
of each test.
This log is in sufficient detail such that any failure can be quickly 
pointed to a specific line of the actual source code where the test failed.
.P
Raw journal reports, if the \f1-k\fP or \f1-K\fP option
(See \(sc5.7, \f(CWverify\fP)
is used,
are placed in the \f(CW./STD/Section/adm\fP directories.
Where \f(CWSection\fP is the name of a particular section, e.g., \f(CWFD\fP.
A separate Raw Journal Report is generated for each component tested.
The name of each Raw Journal Report is the name of the component
preceded by a `.'.
(e.g., The Raw Journal Report for
.Fn link
is \f2.link\fP.
.P
A Raw Journal Report for
.Fn cfgetispeed
is provided in Appendix E.
.H 2 "Output Report"
The \*(Np output report is based on the \*(s3.
The report consists of a test result code for every assertion of each element.
The test result code is one of the following:
.BL 10
.BL 1
.LI
\f2PASS\fP -
successful test of a base assertion.
.LI
\f2FAIL\fP -
unsuccessful test of a base assertion.
.LI
\f2PASS_EXTENDED\fP -
successful test of an extended assertion.
.LI
\f2FAIL_EXTENDED\fP -
unsuccessful test of an extended assertion.
.LI
\f2UNTESTED\fP -
the test for this extended assertion does not exist for the target
system.
.LI
\f2UNSUPPORTED\fP -
the conditional feature was not tested because it was not implemented.
.LI
\f2NOT_APPLICABLE\fP -
The assertion is not applicable to the profile as required by \*(Fp.
.LI
\f2NOT_TESTABLE\fP -
The implementation lacks a PCTS required testing constraint (GTI devices
are not provided) or a PCTS testing parameter's value ({PCTS_NAME_MAX})
is not applicable to the value required by the assertion's testing constraint.
.Ns
When the PCTS determines that the
testing constraint has not been satisfied,
the PCTS uses \f2NOT_TESTABLE\fP instead of the \*(P3 specified
test result code of \f2UNTESTED\fP.
When a \*(P3 testing constraint assertion is extended and the
code to test the assertion is provided,
the test result code is \f2UNTESTED\fP.
.Ne
.LI
\f2UNRESOLVED\fP -
the test for the assertion could not be completed.  Two examples are:
1) a directory was required before the assertion could be tested,
and \f1mkdir()\fP failed;
2)An unexpected signal was sent to the test driver, causing the test to abort.
An \f1UNRESOLVED\fP test result code is an intermediate result
that is required to be resolved to one of the other test result codes.
.LI
\f2NOT_INITIATED\fP -
No test was run for this assertion number.
This \f1test result code\fP
is normally the result of an earlier failure in a test module.
See the Raw Journal Report to determine the reason for failure.
.LE
.LE
.P
The \*(Np shall output \f2UNUSED\fP
instead of a test result code for assertion numbers in the \f2UNUSED\fP
list of each element.
\f2UNUSED\fP
means there is no assertion associated with the number.
.P
Output reports are placed in the \f(CW./STD/Section/adm\fP directories.
Where \f(CWSection\fP is the name of a particular section, e.g., \f(CWFD\fP.
The name of the output reports are \f(CWjournal\fP.
.P
An example of the Output Report
is provided in Appendix F.
.P
A list of potential problems and possible
solutions is provided in Appendix H.
.SK
.rs
.sp |10P
.H 1 "\*(Fp PCTS Conformance Testing"
.SP 2P
.H 2 "Type Checking"
.P
The detection of type checking non-conformity relies upon the ANSI C compiler
to generate an error or a warning message.
.nr ## 1 1
.DS
.BL 10
.BL 1
.I
.LI "<\\n(##>"
.nr ## +1
.LI
Ensure no errors or warnings , which imply errors, are reported in the
compilation phase of the \*(Ps.
When such warning or errors occur, the corresponding final test result codes
are \f1FAIL\fP even if the \*(Np reports them as \f1PASS\fP.
.R
.LE
.LE
.DE
.H 2 "Required Test Result Codes"
The following tables provide the acceptable test result codes other than
\f1PASS\fP for each assertion in \*(S3.
.P
.ce 1
\s14\f3Terminology and General Requirements\fP\s0
.SP 1
.TS
tab (|) center,box;
c l|c|c
c l|c|c
c l|c|c.
Test Code|\&|UNSUPPORTED|UNTESTED
Class|\&|Conditional|Extended
Element|#|other|\&
_
gen_terms|7|\&|1-2,4-7
gen_concept|1|\&|1
err_numbs|3|\&|3
psd_types|5|\&|\&
env_descr|2|\&|\&
c_lang|5|\&|5
num_lmts|17|4-15|16
sym_const|8|6-8|\&
.TE
.SP 1
Additional test result codes:
.BL 6
\f1PASS_EXTENDED\fR\ \(em \ 
.At psd_types 5
.LE
.SK
.ce 1
\s14\f3Process Primitives\fP\s0
.SP 1
.TS
tab (|) center,box;
l l|c s|c s|l
c l|c s|c s|l
l l|l l|l l|l.
Test Code|\&|UNSUPPORTED|NOT_TESTABLE|UNTESTED
Class|\&|Conditional|Testing Constraint|Extended
Element|#|macro|other|GTI_dev|other|\&
_
fork|28|2-3|\&|20|\&|5-6,26,28
execl|60|2-3|38|\&|38,44-46,49,54-55|22-23,26-27,52,60
execv|60|2-3|38|\&|38,44-46,49,54-55|22-23,26-27,52,60
execle|60|2-3|38|\&|38,44-46,49,54-55|22-23,26-27,52,60
execve|60|2-3|38|\&|38,44-46,49,54-55|22-23,26-27,52,60
execlp|60|2-3|38|\&|38,44-46,49,54-55|22-23,26-27,52,60
execvp|60|2-3|38|\&|38,44-46,49,54-55|22-23,26-27,52,60
wait|15|2-3|\&|\&|\&|\&
waitpid|27|2-3|\&|\&|\&|\&
_exit|13|2-3|\&|11-12|\&|6
signal_con|61|\&|\&|14-15,46|46,53,56|3,9,39,47-48,54,57
\&|\&|\&|\&|53,56,59,61|\&|\&
kill|20|2-3|\&|\&|16|\&
sigemptyset|4|2-3|\&|\&|\&|\&
sigfillset|4|2-3|\&|\&|\&|\&
sigaddset|6|2-3|\&|\&|5|6
sigdelset|6|2-3|\&|\&|5|6
sigismember|6|2-3|\&|\&|5|6
sigaction|21|2-3|\&|\&|\&|\&
sigprocmask|14|2-3|\&|\&|\&|\&
sigpending|5|2-3|\&|\&|\&|5
sigsuspend|9|2-3|\&|\&|\&|\&
alarm|9|2-3|\&|\&|\&|\&
pause|7|2-3|\&|\&|\&|\&
sleep|6|2-3|\&|\&|\&|\&
.TE
.P
Additional test result codes:
.BL 6
\f1NOT_APPLICABLE\fR\ \(em \ 
.At execl 32 ,
.At execv 32 ,
.At exele 32 ,
.At execve 32 ,
.At execlp 32 ,
.At execvp 32 ,
.At kill 19
.br
\f1UNUSED\fR\ \(em \ 
.At execl 9-15 ,
.At execv 9-15 ,
.At exele 9-15 ,
.At execve 9-15 ,
.At execlp 59 ,
.At execvp 59
.LE
.SK
.ce 1
\s14\f3Process Environments\fP\s0
.SP 1
.TS
tab (|) center,box;
l l|c s|c s|l
c l|c s|c s|l
l l|l l|l l|l.
Test Code|\&|UNSUPPORTED|NOT_TESTABLE|UNTESTED
Class|\&|Conditional|Constraint|Extended
Element|#|macro|other|GTI_dev|other|\&
_
getpid|4|2-3|\&|\&|\&|\&
getppid|4|2-3|\&|\&|\&|\&
getuid|4|2-3|\&|\&|\&|\&
geteuid|4|2-3|\&|\&|\&|\&
getgid|4|2-3|\&|\&|\&|\&
getegid|4|2-3|\&|\&|\&|\&
setuid|9|2-3|\&|6-7|8|\&
setgid|9|2-3|\&|6-7|8|\&
getgroups|7|2-3|\&|\&|\&|\&
getlogin|5|2-3|\&|\&|\&|5
getpgrp|4|2-3|\&|\&|\&|\&
setsid|9|2-3|\&|6,8-9|\&|9
setpgid|16|2-3|\&|\&|11|\&
uname|7|2-3|\&|\&|\&|7
time|6|2-3|\&|\&|\&|6
times|11|2-3|\&|\&|\&|11
getenv|8|2-3|\&|\&|\&|8
ctermid|8|2-3|\&|4-5,7|\&|8
ttyname|6|2-3|\&|4|\&|6
isatty|5|2-3|\&|4|\&|\&
sysconf|19|2-3|\&|15-16|\&|17
.TE
.P
Additional test result codes:
.BL 6
\f1NOT_APPLICABLE\fR\ \(em \ 
.At setuid 6-7 ,
.At setgid 6-7 ,
.At setpgid 12 ,
.At sysconf 15-16
.LE
.SK
.ce 1
\s14\f3Files and Directories\fP\s0
.SP 1
.TS
tab (|) center,box;
l l|c s s s|c s|l
c l|c s s s|c s|l
l l|l l l l|l l|l.
Test Code|\&|UNSUPPORTED|NOT_TESTABLE|UNTESTED
Class|\&|Conditional|Testing Constraint|Extended
Element|#|macro|ROFS|modem|other|GTI_dev|other|\&
_
dirent.h|3|\&|\&|\&|\&|\&|\&|3
opendir|35|2-3|\&|\&|31|\&|24-25,30-31|33-35
readdir|13|2-3|\&|\&|\&|\&|12|7,13
rewinddir|5|2-3|\&|\&|\&|\&|\&|2
closedir|7|2-3|\&|\&|5-6|\&|\&|7
chdir|27|2-3|\&|\&|\&|\&|21-22|\&
getcwd|8|2-3|\&|\&|\&|\&|\&|\&
open|67|2-3|65-67|\&|\&|44|56-57|23,34,38-41,
\&|\&|\&|\&|\&|\&|\&|\&|51,59,62
creat|53|2-3|52-53|\&|\&|36|45-46|5-6,22,
\&|\&|\&|\&|\&|\&|\&|\&|32-33,48,50
umask|6|2-3|\&|\&|\&|\&|\&|\&
link|66|2-3|64|\&|9-10,23-24,|\&|5-6,46-52,65|5-6,19-20,35,
\&|\&|\&|\&|\&|37,40,|\&|\&|44,58
\&|\&|\&|\&|\&|61-62,65|\&|\&|\&
mkdir|36|2-3|36|\&|\&|\&|27-30|5-6,9-10,34
mkfifo|33|2-3|33|\&|\&|\&|26-27|5-6,9-10,31
unlink|39|2-3|39|\&|9-10,23,37|\&|30-31|5-6,20-21,
\&|\&|\&|\&|\&|\&|\&|\&|26,29,38
rmdir|36|2-3|36|\&|\&|\&|29-30|5-6,20,22,
\&|\&|\&|\&|\&|\&|\&|\&|24,27
rename|69|2-3|67|\&|40,42,46,|\&|52-55,61-62,68|5-6,19-20,35,
\&|\&|\&|\&|\&|61-62,68|\&|\&|38,41,48,64
stat.h|19|\&|\&|\&|4-5|\&|\&|19
stat|37|2-3|\&|\&|\&|\&|31-32|\&
fstat|14|2-3|\&|\&|\&|\&|\&|\&
access|37|2-3|35|\&|23,24,36,37|\&|28-29|37
chmod|34|2-3|34|\&|\&|\&|5-6,21-23,26-27|\&
chown|37|2-3|35|\&|19|\&|5-6,22,25-26,36|37
utime|36|2-3|36|\&|22-23|\&|5-6,28-29|\&
pathconf|57|2-3|\&|\&|21,23,25,|40-41|22-25,51-52|4-14
\&|\&|\&|\&|\&|27,30,33|\&|\&|\&
\&|\&|\&|\&|\&|36,38,40|\&|\&|\&
fpathconf|25|2-3|\&|\&|5,7,9,11,14|6-9,|\&|\&
\&|\&|\&|\&|\&|17,20,|24-25,|\&|\&
\&|\&|\&|\&|\&|22,24|28-29|\&|\&
.TE
.P
Additional test result codes:
.BL 6
\f1NOT_APPLICABLE\fR\ \(em \ 
.At opendir 16,32 ,
.At chdir 15 ,
.At open 15 ,
.At creat 15 ,
.At link 15,29,38,63,66 ,
.At mkdir 15 ,
.At mkfifo 15 ,
.At unlink 15 ,
.At rmdir 15 ,
.At rename 15,29,47,63,69 ,
.At stat 15 ,
.At access 15 ,
.At chmod 15 ,
.At chown 15,21 ,
.At utime 15 ,
.At pathconf 15
.P
\f1PASS_EXTENDED\fR\ \(em \ 
.At creat 9-10 ,
.At pathconf 17
.LE
.SK
.ce 1
\s14\f3Input and Output Primitives\fP\s0
.SP 1
.TS
tab (|) center,box;
l l|c s|c s|l
c l|c s|c s|l
l l|l l|l l|l.
Test Code|\&|UNSUPPORTED|NOT_TESTABLE|UNTESTED
Class|\&|Conditional|Testing Constraint|Extended
Element|#|macro|other|GTI_dev|other|\&
_
pipe|13|2-3|\&|\&|\&|13
dup|6|2-3|\&|\&|\&|\&
dup2|9|2-3|\&|\&|\&|9
close|13|2-3|\&|\&|\&|6,8-10,13
read|33|2-3|\&|11|\&|14-17,21-22,28
write|42|2-3|\&|12,40|27-29|11,15-17,20,22-24,
\&|\&|\&|\&|\&|41|30-33,38,41
fcntl.h|7|\&|\&|\&|\&|\&
fcntl|52|2-3|46,51|\&|\&|50,52
lseek|13|2-3|\&|\&|\&|\&
.TE
.P
Additional test result codes:
.BL 6
\f1NOT_APPLICABLE\fR\ \(em \ 
.At read 12-13,16 ,
.At write 13-14 ,
.At fcntl 47
.P
\f1PASS_EXTENDED\fR\ \(em \ 
.At write 11,22,41
.LE
.SP 2
.ce 1
\s14\f3Device- and Class- Specific Functions\fP\s0
.SP 1
.TS
tab (|) center,box;
l l|c s s|c s|l
c l|c s s|c s|l
l l|l l l|l l|l.
Test Code|\&|UNSUPPORTED|NOT_TESTABLE|UNTESTED
Class|\&|Conditional|Testing Constraint|Extended
Element|#|macro|modem|other|GTI_dev|other|\&
_
interface|69|\&|64-66,69|9,59,63|1-69|\&|2,68
settable|71|\&|54,|22-23,39-40,|8-34,36,39-58,|33-34|5,13,15,18
\&|\&|\&|56-58|47,48|60-68,71|\&|43-44,49,51
cfgetospeed|6|2-3|\&|\&|\&|\&|\&
cfsetospeed|5|2-3|\&|\&|5|\&|\&
cfgetispeed|4|2-3|\&|\&|\&|\&|\&
cfsetispeed|5|2-3|\&|\&|5|\&|\&
tcgetattr|8|2-3|\&|5,6|4-6|\&|\&
tcsetattr|21|2-3|10|11-12,20|5-21|\&|6,9
tcsendbreak|11|2-3|\&|\&|4-9|\&|\&
tcdrain|11|2-3|\&|\&|4-8,11|11|\&
tcflow|16|2-3|\&|\&|5-13,15|\&|\&
tcflush|14|2-3|\&|\&|5-11,13|\&|\&
tcgetpgrp|9|2-3|\&|\&|4-5,8|\&|\&
tcsetpgrp|11|2-3|\&|6|4,6,8,10-11|\&|\&
.TE
.P
Additional test result codes:
.BL 6
\f1NOT_APPLICABLE\fR\ \(em \ 
.At tcgetpgrp 7 ,
.At tcsetpgrp 7
.P
\f1PASS_EXTENDED\fR\ \(em \ 
.At interface 2
.P
\f1UNUSED\fR\ \(em \ 
.At settable 53
.LE
.SK
.ce 1
\s14\f3Language-Specific Services for C\fP\s0
.SP 1
.TS
tab (|) center,box;
l l|c s s|c s|l
c l|c s s|c s|l
l l|l l l|l l|l.
Test Code|\&|UNSUPPORTED|NOT_TESTABLE|UNTESTED
Class|\&|Conditional|Testing Constraint|Extended
Element|#|macro|ROFS|other|GTI_dev|other|\&
_
assert|2|\&|\&|\&|\&|\&|2
isalnum|4|2-3|\&|\&|\&|\&|4
isalpha|4|2-3|\&|\&|\&|\&|4
iscntrl|4|2-3|\&|\&|\&|\&|4
isdigit|4|2-3|\&|\&|\&|\&|4
isgraph|4|2-3|\&|\&|\&|\&|4
islower|4|2-3|\&|\&|\&|\&|4
isprint|4|2-3|\&|\&|\&|\&|4
ispunct|4|2-3|\&|\&|\&|\&|4
isspace|4|2-3|\&|\&|\&|\&|4
isupper|4|2-3|\&|\&|\&|\&|4
isxdigit|4|2-3|\&|\&|\&|\&|4
tolower|4|2-3|\&|\&|\&|\&|4
toupper|4|2-3|\&|\&|\&|\&|4
acos|4|2-3|\&|\&|\&|\&|4
asin|4|2-3|\&|\&|\&|\&|4
atan|4|2-3|\&|\&|\&|\&|4
cos|4|2-3|\&|\&|\&|\&|4
sin|4|2-3|\&|\&|\&|\&|4
tan|4|2-3|\&|\&|\&|\&|4
cosh|4|2-3|\&|\&|\&|\&|4
sinh|4|2-3|\&|\&|\&|\&|4
tanh|4|2-3|\&|\&|\&|\&|4
exp|4|2-3|\&|\&|\&|\&|4
log|4|2-3|\&|\&|\&|\&|4
log10|4|2-3|\&|\&|\&|\&|4
sqrt|4|2-3|\&|\&|\&|\&|4
ceil|4|2-3|\&|\&|\&|\&|4
fabs|4|2-3|\&|\&|\&|\&|4
floor|4|2-3|\&|\&|\&|\&|4
atan2|4|2-3|\&|\&|\&|\&|4
pow|4|2-3|\&|\&|\&|\&|4
fmod|4|2-3|\&|\&|\&|\&|4
frexp|4|2-3|\&|\&|\&|\&|4
ldexp|4|2-3|\&|\&|\&|\&|4
modf|4|2-3|\&|\&|\&|\&|4
setjmp|4|1-3|\&|\&|\&|\&|4
longjmp|4|2-3|\&|\&|\&|\&|2,4
clearerr|4|2-3|\&|\&|\&|\&|4
fclose|19|2-3|\&|\&|19|\&|6,8-10,16-17
feof|5|2-3|\&|\&|\&|\&|\&
ferror|5|2-3|\&|\&|\&|\&|\&
.TE
.SK
.ce 1
\s14\f3Language-Specific Services for C (Cont'd)\fP\s0
.SP 1
.TS
tab (|) center,box;
l l|c s s|c s|l
c l|c s s|c s|l
l l|l l l|l l|l.
Test Code|\&|UNSUPPORTED|NOT_TESTABLE|UNTESTED
Class|\&|Conditional|Testing Constraint|Extended
Element|#|macro|ROFS|other|GTI_dev|other|\&
_
fflush|11|2-3|\&|\&|11|\&|8-9
fgetc|9|2-3|\&|\&|8-9|\&|\&
fgets|9|2-3|\&|\&|8-9|\&|\&
fopen|34|2-3|34|\&|\&|25-28|29,32
fputc|11|2-3|\&|\&|11|\&|8-9
fputs|11|2-3|\&|\&|11|\&|8-9
fread|9|2-3|\&|\&|8-9|\&|\&
freopen|35|2-3|43|\&|\&|34-37|21,23-25,38,41
fseek|11|2-3|\&|\&|\&|\&|8-10
ftell|6|2-3|\&|\&|\&|\&|\&
fwrite|12|2-3|\&|\&|12|5|9-10
getc|9|2-3|\&|\&|8-9|\&|\&
getchar|9|2-3|\&|\&|8-9|\&|\&
gets|9|2-3|\&|\&|8-9|\&|\&
perror|4|2-3|\&|\&|\&|\&|\&
printf|11|2-3|\&|\&|11|\&|8-9
fprintf|11|2-3|\&|\&|11|\&|8-9
sprintf|4|2-3|\&|\&|\&|\&|4
vprintf|12|3-4|\&|\&|12|\&|9-10
vfprintf|12|3-4|\&|\&|12|\&|9-10
putc|11|2-3|\&|\&|11|\&|8-9
putchar|11|2-3|\&|\&|11|\&|8-9
puts|11|2-3|\&|\&|11|\&|8-9
remove|19|2-3|\&|\&|\&|\&|5-6
rewind|4|2-3|\&|\&|\&|\&|\&
scanf|9|2-3|\&|\&|8-9|\&|\&
fscanf|9|2-3|\&|\&|8-9|\&|\&
sscanf|4|2-3|\&|\&|\&|\&|4
setbuf|4|2-3|\&|\&|\&|\&|4
tmpfile|7|2-3|\&|\&|\&|\&|5,7
tmpnam|4|2-3|\&|\&|\&|\&|4
ungetc|4|2-3|\&|\&|\&|\&|4
abs|4|2-3|\&|\&|\&|\&|4
atof|4|2-3|\&|\&|\&|\&|4
atoi|4|2-3|\&|\&|\&|\&|4
atol|4|2-3|\&|\&|\&|\&|4
.TE
.P
Additional test result codes:
.BL 6
\f1NOT_APPLICABLE\fR\ \(em \ 
.At fopen 15 ,
.At remove 15 ,
.At freopen 15 ,
.At vprintf 2 ,
.At vfprintf 2
.P
\f1PASS_EXTENDED\fR\ \(em \ 
.At freopen 34
.LE
.SK
.ce 1
\s14\f3Language-Specific Services for C (Cont'd)\fP\s0
.SP 1
.TS
tab (|) center,box;
l l|c s s|c s|l
c l|c s s|c s|l
l l|l l l|l l|l.
Test Code|\&|UNSUPPORTED|NOT_TESTABLE|UNTESTED
Class|\&|Conditional|Testing Constraint|Extended
Element|#|macro|ROFS|other|GTI_dev|other|\&
_
rand|4|2-3|\&|\&|\&|\&|4
srand|4|2-3|\&|\&|\&|\&|4
calloc|4|2-3|\&|\&|\&|\&|4
free|4|2-3|\&|\&|\&|\&|4
malloc|4|2-3|\&|\&|\&|\&|4
realloc|4|2-3|\&|\&|\&|\&|4
abort|15|2|\&|\&|12-13|\&|7
exit|17|2-3|\&|\&|15-16|\&|2,9
bsearch|4|2-3|\&|\&|\&|\&|4
qsort|4|2-3|\&|\&|\&|\&|4
strcpy|4|2-3|\&|\&|\&|\&|4
strcat|4|2-3|\&|\&|\&|\&|4
strtok|4|2-3|\&|\&|\&|\&|4
strncpy|4|2-3|\&|\&|\&|\&|4
strncat|4|2-3|\&|\&|\&|\&|4
strcmp|4|2-3|\&|\&|\&|\&|4
strncmp|4|2-3|\&|\&|\&|\&|4
strchr|4|2-3|\&|\&|\&|\&|4
strrchr|4|2-3|\&|\&|\&|\&|4
strcspn|4|2-3|\&|\&|\&|\&|4
strspn|4|2-3|\&|\&|\&|\&|4
strpbrk|4|2-3|\&|\&|\&|\&|4
strstr|4|2-3|\&|\&|\&|\&|4
strlen|4|2-3|\&|\&|\&|\&|4
ctime|15|2-3|\&|\&|\&|\&|\&
asctime|4|2-3|\&|\&|\&|\&|\&
gmtime|4|2-3|\&|\&|\&|\&|\&
localtime|16|2-3|\&|\&|\&|\&|\&
mktime|4|2-3|\&|\&|\&|\&|16
strftime|8|2-3|\&|\&|\&|\&|\&
setlocale|16|2-3|\&|\&|\&|\&|\&
fileno|10|2-3|\&|\&|\&|\&|10
fdopen|13|2-3|\&|\&|\&|\&|13
sigsetjmp|6|1-3|\&|\&|\&|\&|4
siglongjmp|6|2-3|\&|\&|\&|\&|\&
tzset|5|2|\&|\&|\&|\&|2
.TE
.P
Additional test result codes:
.BL 6
\f1NOT_APPLICABLE\fR\ \(em \ 
.At exit 14,18
.SK
.ce 1
\s14\f3 System Database\fP\s0
.SP 1
.TS
tab (|) center,box;
l l|c s|c s|l
c l|c s|c s|l
l l|l l|l l|l.
Test Code|\&|UNSUPPORTED|NOT_TESTABLE|UNTESTED
Class|\&|Conditional|Testing Constraint|Extended
Element|#|macro|ROFS|GTI_dev|other|\&
_
getgrgid|6|2-3|\&|\&|\&|7
getgrnam|1|2-3|\&|\&|\&|6
getpwuid|3|2-3|\&|\&|\&|7
getpwnam|5|2-3|\&|\&|\&|6
.TE
.SP 2
.ce 1
\s14\f3Data Interchange Format\fP\s0
.SP 1
.TS
tab (|) center,box;
l l|c s|l
c l|c s|l
l l|l l|l.
Test Code|\&|UNSUPPORTED|UNTESTED
Class|\&|Conditional|Extended
Element|#|Char Sp|Block Sp|\&
_
dif|11|\&|\&|4-11
tar|45|25,33|26,34|4,8,11,20,30,37,42-43,45
cpio|25|\&|\&|9,16,22-25
.TE
.H 2 "Validation Utility"
.P
The test result codes from the output journal can be compared with the
expected result codes from the tables provided above.
The utility \f(CWvalidate\fP will report all discrepancies to \f(CWstdout\fP.
.nr ## 1 1
.DS
.BL 10
.BL 1
.I
.LI "<\\n(##>"
.nr ## +1
.LI
Execute:\h'0.3i'./bin/validate
.R
.LE
.LE
.DE
.SK
.rs
.sp |10P
.H 1 "\*(Fp PCD Conformance Testing"
.SP 2P
.H 2 "PCD.1 Audit"
\*(Px specifies requirements for the PCD.1 in \*(Px, 1.3.1.2.
These requirements are detailed by the specifications of the documentation
assertions in POSIX.3.1.
A PCD.1 audit had been required for FIPS 151-2 conformance.
IEEE Interpretations stated in August 94,  that a 
conflict exists between the requirements of \*(P1 and the documentation
assertions of POSIX.3.1.
Since NIST/CSL no longer has a document to cite for PCD.1 requirements
the FIPS 151-2 PCD.1 audit requiremnt was dropped on 8/29/94
(see \f2Certificate of Validation Requirements for FIPS 151-2\fP,
7.6 PCD.1 Requirement Dropped).
.H 3 "PCD.1 versus <config>"
The tester must ensure that the parameters specified for \f(CW<config>\fP
are consistent with the specifications of the IUT's PCD.1.
.H 3 "\*(Px Documentation Requirements"
The tester should ensure that the implementation's PCD.1 complies
with the "current understanding" of the documentation requirements
of \*(Px.
.H 3 "Additional \*(Fp Documentation Requirements"
The tester must ensure that the additional documentation requirement
as specified in \*(Fp (See \(scA.p) is met.
See \f2\*(Ft\fP for the specific set of  requirements.
.SK
.rs
.sp |10P
.H 1 "Collecting Data for an \*(Nl Certification Report"
.SP 2P
.H 2 "Generating \f(CWCV_report\fP"
When the APTL desires a Certification Report to request a Validation
Certificate for \*(Fp,
the script \f(CW./bin/gen_cert_rept\fP will generate the required report.
The \f2\*(Ft\fP, \(sc6.*.* provides specific information on the contents of the
required files that constitute the Certification Report.
.P
The file \f(CW./NIST-PCTS/CERT_data/LIST_FILES\fP lists all the files to be
included in the Certification Report.
The APTL must as a minimum collect, from the IUT,
all the required files via the
extended \f(CWcpio\fP format creating utility once testing is completed.
Once this data is collected, only the files:
.in +1.0i
.nf
\&\f(CW./CERT_data/NPTP_appendixA
\&./CERT_data/NPTP_appendixB
\&./CERT_data/alt_svcomp
\&./CERT_data/resolved_trc
\&./CERT_data/imp_features
\&./CERT_data/config_IUT\fP
.in -1.0i
.br
.fi
may be altered by the APTL.
All files configured for the execution of the \*(Np,
and those created on execution and then archived by the APTL for
submission to \*(Nl as part of the Certification Report, must retain
the times at which they were last modified by the \*(Ps.
