#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<termios.h>
#include	<pcts_timer.h>


int main(void)
{
	int		tty_fildes, loop_fildes;
	struct	termios tty_termios, *tty_termios_p = &tty_termios,
					loop_termios, *loop_termios_p = &loop_termios;
	pid_t	pid;
	char	inbuf[_POSIX_MAX_INPUT];
	sigset_t  set;

	setup(E_INTERFACE);
	funct(E_INTERFACE);

	if(GTI_NOT_PROVIDED) {
		GTI_NOT_PROVIDED_RESULT(46, BASE)
		GTI_NOT_PROVIDED_RESULT(47, BASE)
		GTI_NOT_PROVIDED_RESULT(48, BASE)
		GTI_NOT_PROVIDED_RESULT(49, BASE)
		GTI_NOT_PROVIDED_RESULT(50, BASE)
		GTI_NOT_PROVIDED_RESULT(51, BASE)
		done();
	}

	continue_on(ERROR);

	setup(E_INTERFACE);
	funct(E_INTERFACE);

								/* interrupt control character */
	assertion(46, BASE);
	start
		prep("Establish PORT_TTY as the control terminal and PORT_LOOP.");
		prep("Catch SIGINT, set ISIG and clear NOFLSH on PORT_TTY.");
		start
			exit_on(ERROR);
			termios_prep(&tty_fildes, tty_termios_p, &loop_fildes,
							loop_termios_p);
			tty_termios_p->c_lflag |= ISIG;
								/* Disable ICANON on PORT_TTY. */
			tty_termios_p->c_lflag &= ~(ICANON | NOFLSH);
			tty_termios_p->c_cc[VTIME] = 0;
			tty_termios_p->c_cc[VMIN] = 0;
			Ztcsetattr(tty_fildes, TCSANOW, tty_termios_p);

			sig_rcvd = 0;
			SIGNAL(SIGINT, sig_int);
			SIGBLOCK(set, SIGINT);
		finis

		FORK
			prep("Send INTR to PORT_TTY.");
			start
				SIGUNBLOCK(set, SIGINT);
				SET_TIMEOUT
					WRITE_TTY_SCC(loop_fildes, tty_termios_p->c_cc[VINTR]);
					tcdrain(loop_fildes);		/* allow for interrupt */
				CLEAR_ALARM
			finis

			testing("SIGINT sent to process group.");
			start
				if(sig_rcvd == 0) {
					sleep(1);
					if(sig_rcvd == 0)
						error("SIGINT was not received by child.");
				}
			finis
		KROF

		testing("SIGINT sent to process group.");
		start
			SIGUNBLOCK(set, SIGINT);
			if(sig_rcvd == 0)
				error("SIGINT was not received by parent.");
		finis

	finis

								/* interrupt control character discarded */
	assertion(47, BASE);
	start
								/* Assertion #47 has just wrote INTR. */
		testing("ISIG set, NOFLSH clear, and INTR input, the INTR character");
		testing("is discarded.");
		start
								/* MIN = TIME = 0, non-canonical */
			SET_ALARM(WAIT_FOR_TIME)
				expecting(0);
				Zread(tty_fildes, inbuf, sizeof(char));
			CLEAR_ALARM
		finis
	finis


								/* QUIT control character */
	assertion(48, BASE);
	start
		prep("Catch SIGQUIT, ensure ISIG set, and queues are flushed.");
		start
			tty_termios_p->c_lflag |= ISIG;
			Ztcsetattr(tty_fildes, TCSAFLUSH, tty_termios_p);
			Ztcflush(loop_fildes, TCIOFLUSH);

			sig_rcvd = 0;
			SIGNAL(SIGQUIT, sig_int);
			SIGBLOCK(set, SIGQUIT);
		finis

		FORK
			prep("Send QUIT to PORT_TTY.");
			start
				SIGUNBLOCK(set, SIGQUIT);
				SET_TIMEOUT
					WRITE_TTY_SCC(loop_fildes, tty_termios_p->c_cc[VQUIT]);
					tcdrain(loop_fildes);		/* allow for interrupt */
				CLEAR_ALARM
			finis

			testing("SIGQUIT sent to process group.");
			start
				if(sig_rcvd == 0) {
					sleep(1);
					if(sig_rcvd == 0)
						error("SIGQUIT was not received by child.");
				}
			finis
		KROF

		testing("SIGQUIT sent to process group.");
		start
			SIGUNBLOCK(set, SIGQUIT);
			if(sig_rcvd == 0)
				error("SIGQUIT was not received by parent.");
		finis

	finis

								/* QUIT control character discarded */
	assertion(49, BASE);
	start
								/* Assertion #49 has just wrote QUIT. */
		testing("ISIG set, NOFLSH clear, and QUIT on input, the QUIT");
		testing("character is discarded.");
		start
								/* MIN = TIME = 0, non-canonical */
			SET_ALARM(WAIT_FOR_TIME)
				expecting(0);
				Zread(tty_fildes, inbuf, sizeof(char));
			CLEAR_ALARM
		finis
	finis


								/* SUSP control character */
	assertion(50, BASE);
	start
		prep("Catch SIGTSTP, ensure ISIG set, and queues are flushed.");
		start
			tty_termios_p->c_lflag |= ISIG;
			Ztcsetattr(tty_fildes, TCSAFLUSH, tty_termios_p);
			Ztcflush(loop_fildes, TCIOFLUSH);

			sig_rcvd = 0;
			SIGNAL(SIGTSTP, sig_int);
			SIGBLOCK(set, SIGTSTP);
		finis

		FORK
			prep("Send SUSP to PORT_TTY.");
			start
				SIGUNBLOCK(set, SIGTSTP);
				SET_TIMEOUT
					WRITE_TTY_SCC(loop_fildes, tty_termios_p->c_cc[VSUSP]);
					tcdrain(loop_fildes);		/* allow for interrupt */
				CLEAR_ALARM
			finis

			testing("SIGTSTP sent to process group.");
			start
				if(sig_rcvd == 0) {
					sleep(1);
					if(sig_rcvd == 0)
						error("SIGTSTP was not received by child.");
				}
			finis
		KROF

		testing("SIGTSTP sent to process group.");
		start
			SIGUNBLOCK(set, SIGTSTP);
			if(sig_rcvd == 0)
				error("SIGTSTP was not received by parent.");
		finis
	finis

								/* SUSP control character discarded */
	assertion(51, BASE);
	start
								/* Assertion #51 has just wrote SUSP. */
		testing("ISIG set, NOFLSH clear, and SUSP on input, the SUSP");
		testing("character is discarded.");
		start
								/* MIN = TIME = 0, non-canonical */
			SET_ALARM(WAIT_FOR_TIME)
				expecting(0);
				Zread(tty_fildes, inbuf, sizeof(char));
			CLEAR_ALARM
		finis
	finis

	cleanup("Close terminal devices.");
	start
		SIGNAL(SIGHUP, SIG_IGN);
		Zclose(tty_fildes);
		Zclose(loop_fildes);
		SLEEP(GTI_LINE_CLOSE_DELAY)
	finis

	done();
}
