#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<termios.h>
#include	<pcts_timer.h>

char String[] = "This is standard output.\n";
#define Strlen	(sizeof(String)-1)


int main(void)
{
	int		tty_fildes, loop_fildes;
	struct	termios tty_termios, *tty_termios_p = &tty_termios,
					loop_termios, *loop_termios_p = &loop_termios;
	pid_t	pid;
	char	inbuf[_POSIX_MAX_INPUT];

	setup(E_INTERFACE);
	funct(E_INTERFACE);

	if(GTI_NOT_PROVIDED) {
		GTI_NOT_PROVIDED_RESULT(69, BASE)
		done();
	}

	exit_on(ERROR);

	/*	When the last close on terminal device occurs and
		the HUPCL flag is set, then a disconnect occurs.

		ISO/IEC 9945-1:1990, 7.1.1.10 and 7.1.1.11:

		General comments:
  		  The point of asynchronous serial communication is to
  		  allow two digital devices to communicate with each other.
  		  The two devices might be two computers, or a computer and
  		  a terminal, or a computer and a modem, or a terminal and a
  		  modem, or a terminal and a printer, or something else.

  		  Modem control allows the machine at one end of a serial
  		  connection to tell, by the use of hardware signals, whether the
  		  system at the other end of the connection is still there.  When
  		  a modem connection is established, each port asserts the pin
  		  that says that it is providing a data carrier, and each port detects
  		  the carrier signal from the other port.  When the carrier signal
  		  for one port is dropped for any reason, the other port detects
  		  that a modem disconnect has occurred and shuts down its end of
  		  the connection.

		Assertions interface[64-66]:
  		  7.1.1.10 defines "modem disconnect", from the point of view of
  		  the "local" terminal.  It says what happens when the
  		  hardware detects that there's no longer a carrier signal from
  		  the other end of the connection.

  		  The first two sentences of 7.1.1.10 (lines 268-271) apply only
  		  to the controlling process.  It is not completely clear whether
  		  the rest of 7.1.1.10 applies only to controlling terminals or to
  		  all terminals.  It should apply to all terminals.  Consider what's
  		  said here:
    		Any subsequent call to the read() function shall return
    		the value zero, indicating end of file.

  		  The modem disconnect is detected on the basis of whether the
  		  other terminal is asserting carrier.  As soon as carrier is
  		  dropped by the other terminal, the input data is not available
  		  to new read() calls on the local terminal.  Note that the quoted
  		  sentence is not conditional: it does NOT say "read() shall
  		  return the value zero after the input queue and any buffers are
  		  drained".  It says "the read() function shall return the value
  		  zero".

  		  It is intentional that this happen immediately on a controlling
  		  terminal, and that buffered data not be processed.  This is an
  		  important security feature.  It ensures that loss of carrier for
  		  a remote login causes the user's login session to be terminated
  		  immediately, without a chance for a bad guy to hijack the
  		  session while it waits for buffers to drain.  

  		  My reading of the standard is that a read() that is in progress
		  when the disconnect is detected may be (but need not be) completed,
		  but that a read() that comes after the disconnect must return zero
		  regardless of what's in the terminal's input buffers at the time of
		  the disconnect.

		Assertions interface[67-69]:
  		  7.1.1.11 tells what happens on the "remote" terminal
		  (non-controlling and controlling) to cause a disconnect that can be
		  sensed on the "local" terminal; i.e., the conditions that cause the
		  "remote" terminal to drop carrier.  It also describes the means by
		  which a process can cause a modem disconnect, and specifies some
		  side effects (notably, flushing the output buffers).

  		  Whether data is transmitted before the modem disconnect occurs is not
  		  specified in POSIX.1.  In fact, the rationale for 7.1.1.11 points
		  this out:
    			POSIX.1 is silent on whether a close() will block on waiting
    			for transmission to drain, or even if a close() might cause
    			a flush of pending output.  If the application is concerned
    			about this, it should call the appropriate function, such as
    			tcdrain(), to ensure the desired behavior.

  		  Whether data actually exists in the input buffers of the receiving
  		  terminal when the disconnect occurs is immaterial--although, the
		  test allows time for data to be transmitted--,since 7.1.1.10
		  requires that a subsequent read() from the terminal return 0 no
		  matter what's in the buffer.
	*/

	assertion(69, BASE);
	start
		ENABLE_PSYNC
		if(GTI_MODEM_CONTROL) {
			prep("Fork a child  process.");
			start
				sv_flush();
				pid = Zfork();
			finis
	
			if(pid == (pid_t)0) {
				prep("Open PORT_LOOP, set HUPCL, clear CLOCAL, and send data");
				prep("to ensure connection has been properly established.");
				start 
					loop_prep(&loop_fildes, loop_termios_p);
					loop_termios_p->c_cflag |= HUPCL;
											/* Monitor modem status lines */
					loop_termios_p->c_cflag &= ~CLOCAL;
					Ztcsetattr(loop_fildes, TCSAFLUSH, loop_termios_p);

					SET_TIMEOUT
										/* Wait for PORT_TTY to complete. */
						WAIT_PSYNC

						expecting(Strlen);
						Zwrite(loop_fildes, String, Strlen);
						Ztcdrain(loop_fildes);
					CLEAR_ALARM

				finis
	
				testing("Write to terminal, then close device.");
				start
					SET_TIMEOUT
						WAIT_PSYNC
						expecting(Strlen);
						Zwrite(loop_fildes, String, Strlen);
						Zclose(loop_fildes);
					CLEAR_ALARM

					SLEEP(GTI_LINE_CLOSE_DELAY)
											/* close() completed. */
					SEND_PSYNCA
					SET_TIMEOUT
											/* Wait for testing to complete. */
						WAIT_PSYNC
					CLEAR_ALARM
				finis
	
				done();
			}

			prep("Open PORT_TTY, set HUPCL, clear CLOCAL, process initial");
			prep("data message, and wait for child to close PORT_LOOP.");
			start
				tty_prep(&tty_fildes, tty_termios_p);
				tty_termios_p->c_cflag |= HUPCL;
											/* Monitor modem status lines */
				tty_termios_p->c_cflag &= ~CLOCAL;
				Ztcsetattr(tty_fildes, TCSAFLUSH, tty_termios_p);

				SEND_PSYNC

				SET_TIMEOUT
					expecting(Strlen);
					Zread(tty_fildes, inbuf, _POSIX_MAX_INPUT);
					SEND_PSYNC
				CLEAR_ALARM

				SET_TIMEOUT
					WAIT_PSYNCA
				CLEAR_ALARM
			finis
	
			testing("When the last close on the device occurs, a disconnect");
			testing("occurs.");
			start
						/*	Allow disconnect to be detected
							and data in input queue to be discarded.
						*/
				dt_secs(TIME_ACQUIRE_XMITTED_DATA);

				SET_TIMEOUT
					expecting(0);
					Zread(tty_fildes, inbuf, sizeof(inbuf));
				CLEAR_ALARM
				SEND_PSYNC
			finis

			WAIT_FOR_PID(pid);
		}
		else
			not_implemented("modem control");
		DISABLE_PSYNC
	finis

	done();
}
