#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<termios.h>
#include	<pcts_timer.h>

const unsigned char eight_bit_set[]= {
										(unsigned char) '\377',
										(unsigned char) '\277',
										(unsigned char) '\200',
									 };
#define Strlen	(sizeof(eight_bit_set)-1)

unsigned char NL		= '\n';
unsigned char CR		= '\r';

int main(void)
{
	int		tty_fildes, loop_fildes;
	struct	termios tty_termios, *tty_termios_p = &tty_termios,
					loop_termios, *loop_termios_p = &loop_termios;
	char	inbuf[_POSIX_MAX_INPUT];

	setup(E_SETTABLE);
	funct(E_SETTABLE);

	if(GTI_NOT_PROVIDED) {
		GTI_NOT_PROVIDED_RESULT(24, BASE)
		GTI_NOT_PROVIDED_RESULT(25, BASE)
		GTI_NOT_PROVIDED_RESULT(26, BASE)
		GTI_NOT_PROVIDED_RESULT(27, BASE)
		GTI_NOT_PROVIDED_RESULT(28, BASE)
		done();
	}

									/* INLCR */
	assertion(24, BASE);
	start
		prep("Establish PORT_TTY as the control terminal and PORT_LOOP.");
		prep("Set PORT_TTY to non-canonical, MIN=TIME=0.");
		prep("Set INLCR flag for PORT_TTY.");
		start
			exit_on(ERROR);
			termios_prep(&tty_fildes, tty_termios_p, &loop_fildes,
							loop_termios_p);
			tty_termios_p->c_lflag &= ~ICANON;
					/* Return number of bytes requested or that available. */
			tty_termios_p->c_cc[VMIN] = 0;
			tty_termios_p->c_cc[VTIME] = 0;

			tty_termios_p->c_iflag |= INLCR;
			Ztcsetattr(tty_fildes, TCSAFLUSH, tty_termios_p);
		finis

		testing("When INLCR is set, a received NL character is translated");
		testing("into a CR character.");
		start
			SET_TIMEOUT
				expecting(sizeof(char));
				Zwrite(loop_fildes, (char *)&NL, sizeof(char));
				Ztcdrain(loop_fildes);
				dt_secs(TIME_ACQUIRE_XMITTED_DATA);
			CLEAR_ALARM

			SET_TIMEOUT
				expecting(sizeof(char));
				Zread(tty_fildes, inbuf, _POSIX_MAX_INPUT);
			CLEAR_ALARM

			if(inbuf[0] != CR)
				error("Expected: %o\nReceived: %o", CR, inbuf[0]);
		finis
	finis

									/* ~INLCR */
	assertion(25, BASE);
	start
		testing("When INLCR is clear, a received NL character is placed");
		testing("in the input queue unaltered.");
		start
			tty_termios_p->c_iflag &= ~INLCR;
			Ztcsetattr(tty_fildes, TCSAFLUSH, tty_termios_p);

			SET_TIMEOUT
				expecting(sizeof(char));
				Zwrite(loop_fildes, (char *)&NL, sizeof(char));
				Ztcdrain(loop_fildes);
				dt_secs(TIME_ACQUIRE_XMITTED_DATA);
			CLEAR_ALARM

			SET_TIMEOUT
				expecting(sizeof(char));
				Zread(tty_fildes, inbuf, _POSIX_MAX_INPUT);
			CLEAR_ALARM

			if(inbuf[0] != NL)
				error("Expected: %o\nReceived: %o", NL, inbuf[0]);
		finis
	finis


									/* ~IGNCR */
	assertion(26, BASE);
	start
		testing("When IGNCR is set, a received CR character is not placed");
		testing("in the input queue.");
		start
			tty_termios_p->c_iflag |= IGNCR;
			Ztcsetattr(tty_fildes, TCSAFLUSH, tty_termios_p);

			SET_TIMEOUT
				expecting(sizeof(char));
				Zwrite(loop_fildes, (char *)&CR, sizeof(char));
				Ztcdrain(loop_fildes);
				dt_secs(TIME_ACQUIRE_XMITTED_DATA);
			CLEAR_ALARM

			SET_TIMEOUT
				expecting(0);
				Zread(tty_fildes, inbuf, _POSIX_MAX_INPUT);
			CLEAR_ALARM
		finis
	finis

									/* ~IGNCR and ICRNL */
	assertion(27, BASE);
	start
		testing("When IGNCR is clear and ICRNL is set, a received CR");
		testing("character is translated into a NL character.");
		start
			tty_termios_p->c_iflag &= ~IGNCR;
			tty_termios_p->c_iflag |= ICRNL;
			Ztcsetattr(tty_fildes, TCSAFLUSH, tty_termios_p);

			SET_TIMEOUT
				expecting(sizeof(char));
				Zwrite(loop_fildes, (char *)&CR, sizeof(char));
				Ztcdrain(loop_fildes);
				dt_secs(TIME_ACQUIRE_XMITTED_DATA);
			CLEAR_ALARM

			SET_TIMEOUT
				expecting(sizeof(char));
				Zread(tty_fildes, inbuf, _POSIX_MAX_INPUT);
			CLEAR_ALARM

			if(inbuf[0] != NL)
				error("Expected: %o\nReceived: %o", NL, inbuf[0]);
		finis
	finis

									/* ~IGNCR and ~ICRNL */
	assertion(28, BASE);
	start
		testing("When IGNCR and ICRNL are both clear, a received CR");
		testing("character is placed in the input queue unaltered.");
		start
			tty_termios_p->c_iflag &= ~(IGNCR | ICRNL);
			Ztcsetattr(tty_fildes, TCSAFLUSH, tty_termios_p);

			SET_TIMEOUT
				expecting(sizeof(char));
				Zwrite(loop_fildes, (char *)&CR, sizeof(char));
				Ztcdrain(loop_fildes);
				dt_secs(TIME_ACQUIRE_XMITTED_DATA);
			CLEAR_ALARM

			SET_TIMEOUT
				expecting(sizeof(char));
				Zread(tty_fildes, inbuf, _POSIX_MAX_INPUT);
			CLEAR_ALARM

			if(inbuf[0] != CR)
				error("Expected: %o\nReceived: %o", CR, inbuf[0]);
		finis
	finis

	start
		SIGNAL(SIGHUP, SIG_IGN);
		Zclose(tty_fildes);
		Zclose(loop_fildes);
		SLEEP(GTI_LINE_CLOSE_DELAY)
	finis

	done();
}
