#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<termios.h>
#include	<pcts_timer.h>


int main(void)
{
	int		tty_fildes, loop_fildes;
	struct	termios tty_termios, *tty_termios_p = &tty_termios,
					loop_termios, *loop_termios_p = &loop_termios;
	pid_t	pid;

	setup(E_SETTABLE);
	funct(E_SETTABLE);

	if(GTI_NOT_PROVIDED) {
		GTI_NOT_PROVIDED_RESULT(56, BASE)
		GTI_NOT_PROVIDED_RESULT(57, BASE)
		GTI_NOT_PROVIDED_RESULT(58, BASE)
		done();
	}

	if(GTI_MODEM_CONTROL == 0) {
		assertion(56, BASE);
		start
			not_implemented("modem control");
		finis
		assertion(57, BASE);
		start
			not_implemented("modem control");
		finis
		assertion(58, BASE);
		start
			not_implemented("modem control");
		finis
		done();
	}

	exit_on(ERROR);
	SIGNAL(SIGHUP, SIG_IGN);

											/* open() and ~CLOCAL */
	assertion(56, BASE);
	start
		ENABLE_PSYNC

		prep("Clear CLOCAL for termios.");
		start
			SET_TIMEOUT
				loop_fildes = Zopen(PORT_LOOP, O_RDWR | O_NONBLOCK);
				loop_init(loop_fildes, loop_termios_p);
				loop_termios_p->c_cflag &= ~CLOCAL;
				Ztcsetattr(loop_fildes, TCSANOW, loop_termios_p);
			CLEAR_ALARM
		finis

		prep("Fork a child.");
		start
			sv_flush();
			pid = Zfork();
		finis

		if(pid == (pid_t)0) {
			testing("Attempt to open PORT_LOOP with CLOCAL and O_NONBLOCK");
			testing("clear waits for modem connection to complete.");
			start
				continue_on(ERROR);
				SET_ALARM(WAIT_FOR_TIME)
					loop_fildes = Zopen(PORT_LOOP, O_RDWR, PERM_CTTY);
				EXPECT_ALARM
			finis
	
			testing("Open of PORT_LOOP with CLOCAL and O_NONBLOCK clear");
			testing("completes when modem connection is established.");
			start
				SEND_PSYNC
				SET_ALARM(WAIT_FOR_TIME)
					loop_fildes = Zopen(PORT_LOOP, O_RDWR, PERM_CTTY);
				CLEAR_ALARM
			finis
	
			done();
		}
	
		prep("Open PORT_TTY after child has tested the blocking condition.");
		start
			sv_flush();
			SET_TIMEOUT
				WAIT_PSYNC
				tty_fildes = Zopen(PORT_TTY, O_RDWR, PERM_CTTY);
			CLEAR_ALARM
		finis

		WAIT_FOR_PID(pid);

		cleanup("Close PORT_TTY.");
		start
						/*	Ensure HUPCL is set on PORT_TTY so modem disconnect
							will occur when PORT_TTY is closed.
						*/
			tty_termios_p->c_cflag |= HUPCL;
			Ztcsetattr(tty_fildes, TCSANOW, tty_termios_p);

			Zclose(tty_fildes);
			Zclose(loop_fildes);
			SLEEP(GTI_LINE_CLOSE_DELAY)
		finis


											/* creat() and ~CLOCAL */
		prep("Clear CLOCAL for termios.");
		start
			SET_TIMEOUT
				loop_fildes = Zopen(PORT_LOOP, O_RDWR | O_NONBLOCK);
				loop_init(loop_fildes, loop_termios_p);
				loop_termios_p->c_cflag &= ~CLOCAL;
				Ztcsetattr(loop_fildes, TCSANOW, loop_termios_p);
			CLEAR_ALARM
		finis

		prep("Fork a child.");
		start
			sv_flush();
			pid = Zfork();
		finis

		if(pid == (pid_t)0) {
			testing("Attempt to create PORT_LOOP with CLOCAL and");
			testing("O_NONBLOCK clear waits for modem connection to");
			testing("complete.");
			start
				continue_on(ERROR);
				SET_ALARM(WAIT_FOR_TIME)
					loop_fildes = Zcreat(PORT_LOOP, PERM_CTTY);
				EXPECT_ALARM
			finis
	
			testing("Create of PORT_LOOP with CLOCAL and O_NONBLOCK clear");
			testing("completes when modem connection is established.");
			start
				SEND_PSYNC
				SET_ALARM(WAIT_FOR_TIME)
					loop_fildes = Zcreat(PORT_LOOP, PERM_CTTY);
				CLEAR_ALARM
			finis
	
			done();
		}
	
		prep("Open PORT_TTY after child has tested the blocking condition.");
		start
			sv_flush();
			SET_TIMEOUT
				WAIT_PSYNC
				tty_fildes = Zopen(PORT_TTY, O_RDWR, PERM_CTTY);
			CLEAR_ALARM
		finis

		WAIT_FOR_PID(pid);

		cleanup("Close PORT_TTY.");
		start
			Zclose(tty_fildes);
			Zclose(loop_fildes);
			SLEEP(GTI_LINE_CLOSE_DELAY)
		finis
		DISABLE_PSYNC
	finis

											/* O_NONBLOCK & ~CLOCAL */
	assertion(57, BASE);
	start
		prep("Clear CLOCAL for termios.");
		start
			SET_TIMEOUT
				loop_fildes = Zopen(PORT_LOOP, O_RDWR | O_NONBLOCK);
				loop_init(loop_fildes, loop_termios_p);
				loop_termios_p->c_cflag &= ~CLOCAL;
				Ztcsetattr(loop_fildes, TCSANOW, loop_termios_p);
			CLEAR_ALARM
		finis

		FORK
			testing("Attempt to open PORT_LOOP with CLOCAL clear and");
			testing("O_NONBLOCK set returns without waiting for the modem");
			testing("connection.");
			start
				SET_TIMEOUT
					loop_fildes = Zopen(PORT_LOOP, O_RDWR | O_NONBLOCK,
																PERM_CTTY);
				CLEAR_ALARM
			finis
		KROF

		cleanup("Close PORT_TTY.");
		start
			Zclose(loop_fildes);
			SLEEP(GTI_LINE_CLOSE_DELAY)
		finis
	finis

											/* CLOCAL set */
	assertion(58, BASE);
	start
		prep("Clear CLOCAL for termios.");
		start
			SET_TIMEOUT
				loop_fildes = Zopen(PORT_LOOP, O_RDWR | O_NONBLOCK);
				loop_init(loop_fildes, loop_termios_p);
				loop_termios_p->c_cflag &= ~CLOCAL;
				Ztcsetattr(loop_fildes, TCSANOW, loop_termios_p);
			CLEAR_ALARM
		finis

		prep("Set CLOCAL for termios for PORT_LOOP.");
		start
			loop_termios_p->c_cflag |= CLOCAL;
			Ztcsetattr(loop_fildes, TCSANOW, loop_termios_p);
		finis

		FORK
			testing("Attempt to open PORT_LOOP via creat() with CLOCAL set");
			testing("returns without waiting for the modem connection.");
			start
				SET_ALARM(WAIT_FOR_TIME)
					loop_fildes = Zcreat(PORT_LOOP, PERM_CTTY);
				CLEAR_ALARM
			finis

			testing("Attempt to open PORT_LOOP via open() with CLOCAL set and");
			testing("O_NONBLOCK clear returns without waiting for the modem");
			testing("connection.");
			start
				SET_ALARM(WAIT_FOR_TIME)
					loop_fildes = Zopen(PORT_LOOP, O_RDWR, PERM_CTTY);
				CLEAR_ALARM
			finis

			testing("Attempt to open PORT_LOOP via open() with CLOCAL set and");
			testing("O_NONBLOCK set returns without waiting for the modem");
			testing("connection.");
			start
				SET_ALARM(WAIT_FOR_TIME)
					loop_fildes = Zopen(PORT_LOOP, O_RDWR | O_NONBLOCK,
																PERM_CTTY);
				CLEAR_ALARM
			finis
		KROF

		cleanup("Close PORT_TTY.");
		start
			Zclose(loop_fildes);
			SLEEP(GTI_LINE_CLOSE_DELAY)
		finis
	finis

	done();
}
