#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zfcntl.h>
#include	<Ztermios.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<pcts_timer.h>

char outbuf[] = "Still crazy after all these years...\n";
const int BUFLEN = sizeof(outbuf)-1;

char inbuf[_POSIX_MAX_INPUT];			/* I/O buffers						*/
char stop_char, start_char;				/* START/STOP values				*/


int main(void)
{
	int	tty_fildes, loop_fildes;
	struct termios	tty_termios, *tty_termios_p = &tty_termios,
					loop_termios, *loop_termios_p = &loop_termios;
	int	cnt, i;

	setup(E_TCFLOW);
	funct(E_TCFLOW);

	if(GTI_NOT_PROVIDED) {
		GTI_NOT_PROVIDED_RESULT(12, BASE)
		GTI_NOT_PROVIDED_RESULT(5, BASE)
		GTI_NOT_PROVIDED_RESULT(6, BASE)
		GTI_NOT_PROVIDED_RESULT(7, BASE)
		GTI_NOT_PROVIDED_RESULT(8, BASE)
		done();
	}

	assertion(12, BASE);
	start
		exit_on(ERROR);
		prep("Establish PORT_TTY as the control terminal and PORT_LOOP.");
		start
			termios_prep(&tty_fildes, tty_termios_p, &loop_fildes,
								loop_termios_p);

								/* Enable flow control on input and output. */
			Ztcgetattr(tty_fildes, tty_termios_p);
			tty_termios_p->c_iflag |=(IXOFF|IXON);
			Ztcsetattr(tty_fildes, TCSAFLUSH, tty_termios_p);

								/* Flow control disabled on PORT_LOOP. */
			Ztcgetattr(loop_fildes, loop_termios_p);
			loop_termios_p->c_cc[VMIN] = 1;
			loop_termios_p->c_cc[VTIME] = 0;
			Ztcsetattr(loop_fildes, TCSAFLUSH, loop_termios_p);

			start_char = tty_termios_p->c_cc[VSTART];
			stop_char = tty_termios_p->c_cc[VSTOP];
		finis

		testing("Neither a terminal's input nor output is suspended on");
		testing("initial open().");
		start
			SET_TIMEOUT
				expecting(BUFLEN);
				Zwrite(tty_fildes, outbuf, BUFLEN);
				Ztcdrain(tty_fildes);
				dt_secs(TIME_ACQUIRE_XMITTED_DATA);
				expecting(BUFLEN);
				Zread(loop_fildes, inbuf, BUFLEN);
			CLEAR_ALARM
		finis
	finis


	assertion(5, BASE);
	start
		testing("A call to tcflow(fildes, TCOOFF), on the controlling");
		testing("terminal of the process, suspends output.");
		start
			expecting(0);
			Ztcflow(tty_fildes, TCOOFF);

			if(PCTS_GTI_BUFFERS_OUTPUT) {
				SET_TIMEOUT
					expecting(BUFLEN);
					Zwrite(tty_fildes, outbuf, BUFLEN);
				CLEAR_ALARM
			}
			else {
					/*	Implementations that do not provide a buffering
						mechanism on write() will timeout when terminal
						output is suspended.
					*/
				SET_ALARM(WAIT_FOR_TIME)
					write(tty_fildes, "A", 2);
				EXPECT_ALARM
			}

			SLEEP(WAIT_FOR_TIME);

			cnt = 0;
			SET_TIMEOUT
				cnt = read(loop_fildes, inbuf, BUFLEN);
			EXPECT_ALARM

			if(cnt > 0)
				error("TCOOFF failed, output was not stopped.");
			if(memchr(inbuf, stop_char, cnt))
				error("TCOOFF failed, STOP was sent.");
			if(memchr(inbuf, start_char, cnt))
				error("TCOOFF failed, START was sent.");
		finis
	finis

	assertion(6, BASE);
	start
		testing("A call to tcflow(fildes, TCOON) restarts suspended output.");
		start
			Ztcflow(tty_fildes, TCOON);

			if( ! PCTS_GTI_BUFFERS_OUTPUT) {
				SET_TIMEOUT
					expecting(BUFLEN);
					Zwrite(tty_fildes, outbuf, BUFLEN);
				CLEAR_ALARM
			}

			SET_TIMEOUT
				Ztcdrain(tty_fildes);
				dt_secs(TIME_ACQUIRE_XMITTED_DATA);

				expecting(BUFLEN);
				Zread(loop_fildes, inbuf, BUFLEN);
			CLEAR_ALARM

			if(memchr(inbuf, stop_char, cnt))
				error("TCOON failed, STOP was sent.");
			if(memchr(inbuf, start_char, cnt))
				error("TCOON failed, START was sent.");
		finis
	finis

	assertion(7, BASE);
	start
		break_on(ERROR);
		for(i=0; i<2; i++) {
			prep("Ensure input and output queues are clear and");
			prep("PORT_TTY is set for %s data transmission.", 
											(i==0) ? "active" : "suspended");
			start
							/*	Activate/suspend output data transmission
								(from system to terminal).
							*/
				Ztcflow(tty_fildes, (i==0) ? TCOON : TCOOFF);
				dt_secs(TIME_ACQUIRE_XMITTED_DATA);	/* In case data xmitted. */
				Ztcflush(tty_fildes, TCIOFLUSH);
				Ztcflush(loop_fildes, TCIOFLUSH);
			finis

			testing("A call to tcflow(fildes, TCIOFF) sends a STOP character");
			testing("when the transmission line is %s.",
											(i==0) ? "active" : "suspended");
			start
				expecting(0);
				Ztcflow(tty_fildes, TCIOFF);
				dt_secs(TIME_ACQUIRE_XMITTED_DATA);

				SET_TIMEOUT
					expecting(1);
					Zread(loop_fildes, inbuf, 1);
				CLEAR_ALARM

				if(inbuf[0] != stop_char) {
					if(inbuf[0] == start_char) {
						message("START character detected, check STOP sent.");
						SET_TIMEOUT
							expecting(1);
							Zread(loop_fildes, inbuf, 1);
						CLEAR_ALARM
						if(inbuf[0] != stop_char)
							error("Instead of STOP character, \"%c\" was sent.",
																	inbuf[0]);
					}
					else
						error("Instead of STOP character, \"%c\" was sent.",
																	inbuf[0]);
				}
			finis
		}
	finis

	assertion(8, BASE);
	start
		break_on(ERROR);
		for(i=0; i<2; i++) {
			prep("Ensure input and output queues are clear and");
			prep("PORT_TTY is set for %s data transmission.", 
											(i==0) ? "active" : "suspended");
			start
							/*	Activate/suspend output data transmission
								(from system to terminal).
							*/
				Ztcflow(tty_fildes, (i==0) ? TCOON : TCOOFF);
				dt_secs(TIME_ACQUIRE_XMITTED_DATA);	/* In case data xmitted. */
				Ztcflush(tty_fildes, TCIOFLUSH);
				Ztcflush(loop_fildes, TCIOFLUSH);
			finis

			testing("A call to tcflow(fildes, TCION) sends a START character");
			testing("when the transmission line is %s.",
											(i==0) ? "active" : "suspended");
			start
				expecting(0);
				Ztcflow(tty_fildes, TCION);
				dt_secs(TIME_ACQUIRE_XMITTED_DATA);

				SET_TIMEOUT
					expecting(1);
					Zread(loop_fildes, inbuf, 1);
				CLEAR_ALARM

				if(inbuf[0] != start_char)
					error("Instead of START character, \"%c\" was sent.",
																	inbuf[0]);
			finis
		}
	finis

	cleanup("Close terminal devices.");
	start
		SIGNAL(SIGHUP, SIG_IGN);
		Zclose(tty_fildes);
		Zclose(loop_fildes);
		SLEEP(GTI_LINE_CLOSE_DELAY)
	finis

	done();
}
