#include	<pcts.h>

#include	<Zunistd.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<termios.h>
#include	<pcts_timer.h>

#define	FILENAME	"7.2.3_06"

int main(void)
{
	int		tty_fildes, loop_fildes;
	struct	termios tty_termios, *tty_termios_p = &tty_termios,
					loop_termios, *loop_termios_p = &loop_termios;
	pid_t	pid;

	setup(E_TCGETPGRP);
	funct(E_TCGETPGRP);

	assertion(6, BASE);
	start
		testing("Call to tcgetpgrp() with an invalid file descriptor).");
		testing("Expect a return value of -1 and errno set to [EBADF].");
		start
			expecting(EBADF);
			expecting(-1);
			Ztcgetpgrp(-1);
		finis
	finis

	assertion(7, NOT_APPLICABLE);

	if(GTI_NOT_PROVIDED) {
		GTI_NOT_PROVIDED_RESULT(8, BASE)
	}
	else {
		assertion(8, BASE);
		start
			prep("Establish PORT_TTY as the control terminal and PORT_LOOP.");
			start
				termios_prep(&tty_fildes, tty_termios_p, &loop_fildes,
										loop_termios_p);
			finis

			FORK
				testing("Call to tcgetpgrp() when the calling process");
				testing("does not have a controlling terminal.  Expect");
				testing("a return value of -1 and errno set to [ENOTTY].");
				start
					Zsetsid();
					expecting(-1);
					expecting(ENOTTY);
					Ztcgetpgrp(tty_fildes);
				finis
			KROF
		finis
	}

	assertion(9, BASE);
	start
		char	path[_POSIX_PATH_MAX +1];
		int		fd;

		prep("Establish a regular file.");
		start
			Zstrcpy(path, temp_path());
			Zstrcat(path, FILENAME);
			fd = Zcreat(path, PROT_ALL);
		finis

		testing("Call to tcgetpgrp() when the file associated with the");
		testing("file descriptor is not a terminal device.");
		testing("Expect a return value of -1 and errno set to [ENOTTY].");
		start
			expecting(-1);
			expecting(ENOTTY);
			Ztcgetpgrp(fd);
		finis

		if(GTI_PROVIDED) {
			testing("Call to tcgetpgrp() when the file associated with the");
			testing("file descriptor is not the controlling terminal.");
			testing("Expect a return value of -1 and errno set to [ENOTTY].");
			start
				expecting(-1);
				expecting(ENOTTY);
				Ztcgetpgrp(loop_fildes);
			finis
		}
	finis

	if(GTI_PROVIDED) {
		cleanup("Close terminal devices.");
		start
			SIGNAL(SIGHUP, SIG_IGN);
			Zclose(tty_fildes);
			Zclose(loop_fildes);
			SLEEP(GTI_LINE_CLOSE_DELAY)
		finis
	}

	done();
}
