#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zfcntl.h>
#include	<sys/Ztimes.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<termios.h>
#include	<pcts_timer.h>

const int REPEAT		= 40;	/* Number of zero breaks to test for.		*/
const int TEN_SEC		= 10;	/* Minimum time for REPEAT breaks to occur.	*/
const int TWENTY_SEC	= 20;	/* Maximum time for REPEAT breaks to occur.	*/

static	time_t	first, last;		/* break duration determination times*/
static	int	sigb_rcvd;
static	void sig_break(int);


int main(void)
{
	int		tty_fildes, loop_fildes;
	struct	termios tty_termios, *tty_termios_p = &tty_termios,
					loop_termios, *loop_termios_p = &loop_termios;
	pid_t	pid;
	int		i,j;

	setup(E_TCSENDBREAK);
	funct(E_TCSENDBREAK);

	if(GTI_NOT_PROVIDED) {
		GTI_NOT_PROVIDED_RESULT(4, BASE)
		GTI_NOT_PROVIDED_RESULT(5, BASE)
		done();
	}

	assertion(4, BASE);
	start
		char	inbuf[_POSIX_MAX_INPUT];				/* read() buffer	*/
		char	c_data = 'A';							/* output data		*/

		exit_on(ERROR);

		ENABLE_PSYNC
			prep("Establish PORT_TTY as the control terminal and PORT_LOOP.");
			start
				termios_prep(&tty_fildes, tty_termios_p, &loop_fildes,
								loop_termios_p);
			finis

			for(j=0; j<2; j++) {		/* Run at min and max baud rates. */
			  prep("Fork a child process.");
			  start
				sv_flush();
				pid = Zfork();
			  finis
	
			  if(pid == (pid_t)0) {
				prep("Send a break condition.");
				start
					SIGNAL(SIGINT, SIG_IGN);	/* If generated. */

					SET_TIMEOUT
						WAIT_PSYNC
						Ztcsendbreak(loop_fildes, 0);
						expecting(1);
						Zwrite(loop_fildes, &c_data, 1);
						Ztcdrain(loop_fildes);
					CLEAR_ALARM
				finis
		
				testing("Send a quarter-second break repeated forty times");
				testing("should last about ten seconds.");
				start
					SET_TIMEOUT
						WAIT_PSYNC
					CLEAR_ALARM
	
					SET_ALARM(TWENTY_SEC)
						for(i=0; i<REPEAT; ++i) {
							Ztcsendbreak(loop_fildes, 0);
										/*	Write out unique character.	*/
							expecting(1);
							Zwrite(loop_fildes, &c_data, 1);
							Ztcdrain(loop_fildes);
						}
					CLEAR_ALARM
				finis

				done();
			}
			prep("Have PORT_TTY detect breaks as a single '\\0'.");
			start
				Ztcgetattr(tty_fildes, tty_termios_p);
				tty_termios_p->c_iflag &= ~IGNBRK;
				tty_termios_p->c_iflag &= ~BRKINT;
								/* Set for non-canonical mode. */
				tty_termios_p->c_lflag &= ~(ICANON | ECHO);
				tty_termios_p->c_cc[VMIN] = 1;
				tty_termios_p->c_cc[VTIME] = 10;
				Ztcsetattr(tty_fildes, TCSAFLUSH, tty_termios_p);
	
				SIGNAL(SIGINT, sig_break);	/* Catch break if it occurs. */
			finis
		
			testing("Controlling terminal receives a single '\\0' on break.");
			start
				sigb_rcvd = 0;
				SEND_PSYNC
				SET_TIMEOUT
					expecting(1);
					Zread(tty_fildes, inbuf, 1);
				CLEAR_ALARM
				if(inbuf[0] != '\0')
					error("%s\n%s.\n%s'\\%o\'.",
							"Improper character received.",
							"Expected an input character of value '\\0'.",
							"Actual character received was ",
							inbuf[0]);
				if(sigb_rcvd != 0)
					error("Break signal detected.");

				SET_TIMEOUT
					expecting(1);
					Zread(tty_fildes, inbuf, 1);
				CLEAR_ALARM
				if(inbuf[0] != c_data)
					error("%s\n%s'\\%o'.\n%s'\\%o'.",
							"Improper character received.",
							"Expected input character of value ",
							c_data,
							"Actual character received was ",
							inbuf[0]);
			finis
	
			testing("Forty calls to tcsendbreak() will generate forty");
			testing("breaks, will last about 10 seconds, and the break");
			testing("will consist of no other data but nulls.");
			start
				int	fudge_time;

				continue_on(ERROR);
				sigb_rcvd = 0;
				first = time((void *)0);	/* Establish a start time. */
				SET_ALARM(TWENTY_SEC)
					SEND_PSYNC
					inbuf[0] = c_data;
					for(i=0; i<REPEAT; i++) {
						expecting(1);
						Zread(tty_fildes, inbuf, 1);
						if(inbuf[0] != '\0')
							error("%s\n%s.\n%s'\\%o\'.",
									"Improper character received.",
									"Expected input character of value '\\0'.",
									"Actual character received was ",
									inbuf[0]);
						if(sigb_rcvd != 0) {
							error("Break signal detected.");
							sigb_rcvd = 0;
						}

						expecting(1);
						Zread(tty_fildes, inbuf, 1);
						if(inbuf[0] != c_data)
							error("Break consisted of other than nulls.");
					}
				CLEAR_ALARM
				last = time((void *)0);	/* Establish an end time. */
				if(j=1)
					fudge_time = REPEAT/baud_value(baud_max())/10 +2;
				else
					fudge_time = REPEAT/baud_value(baud_min())/10 +2;
	
				if((last -first < TEN_SEC -fudge_time) || 
								(last -first > TWENTY_SEC +fudge_time))
					error("%s\n%s%d\n%s%ld\n",
							"tcsendbreak() was too short or too long.",
							"Expected time to read 40 breaks is ",
							TEN_SEC,
							"Actual time taken was ",
							(long) (last -first));
			finis
	
			wait_for(pid, TWENTY_SEC +WAIT_FOR_TIME);

			Zcfsetospeed(loop_termios_p, baud_min());
			Zcfsetispeed(loop_termios_p, baud_min());
			Zcfsetospeed(tty_termios_p, baud_min());
			Zcfsetispeed(tty_termios_p, baud_min());
			Ztcsetattr(tty_fildes, TCSAFLUSH, tty_termios_p);
			Ztcsetattr(loop_fildes, TCSAFLUSH, loop_termios_p);
		  }
		DISABLE_PSYNC

		cleanup("Close terminal devices.");
		start
			SIGNAL(SIGHUP, SIG_IGN);
			Zclose(tty_fildes);
			Zclose(loop_fildes);
			SLEEP(GTI_LINE_CLOSE_DELAY)
		finis
	finis

	assertion(5, EXTENDED);

	done();
}

			/* Catch break condition and retain time when it occured. */
static	void sig_break(int i)
{
	extern	time_t	last;
	extern	int	sigb_rcvd;

	last = time((void *)0);

	++sigb_rcvd;
}
