#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zfcntl.h>
#include	<sys/Ztimes.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<termios.h>
#include	<pcts_timer.h>


int main(void)
{
	int		tty_fildes, loop_fildes;
	struct	termios tty_termios, *tty_termios_p = &tty_termios,
					loop_termios, *loop_termios_p = &loop_termios;
	pid_t	pid;
	int		i;
	sigset_t	set;

	setup(E_TCSENDBREAK);
	funct(E_TCSENDBREAK);

	if(GTI_NOT_PROVIDED) {
		GTI_NOT_PROVIDED_RESULT(6, BASE)
		GTI_NOT_PROVIDED_RESULT(7, BASE)
		GTI_NOT_PROVIDED_RESULT(8, BASE)
		done();
	}

	continue_on(ERROR);
	SIGNAL(SIGHUP, SIG_IGN);		/* Ignore hangups on PORTs. */

	assertion(6, BASE);
	start
		ENABLE_PSYNC
		for(i=0; i<2; i++) {
		  FORK
			prep("Establish PORT_TTY as the control terminal and PORT_LOOP.");
			if(i==0)
				prep("Set TOSTOP flag.");
			else
				prep("Clear TOSTOP flag.");
			start
				exit_on(ERROR);
				termios_prep(&tty_fildes, tty_termios_p, &loop_fildes,
															loop_termios_p);
				if(i==0)
					tty_termios_p->c_lflag |= TOSTOP;
				else
					tty_termios_p->c_lflag &= ~TOSTOP;

				loop_termios_p->c_iflag &= ~(IGNBRK | BRKINT);
				loop_termios_p->c_iflag |= PARMRK;
				loop_termios_p->c_lflag |= NOFLSH;
				Ztcsetattr(tty_fildes, TCSAFLUSH, tty_termios_p);
				Ztcsetattr(loop_fildes, TCSAFLUSH, loop_termios_p);

				SIGNAL(SIGTTOU, SIG_IGN);
				SIGNAL(SIGINT, SIG_IGN);
			finis

			FORK
				prep("Establish background process group containing two");
				prep("processes.");
				start
					Zsetpgid((pid_t)0, (pid_t)0);
					SIGNAL(SIGTTOU, sig_int);
					sig_rcvd = 0;
					SIGNAL(SIGINT, SIG_IGN);
				finis
	
				prep("Fork a child process.");
				start
					sv_flush();
					pid = Zfork();
				finis
	
				if(pid == (pid_t)0) {
					testing("Child process in background process group");
					testing("received SIGTTOU signal.");
					start
						sleep(2*WAIT_FOR_TIME);	/*	Interruped by SIGTTOU. */
						if(sig_rcvd == 0)
							error("Child process did not receive SIGTTOU.");
					finis
	
					done();
				}
	
				testing("Call to tcsendbreak(), on controlling terminal");
				testing("of the process, from member of background");
				testing("process group that is not blocking or ignoring");
				testing("the SIGTTOU signal when TOSTOP is");
				if(i==0)
					testing("set is unsuccessful.");
				else
					testing("clear is unsuccessful.");
				start
					SIGUNBLOCK(set, SIGTTOU);
					expecting(FAILURE);
					Ztcsendbreak(tty_fildes, 0);
				finis
	
				sleep(1);
	
				testing("SIGTTOU signal is sent to calling process.");
				start
					if(sig_rcvd == 0)
						error("Calling process did not receive SIGTTOU.");
				finis
	
				WAIT_FOR_PID(pid);
			KROF
	
			testing("Foreground process of controlling terminal");
			testing("did not receive a break condition.");
			start
				char	inbuf[_POSIX_MAX_INPUT];

				SET_TIMEOUT
					expecting(0);
					Zread(loop_fildes, inbuf, _POSIX_MAX_INPUT);
				CLEAR_ALARM
			finis
		  KROF
		  SLEEP(GTI_LINE_CLOSE_DELAY)
		}
		DISABLE_PSYNC
	finis


	assertion(7, BASE);
	start
		ENABLE_PSYNC
		for(i=0; i<2; i++) {
		  FORK
			prep("Establish PORT_TTY as the control terminal and PORT_LOOP.");
			if(i==0)
				prep("Set TOSTOP flag.");
			else
				prep("Clear TOSTOP flag.");
			start
				exit_on(ERROR);
				termios_prep(&tty_fildes, tty_termios_p, &loop_fildes,
															loop_termios_p);
				if(i==0)
					tty_termios_p->c_lflag |= TOSTOP;
				else
					tty_termios_p->c_lflag &= ~TOSTOP;

				loop_termios_p->c_iflag &= ~(IGNBRK | BRKINT);
				loop_termios_p->c_iflag |= PARMRK;
				loop_termios_p->c_lflag |= NOFLSH;

				Ztcsetattr(tty_fildes, TCSAFLUSH, tty_termios_p);
				Ztcsetattr(loop_fildes, TCSAFLUSH, loop_termios_p);

				SIGNAL(SIGTTOU, SIG_IGN);
				SIGNAL(SIGINT, SIG_IGN);
			finis

			FORK
				prep("Establish background process group containing two");
				prep("processes.");
				start
					Zsetpgid((pid_t)0, (pid_t)0);
					SIGNAL(SIGTTOU, sig_int);
					sig_rcvd = 0;
				finis
	
				prep("Fork a child process.");
				start
					sv_flush();
					pid = Zfork();
				finis
	
				if(pid == (pid_t)0) {
					testing("Child process in background process group");
					testing("does not receive SIGTTOU signal.");
					start
						sleep(2*WAIT_FOR_TIME);	/*	Interruped by SIGTTOU. */
						if(sig_rcvd != 0)
							error("Child process received SIGTTOU.");
					finis
	
					done();
				}
	
				testing("Call to tcsendbreak(), on controlling terminal");
				testing("of the process, from member of background");
				testing("process group that is blocking the SIGTTOU");
				if(i==0)
					testing("signal when TOSTOP is set is successful.");
				else
					testing("signal when TOSTOP is clear is successful.");
				start
					SIGBLOCK(set, SIGTTOU);
					expecting(SUCCESS);
					Ztcsendbreak(tty_fildes, 0);
					Ztcdrain(tty_fildes);
				finis
	
				sleep(1);
	
				testing("SIGTTOU signal not sent to calling process.");
				start
					if(sig_rcvd != 0)
						error("Calling process received SIGTTOU.");
				finis
	
				WAIT_FOR_PID(pid);
			KROF
	
			testing("Foreground process of controlling terminal");
			testing("received break condition.");
			start
				char	inbuf[_POSIX_MAX_INPUT];

				SET_TIMEOUT
					expecting(3*sizeof(char));
					Zread(loop_fildes, inbuf, _POSIX_MAX_INPUT);
				CLEAR_ALARM

				if(inbuf[0] != '\377' || inbuf[1] != '\0' || inbuf[2] != '\0')
					error("%s\n%s %o %o %o",
							"Expected: \\377 \\0 \\0",
							"Received:",
							(int)inbuf[0], (int)inbuf[1], (int)inbuf[2]);
			finis
		  KROF
		  SLEEP(GTI_LINE_CLOSE_DELAY)
		}
		DISABLE_PSYNC
	finis


	assertion(8, BASE);
	start
		ENABLE_PSYNC
		for(i=0; i<2; i++) {
		  FORK
			prep("Establish PORT_TTY as the control terminal and PORT_LOOP.");
			if(i==0)
				prep("Set TOSTOP flag.");
			else
				prep("Clear TOSTOP flag.");
			start
				exit_on(ERROR);
				termios_prep(&tty_fildes, tty_termios_p, &loop_fildes,
															loop_termios_p);
				if(i==0)
					tty_termios_p->c_lflag |= TOSTOP;
				else
					tty_termios_p->c_lflag &= ~TOSTOP;

				loop_termios_p->c_iflag &= ~(IGNBRK | BRKINT);
				loop_termios_p->c_iflag |= PARMRK;
				loop_termios_p->c_lflag |= NOFLSH;

				Ztcsetattr(tty_fildes, TCSAFLUSH, tty_termios_p);
				Ztcsetattr(loop_fildes, TCSAFLUSH, loop_termios_p);

				SIGNAL(SIGTTOU, SIG_IGN);
				SIGNAL(SIGINT, SIG_IGN);
			finis

			FORK
				prep("Establish background process group containing two");
				prep("processes.");
				start
					Zsetpgid((pid_t)0, (pid_t)0);
					SIGNAL(SIGTTOU, sig_int);
					sig_rcvd = 0;
				finis
	
				prep("Fork a child process.");
				start
					sv_flush();
					pid = Zfork();
				finis
	
				if(pid == (pid_t)0) {
					testing("Child process in background process group");
					testing("does not receive SIGTTOU signal.");
					start
						sleep(2*WAIT_FOR_TIME);	/*	Interruped by SIGTTOU. */
						if(sig_rcvd != 0)
							error("Child process received SIGTTOU.");
					finis
	
					done();
				}
	
		
				testing("Call to tcsendbreak(), on controlling terminal");
				testing("of the process, from member of background");
				testing("process group that is ignoring the SIGTTOU");
				if(i==0)
					testing("signal when TOSTOP is set is successful.");
				else
					testing("signal when TOSTOP is clear is successful.");
				start
					SIGNAL(SIGTTOU, SIG_IGN);
					expecting(SUCCESS);
					Ztcsendbreak(tty_fildes, 0);
					Ztcdrain(tty_fildes);
				finis
	
				sleep(1);
	
				testing("SIGTTOU signal not sent to calling process.");
				start
					if(sig_rcvd != 0)
						error("Calling process received SIGTTOU.");
				finis
	
				WAIT_FOR_PID(pid);
			KROF
	
			testing("Foreground process of controlling terminal");
			testing("received break condition.");
			start
				char	inbuf[_POSIX_MAX_INPUT];

				SET_TIMEOUT
					expecting(3*sizeof(char));
					Zread(loop_fildes, inbuf, _POSIX_MAX_INPUT);
				CLEAR_ALARM

				if(inbuf[0] != '\377' || inbuf[1] != '\0' || inbuf[2] != '\0')
					error("%s\n%s %o %o %o",
							"Expected: \\377 \\0 \\0",
							"Received:",
							(int)inbuf[0], (int)inbuf[1], (int)inbuf[2]);
			finis
		  KROF
		  SLEEP(GTI_LINE_CLOSE_DELAY)
		}
		DISABLE_PSYNC
	finis

	done();
}
