#include	<pcts.h>

#include	<Zunistd.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<termios.h>
#include	<pcts_timer.h>

#define	FILENAME	"7.2.4_05"


int main(void)
{
	int		tty_fildes, loop_fildes;
	struct	termios tty_termios, *tty_termios_p = &tty_termios,
					loop_termios, *loop_termios_p = &loop_termios;
	pid_t	cur_pgrp, ct_pgrp, par_pgrp, pid;

	setup(E_TCSETPGRP);
	funct(E_TCSETPGRP);

	if(GTI_NOT_PROVIDED) {
		GTI_NOT_PROVIDED_RESULT(11, BASE)
	}
	else {
	  assertion(11, BASE);
	  start
		FORK
			prep("Establish PORT_TTY as the control terminal and PORT_LOOP.");
			start
				par_pgrp = Zgetpgrp();
				termios_prep(&tty_fildes, tty_termios_p, &loop_fildes,
								loop_termios_p);
				ct_pgrp = Ztcgetpgrp(tty_fildes);
			finis

			testing("Call to tcsetpgrp() does not match the process group ID");
			testing("of process in same session).");
			testing("Expect a return value of -1 and errno set to [EPERM].");
			start
				expecting(-1);
				expecting(EPERM);
				Ztcsetpgrp(tty_fildes, par_pgrp);
			finis

			testing("Process group ID of controlling terminal not changed.");
			start
				if(Ztcgetpgrp(tty_fildes) != ct_pgrp)
					error("%s\n%s%ld\n%s%ld",
							"Controlling terminal process group ID changed.",
							"Expected: ",
							(long) ct_pgrp,
							"Actual: ",
							(long) tcgetpgrp(tty_fildes));
			finis
		KROF
		SLEEP(GTI_LINE_CLOSE_DELAY)
	  finis
	}

	assertion(5, BASE);
	start
		testing("Call to tcsetpgrp() with an invalid file descriptor).");
		testing("Expect a return value of -1 and an errno of [EBADF].");
		start
			cur_pgrp = Zgetpgrp();
	
			expecting(-1);
			expecting(EBADF);
			Ztcsetpgrp(-1, cur_pgrp);
		finis
	finis

	assertion(7, NOT_APPLICABLE);

	assertion(9, BASE);
	start
		char	path[_POSIX_PATH_MAX +1];
		int		fd;

		FORK
			prep("Establish a regular file.");
			start
				Zstrcpy(path, temp_path());
				Zstrcat(path, FILENAME);
				fd = Zcreat(path, PROT_ALL);
			finis

			testing("Call to tcsetpgrp() when the file associated with the");
			testing("file descriptor is not a terminal device.");
			testing("Expect a return value of -1 and errno set to [ENOTTY].");
			start
				cur_pgrp = Zgetpgrp();

				expecting(-1);
				expecting(ENOTTY);
				Ztcsetpgrp(fd, cur_pgrp);
			finis

			if(!GTI_NOT_PROVIDED) {
				prep("Establish PORT_TTY as control terminal and PORT_LOOP.");
				start
					termios_prep(&tty_fildes, tty_termios_p, &loop_fildes,
										loop_termios_p);
				finis
	
				FORK
					testing("Call to tcsetpgrp() when fildes is not a");
					testing("controlling terminal.  Expect a return value");
					testing("of -1 and errno set to [ENOTTY].");
					start
						Zsetpgid((pid_t)0, (pid_t)0);
						cur_pgrp = Zgetpgrp();
		
						expecting(-1);
						expecting(ENOTTY);
						Ztcsetpgrp(loop_fildes, cur_pgrp);
					finis
				KROF
			}
		KROF
		if(!GTI_NOT_PROVIDED) {
			SLEEP(GTI_LINE_CLOSE_DELAY)
		}
	finis

	if(GTI_NOT_PROVIDED) {
		GTI_NOT_PROVIDED_RESULT(6, BASE)
		GTI_NOT_PROVIDED_RESULT(8, BASE)
		GTI_NOT_PROVIDED_RESULT(10, BASE)
		done();
	}

	assertion(6, BASE);
	start
		if(INVALID_PGRP_ID_VALUE != 0) {
			prep("Establish PORT_TTY as the control terminal and PORT_LOOP.");
			start
				termios_prep(&tty_fildes, tty_termios_p, &loop_fildes,
							loop_termios_p);
			finis

			FORK
				testing("Call to tcsetpgrp() with invalid process group ID.");
				testing("Expect return value of -1 and an errno of [EINVAL].");
				start
					ct_pgrp = Ztcgetpgrp(tty_fildes);

					expecting(-1);
					expecting(EINVAL);
					Ztcsetpgrp(tty_fildes, (pid_t)-1);
				finis

				testing("Process group ID of the controlling terminal is not");
				testing("changed.");
				start
					if(Ztcgetpgrp(tty_fildes) != ct_pgrp)
						error("%s\n%s%ld\n%s%ld",
							"Controlling terminal process group ID changed.",
							"Expected: ",
							(long) ct_pgrp,
							"Actual: ",
							(long) tcgetpgrp(tty_fildes));
				finis
			KROF
		}
		else
			not_implemented("invalid value for process group ID");
	finis
	
	assertion(8, BASE);
	start
		prep("Save process group ID of controlling terminal.");
		start
			cur_pgrp = Ztcgetpgrp(tty_fildes);
		finis

		FORK
			testing("Call to tcsetpgrp() when calling process does");
			testing("not have a controlling terminal.");
			testing("Expect a return value of -1 and errno set to [ENOTTY].");
			start
				Zsetsid();				/* Release controlling terminal. */
				cur_pgrp = Zgetpgrp();
	
				expecting(-1);
				expecting(ENOTTY);
				Ztcsetpgrp(tty_fildes, cur_pgrp);
			finis
		KROF

		testing("Process group ID of controlling terminal was not changed.");
		start
			if(cur_pgrp != Ztcgetpgrp(tty_fildes))
				error("%s\n%s%ld\n%s%ld",
						"Process group ID of controlling terminal changed.",
						"Expected: ",
						cur_pgrp,
						"Actual: ",
						Ztcgetpgrp(tty_fildes) );
		finis
	finis

	cleanup("Close terminal devices.");
	start
		SIGNAL(SIGHUP, SIG_IGN);
		Zclose(tty_fildes);
		Zclose(loop_fildes);
		SLEEP(GTI_LINE_CLOSE_DELAY)
	finis

	assertion(10, BASE);
	start
		FORK
			prep("Establish new session.");
			prep("Establish PORT_TTY as the control terminal and PORT_LOOP.");
			start
				termios_prep(&tty_fildes, tty_termios_p, &loop_fildes,
							loop_termios_p);
			finis

			prep("Fork a child.");
			start 
				SIGNAL(SIGHUP, SIG_IGN);
				pid = Zfork(); 
			finis 

			if(pid == (pid_t)0) {
				SLEEP(WAIT_FOR_TIME);	/* Allow time for parent to exit(). */

				testing("Call to tcsetpgrp() when the controlling terminal");
				testing("is no longer associated with the session of the");
				testing("calling process.  Expect a return value of -1 and");
				testing("errno set to [ENOTTY].");
				start
					cur_pgrp = Zgetpgrp();
					expecting(-1);
					expecting(ENOTTY);
					Ztcsetpgrp(tty_fildes, cur_pgrp);
				finis

				done();
			}
		KROF
		SLEEP(2 * WAIT_FOR_TIME);	/* Allow time for child to complete test. */
		SLEEP(GTI_LINE_CLOSE_DELAY)
	finis

	done();
}
