#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Zstat.h>
#include	<Zfcntl.h>
#include	<Zdirent.h>

#include	<sv_env.h>
#include	<sv_errno.h>

#define	DIRNAME		"5.6.3_pth.d"
#define	FILENAME	"5.6.3_pth"

static char	*file_names[] = {
							"ABCDEFGHIJLKM",
							"NOPQRSTUVWXYZ",
							"abcdefghijlkm",
							"nopqrstuvwxyz",
							"0123456789._-"
						   };


#define	INITIAL_PATHNAME_ASS_NUM	4
static	int	err_ass;
void	ck_ul(char *);

void main(void)
{
	int		ass_num = INITIAL_PATHNAME_ASS_NUM;
	char	path_cwd[_POSIX_PATH_MAX +1], path_ref[_POSIX_PATH_MAX +1];

	setup(E_ACCESS);
	funct(E_ACCESS);

								/*	Pathname resolution tests:
									  If call to access() is successful, test
									  assumes the path was resolved properly.
								*/
	assertion(ass_num, BASE);
	start
		prep("Change current working directory to \"../tmp/%s.d\".",
									element[(int) E_ACCESS].e_name);
		prep("Create known directory entries and files in working directory.");
		start
			exit_on(ERROR);
			Zstrcpy(path_cwd, temp_path());
			path_cwd[strlen(path_cwd) -1] = '\000'; /* delete trailing '/' */
			Zchdir(path_cwd);
			Zgetcwd(path_cwd, _POSIX_PATH_MAX +1);
			Zclose(Zcreat(FILENAME, S_IRWXU));

			Zmkdir(DIRNAME, S_IRWXU);
			sprintf(path_ref, "%s/%s", DIRNAME, FILENAME);
			Zclose(Zcreat(path_ref, S_IRWXU));

										/*
												Working_Directory
												 |              |
												 |              |
											  DIRNAME        FILENAME
												 |
											  FILENAME
										*/
		finis

		testing("When the first filename component of the filename");
		testing("argument is \".\", and the pathname does not begin with");
		testing("a slash, then the pathname is resolved by locating");
		testing("the second filename component in the current working");
		testing("directory.");
		start
			sprintf(path_ref, "%s/%s", ".", FILENAME);
			Zaccess(path_ref, F_OK);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Filename argument points to the string \"/\".");
		start
			Zaccess("/", F_OK);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Filename argument points to the string \"///\".");
		start
			Zaccess("///", F_OK);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Filename argument points to the string beginning");
		testing("with a \"/\".");
		start
			sprintf(path_ref, "%s/%s", path_cwd, FILENAME);
			Zaccess(path_ref, F_OK);
		finis

		testing("Filename argument points to the string beginning");
		testing("with \"///\".");
		start
			sprintf(path_ref, "//%s/%s", path_cwd, FILENAME);
			Zaccess(path_ref, F_OK);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("First filename component of the pathname argument is \"..\".");
		start
			Zchdir(DIRNAME);
			sprintf(path_ref, "../%s/%s", DIRNAME, FILENAME);
			Zaccess(path_ref, F_OK);

			Zchdir(path_cwd);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Filename argument points to the string \"F1/\" and");
		testing("F1 is a directory.");
		start
			sprintf(path_ref, "%s/", DIRNAME);
			Zaccess(path_ref, F_OK);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Filename argument points to the string \"F1//\" and");
		testing("F1 is a directory.");
		start
			sprintf(path_ref, "%s//", DIRNAME);
			Zaccess(path_ref, F_OK);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Filename argument points to the string \"F1/F2\".");
		start
			sprintf(path_ref, "%s/%s", DIRNAME, FILENAME);
			Zaccess(path_ref, F_OK);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Filename argument points to the string \"F1/./F2\".");
		start
			sprintf(path_ref, "%s/./%s", DIRNAME, FILENAME);
			Zaccess(path_ref, F_OK);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Filename argument points to the string \"F1/../F1/F2\".");
		start
			sprintf(path_ref, "%s/../%s/%s", DIRNAME, DIRNAME, FILENAME);
			Zaccess(path_ref, F_OK);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Filename argument points to the string \"F1//F2\".");
		start
			sprintf(path_ref, "%s//%s", DIRNAME, FILENAME);
			Zaccess(path_ref, F_OK);
		finis

		cleanup("Remove test file from test directory.");
		start
			Zunlink(path_ref);
		finis
	finis

	if(pathconf(DIRNAME, _PC_NO_TRUNC) == (long) -1) {
		assertion(++ass_num, BASE);
		start
			testing("Filename component is a string of more than {NAME_MAX}");
			testing("bytes in a directory for which {_POSIX_NO_TRUNC} is not");
			testing("supported.");
			start
				error("{_POSIX_NO_TRUNC} not supported.");
			finis
		finis
	}
	else
		assertion(++ass_num, NOT_APPLICABLE);

	assertion(++ass_num, BASE);
	start
		int	i;

		for(i=0; i<5; i++) {
			testing("Filename of \"%s\" is supported and", file_names[i]);
			testing("the unique identity of upper and lower case");
			testing("letters are retained.");
			start
				sprintf(path_ref, "%s/%s", DIRNAME, file_names[i]);
				Zmkdir(path_ref, PROT_ALL);
				ck_ul(file_names[i]);
				Zaccess(path_ref, F_OK);
				Zrmdir(path_ref);
			finis
		}
	finis

	assertion(++ass_num, BASE);
	start
		testing("Filename retains the unique identity of its upper and lower");
		testing("case letters.");
		start
			if(err_ass)
				error("See previous assertion for specifics of failure.");
		finis
	finis

	done();
}


		/*	For the Filename tests, check that the upper and lower case
			letters are preserved.  Read the directory and match with 
			the expected names.  This procedure is not performed for
			each element, but each element is covered, if all the filename
			tests are successful.
		*/

void	ck_ul(char *f_name)
{
	DIR		*dirp;
	struct	dirent *dirent;
	int	i;
	
	dirp = Zopendir(DIRNAME);
	dirent = Zreaddir(dirp);
	for(i=0; i<2; i++) {	/* Skip dot and dot-dot entries if they exist */
		if( ! ((dirent->d_name)[0] == '.') )
			break;
		dirent = Zreaddir(dirp);
	}
											/* Compare only filename */
	if(strcmp(dirent->d_name, f_name)) {
		error("%s\n%s\"%s\"\n%s\"%s\"\n",
			"Filename entry obtained not what was expected.",
			"Expected:	",
			f_name,
			"Actual:	",
			dirent->d_name);
		err_ass++;
	}
	Zclosedir(dirp);
}
