#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Zstat.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>

/*	This module has S_ISUID mode set to ROOT. */

#define	FILENAME	"5.6.4_19"

#define	S_IS_USER			1
#define	S_IS_APP			2
#define	S_ISUID_SUPPORTED	4
#define	S_ISGID_SUPPORTED	8


void t_modes(char *, int);


int main(void)
{
	char	path[_POSIX_PATH_MAX +1];
	struct	stat st_buf;
	pid_t	pid;

	setup(E_CHMOD);
	funct(E_CHMOD);

	assertion(19, BASE);
	start
		FORK
			prep("Set effective user ID and group ID to real user/group.");
			prep("Create test file.");
			start
				Zsetuid(getuid());
				Zsetgid(getgid());

				Zstrcpy(path, temp_path());
				Zstrcat(path, FILENAME);
				Zclose(Zcreat(path, PROT_ALL));
			finis

			testing("Instances of chmod() for various access modes.");
			start
				t_modes(path, FALSE);
			finis
		KROF
	finis

	assertion(20, BASE);
	start
									/* Appropriate privilege process. */
		prep("Initialize path to data file. Data file's owner and group");
		prep("does not match processes real user ID.");
		start
			Zstrcpy(path, data_path());
			Zstrcat(path, "mode_tests_su");
		finis

		testing("Instances of chmod() for various access modes.");
		start
			t_modes(path, FALSE);
		finis
	finis

	if(PCTS_CHMOD_SET_IDS) {
		assertion(21, BASE);
		start
			prep("Fork a child.");
			start
				pid = Zfork();
			finis

			if(pid == (pid_t)0) {
				prep("Set effective user ID and group ID to real user/group.");
				prep("Create test file.");
				start
					Zsetuid(getuid());
					Zsetgid(getgid());
					Zstrcpy(path, temp_path());
					Zstrcat(path, FILENAME);
				finis

				testing("Instances of chmod() with S_ISUID and/or S_ISGID set");
				testing("for various access modes.");
				start
					t_modes(path, TRUE);
				finis
				done();
			}
			wait_for(pid, 5 * MAX_TIME_TO_RUN);
		finis
	}
	else
		assertion(21, NOT_TESTABLE);

	if(PCTS_CHMOD_SET_IDS) {
		assertion(22, BASE);
		start
			testing("Instances of chmod() with S_ISUID and/or S_ISGID set");
			testing("for various access modes.");
			start
				t_modes(path, TRUE);
			finis
		finis
	}
	else
		assertion(22, NOT_TESTABLE);
	

	if(PCTS_CHMOD_SET_IDS) {
		assertion(23, BASE);
		start
		  FORK
			prep("Set effective user ID and group ID to real user/group.");
			prep("Initialize path to configured data file.");
			start
				Zsetuid(getuid());
				Zsetgid(getgid());

				Zstrcpy(path, data_path());
				Zstrcat(path, "chmod_setgid");
			finis

			prep("Verify S_ISGID bit is set on data file.");
			start
				Zstat(path, &st_buf);
				if( ! (st_buf.st_mode & S_ISGID) )
					error("%s\"%s\"\n%s\n%s",
							"Call to stat() reported unexpected mode for path.",
							path,
							"Expected:	S_ISGID bit set.",
							"Actual:	S_ISGID bit cleared.");
			finis

			testing("S_ISGID bit is cleared when chmod() is executed by");
			testing("a non-privileged process and the group owner of the");
			testing("file does not match the effective group ID or one of");
			testing("the supplementary group IDs.");
			start
				Zchmod(path, PROT_ALL);
				Zstat(path, &st_buf);
				if(st_buf.st_mode & S_ISGID)
						error("%s\n%s\n%s",
					  			"Call to stat() reported unexpected mode.",
					  			"Expected:	S_ISGID bit cleared.",
					  			"Actual:	S_ISGID bit set.");
			finis
		  KROF
		finis

		cleanup("Reset data file permission changed during testing.");
		start
			Zstrcpy(path, data_path());
			Zstrcat(path, "chmod_setgid");
			Zchmod(path, S_ISGID | S_IRWXU | S_IRWXG | S_IRWXO);
		finis
	}
	else
		assertion(23, NOT_TESTABLE);
	
    done();
}


/*	Mode values of Posix	*/ 
mode_t p_mode_o[] = {
	0,
	S_IXOTH,
	S_IXOTH | S_IWOTH,
	S_IXOTH | S_IROTH,
	S_IXOTH | S_IWOTH | S_IROTH,
	S_IWOTH,
	S_IWOTH | S_IROTH,
	S_IROTH,
};
#define	N_OMODES	8

mode_t p_mode_g[] = {
	0,
	S_IXGRP,
	S_IXGRP | S_IWGRP,
	S_IXGRP | S_IRGRP,
	S_IXGRP | S_IWGRP | S_IRGRP,
	S_IWGRP,
	S_IWGRP | S_IRGRP,
	S_IRGRP,
};
#define	N_GMODES	8

mode_t p_mode_u[] = {
	0,
	S_IXUSR,
	S_IXUSR | S_IWUSR,
	S_IXUSR | S_IRUSR,
	S_IXUSR | S_IWUSR | S_IRUSR,
	S_IWUSR,
	S_IWUSR | S_IRUSR,
	S_IRUSR,
};
#define	N_UMODES	8

mode_t p_mode_s[] = {
	S_ISGID,
	S_ISUID,
	S_ISGID | S_ISUID,
};
#define	N_SMODES	3

void t_modes(char *path, int t_f)
{
	mode_t	p_mode, ac_mode;
	mode_t	*p_m_o, *p_m_g, *p_m_u, *p_m_s;
	struct	stat	st_buf;
	int	m_u, m_g, m_o, m_s;

	break_on( ERROR );
	for(m_u = 0, p_m_u = &p_mode_u[0];
			m_u < N_UMODES; m_u++, p_m_u++)
	for(m_g = 0, p_m_g = &p_mode_g[0];
			m_g < N_GMODES; m_g++, p_m_g++)
	for(m_o = 0, p_m_o = &p_mode_o[0];
			m_o < N_OMODES; m_o++, p_m_o++)
	if(t_f == TRUE)
	  for(m_s = 0, p_m_s = &p_mode_s[0];
			m_s < N_SMODES; m_s++, p_m_s++){
		p_mode = *p_m_u |  *p_m_g | *p_m_o;
		if(t_f == TRUE)
			p_mode |= *p_m_s;
		expecting((int) 0);
		Zchmod(path, p_mode);
		expecting(SUCCESS);
		Zstat(path, &st_buf);
		ac_mode = st_buf.st_mode & (PROT_ALL | (mode_t)(S_ISUID | S_ISGID));
		if(ac_mode != p_mode)
			error("%s\n%s%lo\n%s%lo",
			  		"Call to stat() reported unexpected mode.",
			  		"Expected:	",
			  		(unsigned long)p_mode,
			  		"Actual:	",
			  		(unsigned long) ac_mode);
	}
}
