#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Zstat.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>

#define	String		"Test"
#define	Strlen		(sizeof(String) -1)
#define	FILENAME	"5.3.2_20"


int main(void)
{
	char	path[_POSIX_PATH_MAX +1];
	char	inbuf[Strlen +1];
	struct	stat st_buf1, st_buf2;
	int		fd;

	setup(E_CREAT);
	funct(E_CREAT);

	assertion(20, BASE);
	start
		testing("Call to creat() returns a file descriptor which can be used");
		testing("by other I/O functions to refer to the file specified by");
		testing("argument path.");
		start
			Zstrcpy(path, temp_path());
			Zstrcat(path, FILENAME);
			fd = Zcreat(path, PROT_ALL);
			Zwrite(fd, String, Strlen);
			Zstat(path, &st_buf1);
			Zfstat(fd, &st_buf2);
			if(st_buf1.st_ino != st_buf2.st_ino)
				error("%s\n%s%lu\n%s%lu.\n",
						"Inodes obtained from path and file descriptor differ.",
						"Expected: ",
						(unsigned long) st_buf1.st_ino,
						"Actual: ",
						(unsigned long) st_buf2.st_ino);
			Zclose(fd);
		finis
	finis

	assertion(21, BASE);
	start
		prep("Close file descriptor STDIN_FILENO.");
		start
			close(STDIN_FILENO);
		finis

		testing("Lowest file descriptor is returned on successful open().");
		start
			expecting(STDIN_FILENO);
			Zcreat(path, PROT_ALL);
			Zclose(STDIN_FILENO);
		finis
	finis

	assertion(22, EXTENDED);

	assertion(23, BASE);
	start
		testing("File offset for creat() is initially set to the beginning");
		testing("of the file.");
		start
			off_t	offset;

			fd = Zcreat(path, PROT_ALL);
			if(offset = Zlseek(fd, (off_t)0, SEEK_CUR))
				error("The file offfset for creat() was set to %lu.",
						(unsigned long) offset);
			Zclose(fd);
		finis
	finis

	assertion(24, BASE);
	start
		testing("Call to creat() returns a file desriptor which has the");
		testing("FD_CLOEXEC flag clear.");
		start
			fd = Zcreat(path, PROT_ALL);
			if((Zfcntl(fd, F_GETFD, 0) & FD_CLOEXEC) != 0)
				error("FD_CLOEXEC flag is set.");
		finis
	finis

	assertion(25, BASE);
	start
		testing("File descriptor opened by creat() can be written and");
		testing("cannot be read.");
		start
			fd = Zcreat(path, PROT_ALL);
			expecting(Strlen);
			Zwrite(fd, String, Strlen);

			Zlseek(fd, (off_t) 0, SEEK_SET);	/* Seek to beginning of file. */
			expecting(EBADF);
			Zread(fd, inbuf, Strlen);
		finis

		cleanup("Close and unlink test file.");
		start
			Zclose(fd);
			Zunlink(path);
		finis
	finis


	assertion(26, BASE);
	start
		time_t	s_time;

		prep("Establish path to data test file.");
		start
			Zstrcpy(path, data_path());
			Zstrcat(path, "ownership");
			Zstat(path, &st_buf1);
		finis

		prep("Obtain a starting time, then call creat() to open an existent");
		prep("test file");
		start
			Ztime(&s_time);				/* Obtain minimum reference time. */
			dt_secs(TRF_DELAY);			/* Delay a fixed interval.			*/

			fd = Zcreat(path, PROT_ALL);
		finis

		testing("Time related element \"st_atime\" of file is not updated.");
		start
			trf_ck(CHK_ATIME, FALSE, s_time, NO_FILDES, path);
		finis

		testing("File permission bits of file are not changed.");
		start
			Zstat(path, &st_buf2);
			if(st_buf2.st_mode != st_buf1.st_mode)
				error("%s\n%s%lu\n%s%lu.\n",
						"The test file mode has changed.",
						"Expected file mode:	",
						(unsigned long) st_buf1.st_mode,
						"Actual file mode:	",
						(unsigned long) st_buf2.st_mode);
		finis

		testing("Owner of file is not changed.");
		start
			if(st_buf2.st_uid != st_buf1.st_uid)
				error("%s\n%s%lu\n%s%lu.\n",
						"The test file uid has changed.",
						"Expected file uid:	",
						(unsigned long) st_buf1.st_uid,
						"Actual file uid:	",
						(unsigned long) st_buf2.st_uid);
			if(st_buf2.st_gid != st_buf1.st_gid)
				error("%s\n%s%lu\n%s%lu.\n",
						"The test file gid has changed.",
						"Expected file gid:	",
						(unsigned long) st_buf1.st_gid,
						"Actual file gid:	",
						(unsigned long) st_buf2.st_gid);
		finis
	finis

	done();
}
