#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Zstat.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<pcts_timer.h>

#define	String	"This is a test!!!"
#define	Strlen	(sizeof(String) -1)


int main(void)
{
	char	path[_POSIX_PATH_MAX +1];
	char	inbuf[Strlen +1];
	int		fd;
	time_t	s_time;

	setup(E_CREAT);
	funct(E_CREAT);

	assertion(39, BASE);
	start
		prep("Initialize path name for a file where search permission");
		prep("is denied on a component of the path prefix.");
		start
			Zstrcpy(path, data_path());
			Zstrcat(path, "no_perm.d");
			Zstrcat(path, "/open_tryme");
		finis

		testing("A call to creat() when search permission is denied.");
		start
			expecting(-1);
			expecting(EACCES);
			Zcreat(path, PROT_ALL);
		finis
	finis

	assertion(40, BASE);
	start
		testing("Call to creat() when path to existent file denies write");
		testing("permission to owner.");
		start
			Zstrcpy(path, data_path());
			Zstrcat(path, "no_write");

			expecting(-1);
			expecting(EACCES);
			Zcreat(path, PROT_ALL);
		finis

		testing("File referenced is not truncated.");
		start
			fd = Zopen(path, O_RDONLY);
			if(Zread(fd, inbuf, Strlen) == 0)
				error("File was truncated.");
		finis
	finis

	assertion(41, BASE);
	start
		prep("Initialize a path name.");
		start
			Zstrcpy(path, data_path());
			Zstrcat(path, "no_write.d");
			Zstrcat(path, "/new_file");
		finis

		testing("File does not exist and parent directory does not");
		testing("permit writing.");
		start
			expecting(-1);
			expecting(EACCES);
			Zcreat(path, PROT_ALL);
		finis

		testing("File is not created.");
		start
			expecting(-1);
			expecting(ENOENT);
			Zaccess(path, F_OK);
		finis
	finis

	assertion(42, BASE);
	start
		pid_t	pid;
		struct	stat st_buf;

		prep("Create a FIFO file and obtain a starting time.");
		start
			Zstrcpy(path, temp_path());
			Zstrcat(path, "er_eintr");
			Zmkfifo(path, PROT_ALL);

			Zstat(path, &st_buf);			/*	Set base-line times for FIFO */
			Zstrcpy(path, temp_path());
			Zstat(path, &st_buf);			/*	and parent directory.		*/

			Zstrcpy(path, temp_path());
			Zstrcat(path, "er_eintr");

			Ztime(&s_time);				/* Obtain minimum reference time. */
			dt_secs(TRF_DELAY);			/* Delay a fixed interval.			*/
		finis

		prep("Fork a child.");
		start
			sv_flush( );
			pid = Zfork();
		finis
	
		if(pid == (pid_t)0) {
			SIGNAL(SIGINT, sig_int);
			prep("When creat() is interrupted, errno is set to [EINTR].");
			start
				expecting(-1);
				expecting(EINTR);
				Zcreat(path, PROT_ALL);
			finis

			testing("Time related fields \"st_ctime\", \"st_mtime\", and");
			testing("\"st_atime\" of the file and \"st_ctime\"");
			testing("and \"st_mtime\" of the parent directory are not marked");
			testing("for update.");
			start
				trf_ck(CHK_CTIME, FALSE, s_time, NO_FILDES, path);
				trf_ck(CHK_MTIME, FALSE, s_time, NO_FILDES, path);
				trf_ck(CHK_ATIME, FALSE, s_time, NO_FILDES, path);

				Zstrcpy(path, temp_path());
				strcat(path, ".");
				trf_ck(CHK_CTIME, FALSE, s_time, NO_FILDES, path);
				trf_ck(CHK_MTIME, FALSE, s_time, NO_FILDES, path);
			finis

			done();
		}

		SLEEP(1);
		Zkill(pid, SIGINT);			/* Send signal to child. */
		WAIT_FOR_PID(pid);
	finis

	assertion(43, BASE);
	start
		struct	stat st_buf;

		prep("Initialize a path name.");
		prep("Obtain a starting time.");
		start
			Zstrcpy(path, temp_path());
			Zstrcat(path, "open_dir");
			Zmkdir(path, PROT_ALL);

			Zstat(path, &st_buf);		/*	Set base-line times for directory */
			strcat(path, "/..");
			Zstat(path, &st_buf);		/*	and parent directory.			*/

			Zstrcpy(path, temp_path());
			Zstrcat(path, "open_dir");

			Ztime(&s_time);				/* Obtain minimum reference time. */
			dt_secs(TRF_DELAY);			/* Delay a fixed interval.			*/
		finis

		testing("Call to creat() when named file is a directory.");
		start
			expecting(-1);
			expecting(EISDIR);
			Zcreat(path, PROT_ALL);
		finis

		testing("Time related fields \"st_ctime\", \"st_mtime\", and");
		testing("\"st_atime\" of the file and \"st_ctime\"");
		testing("and \"st_mtime\" of the parent directory are not marked");
		testing("for update.");
		start
			trf_ck(CHK_CTIME, FALSE, s_time, NO_FILDES, path);
			trf_ck(CHK_MTIME, FALSE, s_time, NO_FILDES, path);
			trf_ck(CHK_ATIME, FALSE, s_time, NO_FILDES, path);

			strcat(path, "/..");
			trf_ck(CHK_CTIME, FALSE, s_time, NO_FILDES, path);
			trf_ck(CHK_MTIME, FALSE, s_time, NO_FILDES, path);
		finis
	finis

	assertion(44, BASE);
	start
		int		open_max, topen_max, i;

		break_on(ERROR);

		prep("Initialize a path name.");
		start
			Zstrcpy(path, temp_path());
			Zstrcat(path, "fill_up");
		finis

		open_max = g_open_max();	/* Get value for {OPEN_MAX} */
									/* Get testable value for {OPEN_MAX} */
		topen_max = (open_max <= PCTS_OPEN_MAX) ? open_max : PCTS_OPEN_MAX;

		prep("Close all file descriptors except 0, 1 and 2.");
		start
			for(i=3; i<topen_max; i++)
				(void) close(i);
		finis

		if(open_max <= PCTS_OPEN_MAX) {
			prep("{OPEN_MAX} equal to {PCTS_OPEN_MAX}.");
			prep("Consume all available file descriptors.");
			start
				for(i=3; i<open_max; i++)
					fd = Zcreat(path, PROT_ALL);
			finis

			testing("Call to creat() when {OPEN_MAX} file descriptors in use");
			testing("and file exists.");
			testing("File is not truncated.");
			start
				Zwrite(fd, String, Strlen);			/* Write data to file. */
				Zlseek(fd, (off_t)0, SEEK_SET);

				expecting(-1);
				expecting(EMFILE);
				Zcreat(path, PROT_ALL);

											/* Allow test file to be read. */
				Zclose(fd);
				fd = Zopen(path, O_RDONLY);
				expecting(Strlen);
				Zread(fd, inbuf, Strlen);
			finis

			testing("Call to creat() when {OPEN_MAX} file descriptors in use");
			testing("and file does not exist.");
			testing("File is not created.");
			start
				Zstrcpy(path, temp_path());
				Zstrcat(path, "emfile_tst");

				expecting(-1);
				expecting(EMFILE);
				Zcreat(path, PROT_ALL);

				expecting(-1);
				expecting(ENOENT);
				Zaccess(path, F_OK);
			finis
		}
		else {
			prep("Testing for {PCTS_OPEN_MAX} < {OPEN_MAX}.");
			prep("Consume PCTS_OPEN_MAX -1 file descriptors."); 
			start
				for(i=3; i<(topen_max -1); i++)
					(void) Zcreat(path, PROT_ALL);
			finis

			prep("Initialize a path name.");
			start
				Zstrcpy(path, temp_path());
				Zstrcat(path, "emfile_tst");
			finis
	
			testing("Call to creat() on {PCTS_OPEN_MAX} -1 file descriptor.");
			start
				expecting(SUCCESS);
				Zcreat(path, PROT_ALL);
			finis
		}

		cleanup("Close all file descriptors except 0, 1 and 2.");
		start
			for(i=3; i<topen_max; i++)
				(void) close(i);
		finis
	finis

	if(tc_name_max()) {
		assertion(45, BASE);
		start
			e_NO_TRUNC(E_CREAT);
		finis
	}
	else
		assertion(45, NOT_TESTABLE);

	if(!tc_name_max()) {
		assertion(46, BASE);
		start
			e_NO_TRUNC(E_CREAT);
		finis
	}
	else
		assertion(46, NOT_TESTABLE);

	assertion(47, BASE);
	start
		e_ENAMETOOLONG(E_CREAT);
	finis

	assertion(48, EXTENDED);

	assertion(49, BASE);
	start
		testing("Call to creat() when the path prefix of path does not exist.");
		start
			Zstrcpy(path, temp_path());
			Zstrcat(path, "_open_/_open_");

			expecting(-1);
			expecting(ENOENT);
			Zcreat(path, PROT_ALL);
		finis

		testing("Call to creat() when path points to an empty string.");
		start
			expecting(-1);
			expecting(ENOENT);
			Zcreat("", PROT_ALL);
		finis
	finis

	assertion(50, EXTENDED);

	assertion(51, BASE);
	start
		testing("Call to creat() when pathname contains a non-directory");
		testing("component.");
		start
			Zstrcpy(path, data_path());
			Zstrcat(path, "Open_Test");
			Zstrcat(path, "/dummy");

			expecting(-1);
			expecting(ENOTDIR);
			Zcreat(path, PROT_ALL);
		finis
	finis

	done();
}
