#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Zstat.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<pcts_timer.h>

#define	FILENAME		"5.3.4_41"
#define	FILENAME_NEW	"5.3.4_41_n"
#define	String	"This is a test!!!"
#define	Strlen	(sizeof(String) -1)

void el_NO_TRUNC(int);
void el_ENAMETOOLONG(int);

void main(void)
{
	char	path_ex[_POSIX_PATH_MAX +1], path_new[_POSIX_PATH_MAX +1];
	struct	stat st_buf1, st_buf2;

	setup(E_LINK);
	funct(E_LINK);

	assertion(41, BASE);
	start
		prep("Initialize path names.");
		start
			Zstrcpy(path_ex, temp_path());
			Zstrcat(path_ex, FILENAME);
			Zclose(Zcreat(path_ex, PROT_ALL));		/* Create test file. */
			Zstrcpy(path_ex, temp_path());
			Zstrcat(path_ex, FILENAME_NEW);
			Zclose(Zcreat(path_ex, PROT_ALL));		/* Create test file. */

			Zstrcpy(path_ex, data_path());
			Zstrcat(path_ex, "no_srch.d");
			Zstrcat(path_ex, "/link_tryme");

			Zstrcpy(path_new, temp_path());
			Zstrcat(path_new, "link_new");
		finis

		testing("Call to link() when search permission is denied for");
		testing("component of the path prefix of existing.");
		start
			expecting(EACCES);
			Zlink(path_ex, path_new);
		finis
	finis

	assertion(42, BASE);
	start
		prep("Initialize path names.");
		start
			Zstrcpy(path_ex, temp_path());
			Zstrcat(path_ex, FILENAME);

			Zstrcpy(path_new, data_path());
			Zstrcat(path_new, "no_srch.d");
			Zstrcat(path_new, "/link_try");
		finis

		testing("Call to link() when search permission is denied for");
		testing("component of the path prefix of new.");
		start
			expecting(-1);
			expecting(EACCES);
			Zlink(path_ex, path_new);
		finis

		testing("No link was created and link count is unchanged.");
		start
			Zstrcpy(path_new, data_path());
			Zstrcat(path_new, "no_srch.d");
			Zchmod(path_new, S_IRWXU);	/* Get access to read directory. */
			Zstrcat(path_new, "/link_try");
			expecting(ENOENT);
			Zaccess(path_new, F_OK);

									/* Link count unchanged. */
			Zstat(path_ex, &st_buf1);
			if(st_buf1.st_nlink != (nlink_t)1)
				error("Link count set to %lu.",
								(unsigned long) st_buf1.st_nlink);
		finis

		cleanup("Reset data file permisssion changed during testing.");
		start
			Zstrcpy(path_new, data_path());
			Zstrcat(path_new, "no_srch.d");
			Zchmod(path_new, S_IRUSR | S_IWUSR);
		finis
	finis

	assertion(43, BASE);
	start
		prep("Initialize path names.");
		start
			Zstrcpy(path_ex, temp_path());
			Zstrcat(path_ex, FILENAME);

			Zstrcpy(path_new, data_path());
			Zstrcat(path_new, "no_write.d");
			Zstrcat(path_new, "/link_try");
		finis

		testing("Call to link() when write permission for directory of new");
		testing("is denied.");
		start
			expecting(EACCES);
			Zlink(path_ex, path_new);

									/* Link count unchanged. */
			Zstat(path_ex, &st_buf1);
			if(st_buf1.st_nlink != (nlink_t)1)
				error("Link count set to %lu.",
								(unsigned long) st_buf1.st_nlink);
		finis
	finis

	assertion(44, EXTENDED);

	assertion(45, BASE);
	start
		prep("Initialize path names.");
		start
			Zstrcpy(path_ex, temp_path());
			Zstrcat(path_ex, FILENAME);
			Zstrcpy(path_new, temp_path());
			Zstrcat(path_new, FILENAME_NEW);
		finis

		testing("Call to link() when path to new already exists.");
		start
			expecting(EEXIST);
			Zlink(path_ex, path_new);

									/* Link count unchanged. */
			Zstat(path_ex, &st_buf1);
			if(st_buf1.st_nlink != (nlink_t)1)
				error("Link count set to %lu.",
								(unsigned long) st_buf1.st_nlink);
		finis
	finis

	assertion(46, BASE);
	start
		int	link_cnt, i;

		break_on(ERROR);
		prep("Initialize a path name.");
		start
			Zstrcpy(path_ex, temp_path());
			Zstrcat(path_ex, "link_target");
			Zclose(Zcreat(path_ex, PROT_ALL));
		finis

		link_cnt = pathconf(path_ex, _PC_LINK_MAX);
		if((link_cnt > PCTS_LINK_MAX) || (link_cnt == -1)) {
			link_cnt = PCTS_LINK_MAX;
			testing("Ensure at least %lu links can be performed successfully.",
							(unsigned long) PCTS_LINK_MAX);
		}
		else {
			testing("Errno [EMLINK] is detected.");
		}
		start
			for(i = 1; i < link_cnt; i++){
				sprintf(path_new, "%s%d", temp_path(), i);
				Zlink(path_ex, path_new);
			}

			if(link_cnt != PCTS_LINK_MAX) {
				i++;
				Zstat(path_ex, &st_buf1);
				sprintf(path_new, "%s%d", temp_path(), i);
				expecting(EMLINK);
				Zlink(path_ex, path_new);

										/* No link was created. */
				expecting(ENOENT);
				Zaccess(path_new, F_OK);

										/* Link count unchanged. */
				Zstat(path_ex, &st_buf2);
				if(st_buf1.st_nlink != st_buf2.st_nlink)
					error("Link count changed from %lu to %lu.",
									(unsigned long) st_buf1.st_nlink,
									(unsigned long) st_buf2.st_nlink);
			}
		finis

		cleanup("Unlink temp files.");
		start
			for(i = 1; i < link_cnt; i++){
				(void) sprintf(path_new, "%s%d", temp_path(), i);
				(void) Zunlink(path_new);
			}
		finis
	finis


	if(tc_name_max()) {
		assertion(47, BASE);
		start
			el_NO_TRUNC(TRUE);
		finis
	}
	else
		assertion(47, NOT_TESTABLE);

	if(!tc_name_max()) {
		assertion(48, BASE);
		start
			e_NO_TRUNC(TRUE);
		finis
	}
	else
		assertion(48, NOT_TESTABLE);

	if(tc_name_max()) {
		assertion(49, BASE);
		start
			el_NO_TRUNC(FALSE);
		finis
	}
	else
		assertion(49, NOT_TESTABLE);

	if(!tc_name_max()) {
		assertion(50, BASE);
		start
			e_NO_TRUNC(FALSE);
		finis
	}
	else
		assertion(50, NOT_TESTABLE);

	assertion(51, BASE);
	start
		el_ENAMETOOLONG(TRUE);
	finis

	assertion(52, BASE);
	start
		el_ENAMETOOLONG(FALSE);
	finis

	assertion(53, BASE);
	start
		prep("Initialize a path name.");
		start
			Zstrcpy(path_ex, temp_path());
			Zstrcat(path_ex, "no_link_dir/_link");
			Zstrcpy(path_new, temp_path());
			Zstrcat(path_new, FILENAME_NEW);
			unlink(path_new);
		finis

		testing("Call to link() when component of existing does not exist.");
		start
			expecting(-1);
			expecting(ENOENT);
			Zlink(path_ex, path_new);
										/* No link was created. */
			expecting(-1);
			expecting(ENOENT);
			Zaccess(path_new, F_OK);
		finis
	finis

	assertion(54, BASE);
	start
		prep("Initialize a path name.");
		start
			Zstrcpy(path_ex, temp_path());
			Zstrcat(path_ex, FILENAME);

			Zstrcpy(path_new, temp_path());
			Zstrcat(path_new, "no_link_dir/__link__");
		finis

		testing("Call to link() when component of new does not exist.");
		start
			Zstat(path_ex, &st_buf1);
			expecting(-1);
			expecting(ENOENT);
			Zlink(path_ex, path_new);

										/* Link count unchanged. */
			expecting(SUCCESS);
			Zstat(path_ex, &st_buf2);
			if(st_buf1.st_nlink != st_buf2.st_nlink)
				error("Link count changed from %lu to %lu.",
								(unsigned long) st_buf1.st_nlink,
								(unsigned long) st_buf2.st_nlink);
		finis
	finis

	assertion(55, BASE);
	start
		prep("Initialize a path name.");
		start
			Zstrcpy(path_ex, temp_path());
			Zstrcat(path_ex, "_link");
			Zstrcpy(path_new, temp_path());
			Zstrcat(path_new, FILENAME_NEW);
		finis

		testing("Call to link() when existing does not exist.");
		start
			expecting(ENOENT);
			Zlink(path_ex, path_new);
										/* No link was created. */
			expecting(-1);
			expecting(ENOENT);
			Zaccess(path_new, F_OK);
		finis
	finis

	assertion(56, BASE);
	start
		testing("Call to link() when existing points to an empty string.");
		start
			expecting(-1);
			expecting(ENOENT);
			Zlink("", path_new);
										/* No link was created. */
			expecting(-1);
			expecting(ENOENT);
			Zaccess(path_new, F_OK);
		finis
	finis

	assertion(57, BASE);
	start
		testing("Call to link() when new points to an empty string.");
		start
			Zstrcpy(path_ex, temp_path());
			Zstrcat(path_ex, FILENAME);
			Zstat(path_ex, &st_buf1);

			expecting(-1);
			expecting(ENOENT);
			Zlink(path_ex, "");
										/* Link count unchanged. */
			expecting(SUCCESS);
			Zstat(path_ex, &st_buf2);
			if(st_buf1.st_nlink != st_buf2.st_nlink)
				error("Link count changed from %lu to %lu.",
								(unsigned long) st_buf1.st_nlink,
								(unsigned long) st_buf2.st_nlink);
		finis
	finis

	assertion(58, EXTENDED);

	assertion(59, BASE);
	start
		prep("Initialize a path name.");
		start
			Zstrcpy(path_ex, temp_path());
			Zstrcat(path_ex, "no_dir");
			Zclose(Zcreat(path_ex, PROT_ALL));
			Zstrcat(path_ex, "/AA");

			Zstrcpy(path_new, temp_path());
			Zstrcat(path_new, FILENAME_NEW);
		finis

		testing("Call to link() when component of existing is not a directory.");
		start
			expecting(-1);
			expecting(ENOTDIR);
			Zlink(path_ex, path_new);
										/* No link was created. */
			expecting(ENOENT);
			Zaccess(path_new, F_OK);
		finis
	finis

	assertion(60, BASE);
	start
		prep("Establish test files.");
		start
			Zstrcpy(path_new, path_ex);

			Zstrcpy(path_ex, temp_path());
			Zstrcat(path_ex, FILENAME);
		finis

		testing("Call to link() when component of new is not a directory.");
		start
			Zstat(path_ex, &st_buf1);
			expecting(-1);
			expecting(ENOTDIR);
			Zlink(path_ex, path_new);
										/* Link count unchanged. */
			expecting(SUCCESS);
			Zstat(path_ex, &st_buf2);
			if(st_buf1.st_nlink != st_buf2.st_nlink)
				error("Link count changed from %lu to %lu.",
								(unsigned long) st_buf1.st_nlink,
								(unsigned long) st_buf2.st_nlink);
		finis
	finis

	done();
}


				/*	Test assertions asociated with {NAME_MAX} for link().
					When ex_path is TRUE, path_ex is the path where
					the long name is generated for.
				*/
void el_NO_TRUNC(int ex_path)
{
	char	*lpath;
	long	res_p;
	char	path[_POSIX_PATH_MAX +1];
	struct	stat st_buf1, st_buf2;
	char	*path_tmp;

	Zstrcpy(path, temp_path());
	res_p = pathconf(path, _PC_NO_TRUNC);
	if(res_p != (long) -1 )  {

		lpath = g_lname();

		prep("Initialize a second pathname.");
		start
			path_tmp = (char *) Zmalloc(_POSIX_PATH_MAX +1);
			Zstrcpy(path_tmp, temp_path());
			Zstrcat(path_tmp, "path_2nd");
			if(!ex_path)
				Zclose(Zcreat(path_tmp, PROT_ALL));
		finis

		if(tc_name_max() == TRUE) {
			testing("Path component longer than {NAME_MAX}.");
			start
				if(ex_path) {
					expecting(ENAMETOOLONG);
					Zlink(lpath, path_tmp);
											/* No link was created. */
					expecting(ENOENT);
					Zaccess(path_tmp, F_OK);
				}
				else {
					Zstat(path_tmp, &st_buf1);
					expecting(ENAMETOOLONG);
					Zlink(path_tmp, lpath);
					expecting(SUCCESS);
					Zstat(path_tmp, &st_buf2);

											/* Link count unchanged. */
					if(st_buf1.st_nlink != st_buf2.st_nlink)
						error("Link count changed from %lu to %lu.",
								(unsigned long) st_buf1.st_nlink,
								(unsigned long) st_buf2.st_nlink);
				}
			finis
		}
		else {
			testing("Path component equal to {PCTS_NAME_MAX}.");
			start
				if(ex_path)
					Zlink(lpath, path_tmp);
				else
					Zlink(path_tmp, lpath);
			finis
		}

		cleanup("Remove buffer space allocated for long path.");
		start
			if(!ex_path)
				Zunlink(path_tmp);
			Zfree(lpath);
			Zfree(path_tmp);
		finis
	}
	else
		error("{_POSIX_NO_TRUNC} not supported.");
}


				/*	Test [ENAMETOOLONG] for link() 
					When ex_path is TRUE, path_ex is the path where
					the long name is generated for.
				*/
void el_ENAMETOOLONG(int ex_new)
{
	char	*lpath, *path_tmp;
	int		test_pm;

	if(test_pm = tc_path_max()) {
		prep("Establish pathname of {PATH_MAX} +2 bytes.");
		start
			lpath = g_lpath();
		finis
	}
	else {
		prep("Establish pathname of {PCTS_PATH_MAX} +2 bytes.");
		start
			char	*ptr, *ptr_end;

					/* Allocate buffer for {PCTS_PATH_MAX} +1 + "/X" */
			lpath = (char *) Zmalloc(PCTS_PATH_MAX +3);

			Zstrcpy(lpath, temp_path());
			Zstrcat(lpath, "ENAMETOOLONG");
			if(access(lpath, F_OK))
				Zmkdir(lpath, PROT_ALL);
			ptr = lpath +Zstrlen(lpath);

			ptr_end = PCTS_PATH_MAX +lpath;
			for( ; ptr +16 < ptr_end; ptr += 16 )
        		(void) strcpy(ptr, "/../ENAMETOOLONG");

			for( ; ptr < ptr_end; ptr += 2 )
        			(void) strcpy(ptr, "/.");

							/* End of path may be "/.//X" "or /./X". */
        	(void) strcpy(ptr_end, "/X");
		finis
	}

	prep("Initialize second pathname.");
	start
		path_tmp = (char *) Zmalloc(_POSIX_PATH_MAX +1);
		Zstrcpy(path_tmp, temp_path());
		Zstrcat(path_tmp, "path_scnd");
		if(!ex_new)
			Zclose(Zcreat(path_tmp, PROT_ALL));
	finis

	if(test_pm = tc_path_max()) {
		testing("For path exceeds {PATH_MAX}.");
		start
			expecting(ENAMETOOLONG);
			if(ex_new)
				Zlink(lpath, path_tmp);
			else
				Zlink(path_tmp, lpath);
		finis
	}
	else {
		testing("For path length of {PCTS_PATH_MAX}.");
		start
			if(ex_new)
				Zlink(lpath, path_tmp);
			else
				Zlink(path_tmp, lpath);
		finis
	}

	cleanup("Remove buffer space allocated for long path.");
	start
		if(!ex_new)
			Zunlink(path_tmp);
		Zfree( lpath );
		Zfree(path_tmp);
	finis
}
