#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Zstat.h>
#include	<Zfcntl.h>
#include	<Zdirent.h>

#include	<sv_env.h>
#include	<sv_errno.h>

#define	FILENAME	"5.4.1_24"
#define	DIRNAME		"5.4.1_24.d"


int main(void)
{
	char	path[_POSIX_PATH_MAX +1];

	setup(E_MKDIR);
	funct(E_MKDIR);

	assertion(24, BASE);
	start
		testing("Search permission for path component is denied.");
		start
			Zstrcpy(path, data_path());
			Zstrcat(path, "no_srch.d");
			Zstrcat(path, "/mkdir_tryme");

			expecting(-1);
			expecting(EACCES);
			Zmkdir(path, PROT_ALL);
		finis
	finis

	assertion(25, BASE);
	start
		testing("Write permission for parent directory is denied.");
		start
			Zstrcpy(path, data_path());
			Zstrcat(path, "no_write.d");
			Zstrcat(path, "/mkdir_tryme");

			expecting(-1);
			expecting(EACCES);
			Zmkdir(path, PROT_ALL);
		finis
	finis

	assertion(26, BASE);
	start
		testing("Call to mkdir() in a directory which contains a file of");
		testing("the same name.");
		start
			Zstrcpy(path, temp_path());
			Zstrcat(path, DIRNAME);
			Zmkdir(path, PROT_ALL);

			expecting(-1);
			expecting(EEXIST);
			Zmkdir(path, PROT_ALL);
		finis
	finis

	{										/*	Assertions 27 and 28. */
		long	link_cnt, lncnt;
		long	link_fst, link_lst;
    	DIR		*dirp;
    	struct	dirent	*dirent;

								/*	Determine whether \".\" and/or \"..\"
									entries are created in test directory.
								*/
 		dirp = Zopendir(path);
		link_fst = 0;
  		while((dirent = readdir(dirp)) > (struct dirent *)0)
			link_fst++;
		Zclosedir(dirp);

		link_cnt = pathconf(path, _PC_LINK_MAX);

		if((link_cnt > PCTS_LINK_MAX) || (link_cnt == (long) -1)) {
			link_cnt = (long) -1;

			assertion(27, NOT_TESTABLE);
			assertion(28, BASE);
			start
				testing("To insure that at least {PCTS_LINK_MAX} [%lu]",
													PCTS_LINK_MAX);
				testing("links can be placed in a directory without");
				testing("attaining the errno [EMLINK].");
				start
					link_lst = (link_cnt == (long) -1)?PCTS_LINK_MAX:link_cnt;
					for(lncnt = link_fst; lncnt < link_lst; lncnt++){
						sprintf(path, "%s%s/%ld", temp_path(), DIRNAME, lncnt);
						Zmkdir(path, PROT_ALL);
					}

					if(link_cnt != (long)-1) {
						lncnt++;
						sprintf(path, "%s%s/%ld", temp_path(), DIRNAME, lncnt);
						expecting(EMLINK);
						Zmkdir(path, PROT_ALL);
					}
				finis

				if(link_cnt != (long)-1) {
					testing("No directory is created.");
					start
						expecting(ENOENT);
						Zaccess(path, F_OK);
					finis
				}


				cleanup("Remove temp directories.");
				start
					for(lncnt = link_fst; lncnt < link_lst; lncnt++){
						sprintf(path, "%s%s/%ld", temp_path(), DIRNAME, lncnt);
						Zrmdir(path);
					}
				finis
			finis
		}
		else {
			assertion(28, NOT_TESTABLE);

			assertion(27, BASE);
			start
				testing("To generate [EMLINK].");
				start
					link_lst = (link_cnt == (long) -1)?PCTS_LINK_MAX:link_cnt;
					for(lncnt = link_fst; lncnt < link_lst; lncnt++){
						sprintf(path, "%s%s/%ld", temp_path(), DIRNAME, lncnt);
						Zmkdir(path, PROT_ALL);
					}

					if(link_cnt != (long)-1) {
						lncnt++;
						sprintf(path, "%s%s/%ld", temp_path(), DIRNAME, lncnt);
						expecting(EMLINK);
						Zmkdir(path, PROT_ALL);
					}
				finis

				if(link_cnt != (long)-1) {
					testing("No directory is created.");
					start
						expecting(ENOENT);
						Zaccess(path, F_OK);
					finis
				}

				cleanup("Remove temp directories.");
				start
					for(lncnt = link_fst; lncnt < link_lst; lncnt++){
						sprintf(path, "%s%s/%ld", temp_path(), DIRNAME, lncnt);
						Zrmdir(path);
					}
				finis
			finis
		}
	}

	if(tc_name_max()) {
		assertion(29, BASE);
		start
			e_NO_TRUNC(E_MKDIR);
		finis
	}
	else
		assertion(29, NOT_TESTABLE);

	if(!tc_name_max()) {
		assertion(30, BASE);
		start
			e_NO_TRUNC(E_MKDIR);
		finis
	}
	else
		assertion(30, NOT_TESTABLE);

	assertion(31, BASE);
	start
		e_ENAMETOOLONG(E_MKDIR);
	finis

	assertion(32, BASE);
	start
		testing("Call mkdir() for a path that does not exist.");
		start
			Zstrcpy(path, temp_path());
			Zstrcat(path, "_mkdir/_mkdir_");

			expecting(-1);
			expecting(ENOENT);
			Zmkdir(path, PROT_ALL);
		finis
	finis

	assertion(33, BASE);
	start
		testing("Call mkdir() with an empty string for path.");
		start
			expecting(-1);
			expecting(ENOENT);
			Zmkdir("", PROT_ALL);
		finis
	finis

	assertion(34, EXTENDED);

	assertion(35, BASE);
	start
		testing("Call mkdir() when path component is not a directory.");
		start
			Zstrcpy(path, temp_path());
			Zstrcat(path, FILENAME);
			Zclose(Zcreat(path, PROT_ALL));
			Zstrcat(path, "/");
			Zstrcat(path, FILENAME);

			expecting(-1);
			expecting(ENOTDIR);
			Zmkdir(path, PROT_ALL);
		finis
	finis

	done();
}
