#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Zstat.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>

mode_t p_mode_o[] = {
	0,
	S_IXOTH,
	S_IXOTH | S_IWOTH,
	S_IXOTH | S_IROTH,
	S_IXOTH | S_IWOTH | S_IROTH,
	S_IWOTH,
	S_IWOTH | S_IROTH,
	S_IROTH,
};
#define	N_OMODES	8

mode_t p_mode_g[] = {
	0,
	S_IXGRP,
	S_IXGRP | S_IWGRP,
	S_IXGRP | S_IRGRP,
	S_IXGRP | S_IWGRP | S_IRGRP,
	S_IWGRP,
	S_IWGRP | S_IRGRP,
	S_IRGRP,
};
#define	N_GMODES	8

mode_t p_mode_u[] = {
	0,
	S_IXUSR,
	S_IXUSR | S_IWUSR,
	S_IXUSR | S_IRUSR,
	S_IXUSR | S_IWUSR | S_IRUSR,
	S_IWUSR,
	S_IWUSR | S_IRUSR,
	S_IRUSR,
};
#define	N_UMODES	8

#define	FIFONAME	"5.4.2_19_f"
#define	DIRNAME		"5.4.2_19.d"


int main(void)
{
	char	path[_POSIX_PATH_MAX +1];
	struct	stat st_buf;

	setup(E_MKFIFO);
	funct(E_MKFIFO);

	assertion(19, BASE);
	start
		prep("Determine path for testing directory.");
		start
			Zstrcpy(path, temp_path());
			Zstrcat(path, DIRNAME);
		finis

		testing("All combinations of mode and creation masks");
		testing("of null and \"S_IRWXU | S_IRWXG | S_IRWXO\".");
		start
			mode_t	p_mode, u_mode;
			mode_t	*p_m_o, *p_m_g, *p_m_u;
			int	m_u, m_g, m_o;
			int	ret_val;

			break_on(ERROR);
			for(m_u = 0, p_m_u = &p_mode_u[0];
					m_u < N_UMODES; m_u++, p_m_u++)
			for(m_g = 0, p_m_g = &p_mode_g[0];
					m_g < N_GMODES; m_g++, p_m_g++)
			for(m_o = 0, p_m_o = &p_mode_o[0];
					m_o < N_OMODES; m_o++, p_m_o++) {
				p_mode = *p_m_u |  *p_m_g | *p_m_o;

				/*  DEFINE UMASK VALUES */
				for(u_mode = (mode_t)0; u_mode <= PROT_ALL;
						u_mode += PROT_ALL){

					(void)umask(u_mode);

					ret_val = Zmkfifo(path, p_mode);
					Zstat(path, &st_buf);
					if((st_buf.st_mode & (mode_t)(PROT_ALL)) !=
							(p_mode & ~u_mode))
						error("%s\n%s%o\n%s%o\n",
					  		"The stat() call reported an unexpected mode.",
					  		"Expected:	",
					  		p_mode & ~u_mode,
					  		"Actual:	",
					  		(st_buf.st_mode &
							(mode_t)(PROT_ALL)));
					if(ret_val != (int) 0)
						error("%s\n%s\n%s%d\n",
					  		"The mkfifo() call return an unexpected value.",
					  		"Expected:	0",
					  		"Actual:	",
					  		ret_val);
					Zunlink(path);
				}
			}
		finis

		cleanup("Reset \"umask\".");
		start
			(void)umask(0);
		finis
	finis

	assertion(20, BASE);
	start
		prep("Create a FIFO file in a directory whose group owner");
		prep("is different than the process effective group ID.");
		start
			Zstrcpy(path, data_path());
			Zstrcat(path, "group_bin.d");
			Zstrcat(path, "/");
			Zstrcat(path, FIFONAME);
			unlink(path);				/* Ensure FIFO does not exist. */
			Zmkfifo(path, PROT_ALL);
		finis

		testing("Effective user ID matches st_uid of new directory file.");
		start
			Zstat(path, &st_buf);
			if(st_buf.st_uid  != geteuid())
				error("%s\n%s\"%o\"/n%s\"%o\"\n",
						"Owner of file in error.",
						"Expected owner: ",
						geteuid(),
						"Actual owner: ",
						st_buf.st_uid);
		finis

		cleanup("Remove FIFO created in data directory.");
		start
			unlink(path);
		finis
	finis

	assertion(22, BASE);
	start
		time_t	s_time;

		prep("Record a starting time, then create a FIFO.");
		start
			Ztime(&s_time);				/* Obtain minimum reference time. */
			dt_secs(TRF_DELAY);			/* Delay a fixed interval.			*/
			Zstrcpy(path, temp_path());
			Zstrcat(path, FIFONAME);
			Zmkfifo(path, PROT_ALL);
		finis
	
		testing("Time related elements \"st_ctime\", \"st_mtime\", and");
		testing("\"st_atime\" of the FIFO file are marked for update.");
		start
			trf_ck(CHK_CTIME, TRUE, s_time, NO_FILDES, path);
			trf_ck(CHK_MTIME, TRUE, s_time, NO_FILDES, path);
			trf_ck(CHK_ATIME, TRUE, s_time, NO_FILDES, path);
		finis

		testing("Time related elements \"st_ctime\" and \"st_mtime\"");
		testing("of parent directory are marked for update.");
		start
			trf_ck(CHK_CTIME, TRUE, s_time, NO_FILDES, temp_path());
			trf_ck(CHK_MTIME, TRUE, s_time, NO_FILDES, temp_path());
		finis
	finis

	done();
}
