#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Zstat.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>

#define	String		"Test"
#define	Strlen		(sizeof(String) -1)
#define	FILENAME	"5.3.1_21"

void	o_read_tst(int, int);
void	o_clean(int, char *);

int main(void)
{
	char	path[_POSIX_PATH_MAX +1];
	char	inbuf[Strlen +1];
	struct	stat st_buf1, st_buf2;
	int		fd;

	setup(E_OPEN);
	funct(E_OPEN);

	assertion(21, BASE);
	start
		testing("Call to open() returns a file descriptor which can be used");
		testing("by other I/O functions to refer to the file specified by");
		testing("argument path.");
		start
			Zstrcpy(path, temp_path());
			Zstrcat(path, FILENAME);
			fd = Zopen(path, O_RDWR | O_CREAT, PROT_ALL);
			Zwrite(fd, String, Strlen);
			Zread(fd, inbuf, Strlen);
			Zstat(path, &st_buf1);
			Zfstat(fd, &st_buf2);
			if(st_buf1.st_ino != st_buf2.st_ino)
				error("%s\n%s%lu\n%s%lu.\n",
						"Inodes obtained from path and file descriptor differ.",
						"Expected: ",
						(unsigned long) st_buf1.st_ino,
						"Actual: ",
						(unsigned long) st_buf2.st_ino);
			Zclose(fd);
		finis
	finis

	assertion(22, BASE);
	start
		break_on(ERROR);
		prep("Close file descriptor STDIN_FILENO.");
		start
			close(STDIN_FILENO);
		finis

		testing("Lowest file descriptor is returned on successful open().");
		start
			expecting(STDIN_FILENO);
			Zopen(path, O_RDONLY);
			Zclose(STDIN_FILENO);
		finis
	finis

	assertion(23, EXTENDED);

	assertion(24, BASE);
	start
		testing("File offset on open() is initially set to beginning of file.");
		start
			off_t	offset;
			int		i;

			for(i=1; i<8; i++) {
				switch(i) {
						case 1:
								fd = Zopen(path, O_RDONLY);
								break;
						case 2:
								fd = Zopen(path, O_WRONLY);
								break;
						case 3:
								fd = Zopen(path, O_RDWR);
								break;
						case 4:
								fd = Zopen(path, O_WRONLY | O_APPEND);
								break;
						case 5:
								fd = Zopen(path, O_WRONLY | O_NONBLOCK);
								break;
						case 6:
								fd = Zopen(path, O_WRONLY | O_CREAT, PROT_ALL);
								break;
						case 7:
								Zunlink(path);
								fd = Zopen(path, O_WRONLY | O_CREAT | O_EXCL,
												PROT_ALL);
								break;
						default:break;
				}

				if(offset = Zlseek(fd, (off_t)0, SEEK_CUR))
					error("%s%d%s\n%s\n%s%lu.\n",
							"The file offfset on open() for CASE ",
							i,
							" is incorrect.",
							"Expected offset of 0.",
							"Actual offset is ",
							(unsigned long) offset);
				Zclose(fd);
			}
		finis
	finis

	assertion(25, BASE);
	start
		testing("Call to open() returns a file desriptor which has the");
		testing("FD_CLOEXEC flag clear.");
		start
			fd = Zopen(path, O_RDWR | O_CREAT, PROT_ALL);
			if((Zfcntl(fd, F_GETFD, 0) & FD_CLOEXEC) != 0)
				error("FD_CLOEXEC flag is set.");
		finis
	finis

	assertion(26, BASE);
	start
		prep("Create a test file and write data to it.");
		start
			fd = Zopen(path, O_RDWR | O_CREAT, PROT_ALL);
			Zwrite(fd, String, Strlen);
			Zclose(fd);
		finis

		prep("Open test file with O_RDONLY flag set.");
		start
			fd = Zopen(path, O_RDONLY);
		finis

		testing("Reading of file is successful.");
		start
			o_read_tst(fd, Strlen);
			Zlseek(fd, (off_t) 0, SEEK_SET);	/* Seek to beginning of file. */
		finis

		testing("Writing to file fails with [EBADF].");
		start
			expecting(EBADF);
			Zwrite(fd, String, Strlen);
		finis

		o_clean(fd, path);
	finis

	assertion(27, BASE);
	start
		prep("Create a test file and write data to it.");
		start
			fd = Zopen(path, O_RDWR | O_CREAT, PROT_ALL);
			Zwrite(fd, String, Strlen);
			Zclose(fd);
		finis

		prep("Open test file with O_WRONLY flag set.");
		start
			fd = Zopen(path, O_WRONLY);
		finis

		testing("Writing to file is successful.");
		start
			Zwrite(fd, String, Strlen);
			Zlseek(fd, (off_t) 0, SEEK_SET);	/* Seek to beginning of file. */
		finis

		testing("Reading of file fails with [EBADF].");
		start
			expecting(EBADF);
			Zread(fd, inbuf, Strlen);
		finis

		o_clean(fd, path);
	finis

	assertion(28, BASE);
	start
		prep("Create a test file and write data to it.");
		start
			fd = Zopen(path, O_RDWR | O_CREAT, PROT_ALL);
			Zwrite(fd, String, Strlen);
			Zclose(fd);
		finis

		prep("Open test file with O_RDWR flag set.");
		start
			fd = Zopen(path, O_RDWR);
		finis

		testing("Reading of file is successful.");
		start
			o_read_tst(fd, Strlen);
		finis

		testing("Writing to file is successful.");
		start
			Zwrite(fd, String, Strlen);
		finis

		o_clean(fd, path);
	finis

	assertion(29, BASE);
	start
		time_t	s_time;

		prep("Establish path to data test file.");
		start
			Zstrcpy(path, data_path());
			Zstrcat(path, "ownership");
			Zstat(path, &st_buf1);
		finis

		prep("Obtain a starting time, then open existing test file with");
		prep("O_TRUNC not set.");
		start
			Ztime(&s_time);				/* Obtain minimum reference time. */
			dt_secs(TRF_DELAY);			/* Delay a fixed interval.			*/

			fd = Zopen(path, O_RDWR, (mode_t) S_IRWXU);
		finis

		testing("Time related elements \"st_ctime\", \"st_mtime\", and");
		testing("\"st_atime\" of the file are not updated.");
		start
			trf_ck(CHK_CTIME, FALSE, s_time, NO_FILDES, path);
			trf_ck(CHK_MTIME, FALSE, s_time, NO_FILDES, path);
			trf_ck(CHK_ATIME, FALSE, s_time, NO_FILDES, path);
		finis

		testing("Data in file not altered.");
		start
			o_read_tst(fd, Strlen);
			Zclose(fd);
		finis

		testing("File permission bits of file are not changed.");
		start
			Zstat(path, &st_buf2);
			if(st_buf2.st_mode != st_buf1.st_mode)
				error("%s\n%s%lu\n%s%lu.\n",
						"The test file mode has changed.",
						"Expected file mode:	",
						(unsigned long) st_buf1.st_mode,
						"Actual file mode:	",
						(unsigned long) st_buf2.st_mode);
		finis

		testing("Owner of file is not changed.");
		start
			if(st_buf2.st_uid != st_buf1.st_uid)
				error("%s\n%s%lu\n%s%lu.\n",
						"The test file uid has changed.",
						"Expected file uid:	",
						(unsigned long) st_buf1.st_uid,
						"Actual file uid:	",
						(unsigned long) st_buf2.st_uid);
			if(st_buf2.st_gid != st_buf1.st_gid)
				error("%s\n%s%lu\n%s%lu.\n",
						"The test file gid has changed.",
						"Expected file gid:	",
						(unsigned long) st_buf1.st_gid,
						"Actual file gid:	",
						(unsigned long) st_buf2.st_gid);
		finis
	finis

	done();
}


void o_clean(int fd, char *path)
{
	cleanup("Close and unlink test file.");
	start
		Zclose(fd);
		Zunlink(path);
	finis
}

/*	Read data test */
void o_read_tst(int fd, int cnt)
{
	int	rd_cnt;
	char	rd_buf[10];
	char	exp_buf[10];

	continue_on(ERROR);
	(void) sprintf(exp_buf, "%s%s", String,
				(cnt == Strlen) ? "" : String);
	rd_cnt = Zread(fd, rd_buf, cnt);
	if(rd_cnt != cnt)
		error("%s\n%s%d\n%s%d.\n",
			"Wrong count returned by read().",
			"Expected:	",
			cnt,
			"Actual:	",
			rd_cnt);
	else
		if(strncmp( exp_buf, rd_buf, cnt))
			error("%s\"%s\"\n%s\"%s\"\n",
				"Read of test file returned:  ",
				rd_buf,
				"Expected:	",
				exp_buf);
}
