#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Zstat.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>

/* This module has the S_ISUID mode set to ROOT. */


int main(void)
{
	struct	stat st_buf;
	time_t	s_time;
	char	path[_POSIX_PATH_MAX +1];
	char	*pathm;
	pid_t	pid;
	int		fd;

	setup(E_OPEN);
	funct(E_OPEN);

	if( ! PCD_READ_ONLY_FILE_SYSTEM) {
		assertion(65, BASE);
		start
			not_implemented("read-only file systems");
		finis
		assertion(66, BASE);
		start
			not_implemented("read-only file systems");
		finis
		assertion(67, BASE);
		start
			not_implemented("read-only file systems");
		finis

		done();
	}

	assertion(65, BASE);
	start
		pathm = fs_mount(MOUNT_RO, ROFS_REG_FILE);

		prep("Establish path to file on read-only file system.");
		start
			Zstrcpy(path, pathm);
			Zstrcat(path, "/" );
			Zstrcat(path, ROFS_TESTFILE );
		finis

		testing("As appropriate privilege process, call to open()");
		testing("with the O_WRONLY flag set and path");
		testing("references a file on a read-only file system.");
		testing("Path name is \"%s\".", path);
		start
			expecting(-1);
			expecting(EROFS);
			fd = Zopen(path, O_WRONLY);
			if(fd > 0)
				Zclose(fd);			/*	Close errant open file. */
		finis

		testing("As appropriate privilege process, call to open()");
		testing("with the O_RDWR flag set and path");
		testing("references a file on a read-only file system.");
		start
			expecting(-1);
			expecting(EROFS);
			fd = Zopen(path, O_RDWR);
			if(fd > 0)
				Zclose(fd);			/*	Close errant open file. */
		finis

		FORK
			prep("Set effective user ID to real user ID.");
			start
				Zsetuid(getuid());
			finis

			testing("As ordinary user process, call to open()");
			testing("with the O_WRONLY flag set and path");
			testing("references a file on a read-only file system.");
			start
				expecting(-1);
				expecting(EROFS);
				Zopen(path, O_WRONLY);
			finis

			testing("As ordinary user process, call to open()");
			testing("with the O_RDWR flag set and path");
			testing("references a file on a read-only file system.");
			start
				expecting(-1);
				expecting(EROFS);
				Zopen(path, O_RDWR);
			finis
		KROF

		fs_umount(ROFS_REG_FILE);
	finis

	assertion(66, BASE);
	start
		pathm = fs_mount(MOUNT_RO, NO_FILE);

		prep("Establish path to new file and obtain a starting time.");
		start
			Zstrcpy(path, pathm);
			Zstrcat(path, "/");
			Zstrcat(path, ROFS_TESTFILE);

			Ztime(&s_time);			/* Obtain minimum reference time. */
			dt_secs(TRF_DELAY);		/* Delay a fixed interval.			*/
		finis

		testing("As appropriate privilege process, call to open() with");
		testing("O_WRONLY and O_CREAT flags set and path references");
		testing("a non-existent file on a read-only file system.");
		testing("Path name is \"%s\".", path);
		start
			expecting(-1);
			expecting(EROFS);
			fd = Zopen(path, O_WRONLY | O_CREAT, PROT_ALL);
			if(fd > 0)
				Zclose(fd);			/*	Close errant open file. */
		finis

		testing("As appropriate privilege process, call to open() with");
		testing("O_RDWR and O_CREAT flags set and path references");
		testing("a non-existent file on a read-only file system.");
		start
			expecting(-1);
			expecting(EROFS);
			fd = Zopen(path, O_RDWR | O_CREAT, PROT_ALL);
			if(fd > 0)
				Zclose(fd);			/*	Close errant open file. */
		finis

		FORK
			prep("Set effective user ID to real user ID.");
			start
				Zsetuid(getuid());
			finis

			testing("As ordinary user process, call to open() with");
			testing("O_WRONLY and O_CREAT flags set and path references");
			testing("a non-existent file on a read-only file system.");
			start
				expecting(-1);
				expecting(EROFS);
				Zopen(path, O_WRONLY | O_CREAT, PROT_ALL);
			finis

			testing("As ordinary user process, call to open() with");
			testing("O_RDWR and O_CREAT flags set and path references");
			testing("a non-existent file on a read-only file system.");
			start
				expecting(-1);
				expecting(EROFS);
				Zopen(path, O_RDWR | O_CREAT, PROT_ALL);
			finis

			testing("File is not created.");
			start
				expecting(-1);
				expecting(ENOENT);
				Zaccess(path, F_OK);
			finis

			testing("Time related elements \"st_ctime\" and \"st_mtime\"");
			testing("of the parent directory are not changed.");
			start
				Zstrcpy(path, pathm);
				trf_ck(CHK_CTIME, FALSE, s_time, NO_FILDES, path);
				trf_ck(CHK_MTIME, FALSE, s_time, NO_FILDES, path);
			finis
		KROF

		fs_umount(NO_FILE);
	finis

	assertion(67, BASE);
	start
		pathm = fs_mount(MOUNT_RO, ROFS_REG_FILE);

		prep("Establish path to file on read-only file system");
		prep("and establish a starting time.");
		start
			Zstrcpy(path, pathm);
			Zstrcat(path, "/");
			Zstrcat(path, ROFS_TESTFILE);

			Ztime(&s_time);			/* Obtain minimum reference time. */
			dt_secs(TRF_DELAY);		/* Delay a fixed interval.			*/
		finis

		testing("As appropriate privilege process, call to open()");
		testing("with the O_WRONLY and O_TRUNC flags set and path");
		testing("references a file on a read only file system.");
		testing("Path name is \"%s\".", path);
		start
			expecting(-1);
			expecting(EROFS);
			fd = Zopen(path, O_WRONLY | O_TRUNC);
			if(fd > 0)
				Zclose(fd);			/*	Close errant open file. */
		finis

		testing("As appropriate privilege process, call to open()");
		testing("with the O_RDWR and O_TRUNC flags set and path");
		testing("references a file on a read only file system.");
		start
			expecting(-1);
			expecting(EROFS);
			fd = Zopen(path, O_RDWR | O_TRUNC);
			if(fd > 0)
				Zclose(fd);			/*	Close errant open file. */
		finis

		FORK
			prep("Set effective user ID to real user ID.");
			start
				Zsetuid(getuid());
			finis

			testing("As ordinary user process, call to open()");
			testing("with the O_WRONLY and O_TRUNC flags set and path");
			testing("references a file on a read only file system.");
			start
				expecting(-1);
				expecting(EROFS);
				Zopen(path, O_WRONLY | O_TRUNC);
			finis

			testing("As ordinary user process, call to open()");
			testing("with the O_RDWR and O_TRUNC flags set and path");
			testing("references a file on a read only file system.");
			start
				expecting(-1);
				expecting(EROFS);
				Zopen(path, O_RDWR | O_TRUNC);
			finis

			testing("File is not truncated.");
			start
				expecting(SUCCESS);
				Zstat(path, &st_buf);
				if(st_buf.st_size == (off_t)0)
					error("File was truncated.");
			finis

			testing("Time related elements \"st_atime\", \"st_ctime\",");
			testing("and \"st_mtime\" of the file are not marked for");
			testing("update.");
			start
				trf_ck(CHK_CTIME, FALSE, s_time, NO_FILDES, path);
				trf_ck(CHK_MTIME, FALSE, s_time, NO_FILDES, path);
			finis
		KROF

		fs_umount(ROFS_REG_FILE);
	finis
	
	done();
}
