#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Zstat.h>
#include	<Zfcntl.h>
#include	<Zdirent.h>

#include	<sv_env.h>
#include	<sv_errno.h>

#define	DIRNAME		"opendir_pth.d"
#define	DIRNAME_1	"5.1.2a_pth1.d"
#define	DIRNAME_2	"5.1.2a_pth2.d"
#define	FILENAME	"5.1.2a_pth"

static char	*file_names[] = {
							"ABCDEFGHIJLKM",
							"NOPQRSTUVWXYZ",
							"abcdefghijlkm",
							"nopqrstuvwxyz",
							"0123456789._-"
						   };

static	char	path_cwd[_POSIX_PATH_MAX +1];
static	char	path_ref[_POSIX_PATH_MAX +1];

static	DIR		*dirp;
static	struct	dirent	*dirent;

#define	INITIAL_PATHNAME_ASS_NUM	5
static	int	err_ass;
int	ck_ul(char *);

int main(void)
{
	int		i;
	int		ass_num = INITIAL_PATHNAME_ASS_NUM;
	DIR		*dirp_1, *dirp_3;
	struct	dirent	*dirent_1, *dirent_3;

	setup(E_OPENDIR);
	funct(E_OPENDIR);

	assertion(ass_num, BASE);
	start
		prep("Change current working directory to \"../tmp/%s.d\".",
									element[(int) E_OPENDIR].e_name);
		prep("Create known directory entries and files in working directory.");
		start
			exit_on(ERROR);
			Zstrcpy(path_cwd, temp_path());
			path_cwd[strlen(path_cwd) -1] = '\000'; /* delete trailing '/' */
			Zchdir(path_cwd);
			Zgetcwd(path_cwd, _POSIX_PATH_MAX +1);
			Zmkdir(DIRNAME_1, S_IRWXU);
			sprintf(path_ref, "%s/%s", DIRNAME_1, FILENAME);
			Zclose(Zcreat(path_ref, S_IRWXU));

			Zmkdir(DIRNAME, S_IRWXU);
			sprintf(path_ref, "%s/%s", DIRNAME, DIRNAME_2);
			Zmkdir(path_ref, S_IRWXU);
			sprintf(path_ref, "%s/%s/%s", DIRNAME, DIRNAME_2, FILENAME);
			Zclose(Zcreat(path_ref, S_IRWXU));

										/*
												Working_Directory
												 |              |
												 |              |
											  DIRNAME        DIRNAME_1
												 |              |
											 DIRNAME_2        FILENAME
												 |
											  FILENAME
										*/
		finis

		testing("When the first filename component of the dirname");
		testing("argument is \".\", and the pathname does not begin with");
		testing("a slash, then the pathname is resolved by locating");
		testing("the second filename component in the current working");
		testing("directory.");
		start
			sprintf(path_ref, "./%s", DIRNAME_1);
			dirp = Zopendir(path_ref);
			for(i=0; i<3; i++) {
				dirent = Zreaddir(dirp);
				if(!strcmp(dirent->d_name, FILENAME))
					break;
				if(i == 2)
					error("Expected entry \"%s\" not found in \"%s\".",
								FILENAME, path_ref);
			}
			Zclosedir(dirp);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Dirname argument points to the string \"/\".");
		start
									/*	Assure directory opened returns
										same entries as directory "///".
									*/
			dirp_1 = Zopendir("/");
			dirp_3 = Zopendir("///");
			for(i=0; i<3; i++) {
				dirent_1 = Zreaddir(dirp_1);
				dirent_3 = Zreaddir(dirp_3);
				if(strcmp(dirent_1->d_name, dirent_3->d_name))
					error("%s\nEntry %d: \"%s\" versus \"%s\".",
							"Directory entry differs.",
							i, 
							dirent_1->d_name, dirent_3->d_name);
			}
			Zclosedir(dirp_1);
			Zclosedir(dirp_3);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Dirname argument points to the string \"///\".");
		start
									/*	Assure directory opened returns
										same entries as directory "/".
									*/
			dirp_1 = Zopendir("/");
			dirp_3 = Zopendir("///");
			for(i=0; i<3; i++) {
				dirent_1 = Zreaddir(dirp_1);
				dirent_3 = Zreaddir(dirp_3);
				if(strcmp(dirent_1->d_name, dirent_3->d_name))
					error("%s\nEntry %d: \"%s\" versus \"%s\".",
							"Directory entry differs.",
							i, 
							dirent_1->d_name, dirent_3->d_name);
			}
			Zclosedir(dirp_1);
			Zclosedir(dirp_3);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Dirname argument points to the string beginning");
		testing("with a \"/\".");
		start
			sprintf(path_ref, "%s/%s", path_cwd, DIRNAME_1);
			dirp = Zopendir(path_ref);
			for(i=0; i<3; i++) {
				dirent = Zreaddir(dirp);
				if(!strcmp(dirent->d_name, FILENAME))
					break;
				if(i == 2)
					error("Expected entry \"%s\" not found in \"%s\".",
								FILENAME, path_ref);
			}
			Zclosedir(dirp);
		finis

		testing("Dirname argument points to the string beginning");
		testing("with \"///\".");
		start
			sprintf(path_ref, "//%s/%s", path_cwd, DIRNAME_1);
			dirp = Zopendir(path_ref);
			for(i=0; i<3; i++) {
				dirent = Zreaddir(dirp);
				if(!strcmp(dirent->d_name, FILENAME))
					break;
				if(i == 2)
					error("Expected entry \"%s\" not found in \"%s\".",
								FILENAME, path_ref);
			}
			Zclosedir(dirp);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("First filename component of the dirname argument is \"..\".");
		start
			Zchdir(DIRNAME);

			sprintf(path_ref, "../%s/%s", DIRNAME, DIRNAME_2);
			dirp = Zopendir(path_ref);
			for(i=0; i<3; i++) {
				dirent = Zreaddir(dirp);
				if(!strcmp(dirent->d_name, FILENAME))
					break;
				if(i == 2)
					error("Expected entry \"%s\" not found in \"%s\".",
								FILENAME, path_ref);
			}
			Zclosedir(dirp);
			Zchdir(path_cwd);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Dirname argument points to the string \"F1/\" and");
		testing("F1 is a directory.");
		start
			sprintf(path_ref, "%s/", DIRNAME_1);
			dirp = Zopendir(path_ref);
			for(i=0; i<3; i++) {
				dirent = Zreaddir(dirp);
				if(!strcmp(dirent->d_name, FILENAME))
					break;
				if(i == 2)
					error("Expected entry \"%s\" not found in \"%s\".",
								FILENAME, path_ref);
			}
			Zclosedir(dirp);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Dirname argument points to the string \"F1//\" and");
		testing("F1 is a directory.");
		start
			sprintf(path_ref, "%s//", DIRNAME_1);
			dirp = Zopendir(path_ref);
			for(i=0; i<3; i++) {
				dirent = Zreaddir(dirp);
				if(!strcmp(dirent->d_name, FILENAME))
					break;
				if(i == 2)
					error("Expected entry \"%s\" not found in \"%s\".",
								FILENAME, path_ref);
			}
			Zclosedir(dirp);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Dirname argument points to the string \"F1/F2\".");
		start
			Zchdir(DIRNAME);

			Zchdir(path_cwd);
			sprintf(path_ref, "%s/%s", DIRNAME, DIRNAME_2);
			dirp = Zopendir(path_ref);
			for(i=0; i<3; i++) {
				dirent = Zreaddir(dirp);
				if(!strcmp(dirent->d_name, FILENAME))
					break;
				if(i == 2)
					error("Expected entry \"%s\" not found in \"%s\".",
								FILENAME, path_ref);
			}
			Zclosedir(dirp);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Dirname argument points to the string \"F1/./F2\".");
		start
			Zchdir(DIRNAME);

			Zchdir(path_cwd);
			sprintf(path_ref, "%s/./%s", DIRNAME, DIRNAME_2);
			dirp = Zopendir(path_ref);
			for(i=0; i<3; i++) {
				dirent = Zreaddir(dirp);
				if(!strcmp(dirent->d_name, FILENAME))
					break;
				if(i == 2)
					error("Expected entry \"%s\" not found in \"%s\".",
								FILENAME, path_ref);
			}
			Zclosedir(dirp);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Dirname argument points to the string \"F1/../F1/F2\".");
		start
			Zchdir(DIRNAME);

			Zchdir(path_cwd);
			sprintf(path_ref, "%s/../%s/%s", DIRNAME, DIRNAME, DIRNAME_2);
			dirp = Zopendir(path_ref);
			for(i=0; i<3; i++) {
				dirent = Zreaddir(dirp);
				if(!strcmp(dirent->d_name, FILENAME))
					break;
				if(i == 2)
					error("Expected entry \"%s\" not found in \"%s\".",
								FILENAME, path_ref);
			}
			Zclosedir(dirp);
		finis
	finis

	assertion(++ass_num, BASE);
	start
		testing("Dirname argument points to the string \"F1//F2\".");
		start
			Zchdir(DIRNAME);

			Zchdir(path_cwd);
			sprintf(path_ref, "%s//%s", DIRNAME, DIRNAME_2);
			dirp = Zopendir(path_ref);
			for(i=0; i<3; i++) {
				dirent = Zreaddir(dirp);
				if(!strcmp(dirent->d_name, FILENAME))
					break;
				if(i == 2)
					error("Expected entry \"%s\" not found in \"%s\".",
								FILENAME, path_ref);
			}
			Zclosedir(dirp);
		finis

		cleanup("Remove test files created on test directory.");
		start
			Zchdir(path_cwd);
			sprintf(path_ref, "%s/%s/%s", DIRNAME, DIRNAME_2, FILENAME);
			Zunlink(path_ref);
			sprintf(path_ref, "%s/%s", DIRNAME, DIRNAME_2);
			Zrmdir(path_ref);

			sprintf(path_ref, "%s/%s", DIRNAME_1, FILENAME);
			Zunlink(path_ref);
			Zrmdir(DIRNAME_1);
		finis

	finis

	if(pathconf(DIRNAME, _PC_NO_TRUNC) == (long) -1) {
		assertion(++ass_num, BASE);
		start
			testing("Dirname component is a string of more than {NAME_MAX}");
			testing("bytes in a directory for which {_POSIX_NO_TRUNC} is not");
			testing("supported.");
			start
				error("{_POSIX_NO_TRUNC} not supported.");
			finis
		finis
	}
	else
		assertion(++ass_num, NOT_APPLICABLE);

	assertion(++ass_num, BASE);
	start
		for(i=0; i<5; i++) {
			testing("Dirname of \"%s\" is supported and", file_names[i]);
			testing("the unique identity of upper and lower case");
			testing("letters are retained.");
			start
				sprintf(path_ref, "%s/%s", DIRNAME, file_names[i]);
				Zmkdir(path_ref, PROT_ALL);
				ck_ul(file_names[i]);
				dirp = Zopendir(path_ref);
				Zclosedir(dirp);
				Zrmdir(path_ref);
			finis
		}
	finis

	assertion(++ass_num, BASE);
	start
		testing("Dirname retains the unique identity of its upper and lower");
		testing("case letters.");
		start
			if(err_ass)
				error("See previous assertion for specifics of failure.");
		finis
	finis

	done();
}


		/*	For the Filename tests, check that the upper and lower case
			letters are preserved.  Read the directory and match with 
			the expected names.  This procedure is not performed for
			each element, but each element is covered, if all the filename
			tests are successful.
		*/

int	ck_ul(char *f_name)
{
	int	i;
	
	dirp = Zopendir(DIRNAME);
	dirent = Zreaddir(dirp);
	for(i=0; i<2; i++) {	/* Skip dot and dot-dot entries if they exist */
		if( ! ((dirent->d_name)[0] == '.') )
			break;
		dirent = Zreaddir(dirp);
	}
											/* Compare only filename */
	if(strcmp(dirent->d_name, f_name)) {
		error("%s\n%s\"%s\"\n%s\"%s\"\n",
			"Filename entry obtained not what was expected.",
			"Expected:	",
			f_name,
			"Actual:	",
			dirent->d_name);
		err_ass++;
	}
	Zclosedir(dirp);
}
