#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zfcntl.h>
#include	<sys/Zstat.h>

#include	<sv_env.h>
#include	<sv_errno.h>

/* This module has the S_ISUID mode set to ROOT. */

#define	FILENAME		"5.5.3_67"


int main(void)
{
	char	path_old[_POSIX_PATH_MAX +1], path_new[_POSIX_PATH_MAX +1];
	char	*pathm;
	pid_t	pid;

	setup(E_RENAME);
	funct(E_RENAME);

	assertion(67, BASE);
	start
		time_t	s_time;

		if(PCD_READ_ONLY_FILE_SYSTEM) {
			pathm = fs_mount(MOUNT_RO, ROFS_REG_FILE);
	
			abstract("Ensure mounted file system gets unmounted.");
			start
				break_on(ERROR);
				prep("Establish path to implementation provided");
				prep("ROFS_TESTFILE");
				start
					sprintf(path_new, "%s/%s", pathm, FILENAME);
					sprintf(path_old, "%s/%s", pathm, ROFS_TESTFILE);
				finis
		
				testing("Call to rename(), by process with appropriate");
				testing("privileges, to rename file on read-only file system.");
				testing("Call to rename(%s, %s)", path_old, path_new);
				start
					Ztime(&s_time);		/* Obtain minimum reference time.	*/
					dt_secs(TRF_DELAY);	/* Delay a fixed interval.			*/
	
					expecting(EROFS);
					Zrename(path_old, path_new);
	
												/* old not changed */
					expecting(SUCCESS);
					Zaccess(path_old, F_OK);
				finis
		
				FORK
					prep("Set effective user ID to real user ID.");
					start
						Zsetuid(getuid());
					finis
	
					testing("Call to rename(), by process without appropriate");
					testing("privileges, to rename file on read-only");
					testing("file system.");
					testing("Call to rename(%s, %s)", path_old, path_new);
					start
						expecting(EROFS);
						Zrename(path_old, path_new);
	
													/* old not changed */
						expecting(SUCCESS);
						Zaccess(path_old, F_OK);
					finis
	
					testing("Time related elements \"st_atime\", \"st_ctime\"");
					testing("and \"st_mtime\" of parent directory are");
					testing("not changed.");
					start
						trf_ck(CHK_ATIME, FALSE, s_time, NO_FILDES, pathm);
						trf_ck(CHK_CTIME, FALSE, s_time, NO_FILDES, pathm);
						trf_ck(CHK_MTIME, FALSE, s_time, NO_FILDES, pathm);
					finis
				KROF
			finis
	
			fs_umount(ROFS_REG_FILE);
		}
		else
			not_implemented("read-only file systems");
	finis

	if(strlen(PCTS_FS) == 0)
		assertion(68, NOT_TESTABLE);
	else {
		assertion(68, BASE);
		start
			if(PCD_LINK_FILE_SYSTEM == FALSE) {
				pathm = fs_mount(MOUNT_RW, ROFS_REG_FILE);
			
				abstract("Ensure mounted file system gets unmounted.");
				start
					break_on(ERROR);
					preparation("Establish paths for linking.");
					start
						sprintf(path_new, "%s/%s", pathm, FILENAME);
				
						Zstrcpy(path_old, temp_path());
						Zstrcat(path_old, "exdev_ck");
						Zclose(Zcreat(path_old, PROT_ALL));
					finis
				
					testing("Appropriate privilege process renaming a file");
					testing("onto another file system.");
					testing("Path for new is \"%s\".", path_new);
					start
						expecting(EXDEV);
						Zrename(path_old, path_new);
														/* old not changed */
						expecting(SUCCESS);
						Zaccess(path_old, F_OK);
					finis
	
					FORK
						exit_on(ERROR);
						prep("Set effective user ID to real user ID.");
						start
							Zsetuid(getuid());
						finis
	
						testing("Ordinary user process renaming a file");
						testing("onto another file system.");
						testing("Path for new is \"%s\".", path_new);
						start
							expecting(EXDEV);
							Zrename(path_old, path_new);
														/* old not changed */
							expecting(SUCCESS);
							Zaccess(path_old, F_OK);
	
							Zunlink(path_old); 			/* Cleanup */
						finis
					KROF
				finis
				
				(void) fs_umount(ROFS_REG_FILE);
			}
			else
				not_implemented("{PCD_LINK_FILE_SYSTEM} of FALSE");
		finis
	}

	assertion(69, NOT_APPLICABLE);

	done();
}
