#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Zstat.h>
#include	<Zfcntl.h>

#include	<sv_env.h>

/*	This module has S_ISUID mode set to ROOT. */

void	rd_mode(mode_t, int);
void	wr_mode(mode_t, int);
void	ex_mode(mode_t, int);
void	ex_sugid(mode_t);

#define	FILENAME_x "./5.6.1_09x"
#define	FILENAME_y "./5.6.1_09y"

#define	OWNER	1
#define	GROUP	2
#define	OTHER	3

struct	sym_tbl {
					const char *name;
					const unsigned long mask;
				} sym_table[] = {
						"S_IRUSR",	S_IRUSR,
						"S_IWUSR",	S_IWUSR,
						"S_IXUSR",	S_IXUSR,
						"S_IROTH",	S_IROTH,
						"S_IWOTH",	S_IWOTH,
						"S_IXOTH",	S_IXOTH,
						"S_IRGRP",	S_IRGRP,
						"S_IWGRP",	S_IWGRP,
						"S_IXGRP",	S_IXGRP,
						"S_ISUID",	S_ISUID,
						"S_ISGID",	S_ISGID,
						};
#define	ITEMS_SYM_TBL	(sizeof(sym_table)/sizeof(sym_table[0]))

#define	String	"DaTa"
#define	Strlen	(sizeof(String) -1)

static	pid_t pid;
static	char path[_POSIX_PATH_MAX +1];


int main(void)
{
	setup(E_STAT_H);
	funct(E_STAT_H);

	assertion(9, BASE);
	start
		break_on(ERROR);
		prep("Call umask() to allow all modes.");
		start
			umask(0);
		finis

		rd_mode(S_IRUSR, OWNER);
		wr_mode(S_IWUSR, OWNER);
		ex_mode(S_IXUSR, OWNER);
	finis

	assertion(10, BASE);
	start
		rd_mode(S_IRGRP, GROUP);
		wr_mode(S_IWGRP, GROUP);
		ex_mode(S_IXGRP, GROUP);
	finis

	assertion(11, BASE);
	start
		rd_mode(S_IROTH, OTHER);
		wr_mode(S_IWOTH, OTHER);
		ex_mode(S_IXOTH, OTHER);
	finis

	assertion(12, BASE);
	start
		rd_mode(S_IRWXU, OWNER);
		wr_mode(S_IRWXU, OWNER);
		ex_mode(S_IRWXU, OWNER);

		rd_mode(S_IRWXG, GROUP);
		wr_mode(S_IRWXG, GROUP);
		ex_mode(S_IRWXG, GROUP);

		rd_mode(S_IRWXO, OTHER);
		wr_mode(S_IRWXO, OTHER);
		ex_mode(S_IRWXO, OTHER);
	finis

	assertion(13, BASE);
	start
		ex_sugid(S_ISUID);
	finis

	assertion(14, BASE);
	start
		ex_sugid(S_ISGID);
	finis

	assertion(15, BASE);
	start
		testing("Bits defined by the symbols S_IRUSR, S_IWUSR, S_IXUSR,");
		testing("S_IROTH, S_IWOTH, S_IXOTH, S_IRGRP, S_IWGRP, S_IXGRP,");
		testing("S_ISUID, and S_ISGID are bitwise discrete.");
		start
			BITWISE_DISTINCT(sym_table, ITEMS_SYM_TBL)
		finis
	finis

	assertion(16, BASE);
	start
		testing("Symbol S_IRWXU is the bitwise inclusive OR of symbols");
		testing("S_IRUSR, S_IWUSR, and S_IXUSR.");
		start
			if(S_IRWXU != (S_IRUSR | S_IWUSR | S_IXUSR))
				error("Expected value of %o\nActual value was %o.",
					S_IRWXU,
					S_IRUSR | S_IWUSR | S_IXUSR);
		finis
	finis

	assertion(17, BASE);
	start
		testing("Symbol S_IRWXG is the bitwise inclusive OR of symbols");
		testing("S_IRGRP, S_IWGRP, and S_IXGRP.");
		start
			if(S_IRWXG != (S_IRGRP | S_IWGRP | S_IXGRP))
				error("Expected value of %o\nActual value was %o.",
					S_IRWXG,
					S_IRGRP | S_IWGRP | S_IXGRP);
		finis
	finis

	assertion(18, BASE);
	start
		testing("Symbol S_IRWXO is the bitwise inclusive OR of symbols");
		testing("S_IROTH, S_IWOTH, and S_IXOTH.");
		start
			if(S_IRWXO != (S_IROTH | S_IWOTH | S_IXOTH))
				error("Expected value of %o\nActual value was %o.",
					S_IRWXO,
					S_IROTH | S_IWOTH | S_IXOTH);
		finis
	finis

	assertion(19, EXTENDED);

	done();
}
			

void rd_mode(mode_t mode, int access)
{
	int		fd;
	char	inbuf[Strlen +1];

	prep("Establish path to temp file and set owner and permission modes.");
	start
		Zstrcpy(path, temp_path());
		Zstrcat(path, "rd_mode");
		fd = Zcreat(path, mode);
		expecting(Strlen);
		Zwrite(fd, String, Strlen);
		Zclose(fd);
		Zchown(path, (uid_t)((access == OWNER) ? UID : BIN_UID),
								(gid_t)((access == GROUP) ? GID : BIN_GID));
	finis

	FORK
		prep("Establish proper user-ID and group-ID for test.");
		start
			Zsetgid(GID );
			Zsetuid(UID);
		finis

		testing("\"%s\" of file can read data file.",
				(access == OWNER) ? "owner" :
				(access == GROUP) ? "group" : "other");
		start
			fd = Zopen(path, O_RDONLY);
			expecting(Strlen);
			Zread(fd, inbuf, Strlen);
			Zclose(fd);

			Zunlink(path);						/* Cleanup */
		finis
	KROF
}


void wr_mode(mode_t mode, int access)
{
	int		fd;

	prep("Establish path to temp file and set owner and permission modes.");
	start
		Zstrcpy(path, temp_path());
		Zstrcat(path, "wr_mode");
		Zclose(Zcreat(path, mode));
		Zchown(path, (uid_t)((access == OWNER) ? UID : BIN_UID),
								(gid_t)((access == GROUP) ? GID : BIN_GID));
	finis

	FORK
		prep("Establish proper user-ID and group-ID for test.");
		start
			Zsetgid(GID);
			Zsetuid(UID);
		finis

		testing("\"%s\" of file can write data file.",
				(access == OWNER) ? "owner" :
				(access == GROUP) ? "group" : "other");
		start
			fd = Zopen(path, O_WRONLY);
			expecting(Strlen);
			Zwrite(fd, String, Strlen);
			Zclose(fd);

			Zunlink(path);							/* Cleanup */
		finis
	KROF
}


void ex_mode(mode_t mode, int access)
{
	prep("Establish path to data file.");
	start
		Zstrcpy(path, FILENAME_x);
		Zchown(path, (uid_t)((access == OWNER) ? UID : BIN_UID),
								(gid_t)((access == GROUP) ? GID : BIN_GID));
	finis

	FORK
		prep("Set proper permission mode on executable file."); 
		prep("Establish proper user-ID and group-ID for test.");
		start
			Zchmod(path, mode);
			Zsetgid(GID);
			Zsetuid(UID);
		finis

		testing("\"%s\" of file can execute this data file.",
				(access == OWNER) ? "owner" :
				(access == GROUP) ? "group" : "other");
		start
		finis

		Zexecl(path, path, (char *) 0);
		error("execl() of \"%s\" failed.", path);
	KROF
}


void ex_sugid(mode_t mode)
{
	char	asc_ass_nu[10];

	prep("Establish path to data file.");
	start
		Zstrcpy(path, FILENAME_y);
	finis

	FORK
		if(mode == S_ISUID)
			prep("Set set-user-ID mask and execute file."); 
		else
			prep("Set set-group-ID mask and execute file."); 
		start
			Zchown(path, BIN_UID, BIN_GID);
			Zchmod(path, mode | S_IRWXU | S_IRWXG);
			(void) sprintf(asc_ass_nu, "%d", ass_nu);
		finis

		if(mode == S_ISUID)
			Zexecl(path, path, "S_ISUID", asc_ass_nu, (char *) 0);
		else
			Zexecl(path, path, "S_ISGID", asc_ass_nu, (char *) 0);

		error("execl() of \"%s\" failed.", path);
	KROF
}
