#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Zstat.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<pcts_timer.h>

#define	FILENAME	"5.5.1_19"
#define	String		"DaTa Test"
#define	Strlen		(sizeof(String) -1)

int main(void)
{
	char	path[_POSIX_PATH_MAX +1], path2[_POSIX_PATH_MAX +1];
	char	inbuf[Strlen +1];
	struct	stat st_buf;
	int		fd;
	pid_t	pid;
	time_t	s_time;

	setup(E_UNLINK);
	funct(E_UNLINK);

	assertion(19, BASE);
	start
		prep("Create test file, link to it, ensure link count is 2.");
		start
			Zstrcpy(path, temp_path());
			Zstrcat(path, FILENAME);
			fd = Zcreat(path, PROT_ALL);
			Zwrite(fd, String, Strlen);
			Zclose(fd);

			Zstrcpy(path2, temp_path());
			Zstrcat(path2, "unlink_2");
			Zlink(path, path2);

			Zstat(path, &st_buf);
			if(st_buf.st_nlink != (nlink_t) 2)
				error("%s%lu\n%s\n",
					"Link count of test file is ",
					(unsigned long) st_buf.st_nlink,
					"Expected a count of 2");
		finis

		testing("Call to unlink() returns zero, removes the link, and link");
		testing("count is 1.");
		start
			expecting(0);
			Zunlink(path2);

			expecting(ENOENT);
			Zaccess(path2, F_OK);

			Zstat(path, &st_buf);
			if(st_buf.st_nlink != (nlink_t) 1)
				error("%s\n%s%lu",
					"Expected link count of 1.",
					"Actual link count is ",
					(unsigned long) st_buf.st_nlink);
		finis
	finis

	assertion(20, EXTENDED);
	assertion(21, EXTENDED);

	assertion(22, BASE);
	start
		ENABLE_PSYNC
		prep("Fork a child.");
		start
			sv_flush ();
			pid = Zfork();
		finis

		if(pid == (pid_t)0) {
			prep("Child open's test file and waits for parent to unlink it.");
			start
				fd = Zopen(path, O_RDONLY);
				SEND_PSYNC
				SET_TIMEOUT
					WAIT_PSYNCA
				CLEAR_ALARM
			finis

			testing("Test file contents still accessible.");
			start
				expecting(Strlen);
				Zread(fd, inbuf, Strlen);
			finis

			testing("Test file no longer accessible after all references");
			testing("to the file are closed.");
			start
				Zclose(fd);
				expecting(ENOENT);
				Zaccess(path, F_OK);
			finis

			done();
		}

		prep("Unlink test file after child opens it.");
		start
			SET_TIMEOUT
				WAIT_PSYNC
				Zunlink(path);
			CLEAR_ALARM
		finis

		testing("Test file no longer accessible.");
		start
			expecting(ENOENT);
			Zaccess(path, F_OK);
			SEND_PSYNCA
		finis

		WAIT_FOR_PID(pid); 		/* Wait for child to close test file */
		DISABLE_PSYNC
	finis

	assertion(24, BASE);
	start
		prep("Record a starting time, then unlink test file.");
		start
			Zclose(Zcreat(path, PROT_ALL));
			Zlink(path, path2);

			Ztime(&s_time);				/* Obtain minimum reference time. */
			dt_secs(TRF_DELAY);			/* Delay a fixed interval.			*/
			Zunlink(path2);
		finis
	
		testing("Time related elements \"st_ctime\" and \"st_mtime\"");
		testing("of parent directory are marked for update.");
		start
			trf_ck(CHK_CTIME, TRUE, s_time, NO_FILDES, temp_path());
			trf_ck(CHK_MTIME, TRUE, s_time, NO_FILDES, temp_path());
		finis
	finis

	assertion(25, BASE);
	start
		testing("Time related element \"st_ctime\" of the file is marked");
		testing("for update.");
		start
			trf_ck(CHK_CTIME, TRUE, s_time, NO_FILDES, path);
		finis
	finis

	assertion(26, EXTENDED);

	done();
}
