#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Zstat.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<locks.h>

#define	 String		"What goes up must come down."
#define	 Strlen		(sizeof(String) -1)

#define 	TMP_FILE	"dup2.tmp"

int main(void)
{
	char	path[_POSIX_PATH_MAX +1];
	int 	fd, fd2;
	pid_t	pid;
	struct	stat st_org, st_dup;
	struct	flock flock_org, flock_dup;

	setup(E_DUP2);
	funct(E_DUP2);

	assertion(4, BASE);
	start
		prep("Open a regular type file and set a lock.");
		start
			Zstrcpy(path, temp_path());
			Zstrcat(path, TMP_FILE);
			fd = Zopen(path, O_RDWR | O_CREAT | O_APPEND, PROT_ALL);
			expecting(Strlen);
			Zwrite(fd, String, Strlen);

			SET_LOCK(F_WRLCK, fd, SEEK_SET, ZERO_OFFSET, TOTAL_FILE,
						&flock_org);
		finis

		testing("Call of dup2(fildes, fildes) returns \"fildes\".");
		start
			expecting(fd);
			fd2 = Zdup2(fd, fd);
		finis

				/*	To determine whether a lock exists on a file in a POSIX.1
					implementation, the semantics are associated with
					ANOTHER process determing when a lock exists.
				*/
		FORK
			testing("Call of dup2(fildes, fildes) did not close \"fildes\"--");
			testing("\"fildes\" retains lock.");
			start
				Zfcntl(fd, F_GETLK, &flock_org);
				if(flock_org.l_type != F_WRLCK)
					error("%s\n%s\n%s%s\n",
							"l_type not correct.",
							"Expected: F_WRLCK",
							"Actual:	",
								(flock_org.l_type == F_UNLCK) ? "F_UNLCK" :
								(flock_org.l_type == F_RDLCK) ? "F_RDLCK" :
								"Unknown LOCK type");
				INIT_LOCK_STRUCT(F_WRLCK, SEEK_SET, ZERO_OFFSET, TOTAL_FILE,
												flock_dup);
				Zfcntl(fd2, F_GETLK, &flock_dup);
				same_lock(&flock_org, &flock_dup);
			finis
		KROF
	finis


	assertion(5, BASE);
	start
		prep("Determine the next lowest file descriptor value to be asigned.");
		start
			fd2 = Zopen(path, O_RDWR | O_CREAT | O_APPEND, PROT_ALL);
			Zclose(fd2);
		finis

		testing("Call to dup2(fildes, fildes2) returns the value \"fildes2\".");
		start
			Zfstat(fd, &st_org);

			expecting(fd2);
			fd2 = Zdup2(fd, fd2);
		finis

		testing("New file description refers to the same open file");
		testing("description as the original file description.");
		start
			off_t	off_org, off_dup2;
										/* Test file ofset. */
			if( (off_org = Zlseek(fd, (off_t)0, SEEK_CUR)) != 
						(off_dup2 = Zlseek(fd2, (off_t)0, SEEK_CUR)) )
				error("%s\n%s%lu\n%s%lu\n",
					"The file offsets differ from expected offset:",
					"Expecting:	",
					(unsigned long) off_org,
					"Actual offset for dup2ed file descriptor:	",
					(unsigned long) off_dup2);

			Zfstat(fd2, &st_dup);
										/* Test access mode. */
			if(st_org.st_mode != st_dup.st_mode)
				error("%s\n%s%lo\n%s%lo\n",
					"The file status modes differ:",
					"Expecting:	",
					(unsigned long)st_org.st_mode,
					"Actual:	",
					(unsigned long)st_dup.st_mode);

										/* Test status flags. */
			if( (Zfcntl(fd, F_GETFL) & O_ACCMODE) !=
								(Zfcntl(fd2, F_GETFL) & O_ACCMODE) )
				error("%s\n%s%o\n%s%o\n",
					"File status flags differ.",
					"Expecting:	",
					Zfcntl(fd, F_GETFL) & O_ACCMODE,
					"Actual:	",
					Zfcntl(fd2, F_GETFL) & O_ACCMODE );
		finis

		prep("Initialize lock structure.");
		start
			INIT_LOCK_STRUCT(F_WRLCK, SEEK_SET, ZERO_OFFSET, TOTAL_FILE,
												flock_org);
			INIT_LOCK_STRUCT(F_WRLCK, SEEK_SET, ZERO_OFFSET, TOTAL_FILE,
												flock_dup);
		finis

		FORK
			testing("File descriptors have the same locks.");
			start
				Zfcntl(fd, F_GETLK, &flock_org);
				Zfcntl(fd2, F_GETLK, &flock_dup);
				same_lock(&flock_org, &flock_dup);
			finis

			testing("The FD_CLOEXEC flag for new file descriptor is clear.");
			start
				if((Zfcntl(fd2, F_GETFD) & FD_CLOEXEC) != 0)
					error("FD_CLOEXEC flag is set.");
			finis
		KROF
	finis

	done();
}
