#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zfcntl.h>
#include	<Ztermios.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<pcts_timer.h>

#define	FILENAME	"6.5.2_14"
#define	String		"Woolly bully, woolly bully   \n"
#define	Strlen		(sizeof(String) -1)

#include	<locks.h>

void lock_tst(struct flock *lck, short	l_type, short l_whence, off_t l_start,
				off_t l_len, pid_t l_pid);

int main(void)
{
	int		fd;
	pid_t	pid;
	struct	flock flock1, flock2;

	setup(E_FCNTL);
	funct(E_FCNTL);

		/*	A call to fcntl(fildes, F_GETLK, arg) when a blocking lock
			exists, returns the first lock that blocks the lock description
			pointed to by arg where arg is a pointer to the type structure
			flock.  All elements of the structure flock addressed by arg
			are set and the lock type is placed in the structure flock
			addressed by arg with the value of l_whence set to SEEK_SET.

			NOTE:
				The "first lock" is interpreted to mean the first
				lock based on position when the file to be locked is
				considered as a linear sequence of bytes.
		*/
	assertion(14, BASE);
	start
		prep("Create a temporary file and set three shared locks on it.");
		start
	  		fd = temp_open(FILENAME, O_CREAT | O_TRUNC | O_RDWR, S_IRWXU);
			Zwrite(fd, String, FILE_LENGTH);
			SET_LOCK(F_RDLCK, fd, SEEK_SET, QUARTER_OFFSET +(off_t)1,
												HALF_FILE -(off_t)2, &flock1);
			SET_LOCK(F_RDLCK, fd, SEEK_SET, HALF_OFFSET +QUARTER_OFFSET,
												TOTAL_FILE, &flock1);
			SET_LOCK(F_RDLCK, fd, SEEK_SET, ZERO_OFFSET, QUARTER_FILE, &flock1);
		finis   

		FORK
			prep("Call fcntl(), with l_whence set to SEEK_SET, to get lock");
			prep("info file where first lock was set.");
			start
				Zlseek(fd, (off_t)0, SEEK_SET);
				GET_LOCK(F_WRLCK, fd, SEEK_SET, ZERO_OFFSET +(off_t)1,
												TOTAL_FILE, &flock2);
			finis

			testing("Info returned is from the first blocking lock, and");
			testing("l_whence is set to SEEK_SET.");
			start
				continue_on(ERROR);
				lock_tst(&flock2, F_RDLCK, SEEK_SET, ZERO_OFFSET,
												QUARTER_FILE, getppid());
			finis
		KROF

		FORK
			prep("Call fcntl(), with l_whence set to SEEK_CUR, to get lock");
			prep("info file where lock was set.");
			start
				Zlseek(fd, (off_t)0, SEEK_SET);
				GET_LOCK(F_WRLCK, fd, SEEK_CUR, QUARTER_OFFSET +(off_t)2,
												QUARTER_FILE, &flock2);
			finis

			testing("Info returned is from the blocking lock, and");
			testing("l_whence is set to SEEK_SET.");
			start
				continue_on(ERROR);
				lock_tst(&flock2, F_RDLCK, SEEK_SET, QUARTER_OFFSET +(off_t)1,
												HALF_FILE -(off_t)2, getppid());
			finis
		KROF

		FORK
			prep("Call fcntl(), with l_whence set to SEEK_CUR, to get lock");
			prep("info file where first lock was set.");
			start
				Zlseek(fd, (off_t)0, SEEK_SET);
				GET_LOCK(F_WRLCK, fd, SEEK_CUR, QUARTER_OFFSET +(off_t)2,
												TOTAL_FILE, &flock2);
			finis

			testing("Info returned is from the first blocking lock, and");
			testing("l_whence is set to SEEK_SET.");
			start
				continue_on(ERROR);
				lock_tst(&flock2, F_RDLCK, SEEK_SET, QUARTER_OFFSET +(off_t)1,
												HALF_FILE -(off_t)2, getppid());
			finis
		KROF

		FORK
			prep("Call fcntl(), with l_whence set to SEEK_END, to get lock");
			prep("info file where lock was set.");
			start
				Zlseek(fd, (off_t)0, SEEK_SET);
				GET_LOCK(F_WRLCK, fd, SEEK_END, ZERO_OFFSET, TOTAL_FILE,
																&flock2);
			finis

			testing("Info returned is from the blocking lock, and");
			testing("l_whence is set to SEEK_SET.");
			start
				continue_on(ERROR);
				lock_tst(&flock2, F_RDLCK, SEEK_SET,
											HALF_OFFSET +QUARTER_OFFSET,
											TOTAL_FILE, getppid());
			finis
		KROF

		CLOSE_LOCKED_FILE(fd);

		prep("Create temp file and set three exclusive locks on it.");
		start
	  		fd = temp_open(FILENAME, O_CREAT | O_TRUNC | O_RDWR, S_IRWXU);
			Zwrite(fd, String, FILE_LENGTH);
			SET_LOCK(F_WRLCK, fd, SEEK_SET, QUARTER_OFFSET +(off_t)1,
												HALF_FILE -(off_t)2, &flock1);
			SET_LOCK(F_WRLCK, fd, SEEK_SET, HALF_OFFSET +QUARTER_OFFSET,
												TOTAL_FILE, &flock1);
			SET_LOCK(F_WRLCK, fd, SEEK_SET, ZERO_OFFSET, QUARTER_FILE, &flock1);
		finis   

		FORK
			prep("Call fcntl(), with l_whence set to SEEK_SET, to get lock");
			prep("info file where lock was set.");
			start
				Zlseek(fd, (off_t)0, SEEK_SET);
				GET_LOCK(F_RDLCK, fd, SEEK_SET, ZERO_OFFSET +(off_t)1,
												TOTAL_FILE, &flock2);
			finis

			testing("Info returned is from the blocking lock, and");
			testing("l_whence is set to SEEK_SET.");
			start
				continue_on(ERROR);
				lock_tst(&flock2, F_WRLCK, SEEK_SET, ZERO_OFFSET,
												QUARTER_FILE, getppid());
			finis
		KROF

		FORK
			prep("Call fcntl(), with l_whence set to SEEK_CUR, to get lock");
			prep("info file where lock was set.");
			start
				Zlseek(fd, (off_t)0, SEEK_SET);
				GET_LOCK(F_RDLCK, fd, SEEK_CUR, QUARTER_OFFSET +(off_t)2,
												QUARTER_FILE, &flock2);
			finis

			testing("Info returned is from the blocking lock, and");
			testing("l_whence is set to SEEK_SET.");
			start
				continue_on(ERROR);
				lock_tst(&flock2, F_WRLCK, SEEK_SET, QUARTER_OFFSET +(off_t)1,
												HALF_FILE -(off_t)2, getppid());
			finis
		KROF

		FORK
			prep("Call fcntl(), with l_whence set to SEEK_END, to get lock");
			prep("info file where lock was set.");
			start
				Zlseek(fd, (off_t)0, SEEK_SET);
				GET_LOCK(F_RDLCK, fd, SEEK_END, ZERO_OFFSET, TOTAL_FILE,
																&flock2);
			finis

			testing("Info returned is from the first blocking lock, and");
			testing("l_whence is set to SEEK_SET.");
			start
				continue_on(ERROR);
				lock_tst(&flock2, F_WRLCK, SEEK_SET,
											HALF_OFFSET +QUARTER_OFFSET,
											TOTAL_FILE, getppid());
			finis
		KROF

		CLOSE_LOCKED_FILE(fd);
	finis

		/*	A call to fcntl(fildes, F_GETLK, arg) sets the lock type to
			F_UNLCK and does not change the other structure flock elements
			when no lock exists in the region specified in the structure
			flock addressed by arg that would prevent the lock from being
			created.
		*/
	assertion(15, BASE);
	start
		prep("Create a temp file, write to it, and set a shared lock");
		prep("on a portion of the file.");
		start
	  		fd = temp_open(FILENAME, O_CREAT | O_TRUNC | O_RDWR, S_IRWXU);
			expecting(FILE_LENGTH);
			Zwrite(fd, String, FILE_LENGTH);
			expecting(SUCCESS);
			SET_LOCK(F_RDLCK, fd, SEEK_SET, ZERO_OFFSET, QUARTER_FILE, &flock1);
		finis

		FORK
			prep("Get lock info on file where shared lock was not set.");
			start
				GET_LOCK(F_WRLCK, fd, SEEK_SET, HALF_FILE, HALF_FILE, &flock2);
			finis

			testing("Check area of the file that is not shared locked");
			testing("and expect l_type to be set to F_UNLCK and");
			testing("other elements of lock structure are not altered."); 
			start
				lock_tst(&flock2, F_UNLCK, SEEK_SET, HALF_FILE, HALF_FILE,
															(pid_t)0);
			finis
		KROF

		CLOSE_LOCKED_FILE(fd);
	finis

		/*	A call to fcntl(fildes, F_SETLK, arg) with lock type
			set to F_RDLCK establishes a shared lock for the region
			specified in the structure flock addressed by arg.
		*/
	assertion(16, BASE);
	start
		prep("Create a temp file, write to it, and set a read lock");
		prep("on a portion of the file.");
		start
	  		fd = temp_open(FILENAME, O_CREAT | O_TRUNC | O_RDWR, S_IRWXU);
			expecting(FILE_LENGTH);
			Zwrite(fd, String, FILE_LENGTH);
			SET_LOCK(F_RDLCK, fd, SEEK_SET, ZERO_OFFSET, TOTAL_FILE, &flock1);
		finis

		FORK
			testing("A write lock cannot be established on shared file.");
			start
				expecting(FAILURE);
				SET_LOCK(F_WRLCK, fd, SEEK_SET, HALF_OFFSET, HALF_FILE, &flock2);
			finis

			testing("Check parent's shared lock exists.");
			start
				GET_LOCK(F_WRLCK, fd, SEEK_SET, HALF_OFFSET, HALF_FILE, &flock2);
				lock_tst(&flock2, F_RDLCK, SEEK_SET, ZERO_OFFSET, TOTAL_FILE,
															getppid());
			finis
		KROF

		CLOSE_LOCKED_FILE(fd);
	finis

		/*	fcntl(fildes, F_SETLK, arg) with lock type set to
			F_WRLCK establishes an exclusive lock for the region
			specified in the structure flock addressed by arg.
		*/
	assertion(17, BASE);
	start
		break_on(ERROR);

		prep("Create a temp file, write to it, and set an exclusive lock");
		prep("on the file.");
		start
	  		fd = temp_open(FILENAME, O_CREAT | O_TRUNC | O_RDWR, S_IRWXU);
			expecting(FILE_LENGTH);
			Zwrite(fd, String, FILE_LENGTH);
			SET_LOCK(F_WRLCK, fd, SEEK_SET, ZERO_OFFSET, TOTAL_FILE, &flock1);
		finis

		FORK
			exit_on(ERROR);
			testing("Cannot place a shared lock on the file.");
			start
				expecting(FAILURE);		/* Also tests ASSERTION #19 */
				continue_on(ERROR);
				SET_LOCK(F_RDLCK, fd, SEEK_SET, ZERO_OFFSET, TOTAL_FILE,&flock2);
			finis

			testing("Cannot place an exclusive lock on the file.");
			start
				expecting(FAILURE);		/* Also tests ASSERTION #19 */
				SET_LOCK(F_WRLCK, fd, SEEK_SET, ZERO_OFFSET, TOTAL_FILE,&flock2);
#define	PASS17
			finis

			testing("Check parent's exclusive lock exists.");
			start
				expecting(SUCCESS);
				GET_LOCK(F_RDLCK, fd, SEEK_SET, HALF_OFFSET, HALF_FILE, &flock2);
				lock_tst(&flock2, F_WRLCK, SEEK_SET, ZERO_OFFSET, TOTAL_FILE,
															getppid());
			finis
		KROF

		if(!GTI_NOT_PROVIDED) {
			/* Test implementations claim of lock support on terminal files */
		  if(GTI_LOCKS) {
			int 	tty_fildes, loop_fildes;
			struct	termios tty_termios, *tty_termios_p = &tty_termios;
			struct	termios loop_termios, *loop_termios_p = &loop_termios;

			break_on(ERROR);

			prep("Open terminal device files.");
			start
				termios_prep(&tty_fildes, tty_termios_p, &loop_fildes,
										loop_termios_p);
			finis

			prep("Write to terminal file , then set an exclusive lock.");
			start
				SET_ALARM(TIMEOUT)
					expecting(Strlen);
					Zwrite(tty_fildes, String, Strlen);
				CLEAR_ALARM
				SET_LOCK(F_WRLCK, tty_fildes, SEEK_SET, ZERO_OFFSET,
													TOTAL_FILE, &flock1);
			finis

			FORK
				exit_on(ERROR);
				testing("Cannot place a shared lock on terminal file.");
				start
					expecting(FAILURE);
					continue_on(ERROR);
					SET_LOCK(F_RDLCK, fd, SEEK_SET, ZERO_OFFSET,
													TOTAL_FILE, &flock2);
				finis

				testing("Cannot place an exclusive lock on terminal file.");
				start
					expecting(FAILURE);
					break_on(ERROR);
					SET_LOCK(F_WRLCK, fd, SEEK_SET, ZERO_OFFSET,
													TOTAL_FILE,&flock2);
				finis
			KROF

			cleanup("Close terminal device files.");
			start
				SIGNAL(SIGHUP, SIG_IGN);
				Zclose(tty_fildes);
				Zclose(loop_fildes);
				SLEEP(GTI_LINE_CLOSE_DELAY)
			finis
		  }
		}
	finis

		/*	fcntl(fildes, F_SETLK, arg) with lock type set to
			F_UNLCK removes any locks for the region specified
			in the structure flock addressed by arg.
		*/
	assertion(18, BASE);
	start
		prep("Attempt to clear the previously set lock with F_UNLCK.");
		start
			SET_LOCK(F_UNLCK, fd, SEEK_SET, ZERO_OFFSET, TOTAL_FILE, &flock1);
		finis

		FORK
			testing("Can place an exclusive lock on the file.");
			start
				GET_LOCK(F_WRLCK, fd, SEEK_SET, ZERO_OFFSET, TOTAL_FILE,&flock2);
				lock_tst(&flock2, F_UNLCK, SEEK_SET, ZERO_OFFSET, TOTAL_FILE,
															(pid_t)0);
			finis
		KROF
	finis

		/*	If a shared or exclusive lock cannot be set,
			fcntl(fildes, F_SETLK, arg) returns without waiting.
		*/
	assertion(19, BASE);
	start
		testing("Tested in ASSERTION #17.");
		start
#ifndef	PASS17
			error("Failed test #17.");
#endif
		finis
	finis

	done();
}


					/*	Check lock info for proper settings.	*/
void lock_tst(struct	flock *lck, short	l_type, short l_whence,
			off_t l_start, off_t l_len, pid_t l_pid)
{
	if(lck->l_type != l_type) 
		error("%s\n%s%s\n%s%s\n",
			"l_type not correct.",
			"Expected:	",
			(l_type == F_UNLCK) ? "F_UNLCK" :
					(l_type == F_RDLCK) ? "F_RDLCK" :
					(l_type == F_WRLCK) ? "F_WRLCK" :
					"Unknown LOCK type",
			"Actual:	",
			(lck->l_type == F_UNLCK) ? "F_UNLCK" :
					(lck->l_type == F_RDLCK) ? "F_RDLCK" :
					(lck->l_type == F_WRLCK) ? "F_WRLCK" :
					"Unknown LOCK type");

	
	if(lck->l_whence != l_whence) 
		error("%s\n%s%s\n%s%s\n",
			"l_whence not correct.",
			"Expected:	",
			(l_whence == SEEK_SET) ? "SEEK_SET" :
					(l_whence == SEEK_CUR) ? "SEEK_CUR" :
					(l_whence == SEEK_END) ? "SEEK_END" :
					"Unknown l_whence value",
			"Actual:	",
			(lck->l_whence == SEEK_SET) ? "SEEK_SET" :
					(lck->l_whence == SEEK_CUR) ? "SEEK_CUR" :
					(lck->l_whence == SEEK_END) ? "SEEK_END" :
					"Unknown l_whence value");

	if(lck->l_start != l_start) 
		error("%s\n%s%lu\n%s%lu\n",
			"l_start not correct.",
			"Expected:	",
			(unsigned long) l_start,
			"Actual:	",
			(unsigned long) lck->l_start);

	if(lck->l_len != l_len) 
		error("%s\n%s%lu\n%s%lu\n",
			"l_len not correct.",
			"Expected:	",
			(unsigned long) l_len,
			"Actual:	",
			(unsigned long) lck->l_len);

	if(lck->l_pid != l_pid) 
		error("%s\n%s%ld\n%s%ld\n",
			"l_pid not correct.",
			"Expected:	",
			(long) l_pid,
			"Actual:	",
			(long) lck->l_pid);
	return;
}
