#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zfcntl.h>
#include	<Ztermios.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<pcts_timer.h>

#define	FILENAME	"6.5.2_20"
#define	String		"Woolly bully, woolly bully   \n"
#define	Strlen		(sizeof(String) -1)

#include	<locks.h>


int main(void)
{
	int		fd;
	pid_t	pid;
	struct	flock flock1, flock2;

	setup(E_FCNTL);
	funct(E_FCNTL);

		/*	fcntl(fildes, F_SETLKW, arg) with lock type set to
			F_RDLCK establishes a shared lock for the region
			specified in the structure flock addressed by arg.
		*/
	assertion(20, BASE);
	start
		prep("Create a temp file, set a shared lock on");
		prep("the entire file using F_SETLKW.");
		start
			fd = temp_open(FILENAME, O_CREAT | O_TRUNC | O_RDWR, S_IRWXU);
			Zwrite(fd, String, FILE_LENGTH);
			SET_LOCKW(F_RDLCK, fd, SEEK_SET, ZERO_OFFSET, TOTAL_FILE, &flock1);
		finis

		FORK
			testing("Check that another process could place a");
			testing("shared lock on the locked portion of the file.");
			start
				GET_LOCK(F_RDLCK, fd, SEEK_SET, ZERO_OFFSET,
													QUARTER_FILE, &flock2);
				continue_on(ERROR);
				if(flock2.l_type != F_UNLCK)
					error("%s\n%s%d\n",
						"Expected lock type of F_UNLCK.",
						"Actual lock value was ",
						flock2.l_type);
			finis

			testing("Child cannot place an exclusive lock on the file.");
			start
				GET_LOCK(F_WRLCK, fd, SEEK_SET, ZERO_OFFSET, TOTAL_FILE,&flock2);
				if(flock2.l_type != F_RDLCK)
					error("%s\n%s%d\n",
						"Expected lock type of F_RDLCK.",
						"Actual lock value was ",
						flock2.l_type);
			finis
		KROF

		CLOSE_LOCKED_FILE(fd);
	finis

		/*	fcntl(fildes, F_SETLKW, arg) with lock type set to
			F_WRLCK establishes an exclusive lock for the region
			specified in the structure flock addressed by arg.
		*/
	assertion(21, BASE);
	start
		prep("Create a temp file, set a write lock on");
		prep("the entire file.");
		start
	  		fd = temp_open(FILENAME, O_CREAT | O_TRUNC | O_RDWR, S_IRWXU);
			Zwrite(fd, String, FILE_LENGTH);
			SET_LOCKW(F_WRLCK, fd, SEEK_SET, ZERO_OFFSET, TOTAL_FILE, &flock1);
		finis

		FORK
			testing("Check that another process cannot place a");
			testing("read lock on the locked portion of the file.");
			start
				/* just to be different, close the fd and then open again */
				close(fd);
	  			fd = temp_open(FILENAME, O_RDWR, S_IRWXU);
				GET_LOCK(F_RDLCK, fd, SEEK_SET, ZERO_OFFSET,
														QUARTER_FILE, &flock2);
				if(flock2.l_type != F_WRLCK)
					error("%s\n%s%d\n",
						"Expected lock type of F_WRLCK.",
						"Actual lock value was ",
						flock2.l_type);
		
				expecting(FAILURE);		/* Additional check */
				SET_LOCK(F_RDLCK, fd, SEEK_SET, ZERO_OFFSET,
														QUARTER_FILE, &flock2);
		finis

		testing("Child cannot place an exclusive lock on the file.");
		start
			expecting(SUCCESS);
			GET_LOCK(F_WRLCK, fd, SEEK_SET, QUARTER_FILE, TOTAL_FILE,&flock2);
			if(flock2.l_type != F_WRLCK)
				error("%s\n%s%d\n",
					"Expected lock type of F_WRLCK.",
					"Actual lock value was ",
					flock2.l_type);
		
			expecting(FAILURE);		/* Additional check */
			SET_LOCK(F_WRLCK, fd, SEEK_SET, ZERO_OFFSET, QUARTER_FILE, &flock2);
			finis
		KROF

		CLOSE_LOCKED_FILE(fd);
	finis

		/*	fcntl(fildes, F_SETLKW, arg) with lock type set to
			F_UNLCK removes any locks for the region specified
			in the structure flock addressed by arg.
		*/
	assertion(22, BASE);
	start
		prep("Create a temp file, write to it, set a readlock on one");
		prep("region and a write lock on another.");
		start
	  		fd = temp_open(FILENAME, O_CREAT | O_TRUNC | O_RDWR, S_IRWXU);
			Zwrite(fd, String, FILE_LENGTH);
			SET_LOCK(F_RDLCK, fd, SEEK_SET, ZERO_OFFSET, QUARTER_FILE, &flock1);
			SET_LOCK(F_WRLCK, fd, SEEK_SET, QUARTER_FILE, QUARTER_FILE, &flock1);
		finis

		prep("Call fcntl(fildes, F_SETLKW, arg) to unlock entire file.");
		start
			SET_LOCKW(F_UNLCK, fd, SEEK_SET, ZERO_OFFSET, TOTAL_FILE, &flock1);
		finis

		FORK
			testing("Fork a child process to see if a write lock could be");
			testing("placed on the previously locked regions of the file.");
			start
				GET_LOCK(F_WRLCK, fd, SEEK_SET, ZERO_OFFSET,
														QUARTER_FILE, &flock2);
				if(flock2.l_type != F_UNLCK)
					error("%s\n%s%d\n",
						"Expected lock type of F_UNLCK.",
						"Actual lock value was ",
						flock2.l_type);
			finis
		KROF

		CLOSE_LOCKED_FILE(fd);
	finis

		/*	fcntl(fildes, F_SETLKW, arg), when the shared or exclusive
			lock is blocked by other locks, will wait until the request
			can be satisfied.
		*/
	assertion(23, BASE);
	start
		prep("Create a temp file, set a shared lock on the entire file.");
		start
	  		fd = temp_open(FILENAME, O_CREAT | O_TRUNC | O_RDWR, S_IRWXU);
			Zwrite(fd, String, FILE_LENGTH);
			SET_LOCKW(F_RDLCK, fd, SEEK_SET, ZERO_OFFSET, TOTAL_FILE, &flock1);
		finis

		prep("Fork a child process.");
		start
			sv_flush( );
			pid = Zfork();
		finis

		if(pid == (pid_t)0) {
			SLEEP(WAIT_FOR_TIME);
			testing("Child process waits until a  write lock can be");
			testing("placed on the file.");
			start
				SET_LOCKW(F_WRLCK, fd, SEEK_SET, ZERO_OFFSET,
														QUARTER_FILE, &flock1);
			finis
			done();
		}

		prep("Delay before clearing lock.");
		start
			sv_flush();			/* Log "prep" before "testing"*/
			SLEEP(2 * WAIT_FOR_TIME);
			SET_LOCK(F_UNLCK, fd, SEEK_SET, ZERO_OFFSET, TOTAL_FILE, &flock1);
		finis

		WAIT_FOR_PID(pid);

		CLOSE_LOCKED_FILE(fd);
	finis

	done();
}
