#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<pcts_timer.h>
#include	<locks.h>

#define	FILENAME1	"6.5.2_51a"
#define	FILENAME2	"6.5.2_51b"
#define	String		"Woolly bully, woolly bully\n"
#define	Strlen		(sizeof(String) -1)

static	pid_t	pid;

static	void sigs_int(int i)
{
	Zkill(pid, SIGKILL);		/* Terminate grandchild process */
	return;
}

int main(void)
{
	struct	flock flock1, flock2;
	int		fdA, fdB;
	struct	sigaction	act1, act2;
	int		stat_val;

	setup(E_FCNTL);
	funct(E_FCNTL);

		/*	If [EDEADLK] is supported, "cmd" is F_SETLKW and a
			deadlock condition was detected.
		*/
	assertion(51, BASE);
	start
		if(EDEADLK_FCNTL) {
		  SET_ALARM(MAX_TIME_TO_RUN);
			exit_on(ERROR);
			prep("Create two test files.");
			start
		  		fdA = temp_open(FILENAME1, O_CREAT | O_TRUNC | O_RDWR, S_IRWXU);
				expecting(Strlen);
				Zwrite(fdA, String, Strlen);

		  		fdB = temp_open(FILENAME2, O_CREAT | O_TRUNC | O_RDWR, S_IRWXU);
				expecting(Strlen);
				Zwrite(fdB, String, Strlen);
			finis
	
			/*	Do all setlockw() calls in child processes to avoid using
				timers in case implementations use timers to detect
				[EDEADLK] occurrences.
			*/
			continue_on(ERROR);				/* Cleanup all active processes */
			ENABLE_PSYNC
				FORK
					prep("Fork a grandchild process.");
					start
						sv_flush();
						pid = Zfork();
					finis
	
					if(pid ==(pid_t)0) {
						prep("Grandchild sets exclusive lock on file B.");
						start
							SET_LOCK(F_WRLCK, fdB, SEEK_SET, ZERO_OFFSET,
													TOTAL_FILE, &flock1);
							sv_flush();
						finis

						WAIT_PSYNC
										/* Assure both process are ready */
						SEND_PSYNCA
									/* Have child attempt to set B lock first */
						SLEEP(WAIT_FOR_TIME);

						testing("Grandchild attempts to set shared lock on");
						testing("file A by waiting till lock can be set.");
						testing("Expects an errno return of [EDEADLK].");
						start
							expecting(-1);
							expecting(EDEADLK);
							SET_LOCKW(F_RDLCK, fdA, SEEK_SET, ZERO_OFFSET,
													TOTAL_FILE, &flock2);
						finis

						done();
					}

														/* Child process */
					prep("Have child catch SIGINT signal");
					prep("If SIGINT occurs, terminate grandchild process.");
					start
						Zsigemptyset(&act1.sa_mask);
						act1.sa_handler = sigs_int;
						act1.sa_flags = 0;
						Zsigaction(SIGINT, &act1, &act2);
					finis

					prep("Child sets exclusive lock on file A.");
					start
						SET_LOCK(F_WRLCK, fdA, SEEK_SET, ZERO_OFFSET,
													TOTAL_FILE, &flock1);
					finis

					SEND_PSYNC
					WAIT_PSYNCA

					prep("Child sets shared lock on file B by waiting");
					prep("till lock can be set.");
					start
						expecting(VOID);
						SET_LOCKW(F_RDLCK, fdB, SEEK_SET, ZERO_OFFSET,
													TOTAL_FILE, &flock2);
					finis

					wait(&stat_val);		/* wait for grandchild process */
				KROF
		  	DISABLE_PSYNC
	
		  CLEAR_ALARM
		  if(unused_time == 0) {
							/* Test timed out, terminate child processes */
			kill(pid, SIGINT);
		  }
		}
		else
			not_implemented("[EDEADLK] for fcntl()");
	finis

	assertion(52, EXTENDED);

	done();
}
