#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<pcts_timer.h>

#define FILENAME "6.4.2_5"
#define FIFONAME "6.4.2_5_fifo"

char	String[] = "Hello, world!";
char	String1[] = "Hello,";
char	String2[] = " world!";
#define Strlen	(sizeof(String)-1)
#define Strlen1	(sizeof(String1)-1)
#define Strlen2	(sizeof(String2)-1)


int main(void)
{
	char	path[_POSIX_PATH_MAX +1];
	char	fifop[_POSIX_PATH_MAX +1];
	int 	fd;
	char	*inbuf, *outbuf;
	int 	fifo_rd, fifo_wr;
	size_t	pipe_buf_sz, fifo_buf_sz, max_buf_sz, pipe_max, fifo_max;
	pid_t	pid;

	setup(E_WRITE);
	funct(E_WRITE);

	exit_on(ERROR);

	assertion(5, BASE);
	start
		off_t	offset;
		time_t	s_time;

		prep("Establish path to FIFO and create it.");
		start
			Zstrcpy(fifop, temp_path());
			Zstrcat(fifop, FIFONAME);
			Zmkfifo(fifop, PROT_ALL);
		finis

							/* Maximum bytes written in pipe and FIFO */
		max_buf_sz=g_max_pipe(&pipe_buf_sz, &fifo_buf_sz, &pipe_max, &fifo_max);

		prep("Allocate and initialize I/O buffers.");
		start
			int i;

			inbuf = (char *) Zmalloc(max_buf_sz);
			outbuf = (char *) Zmalloc(max_buf_sz);
			memset(inbuf, '\0', max_buf_sz);

			for(i = 0; i < max_buf_sz; ++i)
				outbuf[i] = i % CHAR_MAX;
		finis

		prep("Establish regular file for testing.");
		start
			Zstrcpy(path, temp_path());
			Zstrcat(path, FILENAME);
			Zclose(Zcreat(path, PROT_ALL));
		finis

		FORK
			testing("Write for nbyte of zero returns zero and no bytes");
			testing("are written and st_ctime nor st_mtime are updated.");
			testing("Test when O_APPEND is set.");
			start
				fd = Zopen(path, O_WRONLY | O_APPEND);
				offset = Zlseek(fd, (off_t)0, SEEK_END);
				Ztime(&s_time);				/* Obtain minimum reference time. */
				dt_secs(TRF_DELAY);			/* Delay a fixed interval.	*/
				expecting(0);
				Zwrite(fd, "Z", 0);
				trf_ck(CHK_CTIME, FALSE, s_time, fd, NO_PATH);
				trf_ck(CHK_MTIME, FALSE, s_time, fd, NO_PATH);
				expecting(offset);
				Zlseek(fd, (off_t)0, SEEK_CUR);
			finis

			testing("Write for nbyte of zero returns zero and no bytes");
			testing("are written and st_ctime nor st_mtime are updated.");
			testing("Test when O_APPEND is clear.");
			start
				fd = Zopen(path, O_WRONLY);
				offset = Zlseek(fd, (off_t)0, SEEK_END);
				Ztime(&s_time);				/* Obtain minimum reference time. */
				dt_secs(TRF_DELAY);			/* Delay a fixed interval.	*/
				expecting(0);
				Zwrite(fd, "Z", 0);
				trf_ck(CHK_CTIME, FALSE, s_time, fd, NO_PATH);
				trf_ck(CHK_MTIME, FALSE, s_time, fd, NO_PATH);
				expecting(offset);
				Zlseek(fd, (off_t)0, SEEK_CUR);
			finis
		KROF
	finis

		/*	On a regular file, write() starts at a position in
			the file given by the file offset associated with the file
			descriptor argument.
		*/
	assertion(6, BASE);
	start
		FORK
			testing("Write() starts at proper offset.");
			start
				fd = Zopen(path, O_RDWR | O_TRUNC);

				Zlseek(fd, (off_t)Strlen1, SEEK_SET);
				expecting(Strlen2);
				Zwrite(fd, String2, Strlen2);

				Zlseek(fd, (off_t)0, SEEK_SET);
				expecting(Strlen1);
				Zwrite(fd, String1, Strlen1);
				Zlseek(fd, (off_t)0, SEEK_SET);

				expecting(Strlen);
				Zread(fd, inbuf, pipe_buf_sz);
				if(strncmp(inbuf, String, Strlen))
					error("%s\n%s%s\n%s%*s\n",
							"Unexpected data was read.",
							"Expected: ",
							String,
							"Actual: ",
							Strlen,
							inbuf);
			finis
		KROF
	finis

		/*	Before write() returns, the file offset is incremented by
			the number of bytes actually written.
		*/
	assertion(7, BASE);
	start
		FORK
			prep("Seek to a known position in test file.");
			start
				fd = Zopen(path, O_RDWR | O_TRUNC);

				Zlseek(fd, (off_t)Strlen, SEEK_SET);
				expecting(Strlen);
				Zwrite(fd, String, Strlen);
			finis

			testing("Checking file offset.");
			start
				expecting((off_t)(Strlen * 2));
				Zlseek(fd,(off_t) 0, SEEK_CUR);
			finis
		KROF
	finis

	assertion(8, BASE);
	start
		struct	stat	st_buf;
		off_t	offset;

		FORK
			prep("Open regular file with O_APPEND flag.");
			start
				fd = Zopen(path, O_WRONLY | O_APPEND);
			finis

			testing("When a call to write() causes the file offset to be");
			testing("greater than the prior length of the file, the");
			testing("length of the file is set to the new offset value.");
			start
				Zwrite(fd, String1, (size_t)Strlen1);
				offset = Zlseek(fd, (off_t)0, SEEK_CUR);
				Zfstat(fd, &st_buf);
				if(st_buf.st_size != offset)
					error("%s%lu\n%s%lu\n",
						"Expected size ",
						(unsigned long) offset,
						"Actual size ",
						(unsigned long) st_buf.st_size);
			finis
		KROF
	finis

		/*	On a file not capable of seeking, write() starts at
			the current position in the file.
		*/
	assertion(9, BASE);
	start
		FORK
			testing("FIFO is not capable of seeking.");
			start
				fifo_rd = Zopen(fifop, O_RDONLY | O_NONBLOCK);
				fifo_wr = Zopen(fifop, O_WRONLY);
				SET_TIMEOUT
					Zwrite(fifo_wr, String1, Strlen1);
				CLEAR_ALARM

				expecting(ESPIPE);
				Zlseek(fifo_wr,(off_t)0, SEEK_SET);
			finis
	
			testing("Write to FIFO starts at current position in file.");
			start
				SET_TIMEOUT
					Zwrite(fifo_wr, String2, Strlen2);
	
					memset(inbuf,'\0',max_buf_sz);

					expecting((int)Strlen);
					Zread(fifo_rd, inbuf, Strlen);
				CLEAR_ALARM
	
				if(strncmp(inbuf, String, Strlen))
					error("%s\n%s%s\n%s%*s\n",
							"Unexpected data was read.",
							"Expected: ",
							String,
							"Actual: ",
							Strlen,
							inbuf);
			finis
		KROF
	finis

		/*	When the O_APPEND flag of the file status flags
			is set, write() sets the file offset to the end of the file
			prior to each write.
		*/
	assertion(10, BASE);
	start
		off_t offset;

		FORK
			prep("Open file with O_APPEND flag set.");
			start
				fd = Zopen(path,(O_CREAT | O_TRUNC | O_RDWR | O_APPEND),
																PROT_ALL);
			finis

			testing("Write, lseek to beginning of file, and write again.");
			start
				Zwrite(fd, outbuf, max_buf_sz);
				Zlseek(fd,(off_t)0, SEEK_SET);
				Zwrite(fd, outbuf, max_buf_sz);
				offset = Zlseek(fd,(off_t)0, SEEK_CUR);
				if(offset != (off_t)(max_buf_sz*2))
					error("write() did not append to end of file.");
			finis
		KROF
	finis

	done();
}
