#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Ztimes.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<pcts_timer.h>

			/* 
				This test attempts to write a large number of bytes to
				the LOOP device and hopes to deliver an interrupt while
				the write() is being processed.  If the write() can be
				interrupted, a check for FIPS 151-1 compliance will
				test that the data written can be read.

				If the write() cannot be interrupted, the test will report
				UNRESOLVED.  The UNRESOLVED in this case is the indication
				that this test code was unable to test the assertion.
				The UNRESOLVED is an intermediate test result and must be
				resolved to a final test result. 
			*/

#define	MAX_OUTPUT	22000		/* Arbitrary large number of bytes to write. */


#define	WRITE_START_COUNT	-1000

#define	DATA_A		'A'
#define	DATA_B		'B'

static	char	*inbuf;
size_t		pipe_buf_sz, fifo_buf_sz, max_buf_sz, max_input, max_output;
int			ct_sleep(clock_t);


int main(void)
{
	int		r_cnt, w_cnt;
	char	*outbuf;
	int		c_delay;
	int		i;
	int 	tty_fildes, loop_fildes;
	struct termios tty_termios, *tty_termios_p = &tty_termios, loop_termios,
										*loop_termios_p = &loop_termios;

	setup(E_WRITE);
	funct(E_WRITE);

		/*	A call to write(), when interrupted by a signal after it
			has successfully written some data, returns the number of
			bytes written.
		*/

	exit_on(ERROR);

	assertion(13, NOT_APPLICABLE);
	assertion(14, NOT_APPLICABLE);
	assertion(15, EXTENDED);
	assertion(16, EXTENDED);
	assertion(17, NOT_APPLICABLE);

	assertion(12, BASE);
	start
		sigset_t	set;

		GTI_NOT_PROVIDED_EXIT

		max_input = g_max_input(PORT_TTY);
		if(max_input > (size_t)((cc_t)-1))
			max_input = (size_t)((cc_t)-1);

		prep("Allocate I/O buffers.");
		start
			max_output = MAX_OUTPUT;	/* Arbitrary large value */
			inbuf = (char *) Zmalloc(max_output);
			outbuf = (char *) Zmalloc(max_output);
		finis

		prep("Open PORT_TTY and PORT_LOOP devices.");
		prep("Set non-canonical mode.");
		start
			termios_prep(&tty_fildes, tty_termios_p, &loop_fildes,
									loop_termios_p);
			tty_termios_p->c_lflag &= ~(ICANON | ECHO);
							/* Read framing error as single character '\0'. */
 			tty_termios_p->c_iflag &= ~(IGNPAR | PARMRK); 
			tty_termios_p->c_iflag |= IXOFF;/* Enable flow control on input. */
											/* Read data available in queue. */
			tty_termios_p->c_cc[VMIN] = 0;
			tty_termios_p->c_cc[VTIME] = 0;
			Ztcsetattr(tty_fildes, TCSANOW, tty_termios_p);

			loop_termios_p->c_iflag |= IXON;/* Enable flow control on output. */
			Ztcsetattr(loop_fildes, TCSANOW, loop_termios_p);

			memset(outbuf, DATA_A, max_output);
			for(i=0; i<max_output; i++) 
				if(i%3)
					outbuf[i] = DATA_B;
		finis

		c_delay = (int) Zsysconf(_SC_CLK_TCK);
		for(i=0; i<c_delay; i+=2) {
			prep("Issue write() of %d bytes to terminal", MAX_OUTPUT);
			prep("device file while expecting an interrupt.");
			start
				sig_rcvd = 0;
				SIGNAL(SIGALRM, sig_int);
				sleep(1);			/*	Attempt to get alarm timer in
										synch with real time clock.
									*/
				sig_rcvd = 0;
				w_cnt = WRITE_START_COUNT;
				alarm(1);
				ct_sleep((clock_t)(c_delay -i));	/* Delay clock ticks */
				errno = 0;
				while(sig_rcvd == 0) {	/*	If no interrupt, keep trying. */
					w_cnt = write(loop_fildes, outbuf, max_output);

					if(w_cnt == MAX_OUTPUT) {
						message("Successful write of %d bytes.\n", w_cnt);
						if(sig_rcvd != 0)
							break;
									/* write() not interrupted, try again. */
						SIGBLOCK(set, SIGALRM);
							Ztcflush(loop_fildes, TCIOFLUSH); /* Flush output */
							ct_sleep(1);					  /*      and     */
							Ztcflush(tty_fildes, TCIOFLUSH);  /* input queues.*/
							w_cnt = 0;
						SIGUNBLOCK(set, SIGALRM);
						continue;
					}
					
					if(w_cnt == -1) {
						if(errno != EINTR)
							error("%s\n%s\n\t\t%s\n",
								"Call to write() returned unexpected results.",
								"Call to write() returned -1 with errno set to",
								err_lookup(errno));
						else {
							Ztcdrain(loop_fildes);
							if((r_cnt=Zread(tty_fildes, inbuf, max_input)) >0 )
								error("%s %s\nAlso transmitted %d bytes.",
										"Call to write() returned -1",
										"with errno set to [EINTR].",
										r_cnt);
							else {
								message("%s%s\nNo data was transmitted.",
										"Call to write() returned -1",
										"with errno set to [EINTR].");
								break;
							}
						}
					}

					break;
				}

				alarm(0);
			finis

			if((w_cnt == MAX_OUTPUT) || (w_cnt == -1) || (w_cnt == 0)) {
				Ztcflush(loop_fildes, TCIOFLUSH);		/* Flush output */
				ct_sleep(1);					  		/*      and     */
				Ztcflush(tty_fildes, TCIOFLUSH);  		/* input queues.*/
				continue;
			}

			if(w_cnt == WRITE_START_COUNT) {
				message("Interrupted too early.  Try again.");
				continue;
			}

			message("The call to write() was interrupted.");
			break;
		}

		if(i >= c_delay) {
			message("%s\n%s\n%s\n%s\n%s",
					"Unable to interrupt write().",
					"This test result code of UNRESOLVED may be converted",
					"to an APTL resolved test code of PCTS_FAILURE if the",
					"APTL acknowleges that write() cannot be interrupted",
					"on this implementation.");
			Ztcflush(loop_fildes, TCIOFLUSH); /* Flush output */
			ct_sleep(1);					  /*      and     */
			Ztcflush(tty_fildes, TCIOFLUSH);  /* input queues.*/
		}
		else {
		  testing("For FIPS 151-2 requirement Item \"n\".");
		  testing("write() of a terminal device file interrupted");
		  testing("by a signal returns count of bytes written by system.");
		  testing("Interrupted write() returned %d.", w_cnt); 
		  start
			int		d_flag = 0;
			int		frame_err;
			ssize_t	total_cnt;

			for(total_cnt=0; total_cnt<w_cnt; total_cnt += r_cnt) {
						/*	Read data in input queue repeatedly
							until all written data is read.
						*/
				SET_TIMEOUT
					r_cnt = Zread(tty_fildes, inbuf +total_cnt, max_input);
					message("Prior read() read %d bytes (total is %d bytes).",
									(int)r_cnt, (int)(total_cnt +r_cnt));
				CLEAR_ALARM
				if(r_cnt == 0) {
					if(d_flag > 0)
						error("Expected to read %d bytes, Received %d bytes.\n",
								(int)w_cnt, (int)total_cnt);
					sleep(1);	/* Wait for additional data to be transmitted.*/
					d_flag++;
					continue;
				}
				d_flag = 0;
			}
								/* Validate data read. */
 			frame_err = 0;
  			for(i=0; i<w_cnt; i++) {
				if(*(inbuf +i) == '\0') {/* Compute framing error count. */
					frame_err++;
					continue;
				}
				if(i%3) {
					if(*(inbuf +i) != DATA_B)
						error("Byte %d: Received \"%c\", Expected \"%c\"\n",
								i,
								*(inbuf +i),
								DATA_B);
				}
				else {
					if(*(inbuf +i) != DATA_A)
						error("Byte %d: Received \"%c\", Expected \"%c\"\n",
								i,
								*(inbuf +i),
								DATA_A);
				}
			}
			if((frame_err==w_cnt) || (frame_err>(w_cnt/_POSIX_MAX_CANON)))
				error("Too many framing errors detected (%d).", frame_err);
			if(frame_err > 0)
				warning("Detected %d framing errors.", frame_err);
		  finis
		}
	finis

	cleanup("Close PORT files.");
	start
		SIGNAL(SIGHUP, SIG_IGN);
		Zclose(tty_fildes);
		Zclose(loop_fildes);
		SLEEP(GTI_LINE_CLOSE_DELAY)
	finis

	done();
}


	/*
		Clock ticks sleep, ct_sleep(clock_ticks_to_sleep)
		Delay elapsed time in CLK_TCKths of a second.
		Accuracy is limited to +0, -1 CLK_TCKths.
	*/
int ct_sleep(clock_t delay)
{
	struct	tms	tmsbuf; 
	clock_t	r_val;

	if(!delay)
		return(0);
	r_val = times(&tmsbuf);
	while(r_val +delay > times(&tmsbuf))
		;
	return(0);
}
