#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<pcts_timer.h>

#define FILENAME "9.6.4_35"
#define FIFONAME "9.6.4_35_fifo"


int main(void)
{
	char	path[_POSIX_PATH_MAX +1], fifop[_POSIX_PATH_MAX +1];
	int		pipfd[2];
	char	*outbuf;
	int 	fifo_rd, fifo_wr;
	size_t	pipe_buf_sz, fifo_buf_sz, max_buf_sz, pipe_max, fifo_max;
	pid_t	pid;
	int 	fd;
	int		i;
	int		flags;

	setup(E_WRITE);
	funct(E_WRITE);

	exit_on(ERROR);


		/*	[EAGAIN], write() to a pipe or FIFO with the O_NONBLOCK
			flag set and no additional data can be written without blocking.
		*/
	assertion(35, BASE);
	start
		prep("Establish path to FIFO and create it");
		start
			Zstrcpy(fifop, temp_path());
			Zstrcat(fifop, FIFONAME);
			Zmkfifo(fifop, PROT_ALL);
		finis

							/* Maximum bytes written in pipe and FIFO */
		max_buf_sz=g_max_pipe(&pipe_buf_sz, &fifo_buf_sz, &pipe_max, &fifo_max);

		prep("Initialize input and output data buffers and create a");
		prep("regular file type.");
		start
			outbuf = (char *) Zmalloc(max_buf_sz);

			for(i = 0; i < max_buf_sz; ++i)
				outbuf[i] = i % CHAR_MAX;

			Zstrcpy(path, temp_path());
			Zstrcat(path, FILENAME);
			Zclose(Zcreat(path, PROT_ALL));
		finis

		FORK
			prep("Create a pipe with O_NONBLOCK flag set.");
			start
				Zpipe(pipfd);
				flags = Zfcntl( pipfd[1], F_GETFL);
				Zfcntl(pipfd[1], F_SETFL, flags | O_NONBLOCK);
			finis

			prep("Fill up pipe's write channel.");
			start
				size_t	n_bytes;
				size_t	nb_wr = 0;

				SET_TIMEOUT
					for(n_bytes = pipe_buf_sz; n_bytes; n_bytes /= 2)
						while(pipe_max -nb_wr >= n_bytes) {
							expecting(n_bytes);
							nb_wr += Zwrite(pipfd[1], outbuf, n_bytes);
						}
				CLEAR_ALARM
			finis

			testing("Writing to pipe.");
			start
				SET_ALARM(TIMEOUT)
					expecting(-1);
					expecting(EAGAIN);
					Zwrite(pipfd[1], outbuf, pipe_buf_sz/2);
				CLEAR_ALARM
			finis
	
			prep("Opening FIFO and setting O_NONBLOCK flag.");
			start
				fifo_rd = Zopen(fifop, O_RDONLY | O_NONBLOCK);
				fifo_wr = Zopen(fifop, O_WRONLY | O_NONBLOCK);
			finis

			prep("Fill up FIFO's write channel.");
			start
				size_t	n_bytes;
				size_t	nb_wr = 0;

				SET_TIMEOUT
					for(n_bytes = fifo_buf_sz; n_bytes; n_bytes /= 2)
						while(fifo_max -nb_wr >= n_bytes) {
							expecting(n_bytes);
							nb_wr += Zwrite(fifo_wr, outbuf, n_bytes);
						}
				CLEAR_ALARM
			finis

			testing("Writing to FIFO.");
			start
				SET_ALARM(TIMEOUT)
					expecting(-1);
					expecting(EAGAIN);
					Zwrite(fifo_wr, outbuf, fifo_buf_sz/2);
				CLEAR_ALARM
			finis
		KROF
	finis

	assertion(36, BASE);
	start
		FORK
			testing("Call to write() with invalid file descriptor returns -1");
			testing("and sets errno to [EBADF].");
			start
				fd = Zopen(path, O_RDONLY | O_CREAT, PROT_ALL);
				Zclose(fd);
				expecting(-1);
				expecting(EBADF);
				Zwrite(fd, outbuf, 1);
			finis
		KROF
	finis

	assertion(37, BASE);
	start
		FORK
			testing("Call to write() with file descriptor not opened for");
			testing("writing returns -1 and sets errno to [EBADF].");
			start
				fd = Zopen(path, O_RDONLY);
				expecting(-1);
				expecting(EBADF);
				Zwrite(fd, outbuf, 1);
			finis
		KROF
	finis

	assertion(38, EXTENDED);

		/*	[EPIPE], attempted to write to a pipe or FIFO that is not
			open for reading by any process. A SIGPIPE signal is sent
			to the process.
		*/
	assertion(42, BASE);
	start
		FORK
											/* pipe test */
			prep("Catch signal, create pipe and close read descriptor.");
			start
				SIGNAL(SIGPIPE, sig_int);
				Zpipe(pipfd);
				Zclose(pipfd[0]);
			finis
	
			testing("Receive SIGPIPE and errno set to [EPIPE].");
			start
				sig_rcvd = 0;
	
				SET_ALARM(TIMEOUT)
					errno = 0;
					expecting(-1);
					expecting(EPIPE);
					Zwrite(pipfd[1], outbuf, 1);
				CLEAR_ALARM
	
				if(sig_rcvd == 0)
					error("Did not receive signal SIGPIPE.");
			finis
	
											/* FIFO test */
			prep("Open FIFO read/write descriptors, close read descriptor.");
			start
				SIGNAL(SIGPIPE, sig_int);
				fifo_rd = Zopen(fifop, O_RDONLY | O_NONBLOCK);
				fifo_wr = Zopen(fifop, O_WRONLY);
				Zclose(fifo_rd);
			finis
	
			testing("A write() to a FIFO receives");
			testing("SIGPIPE and errno is set to [EPIPE].");
			start
				sig_rcvd = 0;
				SET_ALARM(TIMEOUT)
					expecting(-1);
					expecting(EPIPE);
					Zwrite(fifo_wr, outbuf, (unsigned)1);
				CLEAR_ALARM
	
				if(sig_rcvd == 0)
					error("Did not receive SIGPIPE.");
			finis
		KROF
	finis

	done();
}
