#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<pcts_timer.h>

#define FILENAME "8.1.13_05"
#define FIFONAME "8.1.13_05_fifo"


int main(void)
{
	pid_t	pid;
	int		pipfd[2];
	FILE	*sfd;
	char	inbuf[_POSIX_PIPE_BUF];
	int		tty_fildes, loop_fildes;
	struct	termios tty_termios, *tty_termios_p = &tty_termios,
					loop_termios, *loop_termios_p = &loop_termios;

	setup(E_FGETS);
	funct(E_FGETS);

	exit_on(ERROR);

								/*	These assertion tests contain code similar
									to those of fgets(). */
	assertion(5, BASE);
	start
		int		flags;
		int 	fifo_rd, fifo_wr;
		char	fifop[_POSIX_PATH_MAX +1];

		FORK
			prep("Establish a pipe and set the O_NONBLOCK flag.");
			start
				Zpipe(pipfd);
				flags = Zfcntl(pipfd[0], F_GETFL);
				Zfcntl(pipfd[0], F_SETFL, flags | O_NONBLOCK);
			finis
	
			testing("fgets() from an empty pipe with O_NONBLOCK set.");
			start
				sfd = Zfdopen(pipfd[0], "r");
				expecting(0);
				expecting(EAGAIN);
				Zfgets(inbuf, 2, sfd);
			finis
	
			prep("Open FIFO for read and write.");
			prep("Set O_NONBLOCK on read channel.");
			start
				Zstrcpy(fifop, temp_path());
				Zstrcat(fifop, FIFONAME);
				Zmkfifo(fifop, PROT_ALL);
				fifo_rd = Zopen(fifop, O_RDONLY | O_NONBLOCK);
				fifo_wr = Zopen(fifop, O_WRONLY);
			finis
	
			testing("fgets() from an empty FIFO with O_NONBLOCK set.");
			start
				sfd = Zfdopen(fifo_rd, "r");
				expecting(0);
				expecting(EAGAIN);
				Zfgets(inbuf, 2, sfd);
			finis

			cleanup("Remove FIFO file.");
			start
				Zunlink(fifop);
			finis
		KROF
	finis
	
	assertion(6, BASE);
	start
		FORK
			testing("Stream pointer argument addresses a file descriptor that");
			testing("is not open for reading.");
			start
				Zpipe(pipfd);
				sfd = Zfdopen(pipfd[1], "w");
				expecting(0);
				expecting(EBADF);
				Zfgets(inbuf, 2, sfd);
			finis
		KROF
	finis

	assertion(7, BASE);
	start
		FORK
			prep("Establish a pipe for testing.");
			start
				Zpipe(pipfd);
				sfd = Zfdopen(pipfd[0], "r");
			finis

			testing("fgets() interrupted by signal before it reads any data.");
			start
				SIGNAL(SIGALRM, sig_int);
				alarm(2);
				expecting(0);
				expecting(EINTR);
				Zfgets(inbuf, 2, sfd);
				alarm(0);
			finis
		KROF
	finis
	
	if(GTI_NOT_PROVIDED) {
		GTI_NOT_PROVIDED_RESULT(8, BASE);
		GTI_NOT_PROVIDED_RESULT(9, BASE);
		done();
	}

	assertion(8, BASE);
	start
		sigset_t	set;

		prep("Establish PORT_TTY as the control terminal and PORT_LOOP.");
		start
			exit_on(ERROR);
			termios_prep(&tty_fildes, tty_termios_p, &loop_fildes,
								loop_termios_p);
			sfd = Zfdopen(tty_fildes, "r+");
		finis


		FORK
			prep("Establish this process as a background process group.");
			start
				Zsetpgid((pid_t)0, getpid());
			finis
	
			testing("Attempt to fgets() from controlling terminal, when");
			testing("process is ignoring the SIGTTIN signal, returns");
			testing("a NULL pointer and sets errno to [EIO].");
			start
				SIGNAL(SIGTTIN, SIG_IGN);
				expecting(0);
				expecting(EIO);
				Zfgets(inbuf, 2, sfd);
			finis
	
			testing("Attempt to fgets() from controlling terminal, when");
			testing("process is blocking the SIGTTIN signal, returns");
			testing("a NULL pointer and sets errno to [EIO].");
			start
				SIGBLOCK(set, SIGTTIN);
				expecting(0);
				expecting(EIO);
				Zfgets(inbuf, 2, sfd);
			finis
		KROF
	finis
	
	assertion(9, BASE);
	start
		FORK
			FORK
				prep("Establish this process as background process group.");
				prep("Exit on a SIGHUP signal.");
				start
					Zsetpgid((pid_t)0, (pid_t)0);
					SIGNAL(SIGHUP, SIG_DFL);
				finis

				FORK
					prep("Ignore the SIGHUP signal.");
					start
						SIGNAL(SIGHUP, SIG_IGN);
					finis
	
					prep("Kill the grandchild (process group leader) to");
					prep("create an orphaned process group.");
					start
						Zkill(getppid(), SIGHUP);
						sleep(WAIT_FOR_TIME);
					finis
	
					testing("Call to fgets() for controlling terminal.");
					start
						expecting(0);
						expecting(EIO);
						Zfgets(inbuf, 2, sfd);
					finis
				KROF
			KROF
		KROF
		SLEEP(2 * WAIT_FOR_TIME);		/* Ensure test completes. */
	finis

	cleanup("Close terminal devices.");
	start
		SIGNAL(SIGHUP, SIG_IGN);
		Zclose(tty_fildes);
		Zclose(loop_fildes);
		SLEEP(GTI_LINE_CLOSE_DELAY)
	finis
	
	done();
}
