#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<pcts_timer.h>

#define FILENAME "8.1.31_05"
#define FIFONAME "8.1.31_05_fifo"


int main(void)
{
	char	path[_POSIX_PATH_MAX +1], fifop[_POSIX_PATH_MAX +1];
	int		pipfd[2];
	char	*outbuf;
	int 	fifo_rd, fifo_wr;
	size_t	pipe_buf_sz, fifo_buf_sz, max_buf_sz, pipe_max, fifo_max;
	pid_t	pid;
	int 	fd;
	FILE	*sfd;
	int		i;
	long	nbytes;
	int		flags;

	setup(E_PUTC);
	funct(E_PUTC);

	exit_on(ERROR);

								/*	These assertion tests contain code similar
									to those of fflush(). */
	assertion(5, BASE);
	start
		prep("Establish path to FIFO and create it");
		start
			Zstrcpy(fifop, temp_path());
			Zstrcat(fifop, FIFONAME);
			Zmkfifo(fifop, PROT_ALL);
		finis

							/* Maximum bytes written in pipe and FIFO */
		max_buf_sz=g_max_pipe(&pipe_buf_sz, &fifo_buf_sz, &pipe_max, &fifo_max);

		prep("Initialize input and output data buffers and create a");
		prep("regular file type.");
		start
			outbuf = (char *) Zmalloc(max_buf_sz);

			for(i = 0; i < max_buf_sz; ++i)
				outbuf[i] = i % CHAR_MAX;

			Zstrcpy(path, temp_path());
			Zstrcat(path, FILENAME);
			Zclose(Zcreat(path, PROT_ALL));
		finis

		FORK
			prep("Create a pipe with O_NONBLOCK flag set.");
			start
				Zpipe(pipfd);
				flags = Zfcntl( pipfd[1], F_GETFL);
				Zfcntl(pipfd[1], F_SETFL, flags | O_NONBLOCK);
			finis

			prep("Fill up pipe's write channel.");
			start
				size_t	n_bytes;
				size_t	nb_wr = 0;

				SET_TIMEOUT
					for(n_bytes = pipe_buf_sz; n_bytes; n_bytes /= 2)
						while(pipe_max -nb_wr >= n_bytes) {
							expecting(n_bytes);
							nb_wr += Zwrite(pipfd[1], outbuf, n_bytes);
						}
				CLEAR_ALARM
			finis

			prep("Set up an unbuffered stream which writes to filled pipe.");
			start
				sfd = Zfdopen(pipfd[1], "w");
				setbuf(sfd, (char *)0);
			finis

			testing("An putc() on the pipe when insufficient capacity");
			testing("exists to accept the buffered data.");
			start
				SET_ALARM(TIMEOUT)
					expecting(EOF);
					expecting(EAGAIN);
					Zputc((int)'D', sfd);
				CLEAR_ALARM
			finis
	
			prep("Open FIFO and set O_NONBLOCK flag.");
			start
				fifo_rd = Zopen(fifop, O_RDONLY | O_NONBLOCK);
				fifo_wr = Zopen(fifop, O_WRONLY | O_NONBLOCK);
			finis

			prep("Fill up FIFO's write channel.");
			start
				size_t	n_bytes;
				size_t	nb_wr = 0;

				SET_TIMEOUT
					for(n_bytes = fifo_buf_sz; n_bytes; n_bytes /= 2)
						while(fifo_max -nb_wr >= n_bytes) {
							expecting(n_bytes);
							nb_wr += Zwrite(fifo_wr, outbuf, n_bytes);
						}
				CLEAR_ALARM
			finis

			prep("Set up an unbuffered stream which writes to filled FIFO.");
			start
				sfd = Zfdopen(fifo_wr, "w");
				setbuf(sfd, (char *)0);
			finis

			testing("An putc() on the FIFO when insufficient capacity");
			testing("exists to accept the buffered data.");
			start
				SET_ALARM(TIMEOUT)
					expecting(EOF);
					expecting(EAGAIN);
					Zputc((int)'D', sfd);
				CLEAR_ALARM
			finis
		KROF
	finis

	assertion(6, BASE);
	start
		FORK
			testing("Stream pointer argument addresses a file descriptor that");
			testing("is closed.");
			start
				fd = Zopen(path, O_RDONLY | O_CREAT, PROT_ALL);
				sfd = Zfdopen(fd, "r");
				Zclose(fd);
				expecting(EOF);
				expecting(EBADF);
				Zputc((int)'D', sfd);
			finis
		KROF
	finis


							/*	When putc() is terminated due to receipt
								of a signal and no data was transferred, then
								putc() returns EOF and sets errno to [EINTR].
							*/
	assertion(7, BASE);
	start
		FORK
			prep("Create a pipe with O_NONBLOCK flag clear and fill it up.");
			start
				long	nb_wr, nb_to_wr;

				Zpipe(pipfd);
						/*	Must eventually write one byte at a time to ensure
							that if data can be placed in pipe it will be
							transferred to the pipe for all POSIX.1
							implementations.
						*/
				SET_ALARM(5 * TIMEOUT)
					flags = Zfcntl( pipfd[1], F_GETFL);
					Zfcntl(pipfd[1], F_SETFL, flags | O_NONBLOCK);
					for(nb_to_wr=pipe_buf_sz; nb_to_wr; nb_to_wr /= 2)
						while((nb_wr = write(pipfd[1], outbuf, nb_to_wr)) != -1)
							nbytes += nb_wr;
					Zfcntl(pipfd[1], F_SETFL, flags & ~O_NONBLOCK);
				CLEAR_ALARM
			finis

			prep("Set up an unbuffered stream which writes to filled pipe.");
			start
				sfd = Zfdopen(pipfd[1], "w");
				setbuf(sfd, (char *)0);
			finis

			testing("When putc() is terminated by a signal,");
			testing("putc() returns EOF and sets errno to [EINTR].");
			start
				SIGNAL(SIGALRM, sig_int);
				alarm(2);
					expecting(EOF);
					expecting(EINTR);
					Zputc((int)'D', sfd);
				alarm(0);
			finis
		KROF
	finis


	assertion(8, EXTENDED);
	assertion(9, EXTENDED);

	assertion(10, BASE);
	start
		FORK
											/* pipe test */
			prep("Catch signal, create pipe, and close read descriptor.");
			start
				SIGNAL(SIGPIPE, sig_int);
				Zpipe(pipfd);
				sfd = Zfdopen(pipfd[1], "w");
				setbuf(sfd, (char *)0);
				Zclose(pipfd[0]);
			finis
	
			testing("Call to putc(), when attempting to write to a pipe");
			testing("that is not open for reading by any process.");
			start
				sig_rcvd = 0;
	
				SET_ALARM(TIMEOUT)
					errno = 0;
					expecting(EOF);
					expecting(EPIPE);
					Zputc((int)'D', sfd);
				CLEAR_ALARM
	
				if(sig_rcvd == 0)
					error("Did not receive signal SIGPIPE.");
			finis
	
											/* FIFO test */
			prep("Catch signal, create FIFO, and close read descriptor.");
			start
				SIGNAL(SIGPIPE, sig_int);
				fifo_rd = Zopen(fifop, O_RDONLY | O_NONBLOCK);
				fifo_wr = Zopen(fifop, O_WRONLY);
				sfd = Zfdopen(fifo_wr, "w");
				setbuf(sfd, (char *)0);
				Zclose(fifo_rd);
			finis
	
			testing("Call to putc(), when attempting to write to a FIFO");
			testing("that is not open for reading by any process.");
			start
				sig_rcvd = 0;
	
				SET_ALARM(TIMEOUT)
					errno = 0;
					expecting(EOF);
					expecting(EPIPE);
					Zputc((int)'D', sfd);
				CLEAR_ALARM
	
				if(sig_rcvd == 0)
					error("Did not receive signal SIGPIPE.");
			finis
		KROF

		cleanup("Remove FIFO file.");
		start
			Zunlink(fifop);
		finis
	finis

	done();
}
