#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<pcts_timer.h>

#define FILENAME	"8.1.33_05"
#define FIFONAME	"8.1.33_05_fifo"
#define	String		"DaTa\n"


int main(void)
{
	char	path[_POSIX_PATH_MAX +1], fifop[_POSIX_PATH_MAX +1];
	int		fd, pipfd[2], fifo_rd, fifo_wr, fd_stdout, fd_sfd;
	char	*outbuf;
	size_t	pipe_buf_sz, fifo_buf_sz, max_buf_sz, pipe_max, fifo_max;
	pid_t	pid;
	int		i, flags, ret_val, s_errno;
	long	nbytes;

	setup(E_PUTS);
	funct(E_PUTS);

	exit_on(ERROR);

	assertion(5, BASE);
	start
		prep("Following tests require the output from puts() when tested");
		prep("to be unbuffered.");
		start
			sv_flush();
			setbuf(stdout, (char *)0);
		finis

		prep("Establish path to FIFO and create it");
		start
			Zstrcpy(fifop, temp_path());
			Zstrcat(fifop, FIFONAME);
			Zmkfifo(fifop, PROT_ALL);
		finis

							/* Maximum bytes written in pipe and FIFO */
		max_buf_sz=g_max_pipe(&pipe_buf_sz, &fifo_buf_sz, &pipe_max, &fifo_max);

		prep("Initialize input and output data buffers and create a");
		prep("regular file type.");
		start
			outbuf = (char *) Zmalloc(max_buf_sz);

			for(i = 0; i < max_buf_sz; ++i)
				outbuf[i] = i % CHAR_MAX;

			Zstrcpy(path, temp_path());
			Zstrcat(path, FILENAME);
			Zclose(Zcreat(path, PROT_ALL));
		finis

													/* Test pipe */
		FORK
			prep("Create a pipe with O_NONBLOCK flag set.");
			start
				Zpipe(pipfd);
				flags = Zfcntl( pipfd[1], F_GETFL);
				Zfcntl(pipfd[1], F_SETFL, flags | O_NONBLOCK);
			finis

			prep("Fill up pipe's write channel.");
			start
				size_t	n_bytes;
				size_t	nb_wr = 0;

				SET_TIMEOUT
					for(n_bytes = pipe_buf_sz; n_bytes; n_bytes /= 2)
						while(pipe_max -nb_wr >= n_bytes) {
							expecting(n_bytes);
							nb_wr += Zwrite(pipfd[1], outbuf, n_bytes);
						}
				CLEAR_ALARM
			finis

			prep("Set stdout as an unbuffered stream which writes to the");
			prep("filled pipe, then call puts().");
			start
							/*
								Redirect stdout retaining journal file.
								Set stdout as write channel of pipe.
								Lose automatic error check reporting.
							*/
				fd_stdout = Zdup(STDOUT_FILENO);
				close(STDOUT_FILENO);
				fd_sfd = dup(pipfd[1]);
				errno = 0;
				ret_val = puts(String);
				s_errno = errno;

					/* Reset stdout back to journal file. */
				close(fd_sfd);
				dup2(fd_stdout, STDOUT_FILENO);
				clearerr(stdout);
			finis

			testing("Call to puts() on the pipe when insufficient capacity");
			testing("exists to accept the buffered data.");
			start
				continue_on(ERROR);
				if(fd_sfd != STDOUT_FILENO)
					error("%s\n%s%d\n",
							"Expected file descriptor of STDOUT_FILENO",
							"Actual file descriptor was ",
							fd_sfd);
				if(ret_val >= 0)
					error("Call to puts() returned a value of %d.", ret_val);
				if(s_errno != EAGAIN)
					error("%s\n%s%s\n",
							"Expected errno value to be [EAGAIN].",
							"Actual errno value was ",
							err_lookup(s_errno));
			finis
	

													/* Test FIFO */
			prep("Open FIFO and set O_NONBLOCK flag.");
			start
				fifo_rd = Zopen(fifop, O_RDONLY | O_NONBLOCK);
				fifo_wr = Zopen(fifop, O_WRONLY | O_NONBLOCK);
			finis

			prep("Fill up FIFO's write channel.");
			start
				size_t	n_bytes;
				size_t	nb_wr = 0;

				SET_TIMEOUT
					for(n_bytes = fifo_buf_sz; n_bytes; n_bytes /= 2)
						while(fifo_max -nb_wr >= n_bytes) {
							expecting(n_bytes);
							nb_wr += Zwrite(fifo_wr, outbuf, n_bytes);
						}
				CLEAR_ALARM
			finis

			prep("Set stdout as an unbuffered stream which writes to the");
			prep("filled FIFO, then call puts().");
			start
							/*
								Redirect stdout retaining journal file.
								Set stdout as write channel of pipe.
								Lose automatic error checking.
							*/
				fd_stdout = Zdup(STDOUT_FILENO);
				close(STDOUT_FILENO);
				fd_sfd = dup(fifo_wr);
				errno = 0;
				ret_val = puts(String);
				s_errno = errno;

					/* Reset stdout back to journal file. */
				close(fd_sfd);
				dup2(fd_stdout, STDOUT_FILENO);
				clearerr(stdout);
			finis

			testing("Call to puts() on the FIFO when insufficient capacity");
			testing("exists to accept the buffered data.");
			start
				continue_on(ERROR);
				if(fd_sfd != STDOUT_FILENO)
					error("%s\n%s%d\n",
							"Expected file descriptor of STDOUT_FILENO",
							"Actual file descriptor was ",
							fd_sfd);
				if(ret_val >= 0)
					error("Call to puts() returned a value of %d.", ret_val);
				if(s_errno != EAGAIN)
					error("%s\n%s%s\n",
							"Expected errno value to be [EAGAIN].",
							"Actual errno value was ",
							err_lookup(s_errno));
			finis
		KROF
	finis

	assertion(6, BASE);
	start
		FORK
			testing("Stream pointer argument addresses a file descriptor that");
			testing("is not open for writing.");
			start
				continue_on(ERROR);
				fd = Zopen(path, O_RDONLY);

				fd_stdout = Zdup(STDOUT_FILENO);
				close(STDOUT_FILENO);
				fd_sfd = dup(fd);

				errno = 0;
				ret_val = puts(String);
				s_errno = errno;

				close(fd_sfd);
				dup2(fd_stdout, STDOUT_FILENO);

				if(fd_sfd != STDOUT_FILENO)
					error("%s\n%s%d\n",
							"Expected file descriptor of STDOUT_FILENO",
							"Actual file descriptor was ",
							fd_sfd);
				if(ret_val >= 0)
					error("Call to puts() returned a value of %d.", ret_val);
				if(s_errno != EBADF)
					error("%s\n%s%s\n",
							"Expected errno value to be [EBADF].",
							"Actual errno value was ",
							err_lookup(s_errno));
			finis
		KROF
	finis

							/*	When puts() is terminated due to receipt
								of a signal and no data was transferred, then
								puts() returns zero and sets errno to [EINTR].
							*/
	assertion(7, BASE);
	start
		FORK
			prep("Create a pipe with O_NONBLOCK flag clear and fill it up.");
			start
				long	nb_wr, nb_to_wr;

				Zpipe(pipfd);
						/*	Must eventually write one byte at a time to ensure
							that if data can be placed in pipe it will be
							transferred to the pipe for all POSIX.1
							implementations.
						*/
				SET_ALARM(5 * TIMEOUT)
					flags = Zfcntl( pipfd[1], F_GETFL);
					Zfcntl(pipfd[1], F_SETFL, flags | O_NONBLOCK);
					for(nb_to_wr=pipe_buf_sz; nb_to_wr; nb_to_wr /= 2)
						while((nb_wr = write(pipfd[1], outbuf, nb_to_wr)) != -1)
							nbytes += nb_wr;
					Zfcntl(pipfd[1], F_SETFL, flags & ~O_NONBLOCK);
				CLEAR_ALARM
			finis

			testing("When puts() is terminated by a signal,");
			testing("puts() returns zero and sets errno to [EINTR].");
			start
				continue_on(ERROR);
							/*	Setup stdout as unbuffered stream which
								writes to the filled pipe.
							*/
				fd_stdout = Zdup(STDOUT_FILENO);
				close(STDOUT_FILENO);
				fd_sfd = dup(pipfd[1]);

				SIGNAL(SIGALRM, sig_int);
				errno = 0;
				alarm(2);
					ret_val = puts(String);
					s_errno = errno;
				alarm(0);

				close(fd_sfd);
				dup2(fd_stdout, STDOUT_FILENO);

				if(fd_sfd != STDOUT_FILENO)
					error("%s\n%s%d\n",
							"Expected file descriptor of STDOUT_FILENO",
							"Actual file descriptor was ",
							fd_sfd);
				if(s_errno != EINTR)
					error("%s\n%s%s\n",
							"Expected errno value to be [EINTR].",
							"Actual errno value was ",
							err_lookup(s_errno));
				if(ret_val >= 0)
					error("Call to puts() returned a value of %d.", ret_val);
			finis
		KROF
	finis

	assertion(8, EXTENDED);
	assertion(9, EXTENDED);

	assertion(10, BASE);
	start
		FORK
											/* pipe test */
			prep("Catch signal, create pipe, and close read descriptor.");
			start
				SIGNAL(SIGPIPE, sig_int);
				Zpipe(pipfd);
				Zclose(pipfd[0]);
			finis
	
			testing("Call to puts(), when attempting to write to a pipe");
			testing("that is not open for reading by any process.");
			start
				sv_flush();
				continue_on(ERROR);
				sig_rcvd = 0;
	
				fd_stdout = Zdup(STDOUT_FILENO);
				close(STDOUT_FILENO);
							/* Assign STDOUT_FILENO to pipe's output channel. */
				fd_sfd = dup(pipfd[1]);

				errno = 0;
				ret_val = puts(String);
				s_errno = errno;

				close(fd_sfd);
				dup(fd_stdout);		/* Assign STDOUT_FILENO to journal. */

				if(fd_sfd != STDOUT_FILENO)
					error("%s\n%s%d\n",
							"Expected file descriptor of STDOUT_FILENO",
							"Actual file descriptor was ",
							fd_sfd);
				if(ret_val >= 0)
					error("Call to puts() returned a value of %d.", ret_val);
				if(s_errno != EPIPE)
					error("%s\n%s%s\n",
							"Expected errno value to be [EPIPE].",
							"Actual errno value was ",
							err_lookup(s_errno));
	
				if(sig_rcvd == 0)
					error("Did not receive signal SIGPIPE.");
			finis
	
											/* FIFO test */
			prep("Catch signal, create FIFO, and close read descriptor.");
			start
				SIGNAL(SIGPIPE, sig_int);
				fifo_rd = Zopen(fifop, O_RDONLY | O_NONBLOCK);
				fifo_wr = Zopen(fifop, O_WRONLY);
				Zclose(fifo_rd);
			finis
	
			testing("Call to puts(), when attempting to write to a FIFO");
			testing("that is not open for reading by any process.");
			start
				sv_flush();
				continue_on(ERROR);
				sig_rcvd = 0;
	
				fd_stdout = Zdup(STDOUT_FILENO);
				close(STDOUT_FILENO);
				fd_sfd = dup(fifo_wr);

				errno = 0;
				ret_val = puts(String);
				s_errno = errno;

				close(fd_sfd);
				dup2(fd_stdout, STDOUT_FILENO);

				if(fd_sfd != STDOUT_FILENO)
					error("%s\n%s%d\n",
							"Expected file descriptor of STDOUT_FILENO",
							"Actual file descriptor was ",
							fd_sfd);
				if(ret_val >= 0)
					error("Call to puts() returned a value of %d.", ret_val);
				if(s_errno != EPIPE)
					error("%s\n%s%s\n",
							"Expected errno value to be [EPIPE].",
							"Actual errno value was ",
							err_lookup(s_errno));
	
				if(sig_rcvd == 0)
					error("Did not receive signal SIGPIPE.");
			finis
		KROF
	finis

	cleanup("Remove FIFO file.");
	start
		Zunlink(fifop);
	finis

	done();
}
