#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Ztimes.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<pcts_timer.h>


int	main(void)
{
	int tty_fildes, loop_fildes;
	struct termios	tty_termios, loop_termios, *tty_termios_p = &tty_termios,
							*loop_termios_p = &loop_termios;
	pid_t	pid;
	int		fd_stdout, fd_sfd, ret_val, s_errno;

	setup(E_PUTS);
	funct(E_PUTS);


	exit_on(ERROR);

				/*	When TOSTOP is set, the process is in an orphaned
					process group, the writing process is not ignoring
					or blocking SIGTTOU, then an puts() to the controlling
					terminal sets errno to EIO.

	 				parent - new session, new controlling tty
	 			  	  |
	 				  child - new process group
	 			  	    |
	 		  		    great grandchild - created
					    |
					  child - exits
					    |
	 		  		    great grandchild - attempts write to control tty
				*/

	assertion(11, BASE);
	start
		GTI_NOT_PROVIDED_EXIT

		prep("The output from puts() when tested must be unbuffered.");
		start
			sv_flush();
			setbuf(stdout, (char *)0);
		finis

		prep("Establish PORT_TTY as the control terminal and PORT_LOOP.");
		prep("Set TOSTOP flag on PORT_TTY.");
		prep("Write some data to buffer.");
		start
			termios_prep(&tty_fildes, tty_termios_p, &loop_fildes,
								loop_termios_p);
			tty_termios_p->c_lflag |= TOSTOP;
			Ztcsetattr(tty_fildes, TCSANOW, tty_termios_p);
		finis

		FORK
			prep("Setup a new process group and catch SIGTTOU.");
			start
				Zsetpgid((pid_t)0, (pid_t)0);
				SIGNAL(SIGTTOU, sig_int);
			finis

			prep("Fork grandchild.");
			start
				sv_flush();
				pid = Zfork();
			finis

			if(pid == (pid_t)0) {
				prep("Wait while parent process exits.");
				start
					SLEEP(WAIT_FOR_TIME);
				finis

				testing("A call to puts() when stdout is the");
				testing("controlling terminal.");
				start
					continue_on(ERROR);
							/*
								Redirect stdout retaining journal file.
								Get stdout as controlling terminal.
								Lose automatic error checking.
							*/
					fd_stdout = Zdup(STDOUT_FILENO);
					close(STDOUT_FILENO);
					fd_sfd = dup(tty_fildes);

					errno = 0;
					ret_val = puts("Hi");
					s_errno = errno;

							/* Reset stdout back to journal file */
					close(fd_sfd);
					dup2(fd_stdout, STDOUT_FILENO);

					if(fd_sfd != STDOUT_FILENO)
						error("%s\n%s%d\n",
								"Expected file descriptor of STDOUT_FILENO",
								"Actual file descriptor was ",
								fd_sfd);
					if(ret_val >= 0)
						error("Call to puts() returned value of %d.",
									ret_val);
					if(s_errno != EIO)
						error("%s\n%s%s\n",
								"Expected errno value to be [EIO].",
								"Actual errno value was ",
								err_lookup(s_errno));
				finis

				done();
			}

			prep("Child now exits to obtain orphaned background");
			prep("process group for grandchild.");
			start
			finis
		KROF
		SLEEP(2*WAIT_FOR_TIME);	/* Ensure test completes before continuing. */

		cleanup("Close terminal devices.");
		start
			SIGNAL(SIGHUP, SIG_IGN);
			Zclose(tty_fildes);
			Zclose(loop_fildes);
			SLEEP(GTI_LINE_CLOSE_DELAY);
		finis
	finis

	done();
}
