#include	<pcts.h>

#include	<setjmp.h>
#include	<Zsignal.h>

#include	<sv_env.h>

extern	char **environ;
#define	ENV_EXP	"TZ=AAA"
#define	ENV_NEW	"TZ=BBB"

int main(void)
{
	sigjmp_buf	env;
	sigset_t	set;
	int			ret;

	setup(E_SIGLONGJMP);
	funct(E_SIGLONGJMP);

	assertion(4, BASE);
	start
		prep("Establish known set for process's signal mask.");
		start
			Zsigemptyset(&set);
			Zsigaddset(&set, SIGHUP);
			Zsigprocmask(SIG_SETMASK, &set, (sigset_t *)0);
		finis

		prep("Call sigsetjmp(env, savemask) with savemask argument non-zero.");
		start
			ret = sigsetjmp(env, 1);
			if(ret == 0) {
				prep("Change signal mask, then call siglongjmp().");
				start
					Zsigemptyset(&set);
					Zsigaddset(&set, SIGINT);
					Zsigprocmask(SIG_SETMASK, &set, (sigset_t *)0);
	
					siglongjmp(env, 2);
				finis
			}
		finis

		testing("Call to siglongjmp() restores the process's signal mask");
		testing("and the return value is \"val\".");
		start
			Zsigprocmask(SIG_SETMASK, (sigset_t *)0, &set);
			if(!sigismember(&set, SIGHUP))
				error("%s\n%s\n",
						"sigsetjmp() did not save process's signal mask.",
						"SIGHUP signal not set in process's signal mask.");

			if(sigismember(&set, SIGINT))
				error("%s\n%s\n",
						"sigsetjmp() did not save process's signal mask.",
						"SIGINT signal set in process's signal mask.");
			if(ret != 2)
				error("Returned value was %d", ret);
		finis
	finis

	assertion(5, BASE);
	start
		testing("Call to siglongjmp(env, 0) returns the value 1.");
		start
			ret = sigsetjmp(env, 1);
			if(ret == 0)
					siglongjmp(env, 0);
			if(ret != 1)
				error("Returned value was %d.", ret);
		finis
	finis

							/*	The environment to be saved varies for each
								implementation, is undefined in POSIX.1 and
								Standard C, and how it is saved in "env" is
								implementation defined.

								Since the PCTS cannot test what is saved
								in "env", it also cannot test that it is
								restored.  The PCTS can test that a
								siglongjmp() following a sigsetjmp() allows
								the program to continue from the previous
								sigsetjmp() call.
							*/
	assertion(6, BASE);
	start
		testing("Call to siglongjmp() following a call to sigsetjmp() allows");
		testing("the program to continue from the sigsetjmp() call as though");
		testing("sigsetjmp() returned a non-zero value");
		start
			if(!sigsetjmp(env, 1)) {
				siglongjmp(env, 2);
				error("Code following siglongjmp() call erroneously executed.");
			}
		finis
	finis

	assertion(7, BASE);
	start
		prep("Establish known set for process's signal mask.");
		start
			Zsigemptyset(&set);
			Zsigaddset(&set, SIGHUP);
			Zsigprocmask(SIG_SETMASK, &set, (sigset_t *)0);
		finis

		prep("Call sigsetjmp(env, 0).");
		start
			ret = sigsetjmp(env, 0);
			if(ret == 0) {
				prep("Change signal mask, then call siglongjmp().");
				start
					Zsigemptyset(&set);
					Zsigaddset(&set, SIGINT);
					Zsigprocmask(SIG_SETMASK, &set, (sigset_t *)0);
	
					siglongjmp(env, 2);
				finis
			}
		finis

		testing("Call to siglongjmp() leaves process's signal mask unchanged.");
		start
			Zsigprocmask(SIG_SETMASK, (sigset_t *)0, &set);
			if(sigismember(&set, SIGHUP))
				error("%s\n%s\n",
						"sigsetjmp() saved process's signal mask.",
						"SIGHUP signal set in process's signal mask.");

			if(!sigismember(&set, SIGINT))
				error("%s\n%s\n",
						"sigsetjmp() saved process's signal mask.",
						"SIGINT signal not set in process's signal mask.");
		finis
	finis

	done();
}
