#include	<pcts.h>

#include	<setjmp.h>
#include	<Zsignal.h>

#include	<sv_env.h>

int main(void)
{
	sigjmp_buf	env;
	sigset_t	set;
	int			ret;

	setup(E_SIGSETJMP);
	funct(E_SIGSETJMP);

							/*	The environment to be saved varies for each
								implementation, is undefined in POSIX.1 and
								Standard C, and how it is saved in "env" is
								implementation defined.

								Assertion #4 for sigsetjmp() as stated
								requires the PCTS to test that the calling
								environment is saved in "env" which is not
								portably testable.

								This should be an extended assertion as it was
								in Draft 12.0 of P1003.3.1.
							*/
	assertion(4, EXTENDED);

	assertion(5, BASE);
	start
		prep("Establish known set for process's signal mask.");
		start
			Zsigemptyset(&set);
			Zsigaddset(&set, SIGHUP);
			Zsigprocmask(SIG_SETMASK, &set, (sigset_t *)0);
		finis

		prep("Call sigsetjmp(env, savemask) with savemask argument non-zero.");
		start
			if(!sigsetjmp(env, 1)) {
				prep("Change signal mask, then call siglongjmp().");
				start
					Zsigemptyset(&set);
					Zsigaddset(&set, SIGINT);
					Zsigprocmask(SIG_SETMASK, &set, (sigset_t *)0);
	
					siglongjmp(env, 2);
				finis
			}
		finis

		testing("Call to sigsetjmp() saves the process's current");
		testing("signal mask.");
		start
			Zsigprocmask(SIG_SETMASK, (sigset_t *)0, &set);
			if(!sigismember(&set, SIGHUP))
				error("%s\n%s\n",
						"sigsetjmp() did not save process's signal mask.",
						"SIGHUP signal not set in process's signal mask.");

			if(sigismember(&set, SIGINT))
				error("%s\n%s\n",
						"sigsetjmp() did not save process's signal mask.",
						"SIGINT signal set in process's signal mask.");
		finis
	finis

	assertion(6, BASE);
	start
		testing("Call to sigsetjmp() returns with the value zero.");
		testing("Return from sigsetjmp() caused by invocation of");
		testing("siglongjmp() returns a non-zero value.");
		start
			ret = sigsetjmp(env, 0);
			if(ret == 0) {
				siglongjmp(env, 10);
			}

			if(ret != 10)
				error("Returned value was %d", ret);
		finis
	finis

	done();
}
