#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Ztimes.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<pcts_timer.h>

static	int vfprtargs(char *format, ...);
static	int	ret_val;
static	FILE	*sfd;


int	main(void)
{
	int tty_fildes, loop_fildes;
	struct termios	tty_termios, loop_termios, *tty_termios_p = &tty_termios,
							*loop_termios_p = &loop_termios;
	pid_t	pid;

	setup(E_VFPRINTF);
	funct(E_VFPRINTF);


								/*	This assertion test contains code similar
									to that of fputc(). */
	exit_on(ERROR);

				/*	When TOSTOP is set, the process is in an orphaned
					process group, the writing process is not ignoring
					or blocking SIGTTOU, then a vfprintf() to the controlling
					terminal sets errno to EIO.

	 				parent - new session, new controlling tty
	 			  	  |
	 				  child - new process group
	 			  	    |
	 		  		    great grandchild - created
					    |
					  child - exits
					    |
	 		  		    great grandchild - attempts write to control tty
				*/

	assertion(12, BASE);
	start
		GTI_NOT_PROVIDED_EXIT

		prep("Establish PORT_TTY as the control terminal and PORT_LOOP.");
		prep("Set TOSTOP flag on PORT_TTY.");
		prep("Write some data to buffer.");
		start
			termios_prep(&tty_fildes, tty_termios_p, &loop_fildes,
								loop_termios_p);
			tty_termios_p->c_lflag |= TOSTOP;
			Ztcsetattr(tty_fildes, TCSANOW, tty_termios_p);

			sfd = Zfdopen(tty_fildes, "r+");
			setbuf(sfd, (char *)0);
		finis

		FORK
			prep("Setup a new process group and catch SIGTTOU.");
			start
				Zsetpgid((pid_t)0, (pid_t)0);
				SIGNAL(SIGTTOU, sig_int);
			finis

			prep("Fork grandchild.");
			start
				sv_flush();
				pid = Zfork();
			finis

			if(pid == (pid_t)0) {
				prep("Wait while parent process exits.");
				start
					SLEEP(WAIT_FOR_TIME);
				finis

				testing("A vfprintf() to controlling terminal fails.");
				start
					SET_TIMEOUT
					vfprtargs("%s", "Hi\n");
					if(ret_val >= 0)
						error("Call to vfprintf() returned a value of %d.",
													ret_val);
					if(errno != EIO)
						error("%s\n%s%s\n",
							"Expected errno value to be [EIO].",
							"Actual errno value was ",
							err_lookup(errno));
					CLEAR_ALARM
				finis

				done();
			}

			prep("Child now exits to obtain orphaned background");
			prep("process group for grandchild.");
			start
			finis
		KROF
		SLEEP(2*WAIT_FOR_TIME);	/* Ensure test completes before continuing. */

		cleanup("Close terminal devices.");
		start
			SIGNAL(SIGHUP, SIG_IGN);
			Zclose(tty_fildes);
			Zclose(loop_fildes);
			SLEEP(GTI_LINE_CLOSE_DELAY);
		finis
	finis

	done();
}

static	int vfprtargs(char *format, ...)
{
	va_list	ap;

	va_start(ap, format);
	errno = 0;
	ret_val = vfprintf(sfd, format, ap);
	va_end(ap);
}
