#include	<pcts.h>

#include	<Zunistd.h>

#include	<sv_env.h>
#include	<sv_errno.h>

						/*	Define the absolute maximum number of
							supplementary groups an implementation
							may define above {NGROUPS_MAX}.
						*/
#define	MAX_SGA_NGROUPS_MAX	200
						/*	Define the minimu number of supplementary
							groups a FIPS 151-2 conforming implementation
							may define.
						*/
#define	MIN_SGA_NGROUPS_MAX	8


int	main(void)
{
	gid_t	sup_grps[NGROUPS_MAX +MAX_SGA_NGROUPS_MAX];
	gid_t	gr_lst[NGROUPS_MAX +MAX_SGA_NGROUPS_MAX];
	int		ngrps, n_grps, i, j;

	setup(E_GETGROUPS);
	funct(E_GETGROUPS);

	assertion(4, BASE);
	start
		break_on(ERROR);
		n_grps = Zsysconf(_SC_NGROUPS_MAX);
		if(n_grps >= MIN_SGA_NGROUPS_MAX) {
			prep("Check specification of SUPPLEMENTARY_GROUPS in \"config\".");
			start
				char	*p_sg;

				/* Place "config" values in array */
				for(i=0; i<(NGROUPS_MAX +MAX_SGA_NGROUPS_MAX); i++) {
					if(i==0)
						p_sg = strtok(SUPPLEMENTARY_GROUPS, ",");
					else
						p_sg = strtok(NULL, ",");
					if(p_sg == NULL)						/* test for NULL */
						break;
				   	sup_grps[i] = (gid_t) atoi(p_sg);
				}
	
				if(i != n_grps)
					error("%s\n%s%d.\n%s%d.",
						"Initialization of \"config\" parameter incorrect.",
						"Required supplementary groups is ",
						n_grps,
						"The number specified in SUPPLEMENTARY_GROUPS is ",
						i);

				ngrps = Zgetgroups(n_grps, gr_lst);
				if(ngrps != n_grps)
					error("%s\n%s%d\n%s%d\n",
						"getgroups() returned incorrect count.",
						"Expected count of ",
						n_grps,
						"Actual count is ",
						ngrps);

			finis

			testing("Number of supplementary groups are >= to FIPS 151-2");
			testing("required value.");
			start
				if(FIPS_151_NGROUPS_MAX > ngrps)
					error("%s\n%s%d\n",
							"FIPS 151-2 requires {NGROUPS_MAX} to be >= 8.",
							"Actual value for tested process is ",
							ngrps);
			finis

			testing("When \"gidsetsize\" is equal to number of supplementary");
			testing("group IDs of the process, getgroups() places the list of");
			testing("supplementary group IDs in an array and returns number");
			testing("of supplementary group IDs placed into the array.");
			start
				for(i=0; i<ngrps; i++) {
					for(j=0; j<ngrps; j++) {
						if(gr_lst[i] == getegid())
							break;
						if(gr_lst[i] == sup_grps[j])
							break;
					}

					if(j == ngrps)
						error("%s\"%d\"%s\n%s%s\n",
								"Group ID ",
								gr_lst[i],
								" was returned by getgroups().",
								"This group ID was not specifed in config ",
								"parameter SUPPLEMENTARY_GROUPS.");
				}
			finis
	
			testing("Supplementary groups returned are unique.");
			start
				for(i=0; i<ngrps -1; i++) {
					for(j=i +1; j<ngrps; j++) {
						if(gr_lst[i] == gr_lst[j]) {
							error("Duplicate group ID \"%d\" returned",
								gr_lst[i]);
						}
					}
				}
			finis

			testing("When \"gidsetsize\" is greater than the number of");
			testing("supplementary group IDs of the process, getgroups()");
			testing("places the list of supplementary group IDs in an array");
			testing("and returns the number of supplementary group IDs");
			testing("placed into the array.");
			start
				expecting(ngrps);
				(void) Zgetgroups(n_grps +5, gr_lst);

				for(i=0; i<ngrps; i++) {
					for(j=0; j<ngrps; j++) {
						if(gr_lst[i] == getegid())
							break;
						if(gr_lst[i] == sup_grps[j])
							break;
					}

					if(j == ngrps)
						error("%s\"%d\"%s\n%s%s\n",
								"Group ID ",
								gr_lst[i],
								" was returned by getgroups().",
								"This group ID was not specifed in config ",
								"parameter SUPPLEMENTARY_GROUPS.");
				}
			finis
		}
		else {
			exit_on(ERROR); 				/* No reason to continue. */
			error("{NGROUPS_MAX} is %d, FIPS 151-2 requires 8 minimum.",n_grps);
		}
	finis


				/*	When \"gidsetsize\" is zero, getgroups() returns the
					number of supplementary group IDs associated with the
					calling process without modifying the array pointed to
					by the \"grouplist\" argument.
				*/
	assertion(5, BASE);
	start
		break_on(ERROR);
		testing("When \"gidsetsize\" is zero, getgroups() returns proper value.");
		start
			for(i=0; i<n_grps; i++)
				gr_lst[i] = (gid_t)0;
			if((ngrps = Zgetgroups(0, gr_lst)) != n_grps) {
				error("%s\n%s%d\n%s%d.\n",
					"getgroups() returned incorrect count.",
					"Expected count of ",
					n_grps,
					"Actual count is ",
					ngrps);
			}
		finis

		testing("\"grouplist\" not updated.");
		start
			for(i=0; i<n_grps; i++) {
				if(gr_lst[i] != (gid_t) 0)
					error("Array entry %d updated, value %d",
						i, gr_lst[i]);
			}
		finis
	finis

	assertion(6, BASE);
	start
		break_on(ERROR);
		testing("Call to getgroups(gidsetsize, grouplist) returns a value <=");
		testing("to that returned by sysconf(_SC_NGROUPS_MAX).");
		start
			int	grps;

			grps = Zgetgroups(NGROUPS_MAX +MAX_SGA_NGROUPS_MAX, gr_lst);
			if(grps > n_grps)
				error("%s%d\n%s%d\n",
					"Value from getgroups() was ",
					grps,
					"Value from sysconf() was ",
					n_grps);
		finis
	finis

	assertion(7, BASE);
	start
		testing("[EINVAL] returned by getgroups() when \"gidsetsize\"");
		testing("is less than the number of supplementary groups IDs.");
		start
			expecting(-1);
			expecting(EINVAL);
			(void) Zgetgroups(n_grps -1, gr_lst);
		finis
	finis
	
	done();
}
