#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zsignal.h>

#include	<sv_env.h>
#include	<pcts_timer.h>


int	main(void)
{
	pid_t	pid, c_pid, c2_pid, p_pgrp;

	setup(E_SETPGID);
	funct(E_SETPGID);

		/*	When:
				pid is not a session leader,
				pgid and pid are not equal,
				pgid matches the process group ID of a process in the same
					session as the calling process,
				pid matches the process ID of a process in the same session
					as the calling process,
				pid matches the process ID of a child process of the calling
					process which has not executed an "exec" function,
			then call to setpgid(pid, pgid) sets the indicated process's
			group ID to pgid and returns a value of zero.
		*/
	assertion(4, BASE);
	start
		ENABLE_PSYNC

		prep("Get current process group ID then fork a child.");
		start
			p_pgrp = Zgetpgrp();
			c_pid = Zfork();
		finis

		if(c_pid == (pid_t)0) {
			prep("Create a new process group then wait on parent to complete.");
			start
				c_pid = Zgetpid();
				(void) Zsetpgid((pid_t)0, c_pid);
				if(getpgrp() != c_pid)
					error("%s%ld.\n%s%ld.",
							"Expected process group ID of ",
							(long) c_pid,
							"Actual process group ID is ",
							(long) getpgrp());
				sv_flush();
				SEND_PSYNC
				/* Wait for parent to complete tests */
				SET_TIMEOUT
					WAIT_PSYNCA
				CLEAR_ALARM
			finis

			testing("Process group ID of child is now the same as the");
			testing("process group ID of parent.");
			start
				if(getpgrp() != p_pgrp)
					error("%s%ld.\n%s%ld.",
							"Expected process group ID of ",
							(long) p_pgrp,
							"Actual process group ID is ",
							(long) getpgrp());
			finis

			done();
		}

		SET_TIMEOUT
			WAIT_PSYNC
		CLEAR_ALARM

		testing("Set process group of child back to parent's process group.");
		testing("Call to setpgid() returns a value of zero.");
		start
			sv_flush();
			expecting(0);
			(void) Zsetpgid(c_pid, getpgrp());
		finis

		SEND_PSYNCA
		WAIT_FOR_PID(c_pid);
		DISABLE_PSYNC
	finis


		/*	When:
				pid is not a session leader,
				pgid and pid are equal,
				pid matches the process ID of a process in the same session
					as the calling process,
				pid matches the process ID of a child process of the calling
					process which has not executed an "exec" function,
			then call to setpgid(pid, pid) sets the indicated process's
			group ID to pid and returns a value of zero.
		*/
	assertion(5, BASE);
	start
		ENABLE_PSYNC

		prep("Fork a child.");
		start
			c_pid = Zfork();
		finis

		if(c_pid == (pid_t)0) {
			prep("Wait on parent to change process group ID of this");
			prep("child's process.");
			start
				c_pid = Zgetpid();
				sv_flush();
				SEND_PSYNC

				SET_TIMEOUT
					WAIT_PSYNCA
				CLEAR_ALARM
			finis

			testing("Child process group ID now matches child's process ID.");
			start
				if(getpgrp() != c_pid)
					error("%s%ld.\n%s%ld.",
							"Expected process group ID of ",
							(long) c_pid,
							"Actual process group ID is ",
							(long) getpgrp());
			finis

			done();
		}

		SET_TIMEOUT
			WAIT_PSYNC
		CLEAR_ALARM

		testing("Create new process group for child process.");
		testing("Call to setpgid() returns a value of zero.");
		start
			sv_flush();
			expecting(0);
			(void) Zsetpgid(c_pid, c_pid);
		finis

		SEND_PSYNCA
		WAIT_FOR_PID(c_pid);
		DISABLE_PSYNC
	finis

		/*	When:
				pid is not a session leader,
				pgid matches the process group ID of a process in the same
					session as the calling process,
				pid is equal to zero,
				pid matches the process ID of a child process of the calling
					process which has not executed an "exec" function,
			then call to setpgid(0, pgid) sets the calling process's
			group ID to pgid and returns a value of zero.
		*/
	assertion(6, BASE);
	start
		ENABLE_PSYNC

		prep("Fork a child.");
		start
			c_pid = Zfork();
		finis

		if(c_pid == (pid_t)0) {
			prep("Create a new process group then wait on parent to complete.");
			start
				c_pid = Zgetpid();
				(void) Zsetpgid(c_pid, c_pid);
				if(getpgrp() != c_pid)
					error("%s%ld.\n%s%ld.",
							"Expected process group ID of ",
							(long) c_pid,
							"Actual process group ID is ",
							(long) getpgrp());
				sv_flush();
				SEND_PSYNC
				/* Wait for parent to complete test. */
				SET_TIMEOUT
					WAIT_PSYNCA
				CLEAR_ALARM
			finis

			done();
		}

		SET_TIMEOUT
			WAIT_PSYNC
		CLEAR_ALARM

		prep("Fork a second child.");
		start
			c2_pid = Zfork();
		finis

		if(c2_pid == (pid_t)0) {
			testing("Change child's process group.");
			testing("Call to setpgid() returns a value of zero.");
			start
				expecting((pid_t)0);
				(void) Zsetpgid((pid_t)0, c_pid);
				if(getpgrp() != c_pid)
					error("%s%ld.\n%s%ld.",
							"Expected process group ID of ",
							(long) c_pid,
							"Actual process group ID is ",
							(long) getpgrp());
			finis

			done();
		}
		WAIT_FOR_PID(c2_pid);

		SEND_PSYNCA
		WAIT_FOR_PID(c_pid);
		DISABLE_PSYNC
	finis

		/*	When:
				pid is not a session leader,
				pid matches the process ID of a process in the same session
					as the calling process,
				pgid is equal to zero,
				pid matches the process ID of a child process of the calling
					process which has not executed an "exec" function,
			then call to setpgid(pid, 0) sets the target process's
			group ID to pid and returns a value of zero.
		*/
	assertion(7, BASE);
	start
		ENABLE_PSYNC

		prep("Fork a child.");
		start
			c_pid = Zfork();
		finis

		if(c_pid == (pid_t)0) {
			prep("Wait on parent to change process group ID of this");
			prep("child's process.");
			start
				sv_flush();
				c_pid = Zgetpid();
				SEND_PSYNC
				SET_TIMEOUT
					WAIT_PSYNCA
				CLEAR_ALARM
			finis

			testing("Child process group ID now matches parent's process ID.");
			start
				p_pgrp = Zgetpgrp();
				if(p_pgrp != c_pid)
					error("%s%ld.\n%s%ld.",
							"Expected process group ID of ",
							(long) c_pid,
							"Actual process group ID is ",
							(long) p_pgrp);
			finis

			done();
		}

		SET_TIMEOUT
			WAIT_PSYNC
		CLEAR_ALARM

		testing("Create new process group for child process.");
		testing("Call to setpgid() returns a value of zero.");
		start
			sv_flush();
			expecting(0);
			(void) Zsetpgid(c_pid, (pid_t)0);
		finis

		SEND_PSYNCA
		WAIT_FOR_PID(c_pid);
		DISABLE_PSYNC
	finis

		/*	When:
				pid is not a session leader,
				pgid and pid are equal to zero,
			then call to setpgid(0, 0) sets the calling process's
			group ID equal to its process ID and returns a value of zero.
		*/
	assertion(8, BASE);
	start
		prep("Get current process group ID then fork a child.");
		start
			p_pgrp = Zgetpgrp();
			c_pid = Zfork();
		finis

		testing("Call to setpgid(0,0) returns a value of zero.");
		testing("Process group ID now matches process ID.");
		start
			expecting(0);
			(void) Zsetpgid((pid_t)0, (pid_t)0);

			pid = Zgetpid();
			p_pgrp = Zgetpgrp();
			if(p_pgrp != pid)
				error("%s%ld.\n%s%ld.",
						"Expected process group ID of ",
						(long) pid,
						"Actual process group ID is ",
						(long) p_pgrp);
		finis
	finis

	done();
}
