#include	<pcts.h>

#include	<Zunistd.h>

#include	<sv_env.h>
#include	<sv_errno.h>

#define	FILENAME	"./4.3.3_09x"


int	main(void)
{
	pid_t	pid, c_pid, cur_pgrp;

	setup(E_SETPGID);
	funct(E_SETPGID);

	assertion(9, BASE);
	start
		break_on(ERROR);

		prep("Fork a child process.");
		start
			c_pid = Zfork();
		finis

		if(c_pid == (pid_t)0) {
			prep("Perform an execl().");
			start
				sv_flush();
				Zexecl(FILENAME, FILENAME, (char *)0);
			finis

			done();
		}

		SLEEP(WAIT_FOR_TIME);

		testing("[EACCES]");
		start
			cur_pgrp = Zgetpgrp();
			expecting(EACCES);
			Zsetpgid(c_pid, c_pid);
		finis

		testing("Process group ID of calling process has not changed.");
		start
			if(cur_pgrp != Zgetpgrp())
				error("%s\n%s%ld\n%s%ld",
						"Process group ID has changed.",
						"Expected process group ID of ",
						(long) cur_pgrp,
						"Actual process group ID is ",
						(long) Zgetpgrp());
		finis

		WAIT_FOR_PID(c_pid);
	finis

	assertion(10, BASE);
	start
		FORK
			testing("Call to setpgid() with \"pgid\" less than zero.");
			start
				cur_pgrp = Zgetpgrp();
				expecting(EINVAL);
				(void) Zsetpgid((pid_t)0, (pid_t)-1);
			finis

			testing("Process group ID of calling process has not changed.");
			start
				if(cur_pgrp != Zgetpgrp())
					error("%s\n%s%ld\n%s%ld",
							"Process group ID has changed.",
							"Expected process group ID of ",
							(long) cur_pgrp,
							"Actual process group ID is ",
							(long) Zgetpgrp());
			finis
		KROF
	finis

						/* [EINVAL], value of "pgid" not supported. */
	assertion(11, BASE);
	start
		if(EINVAL_SETPGID) {
			FORK
				testing("Call to setpgid() when \"pgid\" is an invalid value");
				testing("with value greater than zero.");
				start
					cur_pgrp = Zgetpgrp();
					expecting(EINVAL);
					(void) Zsetpgid((pid_t)0, EINVAL_SETPGID);
				finis

				testing("Process group ID of calling process has not changed.");
				start
					if(cur_pgrp != Zgetpgrp())
						error("%s\n%s%ld\n%s%ld",
								"Process group ID has changed.",
								"Expected process group ID of ",
								(long) cur_pgrp,
								"Actual process group ID is ",
								(long) Zgetpgrp());
				finis
			KROF
		}
		else
			not_implemented("[EINVAL] is not supported for positive \'pgid\'s");
	finis

	assertion(12, NOT_APPLICABLE);

	assertion(13, BASE);
	start
		pid_t sl_pid, psl_pid, psl_pgrp;

   /*
   		The following illustrates the three setpgid() calls where the value
		of the pid argument matches the process ID of a session leader.

		In call #1, the required errno is [EPERM].
		In calls #2 and #3 the errno can be reported as [EPERM] or [ESRCH].
	   
              session leader process
                 pid  = psl_pid
                 prgp = psl_pgrp
                      |
                      |____________________
                                          |
                                      session leader process
                                        pid  = sl_pid
                                        pgrp = sl_pgrp
                                          |         |
                                          |         |
                                          |     setpgid(sl_pid, psl_pgrp)  (1)
                                          |     setpgid(psl_pid, sl_pgrp)  (2)
                                          |
                                          |
                                        fork
                                          |
                                          |
                            setpgid(sl_pid, psl_pgrp)   (3)

  		This PCTS tests all three cases for this assertion test.
  */

		FORK
			prep("Establish the child process as a session leader.");
			start
				Zsetsid();
				psl_pid = Zgetpid();
				psl_pgrp = Zgetpgrp();
			finis

			FORK
				prep("Establish the grandchild process as a session leader.");
				start
					Zsetsid();
					sl_pid = Zgetpid();
				finis

				testing("Call setpgid() when the process group for \"pid\" is");
				testing("the current session leader for the calling process.");
				start
					cur_pgrp = Zgetpgrp();
					expecting(-1);
					expecting(EPERM);
					(void) Zsetpgid(sl_pid, psl_pgrp);	/* Case #1 */
				finis

				testing("Process group ID of calling process is not changed.");
				start
					if(cur_pgrp != Zgetpgrp())
						error("%s\n%s%ld\n%s%ld",
								"Process group ID has changed.",
								"Expected process group ID of ",
								(long) cur_pgrp,
								"Actual process group ID is ",
								(long) Zgetpgrp());
				finis

				testing("Call setpgid() when the process group for \"pid\" is");
				testing("a session leader.");
				start
					cur_pgrp = Zgetpgrp();
					expecting(-1);
					(void) Zsetpgid(psl_pid, cur_pgrp);		/* Case #2. */
					if( ! ((errno == EPERM) || (errno == ESRCH)) )
						error("%s\n%s%s.",
							"Expected errno of [EPERM] or [ESRCH].",
							"Actual errno returned was ",
							err_lookup(errno));
				finis

				FORK
					testing("Call setpgid() when the process group for");
					testing("\"pid\" is the current session leader for the");
					testing("calling process.");
					start
						cur_pgrp = Zgetpgrp();
						expecting(-1);
						(void) Zsetpgid(sl_pid, psl_pgrp);	/* Case #3. */
						if( ! ((errno == EPERM) || (errno == ESRCH)) )
							error("%s\n%s%s.",
								"Expected errno of [EPERM] or [ESRCH].",
								"Actual errno returned was ",
								err_lookup(errno));

					finis

					testing("Process group ID of calling process not changed.");
					start
						if(cur_pgrp != Zgetpgrp())
							error("%s\n%s%ld\n%s%ld",
									"Process group ID has changed.",
									"Expected process group ID of ",
									(long) cur_pgrp,
									"Actual process group ID is ",
									(long) Zgetpgrp());
					finis
				KROF
			KROF

			testing("Process group ID of session leader was not changed.");
			start
				if(psl_pgrp != Zgetpgrp())
					error("%s\n%s%ld\n%s%ld",
							"Process group ID has changed.",
							"Expected process group ID of ",
							(long) psl_pgrp,
							"Actual process group ID is ",
							(long) Zgetpgrp());
			finis
		KROF
	finis

	assertion(15, BASE);
	start
		FORK
			prep("Establish the child process as a session leader.");
			start
				Zsetsid();
			finis

			FORK
				testing("setpgid() when \"pgid\" does not match any process");
				testing("groups in the session nor the \"pid\" argument.");
				start
					cur_pgrp = Zgetpgrp();
					c_pid = Zgetpid();
					expecting(EPERM);
					(void) Zsetpgid(c_pid, (pid_t)(c_pid +1));
				finis

				testing("Process group ID of calling process has not changed.");
				start
					if(cur_pgrp != Zgetpgrp())
						error("%s\n%s%ld\n%s%ld",
								"Process group ID has changed.",
								"Expected process group ID of ",
								(long) cur_pgrp,
								"Actual process group ID is ",
								(long) Zgetpgrp());
				finis
			KROF
		KROF
	finis

	assertion(16, BASE);
	start
		FORK
			prep("Exit child.");
			start
			finis
		KROF

		testing("setpgid(exited_process, 0)");
		start
			cur_pgrp = Zgetpgrp();
			expecting(ESRCH);
			Zsetpgid(pid, (pid_t)0);
		finis

		testing("Process group ID of calling process has not changed.");
		start
			if(cur_pgrp != Zgetpgrp())
				error("%s\n%s%ld\n%s%ld",
						"Process group ID has changed.",
						"Expected process group ID of ",
						(long) cur_pgrp,
						"Actual process group ID is ",
						(long) Zgetpgrp());
		finis
	finis

	done();
}
