#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Zstat.h>
#include	<sys/Ztimes.h>
#include	<Zfcntl.h>

#include	<sv_env.h>

static	int		e_index;

char	a_cwd[_POSIX_PATH_MAX +1];
char	path[_POSIX_PATH_MAX +1];
void	chk_times(clock_t *, clock_t *, char *);


int	main(int argc, char *argv[])
{
	e_index = atoi(*argv);
	setup(e_index);
	ass_nu = 47;

	testing("Process ID inherited.");
	start
		if(atol(*++argv) != (long) getpid())
			error("%s%ld\n%s%ld",
					"Process ID inherited is ",
					(long) getpid(),
					"Expected: ",
					atol(*argv));
	finis

	testing("Parent process ID inherited.");
	start
		if(atol(*++argv) != (long) getppid())
			error("%s%ld\n%s%ld",
					"Parent process ID inherited is ",
					(long) getppid(),
					"Expected: ",
					atol(*argv));
	finis

	testing("Process group and session membership inherited.");
	start
		if(atol(*++argv) != (long) getpgrp())
			error("%s%ld\n%s%ld",
					"Process group ID inherited is ",
					(long) getpgrp(),
					"Expected: ",
					atol(*argv));
	finis

	testing("Real user ID inherited.");
	start
		if(atol(*++argv) != (long) getuid())
			error("%s%lu\n%s%lu",
					"Real user ID inherited is ",
					(unsigned long) getuid(),
					"Expected: ",
					(unsigned long) atol(*argv));
	finis

	testing("Group ID inherited.");
	start
		if(atol(*++argv) != (long) getgid())
			error("%s%lu\n%s%lu",
					"Group ID inherited is ",
					(unsigned long) getgid(),
					"Expected: ",
					(unsigned long) atol(*argv));
	finis


	testing("Supplementary groups inherited.");
	start
		int		i, fd;
		gid_t	*g_beg, s_gid;
		int		n_grps;

		n_grps = Zgetgroups(0, (gid_t *)0);		/* Get number of groups */
		g_beg = (gid_t *) Zmalloc((unsigned int)(sizeof(gid_t) * n_grps));
		(void) Zgetgroups(n_grps, g_beg);

		Zstrcpy(path, temp_path());
		Zstrcat(path, "sup_grps");
		fd = Zopen(path, O_RDONLY);
		continue_on(ERROR);
		i = 0;
		while(read(fd, (char *)&s_gid, sizeof(gid_t))) {
			if(s_gid != g_beg[i++])
				error("%s\n%s%lu\n%s%lu",
						"Supplementary group ID not inherited.",
						"Expected inheritance of group:	",
						(unsigned long) s_gid,
						"Actual group located in its place is:	",
						(unsigned long) g_beg[i]);
		}
		Zclose(fd);
	finis
		
	testing("Time left on alarm clock signal.");
	start
		unsigned int	tl_alm;

		if((tl_alm = alarm(0)) == (unsigned int) 0)
			error("%s\n%s%d",
				"Expecting time left on alarm signal.",
				"Actual time left was:	",
				tl_alm);
	finis

	testing("Current working directory and root directory inherited.");
	start
		if(strcmp(getcwd(a_cwd, _POSIX_PATH_MAX), *++argv))
			error("%s\n%s\"%s\"\n%s\"%s\"",
				"Proper directories not inherited.",
				"Expected:	",
				*argv,
				"Actual:	",
				a_cwd);
	finis

	testing("File mode creation mask inherited.");
	start
		mode_t	val_umask;

		val_umask = umask((mode_t)0);
		(void) umask(val_umask);			/* Reset */
		if(atol(*++argv) != (long) val_umask)
			error("%s\n%s\"%ld\"\n%s\"%d\"",
				"File mode creation mask not inherited.",
				"Expected:	",
				atol(*argv),
				"Actual:	",
				val_umask);
	finis

	testing("Process signal mask inherited.");
	start
		sigset_t	oset;

		Zsigprocmask(SIG_BLOCK,(sigset_t *)0, &oset);
		++argv;
		if( ! sigismember(&oset, atoi(*argv)) )
			error("%s\n%s\"%d\"\n%s",
				"Process signal mask not inherited.",
				"Expected signal number set:	",
				atoi(*argv),
				"Actual signal number was not set.");
	finis

	testing("Pending signals are inherited.");
	start
		sigset_t	set;

		Zsigpending(&set);
		if( ! sigismember(&set, atoi(*argv)) )
			error("%s\n%s\"%d\"\n%s",
				"Pending signal not inherited.",
				"Expected signal number pending:	",
				atoi(*argv),
				"Actual signal number was not pending.");
	finis

	testing("Current process times are inherited.");
	start
		int	fd;
		struct	tms	c_buffer, p_buffer;

		Ztimes(&c_buffer);
		Zstrcpy(path, temp_path());
		Zstrcat(path, "times");

		fd = Zopen(path, O_RDONLY);
		Zread(fd, (char *)&p_buffer.tms_utime, sizeof(clock_t));
		chk_times(&p_buffer.tms_utime, &c_buffer.tms_utime, "tms_utime");
		Zread(fd, (char *)&p_buffer.tms_stime, sizeof(clock_t));
		chk_times(&p_buffer.tms_stime, &c_buffer.tms_stime, "tms_stime");
		Zread(fd, (char *)&p_buffer.tms_cutime, sizeof(clock_t));
		chk_times(&p_buffer.tms_cutime, &c_buffer.tms_cutime, "tms_cutime");
		Zread(fd, (char *)&p_buffer.tms_cstime, sizeof(clock_t));
		chk_times(&p_buffer.tms_cstime, &c_buffer.tms_cstime, "tms_cstime");
		Zclose(fd);
	finis
		
	done();
}


void	chk_times(clock_t *p_t, clock_t *c_t, char *str)
{
	if(*p_t > *c_t)
		error("%s %s\n%s%lu\n%s%lu",
				str,
				"not inherited.",
				"Expected time >=:	",
				(unsigned long)*p_t,
				"Actual time obtained:	",
				(unsigned long)*c_t);
}
