#include	<pcts.h>

#include	<Zunistd.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<pcts_timer.h>
#include	"./kill.h"

/* This module has S_ISUID set to ANOTHER_UID.  */


int	main(int argc, char *argv[])
{
	pid_t	pid;
	int		sig;
	char	path[_POSIX_PATH_MAX];

	setup(E_KILL);
	ass_nu = 7;

	exit_on(ERROR);
	prep("Ensure sending process user IDs are set properly.");
	prep("Real user ID is UID and effective user ID is ANOTHER_UID.");
	start
		if(getuid() !=  UID)
			error("%s\n%s%lu.",
					"Expected real user ID to be UID.",
					"Actual real user ID is ",
					(unsigned long) getuid());
		if(geteuid() !=  ANOTHER_UID)
			error("%s\n%s%lu.",
					"Expected effective user ID to be ANOTHER_UID.",
					"Actual effective user ID is ",
					(unsigned long) geteuid());
	finis

	prep("Get signal to send child.");
	start
		sig = atoi(*++argv);
	finis

	SET_TIMEOUT
		prep("Wait for receiving process to be ready and get its pid.");
		start
			int	fd;

			Zstrcpy(path, temp_path());
			Zstrcat(path, FILENAME);
			while((fd = open(path, O_RDONLY)) == -1)
				sleep(1);
			Zread(fd, (char *)&pid, (unsigned)sizeof(pid_t));
			Zclose(fd);
			Zunlink(path);
		finis
	CLEAR_ALARM

	testing("When the real user ID of the sending process matches");
	testing("the real user ID of the receiving process a %s", sig_lookup(sig));
	testing("signal can be sent via kill() to the process (%ld).", (long)pid);
	start
		int		stat_val;
		pid_t	r_pid;

		Zkill(pid, sig);

		if((r_pid = Zwait(&stat_val)) != pid)
			error("%s%s.\n%s%ld\n%s%ld",
					"Incorrect child pid returned while testing ",
					sig_lookup(sig),
					"Expected pid of ",
					(long) pid,
					"Actual pid returned ",
					(long) r_pid);
		if( ! WIFSIGNALED(stat_val)) 
			error("Process not terminated by receipt of signal.");
		else {
			if(sig != WTERMSIG(stat_val))
				error("Process terminated by signal %s\n",
								sig_lookup(WTERMSIG(stat_val)) );
		}
	finis

	done();
}
