#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Zstat.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_sigs.h>
#include	<pcts_timer.h>


int main(void)
{
	pid_t 		pid;
	int			stat_val;
	sigset_t	set;
	int			i;

    setup(E_SIGNAL_CON);
    funct(E_SIGNAL_CON);

	assertion(23, BASE);
	start
		FORK
								/* Testing blocking of a signal. */
			prep("Set process signal mask to block a signal.");
			start
				SIGNAL(SIGUSR1, sig_int);
				SIGBLOCK(set, SIGUSR1);
			finis
			
			prep("Send the signal that was blocked to this child process"); 
			prep("to obtain a pending signal.");
			start
				Zkill(getpid(), SIGUSR1);
				SLEEP(WAIT_FOR_TIME);
			finis

			testing("Signal SIGUSR1 pending.");
			start
				Zsigpending(&set);
				if( ! sigismember(&set, SIGUSR1) )
					error("%s\n%s\n%s",
						"Pending signal not reported.",
						"Expected pending signal SIGUSR1.",
						"Actual signal SIGUSR1 not pending.");
			finis

			prep("Unblock the pending signal.");
			start
				SIGUNBLOCK(set, SIGUSR1);
			finis

			testing("Signal SIGUSR1 no longer pending.");
			start
				Zsigpending(&set);
				if(sigismember(&set, SIGUSR1))
					error("Signal SIGUSR1 still pending.");
			finis

			prep("Set process signal mask to block a signal.");
			start
				SIGBLOCK(set, SIGUSR1);
			finis
			
			prep("Send the blocked signal to this process."); 
			start
				Zkill(getpid(), SIGUSR1);
				SLEEP(WAIT_FOR_TIME);
			finis

			testing("Check signal SIGUSR1 pending.");
			start
				Zsigpending(&set);
				if( ! sigismember(&set, SIGUSR1) )
					error("%s\n%s\n%s",
						"Pending signal not reported.",
						"Expected pending signal SIGUSR1.",
						"Actual signal SIGUSR1 not pending.");
			finis


									/* Testing ignoring a signal. */
			prep("Set sigaction to ignore SIGUSR1 signal.");
			start
				SIGNAL(SIGUSR1, SIG_IGN);
			finis

			testing("Signal SIGUSR1 no longer pending.");
			start
				Zsigpending(&set);
				if(sigismember(&set, SIGUSR1))
					error("Signal SIGUSR1 still pending.");
			finis
		KROF
	finis

	assertion(24, BASE);
	start
		FORK
			prep("Set process signal mask to block a signal.");
			start
				SIGBLOCK(set, SIGUSR1);
			finis
			
			prep("Set action to ignore signal.");
			start
				SIGNAL(SIGUSR1, SIG_IGN);
			finis

			prep("Generate the signal for the child process.");
			start
				Zkill(getpid(), SIGUSR1);
				SLEEP(WAIT_FOR_TIME);
			finis

			prep("Unblock signal.");
			start
				SIGUNBLOCK(set, SIGUSR1);
			finis

			prep("Reset action to ignore signal.");
			start
				SIGNAL(SIGUSR1, SIG_IGN);
			finis

			testing("Signal is not pending.");
			start
				Zsigpending(&set);
				if( sigismember(&set, SIGUSR1) )
					error("Signal SIGUSR1 is pending.");
			finis
		KROF
	finis

	assertion(25, BASE);
	start
						/*	The actual requirements of this assertion are
							not definitive.  The PCTS assumption is that the 
							tests of the sigaction() assertions satisfy
							this assertion.
						*/
		testing("See sigaction() assertion tests.");
		start
		finis
	finis

						/*
							When any stop signal (SIGSTOP, SIGTSTP, SIGTTIN,
							and SIGTTOU) is generated for a process, then all
							pending SIGCONT signals for that process are 
							discarded.
						*/
	assertion(26, BASE);
	start
		for(i=NSIGS_STOP; i<NSIGS; i++) {
			prep("Fork a child process.");
			start
				sv_flush();
				pid = Zfork();
			finis

			if(pid == (pid_t)0) {
				prep("Block and set action to catch SIGCONT signal.");
				prep("Issue signal SIGCONT twice.");
				start
					sig_rcvd = 0;
					SIGNAL(SIGCONT, sig_int);
					if(sigs[i].sig != SIGSTOP) {
						SIGNAL(sigs[i].sig, SIG_DFL);
					}
					SIGBLOCK(set, SIGCONT);
					Zkill(getpid(), SIGCONT);
					Zkill(getpid(), SIGCONT);
					SLEEP(WAIT_FOR_TIME);
				finis
			
				prep("Verify signal SIGCONT is pending.");
				start
					Zsigpending(&set);
					if( ! sigismember(&set, SIGCONT) )
						error("%s\n%s",
							"Expected signal SIGCONT pending.",
							"Actually signal SIGCONT is not pending.");
				finis

				prep("Deliver signal %s to stop this process.",
											sigs[i].signame);
				start
									/* Discards pending SIGCONT signals. */
					Zkill(getpid(), sigs[i].sig);
					SLEEP(WAIT_FOR_TIME);	/* Wait for kill() processed. */
				finis

				prep("Unblock pending SIGCONT signal generated by parent.");
				start
					Zsigemptyset(&set);
					Zsigaddset(&set, SIGCONT);
					Zsigprocmask(SIG_UNBLOCK, &set, (sigset_t *)0);
				finis

				testing("SIGCONT signal no longer pending.");
				start
					Zsigpending(&set);
					if(sigismember(&set, SIGCONT) )
						error("%s\n%s",
							"Expected signal SIGCONT not pending.",
							"Actually signal SIGCONT is pending.");
				finis

				testing("Only one SIGCONT signal delivered.");
				start
					if(sig_rcvd != 1)
						error("%s\n%s%d",
							"Expected one SIGCONT signal to be delivered.",
							"Actually delivery of SIGCONT signals was ",
							sig_rcvd);
				finis
		
				done();
			}

			prep("Wait for child process to be stopped.");
			start
				SET_ALARM(TIMEOUT)
					Zwaitpid(pid, &stat_val, WUNTRACED);
					if( ! WIFSTOPPED(stat_val) )
						error("%s\n%s",
							"Expected child process to be stopped.",
							"Actually child process was terminated.");
				CLEAR_ALARM
			finis

			prep("Send SIGCONT to continue child process.");
			prep("The SIGCONT signal also becomes pending.");
			start
				sv_flush();
				Zkill(pid, SIGCONT);
			finis

			WAIT_FOR_PID(pid);
		}	
	finis

	done();
}
