#include	<pcts.h>

#include	<Zunistd.h>
#include	<sys/Zstat.h>
#include	<Zfcntl.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<sv_sigs.h>
#include	<pcts_timer.h>

				/* Testing:  functions() stopped/restarted with interruption. */

#define FILENAME	"3.3.1_55"
#define FIFONAME	"3.3.1_55_fifo"

static	pid_t	pid;
static	int		stat_val, i;
static	void	com_code(void);

int main(void)
{
	int		fd;
	char	inbuf[50];
	struct	flock flock1, flock2;
	int 	fiford, fifowr;
	char    fifop[_POSIX_PATH_MAX +1];
	char    *in_buf;
	size_t	pipe_buf_sz, fifo_buf_sz, max_buf_sz, pipe_max, fifo_max;
	sigset_t	mask;
	
	setup(E_SIGNAL_CON);
	funct(E_SIGNAL_CON);

	assertion(55, BASE);
	start
		prep("Establish path and create a FIFO.");
		start
			Zstrcpy(fifop, temp_path());
			Zstrcat(fifop, FIFONAME);
			Zmkfifo(fifop, PROT_ALL);
		finis
	
							/* Maximum bytes written in pipe and FIFO */
		max_buf_sz=g_max_pipe(&pipe_buf_sz, &fifo_buf_sz, &pipe_max, &fifo_max);
	
		prep("Allocate and initialize output buffer.");
		start
			in_buf = (char *) Zmalloc(max_buf_sz);
			memset(in_buf, 'Z', max_buf_sz);
		finis
	
		ENABLE_PSYNC
		for(i=NSIGS_STOP; i<NSIGS; i++) {
											/* Testing:  fcntl() */
			prep("Create temp file and set shared lock on entire file.");
			start
				flock1.l_type = F_RDLCK;
				flock1.l_whence = SEEK_SET;
				flock1.l_start = ((off_t)0);
				flock1.l_len = ((off_t)20);

				fd = temp_open(FILENAME, O_CREAT | O_TRUNC | O_RDWR, S_IRWXU);
				Zwrite(fd, inbuf, 20);
				Zfcntl(fd, F_SETLK, &flock1);
			finis
	
		 	prep("Fork a child process to test fcntl().");
			start
				sv_flush();
				pid = Zfork();
			finis
		
			if(pid == (pid_t)0) {
				prep("Have child attempt to place exclusive lock on file.");
				start
					if(sigs[i].sig != SIGSTOP) {
						SIGNAL(sigs[i].sig, SIG_DFL);
					}
					SIGNAL(SIGCONT, sig_int);

					flock2.l_type = F_WRLCK;
					flock2.l_whence = SEEK_SET;
					flock2.l_start = ((off_t)0);
					flock2.l_len = ((off_t)5);
	
					SET_ALARM(4 * WAIT_FOR_TIME);
						expecting(EINTR);
						Zfcntl(fd, F_SETLKW, &flock2);
					CLEAR_ALARM
				finis
	
				done();
			}

			com_code();

			cleanup("Close test file.");
			start
				Zclose(fd);
			finis
	
											/* Testing:  open() */
		 	prep("Fork a child process to test open().");
			start
				sv_flush();
				pid = Zfork();
			finis
		
			if(pid == (pid_t)0) {
				prep("Child hangs on open() call.");
				start
					if(sigs[i].sig != SIGSTOP) {
						SIGNAL(sigs[i].sig, SIG_DFL);
					}
					SIGNAL(SIGCONT, sig_int);

					SET_ALARM(4 * WAIT_FOR_TIME);
						expecting(EINTR);
						Zopen(fifop, O_WRONLY);
					CLEAR_ALARM
				finis
	
				done();
			}

			com_code();
	

											/* Testing:  pause() */
		 	prep("Fork a child process to test pause().");
			start
				sv_flush();
				pid = Zfork();
			finis
		
			if(pid == (pid_t)0) {
				prep("Child calls pause().");
				start
					if(sigs[i].sig != SIGSTOP) {
						SIGNAL(sigs[i].sig, SIG_DFL);
					}
					SIGNAL(SIGCONT, sig_int);

					SET_ALARM(4 * WAIT_FOR_TIME);
						expecting(EINTR);
						Zpause();
					CLEAR_ALARM
				finis
	
				done();
			}

			com_code();
	
	
												/* Testing:  read() */
			prep("Fork a child process to test read().");
			start
				sv_flush();
				pid = Zfork();
			finis
	
			if(pid == (pid_t)0) {
				prep("Open the FIFO's read channel.");
				start
					if(sigs[i].sig != SIGSTOP) {
						SIGNAL(sigs[i].sig, SIG_DFL);
					}
					SIGNAL(SIGCONT, sig_int);

					SET_TIMEOUT
						fiford = Zopen(fifop, O_RDONLY);
					CLEAR_ALARM
				finis
	
				testing("Read blocks waiting for data.");
				start
					SET_ALARM(4 * WAIT_FOR_TIME);
						expecting(EINTR);
						Zread(fiford, inbuf, 20);
					CLEAR_ALARM
				finis
	
				done();
			}	
	
			prep("Open the FIFO's write channel.");
			start
				SET_TIMEOUT
					fifowr = Zopen(fifop, O_WRONLY);
				CLEAR_ALARM
			finis
	
			com_code();

			cleanup("Close FIFO's write channel.");
			start
				Zclose(fifowr);
			finis
	
	
											/* Testing:  sleep() */
		 	prep("Fork a child process to test sleep().");
			start
				sv_flush();
				pid = Zfork();
			finis
		
			if(pid == (pid_t)0) {
				prep("Child executes sleep().");
				start
					if(sigs[i].sig != SIGSTOP) {
						SIGNAL(sigs[i].sig, SIG_DFL);
					}
					SIGNAL(SIGCONT, sig_int);

					sleep(4 * WAIT_FOR_TIME);
				finis
	
				done();
			}
	
			com_code();
	

											/* Testing:  sigsuspend() */
		 	prep("Fork a child process to test sigsuspend().");
			start
				sv_flush();
				pid = Zfork();
			finis
		
			if(pid == (pid_t)0) {
				prep("Establish process's signal mask set.");
				start
					Zsigemptyset(&mask);
					Zsigaddset(&mask, SIGUSR1);
					Zsigprocmask(SIG_SETMASK, &mask, (sigset_t *)0);
				finis
	
				prep("Child calls sigsuspend()");
				start
					if(sigs[i].sig != SIGSTOP) {
						SIGNAL(sigs[i].sig, SIG_DFL);
					}
					SIGNAL(SIGCONT, sig_int);

					SET_ALARM(4 * WAIT_FOR_TIME)
						expecting(EINTR);
						Zsigsuspend(&mask);
					CLEAR_ALARM
				finis
	
				done();
			}
	
			com_code();
	

											/* Testing:  wait() */
		 	prep("Fork a child process to test wait().");
			start
				sv_flush();
				pid = Zfork();
			finis
	
			if(pid == (pid_t)0) {
				prep("Fork a grandchild process.");
				start
					sv_flush();
					pid = Zfork();
				finis
		
				if(pid == (pid_t)0) {
					prep("Grandchild goes to sleep.");
					start
						SLEEP(4 * WAIT_FOR_TIME)
					finis
	
					done();
				}
		
				prep("Child calls wait().");
				start
					if(sigs[i].sig != SIGSTOP) {
						SIGNAL(sigs[i].sig, SIG_DFL);
					}
					SIGNAL(SIGCONT, sig_int);
	
					SLEEP(WAIT_FOR_TIME); /* Grandchild time to sleep. */
					expecting(EINTR);
					Zwait(&stat_val);
				finis
	
				done();
				WAIT_FOR_PID(pid);
			}
		
			com_code();
	
	
											/* Testing:  waitpid() */
		 	prep("Fork a child process to test waitpid().");
			start
				sv_flush();
				pid = Zfork();
			finis
	
			if(pid == (pid_t)0) {
				prep("Fork a grandchild process.");
				start
					sv_flush();
					pid = Zfork();
				finis
		
				if(pid == (pid_t)0) {
					prep("Grandchild goes to sleep.");
					start
						SLEEP(4 * WAIT_FOR_TIME)
					finis
	
					done();
				}
		
				prep("Child calls waitpid().");
				start
					if(sigs[i].sig != SIGSTOP) {
						SIGNAL(sigs[i].sig, SIG_DFL);
					}
					SIGNAL(SIGCONT, sig_int);
	
					SLEEP(WAIT_FOR_TIME); /* Grandchild time to sleep. */
					expecting(EINTR);
					Zwaitpid(pid, &stat_val, 0);
				finis
	
				done();
				WAIT_FOR_PID(pid);
			}
		
			com_code();
	
	
												/* Testing:  write() */
			prep("Fork a child process to test write().");
			start
				sv_flush();
				pid = Zfork();
			finis
	
			if(pid == (pid_t)0) {
				prep("Open the FIFO's writing and reading channels.");
				start
					if(sigs[i].sig != SIGSTOP) {
						SIGNAL(sigs[i].sig, SIG_DFL);
					}
					SIGNAL(SIGCONT, sig_int);

					SET_TIMEOUT
						fiford = Zopen(fifop, O_RDONLY | O_NONBLOCK);
						fifowr = Zopen(fifop, O_WRONLY);
					CLEAR_ALARM
				finis

				prep("Fill up FIFO's write channel.");
				start
					size_t	n_bytes;
					size_t	nb_wr = 0;

					SET_TIMEOUT
						for(n_bytes = fifo_buf_sz; n_bytes; n_bytes /= 2)
							while(fifo_max -nb_wr >= n_bytes) {
								expecting(n_bytes);
								nb_wr += Zwrite(fifowr, in_buf, n_bytes);
							}
					CLEAR_ALARM
					SEND_PSYNC
				finis

				prep("Call to write() blocks writing to pipe.");
				start
					sv_flush();
					SET_ALARM(4 * WAIT_FOR_TIME)
						expecting(EINTR);
						Zwrite(fifowr, in_buf, fifo_buf_sz);
					CLEAR_ALARM
				finis
				
				done();
			}	
	
			SET_TIMEOUT
				WAIT_PSYNC
			CLEAR_ALARM
	
			com_code();
		}
		DISABLE_PSYNC
	finis
	
	done();
}

static	void com_code(void)
{
	prep("Issue a %s.",sigs[i].signame);
	start
		SLEEP(2 * WAIT_FOR_TIME);	/* Allow child time to setup. */
		Zkill(pid, sigs[i].sig);
		SLEEP(WAIT_FOR_TIME);
	finis

	testing("Child process stopped by %s.", sigs[i].signame);
	start
		Zwaitpid(pid, &stat_val, WUNTRACED);

		if( ! WIFSTOPPED(stat_val)) 
			error("Process did not stop.");
		else
			if(WSTOPSIG(stat_val) != sigs[i].sig) 
				error("%s\n%s%s\n%s%s",
					"WSTOPSIG not equal to expected signal.",
					"Expected: ",
					sigs[i].signame,
					"Actual: ",
					sig_lookup(stat_val) );
	finis

	prep("Issue signal SIGCONT to child.");
	start
		Zkill(pid, SIGCONT);
		SLEEP(WAIT_FOR_TIME);
	finis

	testing("Child now active and didn't report a %s or SIGCONT.",
						sigs[i].signame);
	start
		Zwaitpid(pid, &stat_val, WUNTRACED);
		if(WIFSTOPPED(stat_val)) {
			error("Child is stopped.");
			Zkill(pid, SIGKILL);
			Zwaitpid(pid, &stat_val, 0);
		}
		else
			if(WIFSIGNALED(stat_val)) 
				error("Child process terminated by %d", WTERMSIG(stat_val));
	finis
}
