#include	<pcts.h>

#include	<Zunistd.h>

#include	<sv_env.h>
#include	<sv_errno.h>
#include	<sv_sigs.h>
#include	<pcts_timer.h>

void	sig_term(int sig_num)
{
	error("Signal %s was caught, it should have been blocked.",
														sig_lookup(sig_num));
}


int	main(void)
{
	pid_t	pid;
	int		i;
	int		stat_val;
	sigset_t	set, oset;

	setup(E_SIGPROCMASK);
	funct(E_SIGPROCMASK);

			/*	A call to sigprocmask(how, set, NULL) changes the signals
				blocked for the process according to the directive how and
				the signal set pointed to by the non-NULL argument set.
			*/
	assertion(9, BASE);
	start
												/* How of SIG_BLOCK */
		prep("Initialize process signals blocked to the empty set.");
		start
			Zsigemptyset(&set);
			Zsigprocmask(SIG_SETMASK, &set, &oset);
		finis

		prep("Fork a process.");
		start
			sv_flush();
			pid = Zfork();
		finis

		if(pid == (pid_t)0) {
			for(i=0; i<NSIGS; i++) {
				if(	(sigs[i].sig == SIGKILL) ||
					(sigs[i].sig == SIGCONT) ||
					(sigs[i].sig == SIGSTOP) ||
					(sigs[i].sig == SIGALRM) )
						continue;

				prep("Catch %s signal if it occurs.", sigs[i].signame);
				start
					SIGNAL(sigs[i].sig, sig_term);
				finis

				prep("Call sigprocmask(SIG_BLOCK, set, NULL)");
				prep("to block signal %s.", sigs[i].signame);
				start
					Zsigaddset(&set, sigs[i].sig);
					Zsigprocmask(SIG_BLOCK, &set, (sigset_t *)0);
				finis

				prep("Send the blocked %s signal", sigs[i].signame);
				prep("to this process to obtain a pending signal.");
				start
					Zkill(getpid(), sigs[i].sig);
					SLEEP(WAIT_FOR_TIME)
				finis
			
				testing("Signal %s is blocked.", sigs[i].signame);
				start
					Zsigpending(&set);
					if( ! sigismember(&set, sigs[i].sig) )
						error("%s\n%s%s\n%s\n",
							"Signal not reported as pending.",
							"Expected pending signal:	",
							sigs[i].signame,
							"Actual signal was not pending.");
				finis
			}
			done();
		}
		wait_for(pid, (NSIGS * WAIT_FOR_TIME) +TIMEOUT);

												/* How of SIG_UNBLOCK */
		prep("Fork a process.");
		start
			sv_flush();
			pid = Zfork();
		finis

		if(pid == (pid_t)0) {
			prep("Initialize process mask to all signals, except SIGALRM,");
			prep("blocked.");
			start
				Zsigfillset(&set);
				Zsigdelset(&set, SIGALRM);
				Zsigprocmask(SIG_SETMASK, &set, &oset);
			finis

			Zsigemptyset(&set);
			for(i=0; i<NSIGS; i++) {
				if(	(sigs[i].sig == SIGKILL) ||
					(sigs[i].sig == SIGSTOP) ||
					(sigs[i].sig == SIGALRM) )
						continue;

				prep("Call sigprocmask(SIG_UNBLOCK, set, NULL) to unblock %s",
														sigs[i].signame);
				prep("signal and to deliver it to this process.");
				start
					Zsigaddset(&set, sigs[i].sig);
					expecting(0);
					Zsigprocmask(SIG_UNBLOCK, &set, (sigset_t *)0);

					SIGNAL(sigs[i].sig, sig_int);
					sig_rcvd = 0;
					Zkill(getpid(), sigs[i].sig);
					SLEEP(WAIT_FOR_TIME)
				finis
			
				testing("Signal %s is not blocked.", sigs[i].signame);
				start
					if( ! sig_rcvd)
						error("Signal %s is blocked.\n");
				finis
			}
			done();
		}
		wait_for(pid, (NSIGS * WAIT_FOR_TIME) +TIMEOUT);

												/* How of SIG_SETMASK */
		prep("Fork a process.");
		start
			sv_flush();
			pid = Zfork();
		finis

		if(pid == (pid_t)0) {
			prep("Initialize signal mask.");
			start
				Zsigemptyset(&set);
								/* Set action to catch SIGCHLD and SIGCONT
								   since default action is to "ignore" and
								   blocked ignored signals may be discarded.
								*/
				SIGNAL(SIGCHLD, sig_int);
				SIGNAL(SIGCONT, sig_int);
			finis

			for(i=0; i<NSIGS; i++) {
				if(	(sigs[i].sig == SIGKILL) ||
					(sigs[i].sig == SIGSTOP) ||
					(sigs[i].sig == SIGALRM) )
						continue;

				prep("Call sigprocmask(SIG_SETMASK, set, NULL)");
				prep("to block signal %s.", sigs[i].signame);
				start
					Zsigaddset(&set, sigs[i].sig);
					expecting(0);
					Zsigprocmask(SIG_SETMASK, &set, &oset);
				finis

				prep("Send the blocked %s signal", sigs[i].signame);
				prep("to this process to obtain a pending signal.");
				start
					Zkill(getpid(), sigs[i].sig);
					SLEEP(WAIT_FOR_TIME);
				finis
			
				testing("Signal %s is blocked.", sigs[i].signame);
				start
					Zsigpending(&set);
					if( ! sigismember(&set, sigs[i].sig) )
						error("%s\n%s%s\n%s\n",
							"Signal not reported as pending.",
							"Expected pending signal:	",
							sigs[i].signame,
							"Actual signal number was not pending.");
				finis
			}
			done();
		}
		wait_for(pid, (NSIGS * WAIT_FOR_TIME) +TIMEOUT);
	finis


							/*	A call to sigprocmask(how, NULL, NULL) does
								not change the process's signal mask.
							*/
	assertion(10, BASE);
	start
		FORK
			prep("Initialize process signal mask to only SIGINT set.");
			start
				Zsigemptyset(&set);
				Zsigaddset(&set, SIGINT);
				Zsigprocmask(SIG_SETMASK, &set, &oset);
			finis

			testing("Call to sigprocmask(how, NULL, oset)");
			testing("does not change the process's signal mask and");
			testing("the previous mask is stored in oset.");
			start
				Zsigprocmask(SIG_BLOCK,   (sigset_t *)0, (sigset_t *)0);
				Zsigprocmask(SIG_UNBLOCK, (sigset_t *)0, (sigset_t *)0);
				Zsigprocmask(SIG_SETMASK, (sigset_t *)0, (sigset_t *)0);
				Zsigprocmask(SIG_SETMASK, (sigset_t *)0, (sigset_t *)0);

				Zkill(getpid(), SIGINT);
				SLEEP(WAIT_FOR_TIME)
				Zsigpending(&set);
				if( ! sigismember(&set, SIGINT) )
					error("Signal SIGINT not pending.");
			finis
		KROF
	finis

	assertion(11, BASE);
	start
		prep("Fork a process.");
		start
			sv_flush();
			pid = Zfork();
		finis

		if(pid == (pid_t)0) {
			prep("Establish a signal mask with SIGKILL set.");
			start
				Zsigemptyset(&set);
				Zsigaddset(&set, SIGKILL);
			finis
	
			testing("Call to sigprocmask() succeeds.");
			start
				sigprocmask(SIG_SETMASK, &set, &oset);
			finis

			testing("SIGKILL is not blocked from delivery.");
			start
				Zkill(getpid(), SIGKILL);
				SLEEP(WAIT_FOR_TIME);
				error("Child process not terminated.");
			finis

			done();
		}

		waitpid(pid, &stat_val, 0);
		if(WIFSIGNALED(stat_val)) {
			if(WTERMSIG(stat_val) != SIGKILL)
				error("%s\n%s%s",
					"Expected child process to be terminated by SIGKILL.",
					"Actually child terminated by ",
					sig_lookup(WTERMSIG(stat_val)) );
		}
		else
			error("Child not terminated by a signal.");
	finis

	assertion(12, BASE);
	start
		prep("Fork a process.");
		start
			sv_flush();
			pid = Zfork();
		finis

		if(pid == (pid_t)0) {
			prep("Establish a signal mask with SIGSTOP set.");
			start
				Zsigemptyset(&set);
				Zsigaddset(&set, SIGSTOP);
			finis
	
			testing("Call to sigprocmask() succeeds.");
			start
				sigprocmask(SIG_SETMASK, &set, &oset);
			finis

			prep("Send SIGSTOP to child process.");
			start
				sv_flush();
				Zkill(getpid(), SIGSTOP);
				sleep(WAIT_FOR_TIME);	/* May get interrupted by SIGSTOP. */
			finis

			done();
		}

		SLEEP(WAIT_FOR_TIME);
		testing("SIGSTOP is not blocked from delivery.");
		start
			break_on(ERROR);
			Zwaitpid(pid, &stat_val, WUNTRACED);
				if( ! WIFSTOPPED(stat_val)) 
					error("Child status was not reported as STOPPED.");
			Zkill(pid, SIGCONT);
			WAIT_FOR_PID(pid);
		finis
	finis

	assertion(13, BASE);
	start
		prep("Fork a process.");
		start
			sv_flush();
			pid = Zfork();
		finis

		if(pid == (pid_t)0) {
			prep("Initialize process signals blocked to the empty set.");
			start
				Zsigemptyset(&set);
				Zsigprocmask(SIG_SETMASK, &set, &oset);
			finis

			prep("Block signal SIGINT.");
			start
				Zsigemptyset(&set);
				Zsigaddset(&set, SIGINT);
				expecting(0);
				Zsigprocmask(SIG_BLOCK, &set, &oset);
			finis

			prep("Send the blocked SIGINT signal to this process to obtain");
			prep("a pending signal and ensure SIGINT is pending.");
			start
				Zkill(getpid(), SIGINT);
				Zsigpending(&set);
				if( ! sigismember(&set, SIGINT) )
					error("%s\n%s\n%s\n",
							"Signal not reported as pending.",
							"Expected pending signal: SIGINT.",
							"Actual signal number was not pending.");
			finis

			prep("Set action to SIG_DFL for SIGINT.");
			start
				SIGNAL(SIGINT, SIG_DFL);
			finis

			testing("Call of sigprocmask() to unblock SIGINT is delivered");
			testing("before sigprocmask() returns.");
			start
				sv_flush();
				sigprocmask(SIG_UNBLOCK, &set, &oset);
				error("Signal not delivered before sigprocmask() returned.");
			finis

			done();
		}

		waitpid(pid, &stat_val, 0);
		if(WIFSIGNALED(stat_val)) {
			if(WTERMSIG(stat_val) != SIGINT)
				error("%s\n%s%s",
					"Expected child process to be terminated by SIGINT.",
					"Actually child terminated by ",
					sig_lookup(WTERMSIG(stat_val)) );
		}
		else
			error("Child not terminated by a signal.");
	finis


			/*	A call to sigprocmask(how, set, oset) when the value of
				"how" is not one of the defined values returns
				a value of -1 and set "errno" to [EINVAL] and the
				process's signal mask is not changed.
			*/
#define	INVALID_HOW (SIG_SETMASK + SIG_BLOCK + SIG_UNBLOCK)
	assertion(14, BASE);
	start
		FORK
			testing("Setting process signal mask with invalid \"how\"");
			testing("value of -1 returned and errno set to [EINVAL].");
			start
										/* Initialize process signal mask. */
				Zsigemptyset(&set);
				Zsigprocmask(SIG_SETMASK, &set, &oset);

				Zsigaddset(&set, SIGHUP);
				expecting(-1);
				expecting(EINVAL);
				Zsigprocmask(INVALID_HOW, &set, &oset);
			finis

			testing("Setting process signal mask with invalid \"how\"");
			testing("did not change the process's signal mask.");
			start
				Zsigprocmask(SIG_SETMASK, (sigset_t *)0, &oset);
				if(sigismember(&oset, SIGHUP))
					error("Process's signal mask altered.");
			finis
		KROF
	finis

	done();
}
