/*
	The routines in this file interface with the NIST-PCTS to
	provide the following "target system support facilities" on
	some implementations:

		mount a file system read-only,
		mount a file system for reading and writing,
		unmount a file system,
		establish a controlling terminal.
*/

#include	<pcts.h>
#include	<unistd.h>
#include	<fcntl.h>
#include	<sys/stat.h>


/*
	Routine to interface the NIST-PCTS to an implementation
	provided routine to mount a file system as read-only.
*/
mtrofs(dirname, fsname)
	char	*dirname, *fsname;
{
	/*	Proper interface on some implementations.	*/
	return(mount(fsname, dirname, 1 ));

	/*	If not supported, then use "return(-1);" instead.	*/
}


/*
	Routine to interface the NIST-PCTS to an implementation
	provided routine to mount a file system for reading and writing.
*/
mtrwfs(dirname, fsname)
	char	*dirname, *fsname;
{
	/*	Proper interface on some implementations.	*/
	return(mount(fsname, dirname, 0));

	/*	If not supported, then use "return(-1);" instead.	*/
}


/*
	Routine to interface the NIST-PCTS to an implementation
	provided routine to mount a file system for reading and writing.
*/
unmtfs(dirname, fsname)
	char	*dirname, *fsname;
{
	return(umount(fsname));

	/*	If not supported, then use "return(-1);" instead.	*/
}


/*
	Routine to interface the NIST-PCTS to an implementation
	provided routine to establish a controlling terminal.
*/
int open_ctty(path, flags, mode)
	char	*path;
	int		flags;
	mode_t	 mode;
{
	if (flags & O_CREAT)
		return(open(path, flags & ~O_NOCTTY, mode));
	else
		return(open(path, flags & ~O_NOCTTY));
}
