#ifndef	_POSIX_SOURCE
#define	_POSIX_SOURCE
#endif

#include <sys/types.h>
#include <limits.h>
#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>

#include <stdarg.h>

#define SECTION (-2)	/*Used by setsect()*/
#define	TEST	(-1)	/*Used by setsect()*/
#define	RBUFSZ	512		/*Size of a read buffer*/

extern	int	p_system(char *);
void	foutf(int, char *, ...);
off_t	svseek(int, off_t, int);
void	doshell(char *);

struct	rdbuf {
	off_t	bufpos;
	int	pos;
	int	len;
	char	buf[RBUFSZ];
};

			/* Set the number of structure elements to [_POSIX_OPEN_MAX] +1. */
			/* [_POSIX_OPEN_MAX] is 16, in POSIX.1 */
static	struct	rdbuf	rbuffs[] = {
	{(off_t)0,RBUFSZ,0,""},
	{(off_t)0,RBUFSZ,0,""},
	{(off_t)0,RBUFSZ,0,""},
	{(off_t)0,RBUFSZ,0,""},
	{(off_t)0,RBUFSZ,0,""},
	{(off_t)0,RBUFSZ,0,""},
	{(off_t)0,RBUFSZ,0,""},
	{(off_t)0,RBUFSZ,0,""},
	{(off_t)0,RBUFSZ,0,""},
	{(off_t)0,RBUFSZ,0,""},
	{(off_t)0,RBUFSZ,0,""},
	{(off_t)0,RBUFSZ,0,""},
	{(off_t)0,RBUFSZ,0,""},
	{(off_t)0,RBUFSZ,0,""},
	{(off_t)0,RBUFSZ,0,""},
	{(off_t)0,RBUFSZ,0,""},
	{(off_t)0,RBUFSZ,0,""},
};

static	int		bytread(int, char *);

/*
** Our own printf-like routine.
*/
void	outf(char * fmt, ...)
{
	char		buff[2048];
	unsigned	len;
	va_list	v;

	va_start(v, fmt);
	(void)vsprintf(buff, fmt, v);
	va_end(v);
	len = (unsigned)strlen(buff);

	if(write(1, buff, len) != len){
		foutf(STDERR_FILENO, "Write failed to stdout.\n");
		exit(0);
	}
}

/*
** Our own fprintf-like routine using file descriptors instead of
** file pointers.
*/
void	foutf(int fd, char * fmt, ...)
{
	char	buff[2048];
	unsigned	len;
	va_list	v;

	va_start(v, fmt);
	(void)vsprintf(buff, fmt, v);
	va_end(v);
	len = (unsigned)strlen(buff);

	if(write(fd, buff, len) != len){
		write(STDERR_FILENO, "Write failed to file.\n", 22);
		exit(0);
	}
}

int	fgetln(int fd, char *str)
{
	int	rval;
	char	*p;

	p = str;
	while((rval = bytread(fd, str)) && (*str != '\n'))
		++str;
	*str = 0;
	return(rval || !(p == str));
}

int	shell;	/*This flag will be set when a shell escape is taken*/

int	getln(char *str)
{
	int	rval;
	char	*p, *q;

	shell = 0;
	q = p = str;
	while((rval = read(0, str, 1)) && (*str != '\n'))
		++str;
	*str = 0;
	while(*q && ((*q == ' ') || (*q == '	')))
		q++;
	if(*q == '!'){
		shell = 1;
		doshell(++q);
		*p = 0;
	}
	return((rval || (p != str)) ? 1 : 0);
}

static int bytread(int fd, char *ptr)
{
	if(fd < 0 ||  fd > _POSIX_OPEN_MAX){
		foutf(STDERR_FILENO, "Invalid file descriptor for input file\n");
		return(0);
	}
	if(rbuffs[fd].pos < rbuffs[fd].len){
		*ptr = rbuffs[fd].buf[rbuffs[fd].pos++];
	}
	else{
		rbuffs[fd].bufpos += rbuffs[fd].len;
		if(!(rbuffs[fd].len = read(fd, &rbuffs[fd].buf[0], RBUFSZ))){
			return(0);
		}
		rbuffs[fd].pos = 0;
		*ptr = rbuffs[fd].buf[rbuffs[fd].pos++];
	}
	return(1);
}

off_t	svseek(int fd, off_t offset, int whence)
{
	if((fd < 0) || (fd > _POSIX_OPEN_MAX)){
		return((long)-1);
	}
	if(whence == SEEK_CUR){
		offset += ((off_t)rbuffs[fd].pos + rbuffs[fd].bufpos);
		whence = SEEK_SET;
	}
	rbuffs[fd].len = 0;
	rbuffs[fd].pos = RBUFSZ;
	rbuffs[fd].bufpos = lseek(fd, offset, whence);
	return(rbuffs[fd].bufpos);
}

void	rstcache(int fd)
{
	rbuffs[fd].bufpos = (long)0;
	rbuffs[fd].pos = RBUFSZ;
	rbuffs[fd].len = 0;
	*rbuffs[fd].buf = 0;
}

void	doshell(char *str)
{
	int	oldsout;

	oldsout = dup(1);
	close(1);
	if(dup(2) != 1){
		foutf(STDERR_FILENO, "Dup failed, standard output was lost!\n");
		foutf(STDERR_FILENO, "Do not use shell escape until dup is fixed!\n");
		exit(-1);
	}
	(void)p_system(str);
	close(1);
	if(dup(oldsout) != 1){
		foutf(STDERR_FILENO, "Dup failed, standard output was lost!\n");
		foutf(STDERR_FILENO, "Do not use shell escape until dup is fixed!\n");
		exit(-1);
	}
	close(oldsout);
}

/*
** our own isspace() like function, returns true for space tab and newline
** false for all others.
*/

int	whitesp(char c)
{
	return((c == ' ') || (c == '	') || (c == '\n'));
}

/*
** nxtok() gets the next token (delimited by white space) from src
** and places it in dst. It then returns a pointer to the next token
** after the one it got.  If no more tokens, it returns a pointer to
** the end of src.
*/

char	*nxtok(char *dst, char *src)
{

	while(*src && (whitesp(*(src)))) ++src;
	if(*src != '\"')
		while(*src && (!whitesp(*(dst) = *(src++)))) ++dst;
	else{
		++src;
		while(*src &&( (*dst = *(src++)) != '\"')) ++dst;
	}
	while(*src && (whitesp(*(src)))) ++src;
	*dst = 0;
	return(src);
}

/*
** If line has any nonwhite charaters, nonblank() returns true, otherwise
** false.
*/

int	nonblank(char *str)
{
	while(*str){
		if(!whitesp(*str))
			return(1);
		++str;
	}
	return(0);
}

/*
** Routine to read run numbers from BegJourn and BegRun lines and
** return their integer values
*/

int	runno(char *str)
{
	char	token[512];
	int	ret;

	(void)nxtok(token, str);
	if(!sscanf(token, "%d", &ret)){
		return(-1);
	}
	return(ret);
}

/*
** Function to reset the setsect function for a new section name.
*/

static	int		havesect = 0;

void	rstsect(void)
{
	havesect = 0;
}

/*
** Function to read the section file and return the mappings for a section
** name one at a time until the section is exhausted.  Returns the mapping
** in nxt.  Receives the section name in name.  If the section name does
** not map to anything, the item is assumed to be a testname, and TEST is
** returned, otherwise, SECTION is returned.
*/

int	setsect(char *nxt, char *name, char *svvs)
{
	static	char	*p, line[512], token[64];
	static	int	sectfile = -1;

	if(!havesect){
		(void)sprintf(line, "%s/data/sections", svvs);
		if(sectfile < 0){
			if((sectfile = open(line, O_RDONLY))< 0){
				foutf(STDERR_FILENO,
				      "Cannot open section list file \"%s\".\n",
				      line);
				exit(-1);
			}
		}
		else{
			svseek(sectfile, (long)0, SEEK_SET);
		}
		while(fgetln(sectfile, line)){
			p = &line[0];
			p = nxtok(token, p);
			if(!strcmp(token, name)){
				p = nxtok(nxt, p);
				havesect = 1;
				return(SECTION);
			}
		}
		return(TEST);
	}
	else {
		p = nxtok(nxt, p);
		return(SECTION);
	}
}
