#ifndef	_POSIX_SOURCE
#define	_POSIX_SOURCE
#endif

#include <unistd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <fcntl.h>

#define	SV_CONFIG
#include <sv_conf.h>
#include <sv_env.h>

#define	EOLN	'\n'
#define	Posix2_LINE_MAX	2048

/*	The sv_config() function reads the file "include/config" which is
 *	made up of lines with keywords on the beginning of each line and
 *	processes that file	according to the rules defined in "include/sv_conf.h".
 *	The header "sv_conf.h defines an array of structures (sv_params)
 *	each containing a keyword and a pointer to a function which performs
 *	the desired	action. The static nparms is the number of keywords
 *	recognized by the parser.
 *
 *	In this case sv_config() is used to set up the tunable values for
 *	the SVVS modules.  It receives a pointer to an array of
 *	data structures defined in svtune.h as an argument.  It then
 *	loads that array based on the configuration file.
 */

extern	void	*sv_malloc(size_t);
static	unsigned	long	val(char *);
static	void	parse(char *);
static	void	assign(int, char *);
static	char	*nxtok(char *, char *, int *);
static	int		isspace(char);
static	void	stripcom(char *);
static	void	commas(char *);
static	int		getln(int, char *);
static	void	reset_cache(void);
static	int		getbyte(int, char *);

static	struct	cnfg	*params;
static	int		nparms;

void sv_config(void)
{
	char	curlin[Posix2_LINE_MAX];
	char	*p, *q;
	int	fd;

	params = &sv_params[0];
	nparms = NPARMS;

/*
 *	CONFIGD is resolved as a pointer.  Lint will complain, no problem
 */
	if(CONFIGD)
		return;

									/* Determine path for <config>. */
	getcwd(curlin, Posix2_LINE_MAX);
	p = strstr(curlin, "NIST-PCTS");
	if(p == NULL)
		severr("Improper path obtained from getcwd() \"%s\"\n", curlin);
									/* Get last occurrence of "NIST-PCTS". */
	while((q = strstr(p +1, "NIST-PCTS")) != NULL)
		p = q;
	strcpy(p +strlen("NIST-PCTS"), "/include/config");
		
	if((fd = open(curlin, O_RDONLY)) < 0)
		severr("Cannot open config file: '%s'.", curlin);
	while(getln(fd, curlin)){
		stripcom(curlin);
		parse(curlin);
	}
	if(close(fd))
		severr("Cannot close config file.");
/*
 *	CONFIGD is resolved as a pointer, lint will complain, no problem
 */
	CONFIGD = 1;	/*Set CONFIGD */
	reset_cache();			/* in case called twic */
}


static void parse(char *curlin)
{
	int	i;
	char	*str;
	char	def[Posix2_LINE_MAX];

	str = curlin;					/* check for blank line */
	while( isspace(*str) ) str++;
	if(*str){
		curlin = nxtok(def,curlin,(int *)0);
		for(i = 0; i < nparms; i++ )
			if(!strcmp(def,params[i].name)) {
				assign(i,curlin);
				return;
			}				
		error("Unrecognized symbol: '%s' in config file",def);
		sv_flush();
	}
}


/*
 *	lint will complain about pointer alignments for assignment functions
 */
static void assign(int pos, char *line)
{
	char	tmp_buf[Posix2_LINE_MAX];
	int		strsize;
	int		vtyp;
	char	*target;
	unsigned long thing;

	if( (vtyp=params[pos].typ) == STRING ) {
		(void) nxtok(tmp_buf,line,&strsize);
		if( !(params[pos].value = target = (char *)sv_malloc(strsize)) )
			severr("Can't malloc space for config variables");
		memcpy(target, tmp_buf, strsize);
		return;
	}
	target = params[pos].value;
	thing = val(line);

	switch( vtyp ) {
	case UCHAR:
 		*(unsigned char *)target=(unsigned char)thing;
		break;
	case CHAR:
		*target=(char)thing;
		break;
	case USHORT:
		*(unsigned short *)target=(unsigned short)thing;
		break;
	case SHORT:
		*(short *)target=(short)thing;
		break;
	case UINT:
		*(unsigned int *)target=(unsigned int)thing;
		break;
	case INT:
		*(int *)target=(int)thing;
		break;
	case ULONG:
		*(unsigned long *)target=thing;
		break;
	case LONG:
		*(long *)target=(long)thing;
		break;
	case UID_T:
		*(uid_t *)target=(uid_t)thing;
		break;
	case GID_T:
		*(gid_t *)target=(gid_t)thing;
		break;
	case PID_T:
		*(pid_t *)target=(pid_t)thing;
		break;
	}
}		


static char *nxtok(char *dst, char *src, int *len)
{
	extern char *c_string(char *, char *, int *);
	int	foo;

	if( !len ) len = &foo;
	
	*len = 1;	/* trailing '\0' ---> 1 char minimum in target */
	while( isspace(*src) ) src++;
	if(*src == '"')
		src = c_string( dst, src, len );/* quoted characters in str */
	else {
		while(*src && !isspace(*dst = *src++)) {
			dst++;
			(*len)++;
		}
		*dst = 0;
	}
	while( isspace(*src) ) src++;
	return(src);
}


static int isspace(char c)
{
	return((c == ' ') || (c == '\t') || (c == '\n'));
}


static unsigned long val(char *str)
{
	char	num[Posix2_LINE_MAX];
	unsigned	long	retval = 0l;
	char *sstr, *styp, *p;

	(void) nxtok(num,str,(int *)0);
	commas(num);
	p=num;
	switch( *num ) {
	case '+': case '-': case '1': case '2': case '3': case '4': 
	case '5': case '6': case '7': case '8': case '9':
		sstr="%ld";
		styp="decimal";
		break;
	case '0':
		if( *(num+1) == 'x' || *(num+1) == 'X' ) {
			p += 2;
			sstr="%lx";
			styp="hexadecimal";
			break;
		}
		sstr="%lo";
		styp="octal";
		break;
	default:
		error("Found: %s when expecting a number in config file",num);
		sv_flush();
		return( retval );
	}
	if(!sscanf(p,sstr,&retval)) {
		error("Bad or missing %s value: '%s' in config file",styp,num);
		sv_flush();
	}
	return(retval);
}


static void stripcom(char *str)
{
	static	int	comment = 0;
	char	*st,*end;
	int	quote;

	end = st = str;
	while(*end){
		if(comment){
			while( *end && (*end != '*' || *(end + 1) != '/')) ++end;
			if(!*end){
				*str = 0;
				return;
			}
			comment = 0;
			if(st != str)
				*st++ = ' ';
			end +=2;
		}
		quote = 0;
		while( *end && (*end != '/' || *(end + 1) != '*' || quote)){
			if( *end == '"')
				quote = !quote;
			*st++ = *end++;
		}
		if( *end ){
			comment = 1;
			end += 2;
		}
	}
	*st = 0;
}


static void commas(char *str)
{
	char	*d;

	d = str;
	while(*str){
		if(*str != ',')
			*d++ = *str;
		str++;
	}
	*d = 0;
}


static int getln(int fd, char *buf)
{
	int i = 0;

	while( getbyte(fd,buf) ) {
		if( *buf == '\n' ) {
			if (i) break;
			continue;	/* strip empty lines */
		}
		if(i++ > Posix2_LINE_MAX)
			severr("sv_config: line too long in config file");
		buf++;
	}
	*buf = '\0';
	return(i);
}


static int cnt = Posix2_LINE_MAX;
static int my_fd = -1;
static int i = Posix2_LINE_MAX;
static char *buf = (char *)0;

static void reset_cache(void)
{
	cnt = Posix2_LINE_MAX;
	my_fd = -1;
	i = Posix2_LINE_MAX;
	if( buf ) free( buf );
	buf = (char *)0;
}	


static int getbyte(int fd, char *p)
{
	if( !buf && !(buf=(char *)sv_malloc(Posix2_LINE_MAX))) 
		severr("sv_config: Cannot malloc space for read buffer.");
	if(my_fd > 0 && fd != my_fd)
		severr("sv_config: internal error");
	else my_fd = fd;
	if( i >= cnt) {
		cnt = read(fd, buf, Posix2_LINE_MAX);
		i = 0;
	}	
	if ( i < cnt ) {
		*p = buf[i++];
		return(1);
	}
	return(0);
}	


char *c_string(char *dst, char *src, int *len)
{
	int	val;

	*len = 1;
	if( *src != '"' )	/* should never happen */
		severr("missing '\"' at start of string in config file.");
	src++;
	while( *src ) {
		switch( *src ) {
		case 	'"':
			*dst='\0';
			return( ++src );
		case	'\n':
			severr("Embedded newline in config file string value.");
		case	'\\':
			src++;
			switch( *src ) {
			case	'\0':
				severr("Unterminated string in config file.");
			case	'n':
				*dst++ = '\n';
				(*len)++;
				src++;
				break;
			case	't':
				*dst++ = '\t';
				(*len)++;
				src++;
				break;
			case	'b':
				*dst++ = '\b';
				(*len)++;
				src++;
				break;
			case	'r':
				*dst++ = '\r';
				(*len)++;
				src++;
				break;
			case	'0':
			case	'1':
			case	'2':
			case	'3':
			case	'4':
			case	'5':
			case	'6':
			case	'7':
			case	'8':	/* accept 8 and 9 even though not octal */
			case	'9':
				val = 0;
				while( *src >= '0' && *src <= '9' ) {
					val *= 8;
					val += *src - '0';
					src++;
				}
				if( !*src )				
					severr("Unterminated string in config file.");
				*dst++ = (char) val;
				(*len)++;
				break;
			default:
				*dst++ = *src++;
				(*len)++;
				break;
			}
			break;
		default:			
			*dst++ = *src++;
			(*len)++;
			break;
		}
	}
	severr("Unterminated string in config file.");
}
