#ifndef	_POSIX_SOURCE
#define	_POSIX_SOURCE
#endif

/*
 *	sv_eval.c	function return value evaluation routine
 *
 */

#include <string.h>

#include <errno.h>
#include <sv_env.h>

extern int sv_gete(void);
extern int sv_sete(int);

#define	SUCC_FUNCT		(fnct->sv_succ)
#define	SUCC_DEFINED	(SUCC_FUNCT && SUCC_FUNCT != VALUE)
#define	SUCC( z )		((*(fnct->sv_succ))(z))
#define	SUCCEEDED		SUCC( ret_val )
#define	FAIL_FUNCT		(fnct->sv_fail)
#define	FAIL_DEFINED	(FAIL_FUNCT && FAIL_FUNCT != VALUE)
#define	FAIL( z )		((*(fnct->sv_fail))(z))
#define	FAILED			FAIL( ret_val )
#define	EXP_RET			((*fnct->sv_xval)())
#define	TEST( z1, z2 )	((*fnct->sv_test)(z1,z2))
#define GOT_EXPECTED	TEST( ret_val, EXP_RET )
#define GOT_ERRNO		(exp_errno == errno)
#define	FUNCT			(fnct->sv_name)
#define EXPECTING		(sv_xenv->sv_expecting)
#define SSTR			(fnct->sv_sstr)
#define FSTR			(fnct->sv_fstr)
#define FMT( z )		((*fnct->sv_fmt)(z))
#define ERRNO_DEFINED	(fnct->sv_errnodef)
#define EXP_RVAL		(EXPECTING & SV_VALU)
#define EXP_SUCCESS		(EXPECTING == SV_SUCC)
#define EXP_FAILURE		(EXPECTING == SV_FAIL || EXPECTING & SV_ERRN)
#define EXP_VOID		(EXPECTING == SV_VOID)
#define	CLR_RVAL		(EXPECTING &= ~SV_VALU)
#define	CLR_ERRNO		((void) sv_sete(0),EXPECTING &= ~SV_ERRN)
#define	PASS			return

#define EFAIL_ERR	sv_prerr( sv_xenv->sv_fsev,"%s() - failed unexpectedly\nExpected: %s\nReturned: %s\nerrno: %s", FUNCT, SSTR, FMT(ret_val),err_lookup(errno))

#define FAIL_ERR	sv_prerr( sv_xenv->sv_fsev,"%s() - failed unexpectedly\nExpected: %s\nReturned: %s", FUNCT, SSTR, FMT(ret_val))

#define ESUCC_ERR	sv_prerr( sv_xenv->sv_fsev,"%s() - succeeded unexpectedly\nExpected: %s\nReturned: %s\nActual errno: %s", FUNCT, FSTR, FMT(ret_val), err_lookup(exp_errno))

#define SUCC_ERR	sv_prerr( sv_xenv->sv_fsev,"%s() - succeeded unexpectedly\nExpected: %s\nReturned: %s", FUNCT, FSTR, FMT(ret_val))

#define XVAL_ERR	( (void)strcpy(exp_str,FMT(EXP_RET)), \
					sv_prerr(sv_xenv->sv_vsev,"%s() - returned unexpected value\nExpected: %s\nReturned: %s",FUNCT,exp_str,FMT(ret_val))	)

#define SF_IND_ERR	sv_prerr(sv_xenv->sv_usev,"%s() - returned indeterminate value outside range of success/failure\nreturned: %s\nSuccess: %s\nFailure: %s",FUNCT, FMT(ret_val), SSTR, FSTR)

#define F_IND_ERR	sv_prerr(sv_xenv->sv_usev,"%s() - returned indeterminate value outside range of failure\nreturned: %s\nSuccess: %s\nFailure: %s",FUNCT, FMT(ret_val), SSTR, FSTR)

#define S_IND_ERR	sv_prerr(sv_xenv->sv_usev,"%s() - returned indeterminate value outside range of success\nreturned: %s\nSuccess: %s\nFailure: %s",FUNCT, FMT(ret_val), SSTR, FSTR)

#define ERRNO_ERR	sv_prerr(sv_xenv->sv_esev,"%s() - set errno incorrectly\nExpected errno: %s\nActual errno: %s",FUNCT, err_lookup(exp_errno), err_lookup(errno) )

void	evaluate(struct ftable *fnct, pointer ret_val )
{
	int	exp_errno;					/* save expected errno				*/
	char exp_str[512];				/* save fmt str before buff re-used	*/

#ifdef TRACE
	sv_printf("\tevaluating: %s\n",FUNCT);
#endif
	exp_errno = sv_gete();

	if( EXP_RVAL ) {
#ifdef TRACE
		sv_printf("\tExpecting: explicit return value %s\n",FMT( EXP_RET ));
#endif
		CLR_RVAL;
		CLR_ERRNO;
		if( GOT_EXPECTED )
			if ( exp_errno )
				if( GOT_ERRNO ) {
					PASS;				/* matched expected val/errno	*/
				}
				else
					ERRNO_ERR;			/* errno mismatch				*/
			else {
				PASS;					/* matched expected ret val 	*/
			}
		else
			XVAL_ERR;					/* didn't match expected ret 	*/
/*
 *			exit point for non-fatal errors when expecting explict value
 */
		return;
	}
	if( EXP_VOID ) {
#ifdef TRACE
		sv_printf("\tExpecting: VOID\n");
#endif
		CLR_RVAL;
		CLR_ERRNO;
		PASS;							/* expect void, don't check		*/
	}
	if( EXP_SUCCESS ) {
#ifdef TRACE
		sv_printf("\tExpecting: SUCCESS with %s\n",SSTR);
#endif
		if( SUCC_DEFINED && SUCCEEDED )	
			PASS;						/* expected and got success 	*/
		else 							/* return val outside success	*/
			if( FAIL_DEFINED )
				if( FAILED )			/* E_FAIL						*/
					if( ERRNO_DEFINED )			
						EFAIL_ERR;		/*exp succ, got fail, errno def*/
					else
						FAIL_ERR;		/* exp succ, got fail, no errno*/
				else 					/* E_UNDEF						*/
					SF_IND_ERR;			/* outside range of succ & fail	*/
			else						/* E_UNDEF						*/
				S_IND_ERR;				/* outside success, fail undef	*/
/*
 *			exit point for non-fatal errors when expecting success
 */
		return;
	}
	if ( EXP_FAILURE || exp_errno) {
#ifdef TRACE
		if( exp_errno)
			sv_printf("\tExpecting: FAILURE with errno: %d\n",exp_errno);
		else
			sv_printf("\tExpecting: FAILURE with %s\n",FSTR);
#endif
		CLR_ERRNO;
		if( FAIL_FUNCT && FAILED )
			if( exp_errno )
				if( GOT_ERRNO ) {
						PASS;			/* matched expected/actual errno*/
					}
					else 				/* E_ERRNO						*/
						ERRNO_ERR;		/* expected errno!=actual errno */
			else 
				PASS;					/* expected failure got failure	*/
		else							/* ret outside of failure		*/
			if( SUCC_DEFINED )
				if( SUCCEEDED )			/* E_SUCCESS					*/
					if( ERRNO_DEFINED )
						ESUCC_ERR;		/* exp fail/errno, got success	*/
					else
						SUCC_ERR;	/* exp failure, got success	*/
				else					/* E_UNDEF						*/
					SF_IND_ERR;			/* outside range of succ & fail	*/
			else						/* E_UNDEF						*/
				F_IND_ERR;				/* ret outside range of failure	*/
/*
 *			exit point for non-fatal errors when expecting failure/errno
 */
		return;
	}
}
/*
 *	Possible outcomes for expected success/failure for functions with
 *	strange properties, or which cannot fail.  
 *
 *	Ex. 
 *		exit:
 *			success:	gone		(success is a gone process)
 *			failure:	(-1)
 *			others:		undefined
 *
 *		sync:
 *			success:	return		(success is a continued process)
 *			failure:	void		(can't be checked)
 *	
 *		tolower:
 *			success:	value		(all are success, req. usr val to chk)
 *			failure:	void		(can't occur)
 *
 *	Note that sv_value() should never be called to check an expected 
 *	outcome, it is here as a convenient unique address for a function ptr.
 */

int sv_gone(void)
{
	return(0);	/* no checking possible, fail if process still running */
}

int sv_return(void)
{
	return(1);	/* no checking possible, pass if process still running */
}

int sv_value(void)
{
	return(1);	/* all vals represent success, but should not be called */
}
