#include <pcts.h>

#include <Zunistd.h>
#include <ctype.h>

#include <sv_env.h>
#include <install.h>
#include <chk_mcro.h>

/*	To verify that a macro protects its result with parentheses.
	A safe macro is one that, when expanded, consists entirely
	of a sequence of expressions in matched parentheses, optionally
	interspersed with identifiers and white space.  Two consecutive
	identifiers are not OK (and would not be legal C, in any case).
	However, expressions like "(long) (short) func(x>>8 + y)" are OK.
	Expressions of the form "(stuff)ident(stuff)ident(stuff)" are
	accepted by this program, although it's not clear that any such
	is ever legal C.
*/

char *skipident(char *);
char *skipspace(char *);
char *match(char *);

#define	MACRO_LOCATOR	"MACRO_RTV"

#define	MAX_MACRO_DATA_SZ	2048
char	mac_data[MAX_MACRO_DATA_SZ];

				/*	Parameters:
						e_index	-> Structure containing element name
						fname   -> Path to file containing the macro
				*/
void	chk_mcro(int e_index, char *clause)
{
	char	*p_start, *p_end, *p_advance, *ptr;
	char	system_cmd[_POSIX_PATH_MAX +1];
	char	*path;
	FILE	*sfd;
	int		n=0, c;

			/*	Calling block has established a "break_on(ERROR)",
				therfore, any errors encountered will terminate 
				testing in the chk_mcro() routine.
			*/

	prep("Generate system command to extract macro.");
	start

										/* Ensure data file exists. */
		Zgetcwd(system_cmd, _POSIX_PATH_MAX +1);
		Zstrcat(system_cmd, "/../src/");
		Zstrcat(system_cmd,  element[e_index].e_name);
		Zstrcat(system_cmd, "/");
		Zstrcat(system_cmd, clause);
		Zstrcat(system_cmd, "_exm.c");
		Zaccess(system_cmd, R_OK);

				/*	Create system command:
					"/bin/cc -E ../src/func/10.13_exm.c |
											grep MACRO_RTV > ./10.13.1_mac"
				*/
		Zstrcpy(system_cmd, MACRO_PREPROCESSOR_COMMAND);
		Zstrcat(system_cmd, " ../src/");
		Zstrcat(system_cmd,  element[e_index].e_name);
		Zstrcat(system_cmd, "/");
		Zstrcat(system_cmd, clause);
		Zstrcat(system_cmd, "_exm.c | ");
		Zstrcat(system_cmd, GREP_COMMAND);
		Zstrcat(system_cmd, " ");
		Zstrcat(system_cmd, MACRO_LOCATOR);
		Zstrcat(system_cmd, " > ");
		path = system_cmd +strlen(system_cmd);
		Zstrcat(system_cmd, "./");
		Zstrcat(system_cmd, clause);
		Zstrcat(system_cmd, "_mac");
/*
		prep("Command: %s", system_cmd);
		prep("File: %s", path);
*/
		system(system_cmd);
		SLEEP(WAIT_FOR_TIME);		/* Ensure system() call has completed. */
	finis

	prep("Locate text for macro %s().", element[e_index].e_name);
	start
		sfd = Zfopen(path, "r");
		ptr = mac_data;
		while((c = Zfgetc(sfd)) != EOF && c != '\n') {
			*ptr++ = c;
			if(n++ >= MAX_MACRO_DATA_SZ)
				error("Macro too large for allocated buffer [%d].",
							MAX_MACRO_DATA_SZ);
		}

		if(n == 0)
			error("Macro has no text.");

		if((p_start = strstr(mac_data, MACRO_LOCATOR)) == NULL)
			error("Macro locator not found, PCTS bug.");

		p_start += strlen(MACRO_LOCATOR);
		p_end = p_start +strlen(p_start) -1;

		p_start = skipspace(p_start);		/* Skip leading white space. */
											/* Skip trailing white space. */
		while((p_end > p_start) && isspace(*p_end))
			p_end--;
	finis

	testing("Macro %s(), protects its result value with extra",
						element[e_index].e_name);
	testing("parentheses when necessary.");
	testing("\t%s(): %s", element[e_index].e_name, p_start);
	start

		p_start = skipident(p_start);		/* Skip any leading identifier. */

				/*	Skip matched parentheses and leading identifiers.
					Should end up at or past end.  If not, there's a
					problem and the assertion should fail.
				*/
		while(*p_start && (p_start < p_end) ) {
			if(*p_start != '(')
				error("Unmatched parentheses encountered in macro.");
				/* Skip any number of matched parenthesized expressions */
			while(*p_start == '(') {
				if((p_advance = match(p_start)) == (char *) NULL)
					error("Unmatched parentheses encountered in macro.");
				p_start = ++p_advance;
				if(isspace(*p_start))
					p_start = skipspace(p_start);
			}
						/* If we're at end, OK */
			if(*p_start == '\0' || p_start >= p_end)
				continue;
				/* Otherwise, we'd better have exactly one identifier here */
			p_advance = skipident(p_start);
			if(p_advance == p_start)
				error("Macro %s() not fully protected by parentheses.",
						element[e_index].e_name);
			p_start = p_advance;
		}
	finis
}


/*
** match() returns a pointer to the right parenthesis that balances the
** left parenthesis pointed to by its argument.
*/
char *match(char *leftp)
{
	int	count;
	char	*p;
	
	count = 0;
	p = leftp;
	if(*p != '(')
		return((char *) NULL);
	while(*++p) {
		if(*p == '(')
			count++;
		else
			if(*p == ')') {
				if(count == 0)
					return(p);
			else
				count--;
		}
	}
	return((char *) NULL);
}

/*
** skipident() skips the sequence of letters, digits and underscores that
** start at its argument, and any white space that follows, and returns a
** pointer to the first character that is not one of these.
*/
char *skipident(char *s)
{
	while(isalnum(*s) || (*s == '_' ))
		s++;
	return(skipspace(s));
}

/*
** skipspace(s) skips white space characters at s, returning a pointer to the
** first non-white-space character after s.
*/
char *skipspace(char *s)
{
	while(*s && isspace(*s))
		s++;
	return(s);
}
