#include	<pcts.h>

#include	<sys/Zstat.h>
#include	<Ztime.h>

#include	<sv_env.h>


/*
	This routine determines if a time related field was updated.

	If "check" is TRUE, then
		Verify identified time-related field has changed such that:
			TRF_entity > s_time (Minimum time)
					and
			TRF_entity < s_time +TRF_TEST_TIME_TO_RUN
	If "check" is FALSE, then
		Verify identified time-related field has not been updated:
			TRF_entity <= time0
*/

void trf_ck(int check, int req, time_t s_time, int fd, char *path)
{
	struct	stat st_buf;
	time_t	cur_time;
	char	s1[26]; /* Avoid calling ctime() twice in single function call. */

	if(fd == NO_FILDES)
		Zstat(path, &st_buf);
	else
		Zfstat(fd, &st_buf);
	dt_secs(TRF_DELAY);
	Ztime(&cur_time);

	switch (check) {
	  case	CHK_ATIME:
		strcpy(s1, ctime(&st_buf.st_atime) +4);
	  	if(req) {
			if(st_buf.st_atime <= s_time) {
				error("%s\n%s%20.20s.\n%s%20.20s.\n",
					"Time-related field \"st_atime\" in error?.",
					"Expected minimum time is ",
					ctime(&s_time) +4,
					"Actual time recorded is ",
					s1);
			}
	
			if(st_buf.st_atime > cur_time) {
				error("%s\n%s%20.20s.\n%s%20.20s.\n",
					"Time-related field \"st_atime\" larger than current time.",
					"Current time is ",
					ctime(&cur_time) +4,
					"Time-related field \"st_atime\" is ",
					s1);
			}
	    }
	    else
			if(st_buf.st_atime > s_time ) {
				error("%s\n%s%20.20s.\n%s%20.20s.\n",
					"Time-related field \"st_atime\" has been updated.",
					"Expected minimum time: ",
					ctime(&s_time) +4,
					"Actual time: ",
					s1);
			}
	    break;

	  case	CHK_MTIME:
		strcpy(s1, ctime(&st_buf.st_mtime) +4);
	  	if(req) {
			if(st_buf.st_mtime <= s_time) {
				error("%s\n%s%20.20s.\n%s%20.20s.\n",
					"Time-related field \"st_mtime\" in error?.",
					"Expected minimum time is ",
					ctime(&s_time) +4,
					"Actual time recorded is ",
					s1);
			}
	
			if(st_buf.st_mtime > cur_time) {
				error("%s\n%s%20.20s.\n%s%20.20s.\n",
					"Time-related field \"st_mtime\" larger than current time.",
					"Current time is ",
					ctime(&cur_time) +4,
					"Time-related field \"st_mtime\" is ",
					s1);
			}
	    }
	    else
			if(st_buf.st_mtime > s_time ) {
				error("%s\n%s%20.20s.\n%s%20.20s.\n",
					"Time-related field \"st_mtime\" has been updated.",
					"Expected minimum time: ",
					ctime(&s_time) +4,
					"Actual time: ",
					s1);
			}
	    break;

	  case	CHK_CTIME:
		strcpy(s1, ctime(&st_buf.st_ctime) +4);
	  	if(req) {
			if(st_buf.st_ctime <= s_time) {
				error("%s\n%s%20.20s.\n%s%20.20s.\n",
					"Time-related field \"st_ctime\" in error?.",
					"Expected minimum time is ",
					ctime(&s_time) +4,
					"Actual time recorded is ",
					s1);
			}
	
			if(st_buf.st_ctime > cur_time) {
				error("%s\n%s%20.20s.\n%s%20.20s.\n",
					"Time-related field \"st_ctime\" larger than current time.",
					"Current time is ",
					ctime(&cur_time) +4,
					"Time-related field \"st_ctime\" is ",
					s1);
			}
	    }
	    else
			if(st_buf.st_ctime > s_time ) {
				error("%s\n%s%20.20s.\n%s%20.20s.\n",
					"Time-related field \"st_ctime\" has been updated.",
					"Expected minimum time: ",
					ctime(&s_time) +4,
					"Actual time: ",
					s1);
			}
	    break;

	  	default:
			error("In calling sequence to \"trf_ck\".\n");
	}
}


#include	<Zunistd.h>
#include	<sys/Ztimes.h>
#include	<Ztime.h>

static	int	time_it(clock_t delay)
{
	struct	tms	tmsbuf; 
	clock_t	r_val;

	r_val = times(&tmsbuf);
	sleep(0);				/*	Attempt to place this process at end-of-queue
								of those processes waiting to run.
								
								This allows non-timesharing systems to perform
								a context switch to process waiting processes.
								Thereby improving the timing aspects of many
								tests which expect a process to perform an
								action while another process is measuring the 
								time taken to perform the action.

								This also expects the PCTS to be runing on a
								dedicated processor.
							*/
	while(r_val +delay > times(&tmsbuf))
		;
	return(0);
}
		/*
			Delay specified seconds.
				del_secs(seconds_to_sleep)
			Accuracy is limited to +0, -1 CLK_TCKths.
		*/

int del_secs(int s_delay)
{
	long	clk_tcks;
	clock_t	delay;

	clk_tcks = Zsysconf(_SC_CLK_TCK);
	delay = clk_tcks * s_delay;

	if(!delay)
		return(0);
	time_it(delay);
	return(0);
}


		/*
			Delay specified seconds.
				dt_secs(tenth_of_seconds_to_sleep)
			Accuracy is limited to +0, -1 CLK_TCKths and divisibility
			of the value of CLK_TCK by 10.
		*/

int dt_secs(int s_delay)
{
	clock_t	delay;
	long	clk_tcks;

	if(!s_delay)
		return(0);
	clk_tcks = Zsysconf(_SC_CLK_TCK);
	if(clk_tcks < 10) {
		warning("CLK_TCK has value of %d, dt_secs() will delay a clock tick.",
					clk_tcks);
		
		delay = 1;
	}
	else {
		if(clk_tcks < 40)
			warning("Accuracy of dt_secs() poor, CLK_TCK small value of (%d).",
					clk_tcks);
		delay = (clk_tcks/10) * s_delay;
	}

	time_it(delay);
	return(0);
}
