/*
	This routine will compare the results of running the NIST-PCTS:151-2
	with the expected allowable results and report all discrepancies.
*/

#define	 _POSIX_SOURCE

#include <limits.h>
#include <ctype.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <time.h>
#include <sys/Zutsname.h>

#include <version.h>

#define	MAX_LINE_LENGTH	100
#define	ELEMENT_LEN	14
#define	LEN_TRC	25

#define	TRUE	1
#define	FALSE	0


char	iut_hdw[MAX_LINE_LENGTH];			/* Hardware of IUT */
char	iut_sys[MAX_LINE_LENGTH];			/* System of IUT */

#define	FILLER	"                    "
#define	INDENT	"   "

FILE	*m_sfd, *j_sfd;
char	m_buf[MAX_LINE_LENGTH], j_buf[MAX_LINE_LENGTH];
char	m_elem[ELEMENT_LEN], j_elem[ELEMENT_LEN];
int		m_count, j_count;
int		m_tests, j_tests;


FILE *p_open(char *);
int	val_hdr(int); 
int	g_hdr(int, char *);
int cmp_trc(int, int);
int g_line(char *, FILE *);
void	date_stamp(void);

#define	TAB_STOP1	16
#define	TAB_STOP2	16 
#define	TAB_STOP3	16

#define	ERROR	"ERROR:"
#define	ERROR_N	"ERROR_N:"


int	er_flag;
int	skip_cnt;

char	*J_files[] = {
						"./STD/DEF/adm/journal",
						"./STD/PP/adm/journal",
						"./STD/PE/adm/journal",
						"./STD/FD/adm/journal",
						"./STD/IO/adm/journal",
						"./STD/DC/adm/journal",
						"./STD/LS/adm/journal",
						"./STD/PW/adm/journal",
						"./STD/DIF/adm/journal",
					  };
#define	N_sects	(sizeof(J_files)/sizeof(char *))

char	*M_files[] = {
						"./src/validate/M_files/DEF",
						"./src/validate/M_files/PP",
						"./src/validate/M_files/PE",
						"./src/validate/M_files/FD",
						"./src/validate/M_files/IO",
						"./src/validate/M_files/DC",
						"./src/validate/M_files/LS",
						"./src/validate/M_files/PW",
						"./src/validate/M_files/DIF",
					  };

int	main(int argc, char **argv)
{
	int		i, j;
	char	dummy[20];
	int		first_line;
	int		srch_mode = FALSE;			/*	Journal report on # of tests for an
										elements matches expected value.
									*/
	struct	utsname name;

	argc--, argv++;
	while (argc > 0 && **argv == '-') {
		(*argv)++;
		while (**argv) switch (*(*argv)++) {
			case 'i':
				break;

			default:
				printf("Usage: validate test_reslts\n");
				exit(1);
		}
		argc--; argv++;
	}

	date_stamp();
	printf("\t\t%s, %s\n", NIST_PCTS_NAME, NIST_PCTS_VERSION);
	if(uname(&name) < 0) {
		printf("Call to uname() failed\n.");
		exit(1);
	}
	printf("\t\tSysname: %s, Version: %s, Release: %s\n\n",
					name.sysname, name.version, name.release);

	for(i=0; i<N_sects; i++) {
		if((m_sfd = p_open(M_files[i])) == NULL)
			continue;
		if((j_sfd = p_open(J_files[i])) == NULL)
			continue;
		m_count = j_count = 0;

		val_hdr(i);

		while(TRUE) {							/* Process a section. */
			if(g_line(m_buf, m_sfd) == EOF) {
				break;
			}
			if(sscanf(m_buf, "%s%[ (]%d", m_elem, dummy, &m_tests) != 3)
				printf("%s, line %d in error: %s", M_files[i], m_count, m_buf);

			if(srch_mode == FALSE) {
				if(g_line(j_buf, j_sfd) == EOF) {
					printf(	"\n%s\nNo data provided for element %s.\n",
							J_files[i],
							m_elem);
					break;
				}
				if(sscanf(j_buf, "%s%[ (]%d", j_elem, dummy, &j_tests) != 3)
					printf("%s, line %d in error: %s",J_files[i],j_count,j_buf);
			}

			if(strcmp(m_elem, j_elem)) {
				skip_cnt = TAB_STOP1 -strlen(m_elem) -5;
				printf("%s%s[ALL]%.*sNo_Test_Results\n", INDENT,
								m_elem, skip_cnt, FILLER);

						/*	Journal's report is missing entries for this
							element.  Skip master's test entries for this
							element to attempt to get files in sync.
						*/
				printf("Skip %d lines in %s\n", m_tests +1, M_files[i]);
				for(j=0; j<m_tests+1; j++)
					g_line(m_buf, m_sfd);
				srch_mode = TRUE;
				continue;					/* Try next Journal element. */
			}
			srch_mode = FALSE;					/* Normal processing mode. */

			first_line = FALSE;
			if(m_tests != j_tests) {
				printf("Error: %s %s(%d) vs %s %s(%d).\n",
							M_files[i], m_elem, m_tests,
							J_files[i],  j_elem, j_tests);
										/* Read expected error_n message. */
				if(g_line(j_buf, j_sfd) == EOF)
					break;
				if(strncmp(j_buf, ERROR_N, strlen(ERROR_N))) {
									/* If no ERROR_N:" message, then j_tests
									   entries are available.
									*/
					first_line = TRUE;
				}
				else {
							/* Determine actual number of entries provided. */
							/* XXXXXXX improve code. */
					*(j_buf +42) = '\000';
					j_tests = atoi(j_buf +39);
					if(m_tests != j_tests)
						printf("Processing only %d entries for %s.\n",
									(m_tests<j_tests)?m_tests:j_tests,
									m_elem);
				}
			}

			if(cmp_trc(i, first_line) == EOF)		/* Compare test results  */
				break;
			fflush(stdout);

										/* Skip blank line between elements. */
			if(g_line(m_buf, m_sfd) == EOF)
				break;
			if(*m_buf != '\n') {
				*(strchr(m_buf, '\n')) = '\000';
				printf("ERROR detected in %s\n%s\n%s\"%s\".\n",
								M_files[i],
								"\tExpected: blank line.",
								"\tActual: ",
								m_buf);
				}
			if(g_line(j_buf, j_sfd) == EOF) {
				printf("ERROR detected in %s\n%s\n%s\n",
								J_files[i],
								"\tExpected: additional test results",
								"\tActual: EOF detected prematurely");
				break;
			}
			if(*j_buf != '\n') {
				*(strchr(j_buf, '\n')) = '\000';
				printf("ERROR detected in %s\n%s\n%s\"%s\".\n",
								J_files[i],
								"\tExpected: blank line.",
								"\tActual: ",
								j_buf);
				}
		}
		fclose(m_sfd);
		fclose(j_sfd);
	}

	return(0);
}

							/*	Open input file. */
FILE *p_open(char *path)
{
	FILE	*sfd;
	struct	stat st_in;

	sfd = fopen(path, "r");
	if(sfd == (FILE *)0) {
		printf("Error detected opening \"%s\"\n", path);
		return(NULL);
	}

	stat(path, &st_in);
	if(S_ISDIR(st_in.st_mode)) {
		printf("cr_revs: %s: Is a directory.\n", path);
		fclose(sfd);
		return(NULL);
	}

	return(sfd);
}

							/*	Validate the header for each section for
								the proper format and NIST-PCTS:151-2
								version and patch.
							*/
int	val_hdr(int i) 
{
	static	int init = FALSE;
	char	tmp[MAX_LINE_LENGTH];

	g_hdr(i, "\n");

	sprintf(tmp, "\t\t%s", NIST_PCTS_NAME);
	g_hdr(i, tmp);
	sprintf(tmp, "\t\tRelease: %s", NIST_PCTS_VERSION);
	g_hdr(i, tmp);
	g_hdr(i, "\t\tBased on: IEEE Std 2003.1-1992");
	g_hdr(i, "\t\tHardware Type:");
		if(init == FALSE)
			strcpy(iut_hdw, j_buf);
		else
			if(strcmp(iut_hdw, j_buf))
				printf("\n%s\n\tExpected: \"%.*s\"\n\tActual:   \"%.*s\"\n",
						J_files[i],
						strlen(iut_hdw) -1,
						iut_hdw,
						strlen(j_buf) -1,
						j_buf);
	g_hdr(i, "\t\tSysname:");
		if(init == FALSE)
			strcpy(iut_sys, j_buf);
		else
			if(strcmp(iut_sys, j_buf))
				printf("\n%s\n\tExpected: \"%.*s\"\n\tActual:   \"%.*s\"\n",
						J_files[i],
						strlen(iut_sys) -1,
						iut_sys,
						strlen(j_buf) -1,
						j_buf);
	init = TRUE;
	if(g_hdr(i, "\t\tDate:") == EOF)
		return(EOF);
	g_hdr(i, "\n");
	return(TRUE);
}

int	g_hdr(int i, char *text)
{
	if(g_line(j_buf, j_sfd) == EOF)
		return(EOF);
	if(strncmp(j_buf, text, strlen(text))) {
		printf(	"\n%s\n\tExpected: \"%s\"\n\tActual:   \"%.*s\"\n",
				J_files[i],
				text,
				strlen(j_buf) -1,
				j_buf);
		return(EOF);
	}
	return(TRUE);
}


	/* Compare test result codes for an element and list disrepancies.  */
int cmp_trc(int indx, int first_line)
{
	int		m_tn, t_tn;
	char	m_code1[LEN_TRC], m_code2[LEN_TRC], m_code3[LEN_TRC],
			jf_code[LEN_TRC], jm_code[LEN_TRC];
	int		i;

	for(i=0; i<((m_tests<=j_tests)?m_tests:j_tests); i++) {
		jf_code[0] = '\000';
		jm_code[0] = '\000';
		if(g_line(m_buf, m_sfd) == EOF) {
			printf("%s, %s\n",
					M_files[indx],
					"EOF detected prematurely section processing terminated.");
			return(EOF);
		}
		*m_code2 = '\0';
		*m_code3 = '\0';
		if(sscanf(m_buf, "%d%s%s%s", &m_tn, m_code1, m_code2, m_code3) < 2)
			printf("%s, line %d in error: %s", M_files[indx], m_count, m_buf);
								/*	Was initial line of test results already
									read into the buffer.
								*/
		if(first_line == TRUE)
			first_line = FALSE;
		else
			if(g_line(j_buf, j_sfd) == EOF) {
				printf("%s, %s\n",
					J_files[indx],
					"EOF detected prematurely section processing terminated.");
				return(EOF);
			}
		while( ! strncmp(ERROR, j_buf, strlen(ERROR))) {
			printf("%s -- %s", m_elem, j_buf);
			if(g_line(j_buf, j_sfd) == EOF) {
				printf("%s, %s\n",
					J_files[indx],
					"EOF detected prematurely section processing terminated.");
				return(EOF);
			}
		}
		if(sscanf(j_buf, "%d%s%s", &t_tn, jf_code, jm_code) < 2) {
			printf("%s, line %d in error: %s", J_files[indx], j_count, j_buf);
			continue;
		}
		if(m_tn != t_tn) {
			printf("%sTest numbers do not match: %s[%2.2d] vs %s[%2.2d].\n",
						INDENT,
						m_elem, m_tn,
						m_elem, t_tn);
		}
		
		if(!jf_code[0]) {
			skip_cnt = TAB_STOP1 -strlen(m_elem) -4;
			printf("%s%s[%2.2d]%.*s%-15.15s%-15.15s%-15.15s%-15.15s\n",
						INDENT,
						m_elem, m_tn, skip_cnt, FILLER,
						"MISSING",
						m_code1, m_code2, m_code3);
		}
		else {
			if(strcmp(m_code1, jf_code))
			  if(strcmp(m_code2, jf_code))
				if(strcmp(m_code3, jf_code)) {
					if(!er_flag) {
						printf("\n\n%s", INDENT);
						printf("Element[Test]   ");
						printf("Test_Result    ");
						printf("---Required_Test_Code_Results---\n\n");
						er_flag++;
					}

					skip_cnt = TAB_STOP1 -strlen(m_elem) -4;
					printf("%s%s[%2.2d]%.*s%-15.15s%-15.15s%-15.15s%-15.15s\n",
							INDENT,
							m_elem, m_tn, skip_cnt, FILLER,
							jf_code,
							m_code1, m_code2, m_code3);
				}
		}
		if(jm_code[0]) {
			if(strcmp(m_code1, jm_code))
			  if(strcmp(m_code2, jm_code))
				if(strcmp(m_code3, jm_code)) {
					if(!er_flag) {
						printf("\n\n%s", INDENT);
						printf("Element[Test]       ");
						printf("Test_Result    ");
						printf("---Required_Test_Code_Results---\n\n");
						er_flag++;
					}

					skip_cnt = TAB_STOP1 -strlen(m_elem) -4 -1;
					printf("%s%s[%2.2d]m%.*s%-15.15s%-15.15s%-15.15s%-15.15s\n",
							INDENT,
							m_elem, m_tn, skip_cnt, FILLER,
							jm_code,
							m_code1, m_code2, m_code3);
				}
		}
		
	}

	if(m_tests != j_tests) {		/* Report results have been skipped. */
		if(m_tests > j_tests) {
			printf("Skipped %d lines in %s\n", m_tests -i, M_files[indx]);
			while(i < m_tests) {	/*	Sync Master file with Journal file. */
				g_line(m_buf, m_sfd);
				i++;
			}
		}
		else {
			printf("Skipped %d lines in %s\n", j_tests -i, J_files[indx]);
			while(i < j_tests) {	/*	Sync Journal file with Master file. */
				g_line(j_buf, j_sfd);
				i++;
			}
		}
	}

	return(0);
}


int g_line(char *ptr, FILE *sfd)
{
	fgets(ptr, MAX_LINE_LENGTH, sfd);
	if(feof(sfd))
		return(EOF);
	if(ferror(sfd) != 0) {
		printf("Error detected reading input file.\n");
		exit(1);
	}

	if(sfd == m_sfd)
		m_count++;
	else
		j_count++;
	
	return(0);
}

void	date_stamp(void)
{
	time_t	tm;
	char	*yr,*dt	;

	tm = time((time_t *)0);
	dt = ctime(&tm);
	*(dt + 24) = 0;
	yr = dt + 20;
	*(dt + 19) = 0;
	dt += 4;
	printf("                                               %s %s\n\n", dt, yr);
}
